/*
** Copyright (c) Oracle Corporation 2001-2003. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/

/**
 * LovInput.js: Javascript support library for the ListOfValues template
 *
 * There are no public functions in this library. Any function may be
 * altered or deleted by the Cabo group at any release. Developers should
 * not directly call any of these functions, or use any variable defined here.
 *
 * There are three interfaces that will be supported: the callback functions
 * onLovValidate, onLovInit and onLovSelect.
 *
 * There is, also, an LovOpenWindowAction clientAction that will generate a
 * call into the _LovInputOLW function.
 *
 * @version $Name:  $ ($Revision: 1.66 $) $Date: 2004/02/19 01:27:56 $
 * @author Bud Osterberg (Bud.Osterberg@Oracle.com)
 */

////
////  Reserved Names
////
//  The following names are used by this library, and the lovTemplate, and
// should not be used by the developer on any pages using this library.
//
// _LovEV, _LovFL, _LovFR, _LovHR, _LovIC, _LovLD, _LovLI,
// _LovNM, _LovPT, _LovSC, _LovSF, _LovSL, _LovSR, _LovST, _LovWN
//

////
////  Globals
////
// EV: EVent     - Key string used to save/fetch the event parameter
var _LovEV = "event";
// FL: FieLd     - Key string used to save/fetch the transformed field name
var _LovFL = "field";
// FR: FoRm      - Key string used to save/fetch the form name
var _LovFR = "form";
// HR: HRef      - The URL of the main calling page (where the lovInupt is).
var _LovHR;
// IC: lastIeClick - used to keep the date of the last onClick event
//                   to determine if a second should be processed.
var _LovIC = 0;
// LD: LastDate - used to keep the date of the last onChange event to determine
// if the onClick event should be processed.
var _LovLD = 0;
// LI: LovId - The ID attribute passed in from the listOfValues element.
// var _LovLI;
// NM: NaMe - An object containing the name of the form, name of the
//            field and the transformed name of the field from/to which
//            the data goes. It also contains the partialTargets.
var _LovNM = null;
// PR: PartialRefresh - non-zero if we should use partial updates.
// PT: PartialTarget- The string used to send the partialTargets
//                    parameter on events
var _LovPT = "partialTargets";
// SC: stateCheck - The stateCheck callback passed in from the listOfValues
//                  element.
// var _LovSC;
// SF: Select function - This variable holds the user specified post select
//                       callback. If other than null, it will be called to
//                       transfer data back to the main page, otherwise, the
//                       default callback below will do what it can.
// The select function looks like:
//     function MyCallback(lovwin, formNm, event)
// where lovwin is the LOV window that is closing down
//
//       formNm is an object with the form name, field name, and
//              transformed field name.
// and
//       event  is the window closing event
//
var _LovSF = null;
// SL: SeLected - flag that tells whether a select has occurred.
// var _LovSL = false;
// SR: SouRce    - Key string used to save/fetch the source parameter
var _LovSR = "source";
// ST: SearchText- The string used to send the searchText parameter on events
var _LovST = "searchText";
// WN: WindowName- The name of the LOV modal window
var _LovWN = 'lovWindow';

////
//// Public Functions
////

/**
 * There are no public functions in this library
 */

////
//// Private Functions
////

///
/// Event and Window related functions
///

// VTF: Validate Text Field
//
// This function handles the onChange and onClick events for the lovInput field.
//
// When this is an onChange handler, we know that the user has made some sort
// of update. However, if the user has just cleared out the field, we don't
// want to fire validation as this may lead to a very expensive blind query
// So, we don't initiate validation on a blank field unless explicitly
// requested to do so.
//
// When this is an onClick handler, the user has explicitly requested a lookup.
// We can't just ignore that request, so if the field is blank, we'll validate
// it if the user requests, otherwise we'll directly open the LOV Window.
//
// Once we know we're going to validate, we just build up an URL pointing back
// to the current page, add a partialTarget id pointing to the lovInput text
// field, and fire an lovValidate event using firePartialChange. The event has
// a source parameter (the name of the lovInput element), and a searchText
// parameter (the contents of the field).
//
// In the handler for the lovValidate event, the client should try to find an
// exact match. If one is found, they will update the provider for the
// lovInput's text element and leave the lovInput's isValid attribute set to
// true. Otherwise, they should set up the data providers for the LOV table,
// and set the lovInput's isValid attribute to false.
//
// When the lovInput starts the partial refresh, it will check isValid. When
// everything is valid, it will just update the field with the new valid data.
// When invalid, a piece of javascript will be generated which makes a call to
// the _LovInputOLW function to open the LOV window.
//
//
// Inputs:
//    partialTargets: The set of partial targets to update.
//    validate: A callback in which the client can add/delete/update parameters
//              which will end up on the URL used to fire the partialChange.
//    input:    The lovInput text field DOM object.
//    validateBlanks: By default, we do not send a validation event if the text
//                    field is null or all whitespace. If the validateBlanks
//                    argument is true, we will validate anything.
//    paramObj: The object containing all the parameters needed to open the LOV
//              window. Since this parameter object will ONLY be sent down if
//              the window may be opened, and since the window can only be
//              directly opened in response to a click of the search icon, we
//              know that a valid parameter object means this is an onClick
//              handler.
//
// Returns: Almost always false (to keep from navigating to the top of the
//          page in response to the href="#"). We only return true in the case
//          of an onChange handler and an null field - otherwise we'd get stuck
//          in the field when running IE.
//
function _LovInputVTF(partialTargets,
                      validate,
                      input,
                      validateBlanks,
                      formSubmitted,
                      paramObj)
{
  var _click = (paramObj != (void 0));

  // OK, we've gotten here because one of the following things has happened:
  //   1.) user has changed the data and tabbed out
  //   2.) user has changed the data and clicked search
  //   3.) user has not changed the data but tabbed out
  //   4.) user has not changed the data but clicked search
  //
  // In all four of these cases, the data could be either:
  //   A.) null or all whitespace
  //   B.) some text
  //
  // Furthermore, this function is only called in the PPR environment (non-PPR
  // ignores the onChange and hard-wires onClick to OLW).
  //
  // In the above cases we get the following events:
  //    I.E.:  cases 1 & 2: onChange only; case 3: nothing;
  //           case 4: onClick only
  //    Mozilla:  case 1: onChange only; case 2: onChange followed by onClick;
  //              case 3: nothing; case 4: onClick only
  //
  // We want to validate EVERY onChange of type B, and onChange of typeA
  // only if validateBlanks is true. If we get an onClick right after an
  // onChange, then we're dealing with Mozilla case #2, and we can safely
  // ignore it (the onChange already fired the validation). Otherwise, we
  // have case #4. Here the user has clicked the icon even though there is an
  // existing value in the text field. They must REALLY want the LOV window.
  // In this case we bring up the LOV window without doing any validation.
  //
  // There is one known bug lurking in here. If the user changes the text to an
  // exact match and clicks the icon, we have case #2. We send the validate
  // event, the server recognizes that it's a match, sends back the same text,
  // and we redraw... it appears that *NOTHING* has happened in response to the
  // click. Sure, it's easy to say "just bring up the window in response to an
  // exact match," but we don't want the user to type an exact match, tab out
  // and have a window come up. We can't wait around for the click to come in
  // and process there, because in the case of a tab out, there won't be an
  // onClick coming. Hmmmph! We've addressed this by ALWAYS bringing up the
  // window. _LovInputOLW will detect an existing window, and ignore a second
  // request (the PPR request from the validation) to bring it up.

  // If this is onClick, check for a processed onClick or onChange a short
  // while ago. This filters out Mozilla case #2 and doubleClicks.
  var newDate = new Date();
  var form = null;

  if (_click)
  {
    if (_LovLD)
    {
      var diff = newDate - _LovLD;
      if ((diff >= 0) && (diff < 2000))
      {
        // This event is the second click of a double click which was processed
        // a short while ago. Everything is taken care of, we're done
        return false;
      }
    }
    // We're going to process this event, remember when it happened.
    _LovLD = newDate;
  }
  else if (_agent.isIE)
  {
    // I hate IE! When we set the timeout below in IE, this event gets
    // delivered twice. Skip the second one.
    if (_LovIC)
    {
      var diff = newDate - _LovIC;
      if ((diff >= 0) && (diff < 500))
      {
        return false;
      }
    }
    _LovIC = newDate;
  }

  // The default is to NOT send the validate event
  var _sendValEvent = false;

  // Assume the string is blank
  var strBlank = true;
  if (input)
  {
    // Check if the input string is all whitespace
    if (input.value)
    {
      // If there is any non-whitespace char, this search will
      // return a positive index.
      strBlank = (input.value.search(/\S/) < 0);
    }

    if (strBlank)
    {
      // OK, we did get a blank string. If the client wants to validate it,
      // go ahead.
      if ((validateBlanks != (void 0)) && validateBlanks)
      {
        // The only time we'll ever validate a blank string is if it's
        // changing to a blank string. If the string was blank before, and is
        // still blank, then we just bring up the LOV window.
        _sendValEvent = !_click;
      }
    }
    else
    {
      // The string is not blank. Here if the user has clicked the icon, we
      // always bring up the LOV, we don't do any validation. However, if the
      // user has changed the text, we DO want to validate it.
      _sendValEvent = !_click;
    }

    if (_sendValEvent)
    {
      // Only send the validate event if we've got some partial targets
      if (partialTargets)
      {
        var params = new Object();
        var URL = 0;
        var origParams = new Object();
        // If possible, get the destination from the form
        if (input.form != (void 0))
        {
          form = input.form;
          if (form.action != (void 0))
            URL = form.action;
        }
        URL = _LovInputDDP(URL, params);

        // clone the form action parameters
        for (var paramName in params)
          origParams[paramName] = params[paramName];

        if (input.id)
          params[_LovSR] = input.id;
        else
          params[_LovSR] = input.name;

        params[_LovST] = input.value;
        params[_LovPT] = partialTargets;

        // check for a validate callback, and let it update the params
        if (_LovInputMPC(validate, params, 'lovValidate', false))
        {
          var useSubmit = _LovInputUSF();

          for (paramName in params)
          {
            // Make sure we don't double encode any parameters that may have
            // already been encoded on the form action.
            if ((origParams[paramName] != null)
                && (origParams[paramName] == params[paramName]))
            {
              // This parameter is already on the action with it's current value
              delete params[paramName];
            }
            else
            {
              if (origParams[paramName] != null)
              {
                // This parameter has been updated by the callback
                delete origParams[paramName];
              }

              // The input field and/or any added parameter could have any
              // random character in it and does need to be encoded
              // (Bug #2720832).
              //
              // However, if we're going to submit the form, or if we'll end up
              // building a second form and submitting that, we should not
              // encode the input.
              if (!(useSubmit || formSubmitted))
                params[paramName] = _LovInputENC(params[paramName]);
            }
          }

          // To avoid Bug 2666324, we will not encode any of the original
          // parameters when adding the args to the URL.
          var encodeParams = false;

          if (useSubmit && !formSubmitted)
          {
            // Send down the original action with duplicates/updates removed
            var action = _LovInputUAA(URL, origParams, encodeParams);

            // we will be doing a partial update
            params[_getPartialParameter()] = true;

            // now build a form out of these parameters, and submit it.
            _LovInputSFS(params, form, action);
          }
          else
          {
            // The parameter objects do not overlap. Build one big
            // parameter list, and then build the URL from that.
            for (paramName in origParams)
              params[paramName] = origParams[paramName];

            // finally, fire the event
            var callStr = 'if (!_lovInputREI(new Date()))';
            if (formSubmitted)
            {
              var pStr = '{ ';
              var notFirst = false;
              for (paramName in params)
              {
                if (notFirst)
                  pStr += ", ";
                pStr += "'" + paramName + "' : '" + params[paramName] + "'";
                notFirst = true;
              }
              pStr += ' }';

              callStr += "_submitPartialChange('"
                         + form.name
                         + "', 0, "
                         + pStr
                         + ")";
              // _submitPartialChange(form, 0, params);
            }
            else
            {
              URL = _LovInputUAA(URL, params, encodeParams);
              callStr += "_firePartialChange('" + URL + "')";
              // _firePartialChange(URL);
            }
            // Bug 2807785
            // OK, this validate has happened because the user has entered
            // something into the textInput field, and then left the field. How
            // they left the field is very important. If they tabbed out, or
            // clicked on some whitespace, then we could just go ahead and send
            // the event. However if they clicked on a submit button, then
            // firing this event would disrupt that submit, so we wait a little
            // bit check to see if a submit has started, and if it hasn't,
            // go ahead and do the validate.
            var delay = 200;
            if (_agent.isIE)
              delay = 250;

            _setRequestedFocusNode(document, params[_LovSR], true);
            window.setTimeout(callStr, delay);
          }
        }
      }
    }
    // EVERY click opens the LOV window.
    if (_click)
      _LovInputOLW(partialTargets, paramObj, 1);
    else
      // every change has dirtied the form
      _navDirty = true;
  }
  return (strBlank && !_click);
}
// REI: Recent Event of Interest
//
// returns true if a reset or submit occurred recently
//
function _lovInputREI(nowDate)
{
  return (_recentReset(nowDate) || _recentSubmit(nowDate));
}
// OLW: Open LOV Window
//
// This function is called from the LovInputRenderer to
// open up the new LOV window. This is the main entry point,
// but it still isn't a public function. The interface is
// subject to change at any time.
//
// Inputs:
//   partialTargets: The partial targets to update
//   params: The parameter object containing all the useful fields.
//   direct: True if we got here by way of a direct click, false if through
//           a PPR script.
//
// Returns: nothing useful
//
function _LovInputOLW(partialTargets, params, direct)
{
  var dep = _getDependent(window, _LovWN);
  var paramObj = params;
  var isDirect = (direct == 1);

  if (paramObj == (void 0))
  {
    paramObj = new Object();
  }

  var isAction = _LovInputGPF(paramObj, 'A', false);
  var formName = _LovInputGPF(paramObj, 'F', 0);
  var fieldName = _LovInputGPF(paramObj, 'N', 0);

  // Field name is required when not an action, formName is always required
  if (!((isAction || fieldName) && formName))
    return;

  var fieldID = _LovInputGPF(paramObj, 'L', 0);
  var transFieldName = _LovInputGPF(paramObj, 'T', fieldName);

  var onInit = _LovInputGPF(paramObj, 'I', 0);
  var onSelect = _LovInputGPF(paramObj, 'S', 0);
  var evt = _LovEV;

  // The URL to open in the launched modal LOV window
  // If no destination specified, loop back to calling page.
  var URL = _LovInputGPF(paramObj, 'D', location.href);

  var sTxt;
  if (!isAction)
  {
    var fld = document[formName][transFieldName];
    if (!fld)
      return;

    // Get the search text out of the text input field
    sTxt = fld.value;
  }
  else
  {
    sTxt = _LovInputGPF(paramObj, 'R', '');
  }

  // Callback may or may not be null
  _LovSF = onSelect;
  _LovNM = new Object();
  // The main page has a function called _LovClose on it. This function
  // will end up submitting the lovClose event. Grab it now and store it away.
  _LovNM[_LovFR] = formName;
  _LovNM[_LovFL] = transFieldName;
  if (fieldID)
    _LovNM[_LovSR] = fieldID;
  else
    _LovNM[_LovSR] = fieldName;
  _LovNM[_LovPT] = partialTargets;

  // Check for an existing LOV window. If it already exists, then the user has
  // entered a value in the text field and then clicked the search icon. We
  // already opened a window in response to the click, this OMW call is a PPR
  // call in response to the validation, it can be ignored, the window is
  // already up.
  // We HAVE to make sure that the _LovNM object has been re-populated
  // BEFORE we exit or else no values will be returned.
  if (dep != (void 0))
  {
    return false;
  }

  var submitPrepare = _LovInputGPF(paramObj, 'P', false);

  if (submitPrepare && isDirect)
  {
    _submitPartialChange(formName, 0, {'event':'lovPrepare',
                                       'source':_LovNM[_LovSR],
                                       'searchText':sTxt,
                                       'partialTargets':partialTargets});
  }
  else
  {
    // build up an object which has all the parameters we'll eventually pass
    // we then pass this object off to the developer's init function for
    // any additions/changes.
    var prmObj = new Object();
    var filter = "lovFilter";
    URL = _LovInputDDP(URL, prmObj);

    prmObj[_LovSR] = _LovNM[_LovSR];
    prmObj[_LovST] = sTxt;

    // do the onInit callback
    if (!submitPrepare)
    {
      if (! _LovInputMPC(onInit, prmObj, filter, true))
        return false;
    }
    else
    {
      prmObj[_LovEV] = filter;
    }

    // build up the final URL
    URL = _LovInputUAA(URL, prmObj, true);

    // and save off the calling page URL
    _LovHR = location.href;

    // Finally, open the new window
    _LovInputOMW(URL, _LovInputWCB);
  }

  return false;
}
// GPF : Get Parameter Field
//
// Gets argument parameters from the parameter object given a particular key.
// The field key should be one of the single character argument abbreviations:
//
// A: isAction:       This openWindow call is from an LovOpenWindowAction,
//                    not an lovInput element.
//
// D: Dest:           The destination URL string.
//
// F: Formname:       The name of the form  which contains the initial
//                    searchText value (this is assumed to be at document level
//                    e.g. if the field is document.form1.txtval1.value,
//                    formName would be form1).
//
// I: onInit:         The user defined onLovInit function.
//
// L: fieLdid:        The ID of the field which contains the initial
//                    searchText value. This ID is used to set this field as
//                    a partial refresh target.
//
// N: fieldName:      The name of the field which contains the initial
//                    searchText value (this is assumed to be a child of the
//                    form given in formName. So, if the field is
//                    document.form1.txtval1.value, fieldName would be txtval1,
//                    and formName better be form1).
//
// R: seaRchtext:     The seeded searchText. Only used if isAction is true.
//
// S: onSelect:       The user defined onLovSelect function. See _LovSF above
//
// T: Transfieldname: The transformed version of fieldName. If this field is
//                    in a table, the name will have the table name, and row
//                    number added on. Accessing the field through the DOM
//                    will require this full transformed name. If the field
//                    name is not transformed (ie. transFieldName ==
//                    fieldName) this value will be null.
//
// Inputs:
// paramObj: The object in which we'll search for parameters.
//    field: A one-character field identifier (see above).
//     dflt: Value to return if no parameter is set.
//
// Returns:
//   The value from the lookup, or the given default if the parameter was not
//   set in the parameter object.
function _LovInputGPF(paramObj, field, dflt)
{
  var retVal;
  if (paramObj != (void 0))
  {
    retVal = paramObj[field];
  }

  if (retVal == (void 0))
    retVal = dflt;

  return retVal;
}
// MPC : Manage Parameter Callback
//
// This function executes one of the callback functions that allow the client
// to change the parameters being sent to an event. It checks for any changes
// to the event parameter, and if requested saves the original event name.
//
//
// Inputs:
//   callback:  The onLov{Validate,Init} callback. The signature is
//              callback(object params) where the params is just a
//              bunch of key/value pairs listing the parameters.
//   params:    The parameter object
//   eventName: The name of the event that will be raiesd with these params
//   keepEvent: If true, the client cannot override the event name.
//
// Returns: Whatever the client callback returned. If the client callback
//          returns TRUE, we'll post-process the parameter object. If it
//          returns false, we return immediately.
//
function _LovInputMPC(callback, params, eventName, keepEvent)
{
  var evt = _LovEV;

  // we don't want to send the event over to the callback, so save it locally
  var saveEvent = null;

  // only save an existing event name if requested
  if (keepEvent)
    saveEvent = params[evt];
  delete params[evt];

  if (callback)
  {
    // If the client callback returns false, no more processing, just return
    var rv = callback(params);
    if (!rv)
      return false;
  }

  // Now put the event parameter in. Precedence is:
  //    saved value (if keepEvent is true)
  //    default value
  if (!params[evt])
  {
    // If the callback didn't set any event name
    if (saveEvent)
    {
      //... and there was one on the URL, use it
      params[evt] = saveEvent;
    }
    else
    {
      // otherwise use the default
      params[evt] = eventName; // The default event name
    }
  }
  return true;
}

// OMW: Open Modal lov Window
//
// This function opens up a modal window displaying the destination page
// with the requested callback function attached.
// Because we want things to stay modal, we need to call through the
// FrameRedirect jsp.
//
// Inputs:
//   destination: URL of the page to display
//   callback: A standard window close callback (no third argument!)
//
// Returns: none you should depend on
//
function _LovInputOMW(destination, callback)
{
  // Only allow redirect to another page on this same machine
  var newURL = location.protocol + '//' + location.host;

  // add in the frame redirect JSP
  newURL += _jspDir + _getQuerySeparator(_jspDir) + '_t=';

  if (_LovInputUSF())
  {
    // If we've got a non-UTF encoding on the page,
    // call the recoding version of fred.
    newURL += 'fredRC&';

    if (_enc)
      newURL += "enc=" + _enc + "&";
  }
  else
    newURL += 'fred&';

  // Bug #2464627: Force the size to at least 750 x 550
  newURL +=  '_minWidth=750&_minHeight=550&';

  if (_configName)
    newURL += "configName=" + _configName + "&";

  if (_contextURI)
    newURL += "contextURI=" + _contextURI + "&";

  newURL += 'redirect=';

  if (destination.charAt(0) != '/')
  {
    // The destination is relative, we have to prepend the current directory
    // because we're about to move our base over to the JSP directory.
    var basePath=location.pathname;
    destination = (basePath.substr(0, basePath.lastIndexOf('/') + 1)
                   + destination);
  }

  // This is no longer the first argument, so gotta replace the '?'
  // FredJSP is pedantic enough to replace the delimiter after the
  // 'redirect=' with a '?' for us. If we didn't replace this ?, we'd end
  // up with two on the final URL which would cause much pain.
  newURL += destination.replace('?', '&');

  // OK, open the LOV window as a modal window
  lovw = openWindow(window, newURL, _LovWN, {width:750, height:550},
                    true, 'dialog', callback);

  // initialize these here because it has to be on the LOV window. At this
  // point, nothing has been selected, and the lovSelect event has not been
  // generated.
  lovw._LovSL = false;
  lovw._LovSF = _LovSF;
  lovw._LovNM = _LovNM;
}

//
// PWP: Pass Window Parameters
//
// This grabs parameters from the most likely places
// Inputs:
//   paramName: name of the parameter to get
//   winObj: The window from which to get the parameter
// Returns:
//   The value of the parameter gotten from the
//   windowObject, top, or windowObject's opener in that order.
//
function _LovInputPWP(paramName, winObj)
{
  var rv = window[paramName];

  if (rv  == (void 0))
  {
    if ((winObj != (void 0)) && winObj[paramName])
    {
      rv = winObj[paramName];
    }
    else if (top[paramName])
    {
      rv = top[paramName];
    }
    else if ((winObj.opener != (void 0)) && (winObj.opener[paramName]))
    {
      rv = winObj.opener[paramName];
    }
  }
  return rv;
}

//
// WCB: Windowclose CallBack
//
// This is the default callback for the closing of the LOV window
// Inputs:
//   lovwin: The window that closed calling this function
//   event: The close event (not used)
// Returns:
//   False in PPR to inhibit a redraw, true otherwise.
//
function _LovInputWCB(lovwin, event)
{
  var rv = true;
  // If there has been no selection, we don't send or submit anything.
  if (!lovwin._LovSL)
  {
    _LovInputSTC(lovwin.opener);
    return false;
  }

  // It's tempting to get rid of these PWP calls, and get the params from the
  // place where you KNOW they live, but they live in different places on
  // different browsers. Best to check all the likely places.
  _LovNM = _LovInputPWP('_LovNM', lovwin);

  // If the name object is null, then there has been a partial update of the
  // main page since we brought up the LOV window. This happens in the following
  // scenario: - The user enters enough text to match a single value
  //           - The user clicks the LOV icon
  //           - We send a validate event, and raise the window
  //           - The validate event completes, initiates a PPR redraw of the
  //             main page which hammers out JavaScript context.
  //           - The user selects the only selection available, and clicks the
  //             select button
  //           - We end up here, but there is no context.
  // This isn't really a problem since the PPR update has already updated the
  // text field with the only possible value. There is a bit of a problem here
  // in that if the user entered "San Fra", the LOV window came up with "San
  // Francisco" as the only selection, the user doesn't want that and presses
  // cancel. The user now expects "San Fra" to be in the text field, but it has
  // been updated with "San Francisco".

  if (_LovNM == null)
    return false;

  // If there's been a selection, assume we've dirtied the page
  lovwin.opener._navDirty = true;

  // if we're still in business...
  if (rv)
  {
    var params = new Object();
    params[_LovEV] = 'lovUpdate';
    params[_LovSR] = _LovNM[_LovSR];
    var partialTargets = _LovNM[_LovPT];

    // When done with an LOV window, the BLAF says we should focus on the
    // lovInput's textInput field.
    _setRequestedFocusNode(lovwin.opener.document, params[_LovSR], false);

    if (partialTargets)
    {
      params[_LovPT] = partialTargets;
    }

    // Look for the close function on the main page. The close function will
    // submit the given form with the given parameters.
    // We can't submit the form from here because submitForm expects the
    // document containing the form to be available as plain old 'document'.
    var cf = _LovInputPWP('_lovClose', lovwin);
    var frm = _LovNM[_LovFR];
    if ((cf != (void 0)) && (frm != (void 0)))
    {
      cf(frm, params, (partialTargets));
    }
    else
    {
      // If no close function found (this should never happen), then
      // fallback to a plain old event submission.
      var URL = _LovInputDDP(0, params);
      if (partialTargets)
      {
        URL = _LovInputUAA(URL, params, true);
        _firePartialChange(URL);
      }
      else
      {
        URL = _LovInputUAA(URL, params, true);
        location = URL;
      }
    }
  }
  return false;
}

///
///   URL manipulation functions
///

// DDP: Decompose Destination into Parts
// Inputs:
//   URL: The starting URL to break up
//   parts: An empty object which will be populated with the
//          parameters off the URL. The value will be indexed by the key.
//
// Returns:
//   The base URL (the original URL stripped of all args).
//
function _LovInputDDP(URL, parts)
{
  var base = URL;

  if (!URL)
  {
    if (_LovHR != (void 0))
    {
        base = _LovHR;
    }
    else if ((location != (void 0))
        && (location.href != (void 0)))
    {
        base = location.href;
    }
    else
    {
      // we have no possible recourse here.
      return "#";
    }
  }

  // We sometimes get a '#' added to the end of URLs, delete it here
  // (ONLY if it's the LAST character of the URL!)
  if (base.charAt(base.length - 1) == '#')
  {
    base = base.substr(0, base.length - 1);
  }

  var newURL = base;

  // Find the beginning of the argument section of the current URL
  var sepIndx = base.indexOf('?');

  // If no separator (and hence, no args) found, just keep the original URL
  if (sepIndx > 0)
  {
    // Otherwise, gotta split the args up
    newURL = base.substr(0, sepIndx);
    var query = base.substr(sepIndx + 1); // get just the args, drop the '?'
    var params = query.split('&');          // each arg separately

    for (var i = 0; i < params.length; i++)
    {
      var indx = params[i].indexOf('=');

      if (indx >= 0)
      {
        // If there's an '=', use everything before it as key,
        // and everything after as the value
        parts[params[i].substr(0, indx)] = params[i].substr(indx + 1);
      }
      else
      {
        // otherwise, use the whole param as a key to an empty string
        parts[params[i]] = "";
      }
    }
  }

  return newURL;
}
//
// UAA: URL Add Args
//
// Add all the args in the given object to the given URL
// Inputs:
//   URL: The base URL to which we are making changes.
//   paramObj: an object containing all the key value pairs.
//
// Returns: (string) the resulting new URL
//
function _LovInputUAA(URL, paramObj, encodeParams)
{
  // This URL has been sanitized, all the parameters are in the paramObj
  // Therefore, we know that the URL doesn't have any args,
  // so start with a '?'
  var sep = '?';
  var newURL = URL;

  if (newURL)
  {
    // go through the parameters adding them to the URL one at a time
    for (var key in paramObj)
    {
      var val = paramObj[key];
      newURL += (sep
                 + (encodeParams ? _LovInputENC(key) : key)
                 + '=');
      if (val)
        newURL += (encodeParams ? _LovInputENC(val) : val);
      // every arg after the first will be prefaced with an '&'
      sep = '&';
    }
  }
  return newURL;
}

// STC: Select Text after Cancel
//
// User is cancelling, select the text in the input field and set focus
//
// Inputs: win: The window containing the lovInput field.
//
// Returns: none you should depend on
//
function _LovInputSTC(win)
{
  // First, set the focus to the textInput and select the text.
  var nameObj = _LovInputPWP('_LovNM', win);

  var doc = null;
  var fld = null;
  if (nameObj && nameObj[_LovFR] && nameObj[_LovFL])
  {
    if ((win != null) && (win["document"]))
    {
      doc = win.document;
      fld = doc[nameObj[_LovFR]][nameObj[_LovFL]];
    }
  }

  if (fld)
  {
    // If fld is non null, we know that doc must be valid
    _pprFocus(fld, doc);
    // select all the text in the input field
    // so the user can blast or change it
    if (fld["select"] != null)
      fld.select();
  }
}

///
/// Button functions
///

// CBF: Cancel Button Function
//
// User is cancelling, close the LOV.
//
// Inputs: none
//
// Returns: none you should depend on
//
function _LovInputCBF()
{
  // We done. Go on home.
  top.close();
  return false;
}
// SBF: Select Button Function
//
// This function handles a click of the select button.
//
// Inputs: none
//
// Returns: none you should depend on
//
function _LovInputSBF(event)
{
  // OK, a select has happened, remember it.
  top._LovSL = true;

  // Another Navigator weirdness is that it won't allow alerts from a closing
  // window. We really have to call the onLovSelect callback here before the
  // window closes. This actually worked so well that we now extend do it the
  // same way on every browser.
  var nameObj = _LovInputPWP('_LovNM', top);
  var opn = top.opener;

  // If there's been a selection, assume we've dirtied the page
  opn._navDirty = true;

  var stateCheck = _LovInputPWP('_LovSC', window);
  var closeWin = true;
  if (stateCheck != (void 0) && stateCheck != '')
  {
    closeWin = eval(stateCheck + '(window, "_LOVResFrm")');
  }

  var selFunc = _LovInputPWP('_LovSF', top);

  // Now run the onLovSelect callback if there is one
  if (selFunc)
  {
    // If the client returns false from this callback, it should be treated as
    // if a select never happened.
    top._LovSL = selFunc(window,
                         opn.document[nameObj[_LovFR]][nameObj[_LovFL]],
                         event, opn);
  }

  // Netscape aborts the submit if the window goes away, so we can't just do a
  // submit followed by a top.close because the top.close happens locally, and
  // often completes before the submit. This causes the submit to be dropped.
  //
  // So, we defer the close of the LOV window until after the submit happens.
  // The submit will initiate a redraw, we catch it when the page unloads
  // and close the entire window then.
  //
  // added the top._unloadUIXDialog call to cover the Google pop-up
  // blocker bug where the body's onunload function does not get called
  // (bug 3184718).
  if (closeWin)
  {
    window.onunload = function(){top._unloadUIXDialog(window.event);
                                 top.close();};
  }

  // This *USUALLY* works... well, it doesn't work under Netscape on Solaris.
  // Netscape on Solaris doesn't redraw or deliver an onUnload event after the
  // submit!! Not only that, but calling the callback when the window is
  // closing leads to a Bus Error, so we call the WCB directly. So there!
  if (_agent.isNav && _agent.isSolaris)
  {
    window.onunload = function(){_LovInputWCB(top, event);};
  }
  submitForm(0, 0, {'event':'lovSelect', 'source':_LovLI});
}

// ENC: ENCode parameter text
//
// Encodes parameter text as well as possible on a given platform.
//
// Inputs: The string to encode
//
// Returns: The encoded string
//
function _LovInputENC(str)
{
  // Just Informational.
  // The following is the shift_jis encoding of "nippon" (means Japan).
  //   %93%FA%96%7B
  // And this is the utf8 one.
  //   %E6%97%A5%E6%9C%AC
  var encStr;
  var useEscape = _agent.isNav || _agent.isMac || _agent.atMost("ie", 5.49);

  if (!useEscape)
  {
    // JavaScript 1.5 added the handy encodeURIComponent which should work
    // with multibyte characters.
    encStr = encodeURIComponent(str);
  }
  else
  {
    // Earlier browsers need a little help.

    // There's really no useful function to encode multibyte strings, we do
    // what we can with what we have.

    // First just escape the string
    var escaped = escape(str);

    // Netscape's escape function doesn't encode the '+' char, and it would be
    // lost. Encode it here.
    var lastIdx = 0;
    var idx;
    encStr = "";

    while((idx = escaped.indexOf('+', lastIdx)) > 0 )
    {
      // explicitly encode each '+' char
      encStr += escaped.substring(lastIdx, idx) + '%2B';
      lastIdx = idx + 1;
    }
    // tack on anything that's left over
    encStr += escaped.substring(lastIdx);
  }
  return encStr;
}

//
// QSF: Quick Select Function
//
// This function gets called when the end-user clicks on the Lov Quick Select
// button in the ListOfValues table. It acts as if the user clicked on the
// single-selection radio button for the row, and then pressed the Select
// button. Therefore, this function programmatically selects the single
// selection's radio button for the table/row and then calls the _LovInputSBF
// function.
// Inputs:
//   tableName: The name of the table for which this button was selected.
//   row: zero-based index of the row.
//
// Returns: nothing
//
function _LovInputQSF(tableName, row)
{

  var proxy = new TableProxy(tableName);

  proxy.setSelectedRow(row);

  _LovInputSBF();
}

// USF: Use SubmitForm
//
// Returns true if the encoding, or some other factor, forces us to submit the
// validation event in a form rather than through a partialChange.
//
// Inputs: none
//
// Returns: true if a form should be used to send the event.
//
function _LovInputUSF()
{
  return _enc.toUpperCase() != "UTF-8";
}

// SFS: Second Form Submit
//
// This is a hack to make up for the fact that javascript can't encode to
// anything other than UTF-8. To support other character sets, we create a
// dummy form, move all the pertinent data over into it, and submit. On the
// server it looks the same, but on the client a new entry has been added to
// the history list.
//
// Inputs: params: all the parameters that would have gone on the URL
//         form: The form that the LOV currently lives in (we'll make a dummy
//               form with a name based on this form's name).
//
// Returns: none you should depend on
//
function _LovInputSFS(params, form, action)
{
  var doc = window.document;

  var formID = "_LovInput";
  if (form.id)
    formID += form.id;
  else if (form.name)
    formID += form.name;
  else
    formID += "DummyForm";

  var newForm = doc.createElement("form");
  newForm.id = formID;
  newForm.name = formID;
  newForm.target = form.target;
  newForm.method = form.method;
  if (action)
    newForm.action = action;
  else
    newForm.action = form.action;

  for (var paramName in params)
  {
    // build a hidden field to hold this parameter
    var hiddenField = doc.createElement("input");
    hiddenField.type = "hidden";
    hiddenField.name = paramName;
    hiddenField.value = params[paramName];
    newForm.appendChild(hiddenField);
  }

  doc.body.appendChild(newForm);

  // Bug #3058770
  // The fix for bug 2752257 relies on a form validater being present
  // It doesn't check for anything useful inside the validater, only that
  // there's something there. So, set up a dummy value.
  var val = "_"+ newForm.name + "Validater";
  var del = false;

  // If there's already something there, it'll serve our purpose.
  // Don't overwrite it.
  if (window[val] == (void 0))
  {
    del = true;
    window[val] = 1;
  }

  _submitPartialChange(newForm, 0, params);

  // Now remove the dummy validater
  // IE doesn't support deletion of window elements
  if (del && (!_agent.isIE))
    delete window[val];

  // and clean up the generated form
  doc.body.removeChild(newForm);
}

// SOE: Submit On Enter
//
// A copy of _submitOnEnter (built by the FormRenderer) which takes an event
// name, and source parameter. This function is used as an onKeyPress handler
// for the single searchText field in the basic search area (the advanced
// search mode would have multiple input fields). If the key pressed was the
// enter key, then the specified form will be submitted with the specified
// eventName and Source parameters, and FALSE will be returned (to halt
// further processing). For any other keypress TRUE will be returned.
//
// Inputs:  event: the user event that trigger the call to this function. This
//                 function is only used as a target of an onKeyPress handler,
//                 so this event will always be a keypress.
//           form: The form to submit - really the only form on the LOV page.
// generatedEvent: The event that will be generated when the form is submitted
//                 (currently this is hard-coded to 'lovFilter' in the
//                 listOfValues template.
//         source: The same source parameter that would go with any other
//                 lovFilter event (i.e. the ID of the lovInput element).
//
// Returns: FALSE, after a submit if the keyPress was a <cr>, TRUE otherwise.
//
function _LovInputSOE(event, form, generatedEvent, source)
{
  if (_getKC(event) == 13)
  {
    submitForm(form, 1, {'event': generatedEvent, 'source': source});
    return false;
  }
  return true;
}
