/*
** Copyright (c) Oracle Corporation 2000-2003. All Rights Reserved.
**
** @version $Name:  $ ($Revision: 1.172 $) $Date: 2004/02/06 03:22:47 $
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/

/**
 * Note that the only function in this file that is supported is the openWindow
 * function.
 * All other functions may change without notice.
 */
// Object containing information about the user agent
var _agent = new Object();

// Object for the last time we submitted
var _lastDateSubmitted;

// Object for the last time we reset a form
var _lastDateReset = 0;

// Variables tracking the last time we validated a field, and the last time the
// validation actually failed.
var _lastDateValidated  = 0;
var _lastValidationFailure  = 0;

// Number of submits done on this page. This gives an accurate
// value of the depth of the history list (or distance to the page
// that preceeded this one). Well, it's acurate on Mozilla, it
// has little resemblance to reality on IE.
var _pprSubmitCount = 0;

// IE doesn't add the first partial refresh into the history list.
// Doesn't matter if it's a partialSubmit or a partialChange. However,
// subsequent submits are counted. This variable keeps track of whether we've
// done any action on this page.
var _pprSomeAction = false;

// Number of outstanding partial page request.  Partial
// page rendering keeps this ref count to distinguish
// between real partial page activity and back/refresh.
var _pprRequestCount = 0;

// Flag used by partial page rendering to mark whether the
// parent window has been unloaded.
var _pprUnloaded = false;

// Flag used by partial page rendering and the back issue
// to indicate whether or not we need to restore the saved inline scripts.
var _pprBackRestoreInlineScripts = false;

// We block input while a PPR request is in transit.
// _pprSavedCursor holds the cursor that the client had before the PPR event
var _pprSavedCursor = null;

// _pprSavedBlocking is true if we're blocked waiting on a PPR event
var _pprBlocking = false;

// We block using a special DIV element. This is it's name
var _pprdivElementName = '_pprBlockingDiv';

// A holder for the pending timeout (Gecko only).
var _pprBlockingTimeout = null;

// Keeps track of the last element to initiate a PPR request
var _pprEventElement = null;

// Keeps track of whether the user has actually made a choice from the popup
var _pprChoiceChanged = false;

// Object containing the initial state of a chosen form
var _initialFormState;
var _initialFormExclude = new Object();
var _initialFormStateName;
var _navDirty;

// this is the id of the component which gets the initial focus when the
// page loads.
var _initialFocusID = null;

// Certain UIX facilities can request that focus be set to a particular node,
// or the node AFTER a particular node following a PPR update. These three
// variables store the values needed to track down that node.
var _UixFocusRequestDoc = null;
var _UixFocusRequestID = null;
var _UixFocusRequestNext = false;

// _checkUnload is set on our body tag and is called when the window
// unloads. In our dialog windows, we call _checkUnload via an intermediary
// function _unloadUIXDialog to work around Google's pop-up blocker
// feature which blocks onUnload event handlers from window's opened
// with window.open.
// We don't want to call _checkUnload twice in a row,
// once from dialog code and once from body's onUnload event handler,
// so we block the second call in _unloadUIXDialog.
var _blockCheckUnloadFromDialog = false;


// variables needed if _submitForm was called before the form had
// completely rendered.
var _saveForm = null;
var _saveDoValidate = null;
var _saveParameters = null;
var _submitRejected = false;
var _inPartialSubmit = false;

/**
 * Return true if the agent is at least the specified agent type and version.
 */
function _atLeast(
  kind,
  version
  )
{
  return (!kind    || (kind    == _agent.kind))    &&
         (!version || (version <= _agent.version));
}


/**
 * Return true if the agent is at most the specified agent type and version.
 */
function _atMost(
  kind,
  version
  )
{
  return (kind == _agent.kind) && (version >= _agent.version);
}

function _supportsDOM()
{
  var retVal = false;

  if (_agent.isIE)
  {
    retVal = _agent.version >= 5.5;
  }
  else if (_agent.isNav)
  {
    retVal = false;
  }
  else if (_agent.isGecko)
  {
    retVal = true;
  }
  return retVal;
}

/**
 * initialize information about the agent
 */
function _agentInit()
{
  // convert all characters to lowercase to simplify testing
  var agentString = navigator.userAgent.toLowerCase();

  var version = parseFloat(navigator.appVersion);

  var isOpera   = false;
  var isIE      = false;
  var isNav     = false;
  var isGecko   = false;
  var isSafari = false;
  var kind      = "unknown";
  var isWindows = false;
  var isSolaris = false;
  var isMac     = false;

  if (agentString.indexOf("msie") != -1)
  {
    isIE = true;

    // extract ie's version from the ie string
    var matches = agentString.match(/msie (.*);/);
    version = parseFloat(matches[1]);
    kind = "ie";
  }
  else if (agentString.indexOf("opera") != -1)
  {
    isOpera = true
    kind = "opera";
  }
  else if ((agentString.indexOf("applewebkit") != -1) ||
           (agentString.indexOf("safari") != -1))
  {
    isSafari = true
    kind = "safari";
  }
  else if (agentString.indexOf("gecko/") != -1)
  {
    isGecko = true;
    kind = "gecko";
    version = 1.0;
  }
  else if ((agentString.indexOf('mozilla')    != -1) &&
           (agentString.indexOf('spoofer')    == -1) &&
           (agentString.indexOf('compatible') == -1))
  {
    if (version >= 5.0)
    {
      isGecko = true;
      kind = "gecko";
      version = 1.0;
    }
    else
    {
      isNav = true;
      kind = "nn";
    }
  }

  if (agentString.indexOf('win') != -1)
  {
    isWindows = true;
  }
  else if (agentString.indexOf('mac') != -1)
  {
    isMac = true;
  }
  else if (agentString.indexOf('sunos') != -1)
  {
    isSolaris = true;
  }

  _agent.isIE = isIE;
  _agent.isNav = isNav;
  _agent.isOpera = isOpera;
  _agent.isGecko = isGecko;
  _agent.isSafari = isSafari;
  _agent.version = version
  _agent.kind = kind;
  _agent.isWindows = isWindows;
  _agent.isSolaris = isSolaris;
  _agent.isMac = isMac;

  _agent.atLeast = _atLeast;
  _agent.atMost  = _atMost;
}


_agentInit();

// available features in ie
var _ieFeatures =
{
  channelmode:1, // ie 5.0
  copyhistory:1,
  directories:1,
  fullscreen:1,  // ie 5.0
  height:1,
  location:1,
  menubar:1,
  resizable:1,
  scrollbars:1,
  status:1,
  titlebar:1,    // ie 5.0 when trusted
  toolbar:1,
  width:1
};

// available features in Netscape
var _nnFeatures =
{
  alwayslowered:1,
  alwaysraised:1,
  copyhistory:1,
  dependent:1,
  directories:1,
  height:1,
  hotkeys:1,
  innerheight:1,
  innerwidth:1,
  location:1,
  menubar:1,
  outerwidth:1,
  outerheight:1,
  resizable:1,
  scrollbars:1,
  status:1,
  titlebar:1,
  toolbar:1,
  width:1,
  "z-lock":1
}

// override values for modeless windows. Values in this
// list can't be overridden by the caller for modeless windows
var _modelessFeatureOverrides =
{
};

// override values for modal windows. Values in this
// list can't be overridden by the caller for modal windows
var _modalFeatureOverrides =
{
};


var _featureDefaults =
{
  // default values for features of document windows
  document:
  {
    channelmode:false,
    copyhistory:true,
    dependent:false,
    directories:true,
    fullscreen:false,
    hotkeys:false,
    location:true,
    menubar:true,
    resizable:true,
    scrollbars:true,
    status:true,
    toolbar:true
  },
  // default values for features of dialog windows
  dialog:
  {
    channelmode:false,
    copyhistory:false,
    dependent:true,
    directories:false,
    fullscreen:false,
    hotkeys:true,
    location:false,
    menubar:false,
    resizable:true,
    scrollbars:true,
    status:true
  }
}


// featues that require signbing in order to be set
var _signedFeatures =
{
  alwayslowered:1,
  alwaysraised:1,
  titlebar:1,
  "z-lock":1
};

// features that are boolean values
var _booleanFeatures =
{
  alwayslowered:1,
  alwaysraised:1,
  channelmode:1,
  copyhistory:1,
  dependent:1,
  directories:1,
  fullscreen:1,
  hotkeys:1,
  location:1,
  menubar:1,
  resizable:1,
  scrollbars:1,
  status:1,
  titlebar:1,
  toolbar:1,
  "z-lock":1
};


/**
 * Gets the preferred width of the content
 */
function _getContentWidth(
  element,
  offsetWidth,
  offsetLeft
  )
{
  var children = element.childNodes;

  var isGecko = _agent.isGecko;

  var hasContentProp = (isGecko)
                         ? "tagName"
                         : "canHaveHTML"
  var maxWidth = 0;

  for (var i = 0; i < children.length; i++)
  {
    var currChild = children[i];

    if (currChild[hasContentProp] && (currChild.offsetWidth > 0))
    {
      var currWidth = 0;
      var currOffsetWidth = currChild["offsetWidth"];

      if (isGecko)
      {
        if (currOffsetWidth == offsetWidth)
        {
          currWidth = _getContentWidth(currChild,
                                       currOffsetWidth,
                                       currChild.offsetLeft);
        }
        else
        {
          currWidth = currOffsetWidth;
        }
      }
      else
      {
        currWidth = currChild["clientWidth"];

        if (currWidth == 0)
        {
          currWidth = _getContentWidth(currChild,
                                       currOffsetWidth,
                                       currChild.offsetLeft);
        }
      }

      if (currWidth > maxWidth)
      {
        maxWidth = currWidth;
      }
    }
  }

  // handle error cases
  if (maxWidth == 0)
    maxWidth = offsetWidth;

  // bogusly double the offset to guess the right margin...
  // However, in right to left languages, the left margin can get very large,
  // and cause the window to become huge (Bug 2846393). Limit it to an
  // empirically reasonable value.
  var marginWidth = 10;

  if (_isLTR() || (offsetLeft <= 5))
  {
      marginWidth = 2 * offsetLeft;
  }

  return maxWidth + marginWidth;
}

function _getTop(
  element
  )
{
  if (!_agent.isGecko)
  {
    return top;
  }
  else
  {
    var currWindow = (element)
                       ? element.window
                       : window;

    while (currWindow.parent && (currWindow.parent != currWindow))
    {
      currWindow = currWindow.parent;
    }

    return currWindow;
  }
}

/**
 * Sizes the window to its preferred size.
 */
function _sizeWin(
  element,
  extraWidth,
  extraHeight,
  params
  )
{
  var isGecko = _agent.isGecko;
  var isIE      = _agent.isIE;

  if (!(isGecko || (isIE && _agent.isWindows)))
    return;

  /*
  // =-= bts theoretically, this would be all we need to do for Mozilla,
  //         but the implementation appears to be sub-optimal for our case
  if (isGecko)
  {
    element.window.sizeToContent();
    return;
  }
  */

  var body =  element.window.document.body;

  if (body)
  {
    // width of the inside
    var newWidth = (!isIE && (body.scrollWidth > body.clientWidth))
                     ? body.scrollWidth
                     : _getContentWidth(body, body.offsetWidth, body.offsetLeft);
    var newHeight = 0;

    if (isGecko)
    {
      newHeight = body.offsetHeight + (window.outerHeight - window.innerHeight);

      // random extra space added in to make this work
      newHeight += 30;

      // add in the window frame plus padding
      newWidth  += (window.outerWidth - body.offsetWidth);
    }
    else
    {
      newHeight = body.scrollHeight + (body.offsetHeight - body.clientHeight);

      // if this method supported windows with toolbars, we would need to
      // add another 67 here, rather than the aribitrary 8 pixels
      newHeight += 21;

      // add in the padding plus bogus extra size
      newWidth += body.offsetWidth - body.clientWidth + 16;

      // add in the margins (MS bogusly uses Strings for these)
      newHeight += parseInt(body.topMargin) + parseInt(body.bottomMargin);
      newWidth  += parseInt(body.leftMargin) + parseInt(body.rightMargin);
    }

    //
    // allow the size to be a little bigger than currently necessary.
    // This is useful when we will be paging through multiple items and
    // later pages  might need a slightly larger window than the initial
    // page.
    if (extraWidth)
      newWidth += extraWidth;

    if (extraHeight)
      newHeight += extraHeight;

    // Make sure that width and height are at larger than min requested
    if (params != (void 0))
    {
      if (params['W'])
      {
        var minWidth = 0 + params['W'];
        if (newWidth < minWidth)
          newWidth = minWidth;
      }

      if (params['H'])
      {
        var minHeight = 0 + params['H'];
        if (newHeight < minHeight)
          newHeight = minHeight;
      }
    }

    // element.window.parent.resizeTo(newWidth, newHeight);
    _getTop(element).resizeTo(newWidth, newHeight);
  }
}


/**
 * onClick override for Netscape when modal windows are up.
 * Handles suppressing mouse clicks.
 */
function _onModalClickNN(
  event
  )
{
  if (_getValidModalDependent(self))
  {
    // eat the mouse click if we have a modal window
    return false;
  }
  else
  {
    // allow the mouse click to propagate down if we
    // don't have a modal window
    self.routeEvent(event);
    return true;
  }
}

//
// mozilla event handler for eating click events
var _mozClickEH = new Object();

function _onModalClickMoz(
  event
  )
{
  dump(event);
}

_mozClickEH["handleEvent"] = _onModalClickMoz;


/**
 * onFocus override when modal windows are up.
 * Handles moving the focus to the top and suppressing IE mouse clicks.
 */
function _onModalFocus()
{
  var theBody = self.document.body;

  var modalDependent = _getValidModalDependent(self);

  var hasCapture = _agent.atLeast("ie", 5) && _agent.isWindows;

  if (modalDependent)
  {
    modalDependent.focus();

    // reset the capture on IE since it is released whenever
    // the focus moves to another window
    if (hasCapture)
    {
      theBody.setCapture();
    }
  }
  else
  {
    if (hasCapture)
    {
      theBody.releaseCapture();
    }
  }
}

/**
 * renders transparent image for spacing
 */
function t(width,height)
{

  // if the transparent url is not null render img tag
  if ( _tURL != void 0 )
  {
    document.write('<img src="' + _tURL + '"');

    if (width!=void 0)
      document.write(' width="' + width + '"');
    if (height!=void 0)
      document.write(' height="' + height + '"');

    // if accessibility mode is not null, render alt attribute
    if (_axm != void 0)
      document.write(' alt=""');

    document.write('>');
  }
}

/**
 * Opens a new window, makes it active, and returns the Window
 * object.
 * <p>
 * @param parentWindow The window opening this window.  For modal
 *                     windows, this is the window that the
 *                     new window will be modal to. REQUIRED.
 * @param srcURL       URL for the contents of the new window.
 *                     REQUIRED.
 * @param windowName   Name that identifies the new window for
 *                     URLS. Required if a closeCallback is to be used.
 * @param features     Javascript object containing name/value pairs
 *                     of features to apply to the new window,
 * @param isModal      True if the window should be modal.
 * @param kind         Either 'document' or 'dialog'.  The default is
 *                     dependent on the value of isModal--'document'
 *                     for modeless, 'dialog' for modal.
 * @param closeCallback  Callback method called when the widnow is closed.
 *                      Note: Under certain adverse conditions (for example,
 *                      if the parentWindow navigates to a new page), it is
 *                      possible that the closeCallback may not be called.
 */
function openWindow(
  parentWindow,
  srcURL,
  windowName,
  features,
  isModal,
  kind,
  closeCallback
  )
{
  if (parentWindow)
  {
    // default modality if none specified
    if (isModal == (void 0))
      isModal = false;

    // if the kind isn't specified, default the kind off of the modality
    if (!kind)
    {
      kind = (isModal) ? "dialog" : "document";
    }

    // default the window name to "_blank" if no
    // name is specified, in order to guarantee that a new window is
    // opened
    if (!windowName)
      windowName = "_blank";

    //
    // pick the correct defaults and overrides
    //
    var defaults = _featureDefaults[kind];

    if (defaults == (void 0))
    {
      kind = "document";
      defaults = _featureDefaults[kind];
    }

    var overrides = (isModal)
                      ? _modalFeatureOverrides
                      : _modelessFeatureOverrides;

    // determine the avialable features
    var agentFeatures = (_agent.isIE)
                          ? _ieFeatures
                          : _nnFeatures;

    // we'll be hammering the features object, so make a copy
    var featuresCopy = null;
    if (features)
    {
      featuresCopy = new Object();
      for (var f in features)
      {
        featuresCopy[f] = features[f];
      }
    }

    //
    // build up the feature string
    //
    var featureString = "";

    // loop through the available features for this platform
    for (var featureName in agentFeatures)
    {
      // get the overridden value of the feature
      var featureValue = overrides[featureName];

      if (featureValue == (void 0))
      {
        // get the value of the feature if it isn't overridden
        if (featuresCopy)
        {
          featureValue = featuresCopy[featureName];
          delete featuresCopy[featureName];
        }

        // if no value, get the default value, if any
        if (featureValue == (void 0))
          featureValue = defaults[featureName];
      }

      if (featureValue != (void 0))
      {
        // check if this is a boolean value
        var isBoolean = _booleanFeatures[featureName] != (void 0);

        // output the value
        if (featureValue || !isBoolean)
        {
          // add the feature name
          featureString += featureName;

          // add the value for nonboolean features
          if (!isBoolean)
          {
            featureString += "=" + featureValue;
          }

          // add separator between this and the next
          featureString += ",";
        }
      }
    }

    // Now tack on all the extra features that the user has requested.
    // These may or may not have meaning to the browser's implementation of
    // window.open().
    for (var f in featuresCopy)
    {
      featureString += f;
      if (featuresCopy[f])
        featureString += "=" + featuresCopy[f];

      featureString += ",";
    }

    // trim off last separator
    if (featureString.length != 0)
    {
      featureString = featureString.substring(0, featureString.length - 1);
    }

    // register the closing callback
    if (closeCallback)
    {
      _setDependent(parentWindow, windowName, closeCallback);
    }

    // detect the bogus ie4 and turn off modal windows
    var atMostIE4 = _agent.atMost("ie", 4.99);
    var alphaFilter = false;

    // body of the aprent window
    var parentBody = parentWindow.document.body;

    if (isModal && !atMostIE4)
    {
      if (_agent.atLeast("ie", 4))
      {
        parentBody.style.filter = "alpha(opacity=50)";
        alphaFilter = true;
      }

      if ((_agent.atLeast("ie", 5) && _agent.isWindows))
      {
        parentBody.setCapture();
      }
      else if (_agent.isNav)
      {
        parentWindow.captureEvents(Event.CLICK);
        parentWindow.onclick = _onModalClickNN;
      }
      else if (_agent.isGecko)
      {
        // this should work, but doesn't appear to
        parentBody.addEventListener(Event.CLICK, _mozClickEH, true);
      }

      parentWindow.onfocus = _onModalFocus;
    }

    // open an empty window
    var newWindow = parentWindow.open(srcURL, windowName, featureString);

    /*
    // create the content for the window.  We use a frameset so that
    // the content can change without firing the window close event.
    var realSrcURL = "<html>";

    //realSrcURL += "<head><scr";
    //realSrcURL += 'ipt src="/images/jsLibs/Window.js"></head>';

    realSrcURL += '<frameset rows="100%,*" border="0" onunload="_checkUnload(event)"><frame src="' +
          srcURL +
          '"></frameset></html>';

    newWindow.document.write(realSrcURL);
    newWindow.document.close();
        */

    if (isModal && !atMostIE4)
    {
      _setDependent(parentWindow, "modalWindow", newWindow);
    }

    // make the active window
    newWindow.focus();


    // Set up a timer to make sure that we reset the alpha filter.
    if (alphaFilter)
    {
      parentWindow.setTimeout("_clearAlphaFilter()", 1000);
    }

    return newWindow;
  }
  else
  {
    return null;
  }
}


/**
 * Returns the object containing the dependent windows.
 */
function _getDependents(
  parentWindow,
  createIfNecessary
  )
{
  var depends;

  if (parentWindow)
  {
    depends = parentWindow["_dependents"];

    if (depends == (void 0))
    {
      if (createIfNecessary)
      {
        depends = new Object();
        parentWindow["_dependents"] = depends;
      }
    }
  }

  return depends;
}

/**
 * Get the named dependent
 */
function _getDependent(
  parentWindow,
  dependentName
  )
{
  var depends = _getDependents(parentWindow);
  var dependent;

  if (depends)
  {
    dependent = depends[dependentName];
  }

  return dependent;
}


/**
 * Sets the value of the named dependent
 */
function _setDependent(
  parentWindow,
  dependentName,
  dependentValue
  )
{
  var depends = _getDependents(parentWindow, true);

  if (depends)
  {
    depends[dependentName] = dependentValue;
  }
}


/**
 * Returns the dependent which is modal.
 */
function _getModalDependent(
  parentWindow
  )
{
  return _getDependent(parentWindow, "modalWindow");
}

/**
 * Returns the dependent which is modal and is still open.
 */
function _getValidModalDependent(
  parentWindow
  )
{
  var modalDependent = _getModalDependent(parentWindow);

  if (modalDependent)
  {
    if (modalDependent.closed)
    {
      _setDependent(parentWindow, "modalWindow", (void 0));
      modalDependent = (void 0);
    }
  }

  return modalDependent;
}


/**
 * Returns true if the passed in dependent is the modal dependent
 * of the parent window,
 */
function _isModalDependent(
  parentWindow,
  dependent
  )
{
  return (dependent == _getModalDependent(parentWindow));
}

// A self-rescheduling function which clears out alpha filtering when
// modal windows are closed.  The alpha filter is normally removed in
// the modal window's onunload() handler.  However, the onunload handler
// may not be called if the modal window is closed by the user before
// loading completes.  This function is used in conjunction with
// Window.setTimeout() to make sure that the alpha filter is cleared
// in this case.
function _clearAlphaFilter()
{
  if (_getValidModalDependent(self) != null)
  {
    // If the modal window is still shown, re-schedule ourselves
    self.setTimeout("_clearAlphaFilter()", 1000);
  }
  else
  {
    // No modal dependent - clear out the alpha filter and
    // don't bother rescheduling.
    self.document.body.style.filter = null;
  }
}


/**
 * Called by our modal windows when changes are applied and
 * the window is closed to make sure that the parent window
 * is updated appropriately. Due to bug 3184718 (Google
 * pop-up blocker does not call onUnload event handlers from
 * dialog windows) we cannot rely on _checkUnload being
 * called by the unload event. We call this function
 * instead which in turn calls _checkUnload.
 */
function _unloadUIXDialog(
  event
  )
{
  // _checkUnload is called from body's
  // unload event when
  // We use this flag to keep it from
  // running through _checkUnload function twice.
  // If Google's pop-up blocker is
  // not enabled, then _checkUnload is called from body's
  // unload event as well as from here.

  _blockCheckUnloadFromDialog = false;
  _checkUnload(event);
  _blockCheckUnloadFromDialog = true;
}

/**
 * Called by the unload handler of modal windows to call the event
 * handler and make sure that the parent window is updated appropriately
 * to show that no modal window is up anymore.
 */
function _checkUnload(
  event
  )
{
  // Make sure we don't run through this function twice
  // when we close a dialog. The
  // _unloadUIXDialog function blocks a second run
  // using the _blockCheckUnloadFromDialog flag.

  if (_blockCheckUnloadFromDialog)
  {
    _blockCheckUnloadFromDialog = false;
    return;
  }


  // Check to see if we are a modal window that has been
  // abandoned (who's parent has changed out from under us).
  // In this case, we skip the unload handler, since the
  // modal window no longer has permission to access its
  // parent, and JavaScript errors may occur
  if (_isModalAbandoned())
    return;

  // Check to see if we have an open modal child window
  var modalWindow = _getModalDependent(window);
  if (modalWindow != null)
  {
    // If we are being unloaded before our modal child has been
    // closed, that means that the user must have navigated
    // to a new page just before the modal window was displayed.
    // In this case, let's just close our modal child.  We need
    // to be extra careful to make sure that the modal child does
    // not try to access any properties on the parent window
    // when closing, because the modal child may no longer have
    // permission to access us at this point.  Set a property
    // on the modal window to let it know that it has been
    // abandoned.
    _setModalAbandoned(modalWindow);

    // Now we can safely close the modal window
    modalWindow.close();
  }

  _pprUnloaded = true;

  var topWindow = _getTop();

  if (!topWindow)
    return;

  var parentWindow = topWindow["opener"];

  if (!parentWindow)
    return;

  var unloader = _getDependent(parentWindow, self.name);

  if (_isModalDependent(parentWindow, self))
  {
    // remove the modal window
    _setDependent(parentWindow, "modalWindow", (void 0));

    parentWindow.onfocus = null;

    var parentBody = parentWindow.document.body;

    // release the ie mouse grab
    if (_agent.atLeast("ie", 4))
    {
      if (_agent.atLeast("ie", 5) && _agent.isWindows)
      {
        parentBody.releaseCapture();
      }
      parentBody.style.filter = null;
    }

    // release the netscape mouse grab
    if (_agent.isNav)
    {
      parentWindow.releaseEvents(Event.CLICK);

      // stop eating all of the mouse clicks
      parentWindow.onclick = null;
    }

    if (_agent.isGecko)
    {
      parentBody.removeEventListener(Event.CLICK,
                                     _mozClickEH,
                                     true);
    }
  }

  if (unloader != (void 0))
  {
    // remove our dependent info
    _setDependent(parentWindow, self.name, (void 0));

    // try the passed in event (netscape way first), then
    // try to get the event the IE way
    if (event == (void 0))
      event = self.event;

    // call the unloader with the unloading window and the event
    unloader(topWindow, event);
  }
}

// Tests whether the current window is an "abandoned"
// modal window.  This is a modal window who's parent
// window has navigated to a new page, in which case
// the modal window is out of context.
function _isModalAbandoned()
{
  // We look for the _abandoned property on the modal window.
  // Note that in the LOV case, we actually have two onunload
  // event handlers that need to be suppressed: The onunload
  // handler for the LOV window and the onunload handler for
  // the actual contents of the LOV window which are nested
  // within a frame.  So, we check for the _abandoned property
  // on the "top" window.
  var topWindow = _getTop();
  return topWindow._abandoned;
}

// Marks the specified modal window as abandoned
function _setModalAbandoned(
  modalWindow
  )
{
  // Just set the _abandoned property on the modal window.
  modalWindow._abandoned = true;
}


function _focusChanging()
{
  if (_agent.isIE)
  {
    return (window.event.srcElement != window.document.activeElement);
  }
  else
  {
    // Netscape gives us no good way of determining this
    return true;
  }
}


/**
 * Function that returns a single key/value pair String
 */
function _getKeyValueString(
  target,
  keyName,
  index
  )
{
  var value = target[keyName];

  if (typeof(value) == "function")
  {
    value = "[function]";
  }

  var separator = (_agent.isGecko)
                    ? ((index + 1) % 3 == 0)
                      ? '\n'
                      : '    '
                    : '\t';

  return keyName + ':' + value + separator;
}

function _dump(
  target
  )
{
  dump(target, {innerText:1, outerText:1, outerHTML:1, innerHTML:1});
}

/**
 * Utility for dumping the contents of a JavaScript object.
 */
function dump(
  target,
  suppressProps,
  name
  )
{
  var props = "";

  if (target)
  {
    // default the name if none provided
    if (!name)
    {
      name = target["name"];
    }

    //
    // Because we need to catch exceptions that IE throws if
    // for some object reads, we need to have separate ie and netscape
    // code for the exception catching.  Unfortunately, "try" and
    // catch are reserved words, so we have to dynamically create
    // our adding function so that Netscape doesn't choke when it
    // parses our file
    //
    // var adderContent = "return i + ':' + target[i] + '\\n';";
    var adderContent = "return _getKeyValueString(target, key, index);";

    // wrap adder content with a try and eat the bogus ie exception
    if (_agent.atLeast("ie", 5) || _agent.isGecko)
      adderContent = "try{" + adderContent + "}catch(e){return '';}";

    var adder = new Function("target", "key", "index", adderContent);
    var propCount = 0;
    var propArray = new Array();

    for (var key in target)
    {
      // don't add properties that should be suppressed
      if ((!suppressProps || !suppressProps[key]) && !key.match(/DOM/))
      {
        propArray[propCount] = key;
        propCount++;
      }
    }

    // sort the array so that we can find stuff
    propArray.sort();

    for (var i = 0; i < propArray.length; i++)
    {
      props += adder(target, propArray[i], i);
    }
  }
  else
  {
    // the object to dump was undefined
    name = "(Undefined)";
  }

  // tell the user that the object has no properties
  if (props == "")
  {
    props = "No properties";
  }

  alert(name + ":\n" + props);
}


/**
 * Calls the correct validations function for the form and returns true
 * if the validation succeeded.
 */
function _validateForm(
  form
  )
{
  var funcName = '_' + form.name + 'Validater';
  var formWind = window[funcName];
  if (formWind)
    return formWind(form);

  return false;
}


/**
 * Returns the next sibling that is not a comment
 */
function _getNextNonCommentSibling(
  parent,
  index
  )
{
  var children = parent.children;

  for (var i = index + 1; i < children.length; i++)
  {
    var child = children[i];

    if (child && (child.tagName != "!"))
    {
      return child;
    }
  }

  return null;
}


/**
 * Validate the specified field.
 */
function _valField(
  formName,
  nameInForm
  )
{
  if (nameInForm)
  {
    // get the target whose validation we want to run
    var target = document.forms[formName][nameInForm];

    // get its onblur function
    var blurFunc = target.onblur;

    if (blurFunc)
    {
      var valFunc = blurFunc.toString();

      // whack off the beginning and end of the function, leaving the content
      var valContents = valFunc.substring(valFunc.indexOf("{") + 1,
                                          valFunc.lastIndexOf("}"));

      var targetString = "document.forms['" +
                         formName +
                         "']['" +
                         nameInForm +
                         "']";

      // replace 'this' with the actual target
      valContents = valContents.replace(/this/, targetString);

      // trim off last argument
      var lastArg = valContents.lastIndexOf(",");

      valContents = valContents.substring(0, lastArg) + ")";

      // perform the validation
      eval(valContents);
    }
  }
}

function _validationAlert(errorString)
{
  // Show the error and note the time we finished this validation.
  // Record the validation both before and after the alert so that we
  // halt any validations caused by events triggered along with this
  // one, or by the closing of this alert.
  _recordValidation(true, 0);
  alert(errorString);
  _recordValidation(true, 0);
}

// Records the time of this validation event.
// If fail is set, this is a validation failure, that is noted also.
function _recordValidation(fail, now)
{
  if (!now)
    now = new Date();

  _lastDateValidated = now;
  if (fail)
    _lastValidationFailure = now;
}


// returns true if a validation has occurred "recently"
// failures: True means only report on recent failures, false means report on
//           any validation.
function _recentValidation(failures)
{
  var retVal = false;
  var timeWindow = 250;

  // Assuming that a reasonable user won't close the dialog and change the
  // text within a quarter of a second, we ignore any validations within
  // 250ms. of the last failed validation. The timings I've seen here range
  // in the 60 - 90 ms. range, but that is on fast development machines. We
  // could probably lower this to 150 if we're seeing dropped validations
  // for really fast, ambitious users.
  // With Macintosh IE, we manage to crash the browser!
  if (_agent.isMac)
  {
    // The iBook can have diffs of up to about 480 ms.
    // Call it 600 to be safe.
    timeWindow = 600;
  }

  var newDate = new Date();
  var diff;

  // If failures are requested, the caller is only interested in failures.
  // If simple validation requested, caller interested in any validation fail
  // or not.
  diff = newDate - _lastValidationFailure;
  if ((diff >= 0) && (diff < timeWindow))
  {
    retVal = true;
  }
  else if (!failures)
  {
    diff = newDate - _lastDateValidated;
    if ((diff >= 0) && (diff < timeWindow))
    {
      retVal = true;
    }
  }
  return retVal;
}

/**
 * Field validation function
 * @param err   The error string
 * @param input The field we are validating
 */
function _validateField(
  input,
  validationIndex,
  errorFormatIndex,
  emptyValidation,
  nextSibOK
  )
{
  var isNN = _agent.isNav;

  // don't validate under Netscape if tabbing to the next sibling is OK
  if (isNN && nextSibOK)
  {
    return;
  }

  // Bug #2205664 (also 2210697, and 2465351): Handling these validations is
  // fraught with peril on Netscape. The problem is that onBlur events stack
  // up. The user leaves the text field, an onBlur fires, we raise an alert
  // causing another onBlur to queue up, we finish handling the first,
  // return, and immediately get the second.
  // Bug 2465351 also mentions Mozilla, so it's been added for completeness.
  if (isNN || _agent.isMac || _agent.isGecko)
  {
    if (_recentValidation(false))
      return;
  }

  // determine whether we need to validate the field
  var doValidate = emptyValidation || (_getValue(input) != "");

  // We only validate if we aren't in the middle of validation in order
  // to avoid infinite lopps caused by the fact that the focus has already
  // moved to a new field when the validation fires, and if the validation
  // on the new field failed also, we would ping-pong between the fields
  // forever
  if (doValidate && !window._validating && _focusChanging())
  {
    if (nextSibOK)
    {
      var activeElement = window.document.activeElement;

      if (activeElement)
      {
        var parent = input.parentElement;

        if (parent == activeElement.parentElement)
        {
          var children = parent.children;

          for (var i = 0; i < children.length; i++)
          {
            if (input == children[i])
            {
              doValidate = (activeElement != _getNextNonCommentSibling(parent, i));
            }
          }
        }
      }
    }

    if (doValidate)
    {
      var validationError = _getValidationError(input, validationIndex);

      if (validationError)
      {
        // mark that we are in the middle of validation
        window._validating = input;

        input.select();

        // move the focus back to the failed field before showing the alert
        // to grab the user's attention about what failed.  We don't
        // do this for netscape because doing so in Netscape will cause
        // an infinite loop because Netscape appears to keep the onFocus
        // caused by moving the focus to the field and the onBlur
        // caused by the display of the alert queued up.  This hoses
        // us because it causes the window._validating flag to be cleared
        // and another round of validation to occur
        // ColorField has required validation on hidden field,
        // but cannot receive focus
        if (!isNN && (input.type != 'hidden'))
        {
          input.focus();

          // See if there's a specific position at which validation
          // failed;  if there is, we'll try to select everything after it.
          if (window["_failedPos"] != (void 0))
          {
            // IE style - createTextRange()
            if (input.createTextRange)
            {
              var rng = input.createTextRange();
              rng.moveStart("character", window["_failedPos"]);
              rng.select();
            }
            // Mozilla style.  Sadly, this won't work for TEXTAREA,
            // because of a Mozilla bug.
            else if (input.selectionStart != (void 0))
            {
              input.selectionStart = window["_failedPos"];
            }

            window["_failedPos"] = (void 0);
          }
        }

        // get the error String, if any
        var errorString = _getErrorString(input, errorFormatIndex,
                                          validationError);

        if (errorString)
        {
          // show the error and note the time we finished this validation.
          _validationAlert(errorString);
        }

        // move the focus back to the field after showing the alert for
        // Netscape
        // ColorField has required validation on hidden field,
        // but cannot receive focus.
        if (isNN && (input.type != 'hidden'))
        {
          input.focus();
        }
      }
    }
  }
}


/**
 * Field unvalidation function.
 *
 * If the input object is the currently validating object, reset the
 * validating object so that field validations can continue.  This method
 * is called from the onFocus handler of validating objects.
 */
function _unvalidateField(
  input
  )
{
  if (window._validating == input)
  {
    window._validating = void 0;
  }
}


/**
 * Attempts to submits the form, potentially firing validation and notifying
 * any Cabo onSubmit handlers registered on the form, returning
 * <code>true</code> if the submission actually occurred.
 * <p>
 * If the <code>doValidate</code> parameter is false, no validation will
 * be performed, and the form is guaranteed to be submitted.  Otherwise,
 * the form will be submitted if both the validation succeeds and any
 * registered Cabo onSubmit handlers do not return <code>false</code>.
 * <p>
 * @param form The form to submit.  This can either be the name of the form
 *             in the current <code>document</code>, the index of the form
 *             in the current <code>document</code> or the form itself.
 * @param doValidate boolean value specifying whether validation should
 *   occur before the form is submitted.  (As per a common Javascript
 *   idiom, it is acceptable to pass true/false as well as 0/1).  If
 *   this parameter is ommitted, it defaults to true.
 * @param parameters a single Javascript object that specifies
 *   all the additional key-value pairs to submit.  There must be
 *   pre-existing &lt;input type="hidden"&gt; elements as targets
 *   for each of these parameters.
 */
function submitForm(
  form,
  doValidate,
  parameters
  )
{
  // if the form was passed as a form name, get the form object
  if ((typeof form) == "string")
  {
    form = document[form];
  }
  // if the form was passed as a form index, get the form object
  else if ((typeof form) == "number")
  {
    form = document.forms[form];
  }

  // we had better have a form now
  if (!form)
    return false;

  // Check to see if submitForm is called before the form
  // has been rendered completely. If so, save the parameters
  // and return. At the end of the form, we always call _submitFormCheck
  // which will re-call submitForm if it had been rejected.
  // This is for bug 2752257.

  // Bug #3058770: In LovInput, we have to hack up a form for NLS. It's not
  // validated, so there is no real validater, we've just hacked one. The
  // submit always sets doValidate to false. Just make sure that you never use
  // this validater if doValidate is false (it might just be the value '1').
  var formComplete = window["_"+ form.name + "Validater"];

  if (formComplete == (void 0))
  {
    _saveForm = form;
    _saveDoValidate = doValidate;
    _saveParameters = parameters;
    if (form.target == "_pprIFrame")
    {
      _inPartialSubmit = true;
    }
    _submitRejected = true;
    // Do not submit the form,
    // since the form hasn't been rendered completely yet.
    return false;
  }
  _submitRejected = false; //just in case, clear it.
  _inPartialSubmit = false; //just in case, clear it.

  // Bug 1789483: ignore a second form submission that happens
  // less than 0.5 seconds after the first one
  var newDate = new Date();
  if (_recentSubmit(newDate))
    return;

  _lastDateSubmitted = newDate;

  //
  // default value for doValidate is true
  if (doValidate == (void 0))
    doValidate = true;

  // assume that we should submit the form
  var doSubmit = true;

  // validate the form if necessary, and don't submit the
  // form if validation fails
  if (doValidate && !_validateForm(form))
    doSubmit = false;

  //
  // If we have an onSubmit handler, call it
  //
  var onSubmit = window["_" + form.name + "_Submit"];

  if (onSubmit != (void 0))
  {
    // create function so that "return" is handled correctly,
    var func = new Function("doValidate", onSubmit);

    // install the function on the object so that "this" is
    // handled correctly
    form._tempFunc = func;

    // call the submit handler with the doValidate flag,
    var handlerResult = form._tempFunc(doValidate);

    // uninstall the temporary function
    form._tempFunc = (void 0);

    // if we're validating and the handler returns false,
    // don't submit the form
    if (doValidate && (handlerResult == false))
    {
      doSubmit = false;
    }
  }

  if (doSubmit)
  {
    // reset any hidden form values before submitting
    _resetHiddenValues(form);

    //
    // assign any dynamic values before submitting
    //
    var isDOM = _supportsDOM();
    var tempParams = new Object();

    if (parameters)
    {
      for (var paramName in parameters)
      {
        var paramValue = parameters[paramName];

        if (paramValue != (void 0))
        {
          var hiddenField = form[paramName];

          if (hiddenField)
          {
            hiddenField.value = paramValue;
          }
          else if (isDOM)
          {
            // as a convenience to the client, build a hidden field to hold
            // this parameter.
            var tmpField = document.createElement("input");
            tmpField.type = "hidden";
            tmpField.name = paramName;
            tmpField.value = parameters[paramName];
            form.appendChild(tmpField);
            tempParams[paramName] = tmpField;
          }

        }
      }
    }

    // finally submit the form
    form.submit();

    // And cleanup any temporary fields we provided for the client.
    // ...probably don't need to do this, but...
    for (var paramName in tempParams)
      form.removeChild(tempParams[paramName]);
  }

  return doSubmit;
}

/**
 * Checks if _submitForm had been called before the form had completely
 * rendered, and if so, recall it. This function is rendered at the end of the
 * form, so it is guaranteed that the form is complete when this is called.
 */
function _submitFormCheck()
{

  if (_submitRejected)
  {
    if (_inPartialSubmit)
    {
      _submitPartialChange(_saveForm, _saveDoValidate, _saveParameters);
      _inPartialSubmit = false;
    }
    else
    {
      submitForm(_saveForm, _saveDoValidate, _saveParameters);
    }
    _saveForm = null;
    _saveDoValidate = null;
    _saveParameters = null;
  }
}

/**
 * Attempts to reset the form, calling
 * any reset function calls registered on the form.
 * The form will be reloaded if any
 * reset function call returns <code>true</code>.
 * This function returns <code>true</code> if the page
 * had to be reloaded, and false otherwise.
 * <p>
 * @param form The form to submit.  This can either be the name of the form
 *             in the current <code>document</code>, the index of the form
 *             in the current <code>document</code> or the form itself.
 */
function resetForm(
  form
  )
{
  var doReload = false;

  // if the form was passed as a form name, get the form object
  if ((typeof form) == "string")
  {
    form = document[form];
  }
  // if the form was passed as a form index, get the form object
  else if ((typeof form) == "number")
  {
    form = document.forms[form];
  }

  // we had better have a form now
  if (!form)
    return false;

  var resetCallbacks= window[ "_" + form.name + "_Reset"];

  if (resetCallbacks && !doReload)
  {
    for (var i = 0; i < resetCallbacks.length; i++)
    {
      var trueResetCallback = unescape(resetCallbacks[i]);

      doReload = (eval(trueResetCallback));

    }
  }


  if ( doReload )
  {
    window.document.location.reload();
  }
  else
  {
    form.reset();
  }
  _lastDateReset = new Date();
  return doReload;
}



/**
 * Resets any Marlin-generated hidden values on the form passed in.
 * All hidden form values generated by Marlin on this form are set to
 * the empty string.
 */
function _resetHiddenValues(
  form
  )
{
  var resetFields = window["_reset" + form.name + "Names"];

  if (resetFields)
  {
    for (var i = 0; i < resetFields.length; i++)
    {
      var currField = form[resetFields[i]];

      if (currField)
      {
        currField.value = '';
      }
    }
  }
}


/**
 * Returns the value of a form element.
 */
function _getValue(formElement)
{
  var elementType = formElement.type

  if (!elementType && formElement.length)
  {
    elementType = formElement[0].type;
  }

  if (elementType == "checkbox")
  {
    return formElement.checked;
  }
  else if (elementType.substring(0,6) == "select")
  {
    var selectedIndex = formElement.selectedIndex;

    // selectedIndex exists and non-negative
    if (selectedIndex != (void 0) &&
        selectedIndex != null &&
        selectedIndex >= 0)
    {
      var opt = formElement.options[selectedIndex];
      var value = opt.value;
      if (!value)
      {
        // If there's no value, it could be for two reasons:
        //  (1) The user has only specified "text".
        //  (2) The user explicitly wanted "value" to be empty.
        // We can't really tell the difference between the two,
        // unless we assume that users will be consistent with
        // all options of a choice.  So, if _any_ option
        // has a value, assume this one was possibility (2)
        for (var i = 0; i < formElement.options.length; i++)
        {
          if (formElement.options[i].value)
            return value;
        }

        // OK, none had a value set - this is option (1) - default
        // the "value" to the "text"
        return opt.text;
      }

      return value;
    }

    // no selected value
    return "";
  }
  else if (elementType == "radio")
  {
    if (formElement.length)
    {
      for (var i = 0; i < formElement.length; i++)
      {
        if (formElement[i].checked)
        {
          return formElement[i].value;
        }
      }
    }
    else
    {
      if (formElement.checked)
      {
        return formElement.value;
      }
    }

    // no selected value
    return "";
  }
  else
  {
    return formElement.value;
  }
}


/**
 * Calls an array of validation functions and returns a single error
 * String.
 */
function _multiValidate(
  form,
  validators
  )
{
  var failures = "";

  // We check for any revent validation failures here, not just validations.
  // If a validation has been run on one field in the form (e.g. an onBlur), we
  // still need to run every other validation. However, if that one validation
  // failed, the user has seen one alert, don't bug them with a second til they
  // have fixed the first error.
  if (validators && !_recentValidation(true))
  {
    // get the list of different validations
    var validations = _getValidations(form);

    if (validations)
    {
      var firstFailure = true;

      // loop through the validations, building up the error string
      for (var i = 0; i < validators.length; i += 4)
      {
        // get the current form element to validate
        var currInput = form[validators[i + 1]];

        // if currInput is an array then multiple elements have the same name.
        // Only the first will be validated as subsequent values should be in sync
        var elementType = currInput.type;

        if (!elementType && currInput.length)
        {
          if (currInput[0].type != "radio")
          {
            currInput = currInput[0];
          }
        }

        // determine if we should skip validation if the input value is
        // empty
        var skipIfEmpty = validators[i + 3];

        var value = _getValue(currInput);

        // do the validation if either there isn't an additional
        // required check on this element, or this element has a value
        if (!(skipIfEmpty && (value == "")))
        {
          // evaluate the current submit validation
          var validationError = _getValidationError(currInput, validators[i],
                                                    validations);
          if (validationError)
          {
            // move the focus back to the first failed field
            if (firstFailure)
            {
              // ColorField has required validation on hidden field,
              // but cannot receive focus.
              if (currInput.focus && (currInput.type != 'hidden'))
                currInput.focus();

            if (currInput.type == "text")
                currInput.select();
              firstFailure = false;
            }

             // get the formatted error string for the current input and
             // formatIndex
            var errorString = _getErrorString(currInput,
                                              validators[i + 2],
                                              validationError);

            failures += '\n' + errorString;
          }
        }
      }
    }
    _recordValidation((failures.length > 0), 0);
  }

  return failures;
}


/**
 * Returns the id of an input element on either IE or Netscape, dealing
 * with the fact that Netscape doesn't support IDs locally.
 */
 function _getID(
   input
   )
 {
   if (!_agent.isNav)
   {
     // for non-Netscape return the ID directly
     var id = input.id;

     var inputType = input.type;

     if (!inputType && input.length)
       inputType = input[0].type;

     // for radio buttons, return ID of enclosing <span>
     if (inputType == "radio")
     {
       var inputParent;

       if (input.length)
       {
         inputParent = input[0].parentNode;
       }
       else
       {
         inputParent = input.parentNode;
       }

       id = inputParent.id;
     }

     return id;
   }
   else
   {
     var form = _getForm(input);
     // for loser Netscape, use table lookup
     var nameToID = window["_" + form.name + "_NameToID"];

     if (nameToID)
     {
       var name = _getName(input);
       return nameToID[name];
     }
   }
 }


/**
 * Returns the form of an input element on either IE or Netscape, dealing
 * with the fact that radio inputs do not directly support the form attribute.
 */
 function _getForm(
   input
   )
 {
   var form = input.form;

   if (form == (void 0))
   {
     var inputType = input.type;

     if (!inputType && input.length)
       inputType = input[0].type;

     // for radio buttons, return ID of enclosing <span>
     if (inputType == "radio" && input.length)
     {
       form = input[0].form;
     }
   }

   return form;
 }

/**
 * Returns the name of an input element on either IE or Netscape, dealing
 * with the fact that radio inputs do not directly support the name attribute.
 */
 function _getName(
   input
   )
 {
   var name = input.name;

   if (name == (void 0))
   {
     var inputType = input.type;

     if (!inputType && input.length)
       inputType = input[0].type;

     // for radio buttons, return ID of enclosing <span>
     if (inputType == "radio" && input.length)
     {
       name = input[0].name;
     }
   }

   return name;
 }

/**
 * Return true if the object or any of its prototypes'
 * are an instance of the specified object type.
 */
function _instanceof(
  obj,  // the object instance
  type  // the constructor function
)
{
  if (type == (void 0))
    return false;

  while (typeof(obj) == "object")
  {
    if (obj.constructor == type)
      return true;

    // walk up the prototype hierarchy
    obj = obj.prototype;
  }

  return false;
}

/**
 * Return the formatted error string for an input field
 * and an errorFormatIndex
 */
function _getErrorString(
  input,
  errorFormatIndex,
  validationError
  )
{
  var errorFormat;

  var form = _getForm(input);
  var value = _getValue(input);

  // use the message embedded in the validationError, if any
  if (_instanceof(validationError, window["ParseException"]))
  {
    errorFormat = validationError.parseString;
  }
  else
  {
    // get the list of different error formats
    var errorFormats = window["_" + form.name + "_Formats"];

    if (errorFormats)
    {
      // get the appropriate error format
      errorFormat = errorFormats[errorFormatIndex];
    }
  }

  if (errorFormat)
  {
    // get the mapping of id's to labels
    var labelMap = window["_" + form.name + "_Labels"];

    var label;

    // get the label for this input element, if one has been
    // associated using the ID of the input element
    if (labelMap)
    {
      label = labelMap[_getID(input)];
    }

    // get the mapping of id's to patterns
    var patternMap = window["_" + form.name + "_Patterns"];

    var pattern;

    // get the pattern for this input element, if one has been
    // associated using the ID of the input element
    if (patternMap)
    {
      pattern = patternMap[_getID(input)];
    }

    // format the error string, replacing the following tokens
    //   %value%    the value of the input element
    //   %label%    the value of the label
    //   %pattern%  the validation pattern
    var errorString = _formatErrorString(errorFormat,
                                         {
                                           "value":value,
                                           "label":label,
                                           "pattern":pattern
                                         });
    // return the error
    return errorString;
  }
}


/**
 * Returns the array of validation information used to validate the form at
 * submission time.
 */
function _getValidations(
  form
  )
{
  return window["_" + form.name + "_Validations"];
}


/**
 * Perform the error validation and return true if there is an error.
 */
function _getValidationError(
  input,           // form element to be validated
  validationIndex, // index of validation code
  validations      // the validations array, if available
  )
{
  if (!validations)
  {
    // get the list of different validations
    validations = _getValidations(input.form);
  }

  if (validations)
  {
    var validator = validations[validationIndex];

    if (validator)
    {
      // get the true validator by replacing any value token with
      // the value of the input
      var trueValidator = validator.replace(/%value%/g, "_getValue(input)");

      // return true/ParseException if a validation error has occurred
      return (eval(trueValidator));
    }
  }

  // no error
  return (void 0);
}


/**
 * Performs token replacement on the the error format, replacing each
 * token found in the token Object with the value for that token.
 */
function _formatErrorString(
  errorFormat, // error format string with embedded tokens to be replaced
  tokens       // tokens Object containin token names and values to be replaced
  )
{
  var currString = errorFormat;

  // loop through all of the tokens, replacing them if they are present
  for (var currToken in tokens)
  {
    var currValue = tokens[currToken];

    // if the token has no value, replace it with the empty string
    if (!currValue)
    {
      currValue = "";
    }

    // the tokens are delimited by '%' before and after the token
    var currRegExp = "%" + currToken + "%";

    // support tokens of the form {token} as well as %token%
    currString = currString.replace(new RegExp('{' + currToken + '}', 'g'),
                                    currRegExp);

    // Replace the token.  Don't use String.replace, as the value may
    // include dollar signs, which leads Netscape astray (bug 2242675)
    var indexOf = currString.indexOf(currRegExp);
    if (indexOf >= 0)
    {
      currString = currString.substring(0, indexOf) +
                   currValue +
                   currString.substring(indexOf + currRegExp.length);
    }
  }

  return currString;
}

/**
 * Chain two functions together returning whether the default
 * event handling should occur
 */
function _chain(
  evh1,        // event handler 1 string
  evh2,        // event handler 2 string
  target,      // target of event
  event,       // the fired event
  shortCircuit // shortcircuit if handler 1 false
  )
{
  var result1 = _callChained(evh1, target, event);

  if ( shortCircuit && (result1 == false))
    return false;

  var result2 = _callChained(evh2, target, event);

  // since undefined results should be evaluated as true,
  // return false only if either result1 or result2 return false
  return !((result1 == false) || (result2 == false));
}

function _callChained(
  handler,
  target,
  event
  )
{
  if (handler && (handler.length > 0))
  {
    // handle ie case, where we have no event parameter
    if (event == (void 0))
    {
      event = target.window.event;
    }

    // create function so that "return" is handled correctly,
    // use event parameter so that both ie and netscape
    // functions work
    var func = new Function("event", handler);

    // install the function on the object so that "this" is
    // handled correctly
    target._tempFunc = func;

    // evaluate the result
    var result = target._tempFunc(event);

    // clear the temporary function
    target._tempFunc = (void 0);

    // undefined results should be evaluated as true,
    return !(result == false);
  }
  else
  {
    return true;
  }
}

// Enforce the maximum length of a form element
function _checkLength(formElement, length, event)
{
  elementLength = formElement.value.length;
  if (elementLength > length)
  {
    formElement.value = formElement.value.substr(0, length);
    return false;
  }

  if (elementLength < length)
    return true;

  if ((elementLength == length) && (event.type == 'change'))
    return true;

  if (event && (event.which < 32))
     return true;

  return false;
}


/**
 * Cover for document.getElementById that works on IE 4.x
 */
function _getElementById(
  doc,
  id
  )
{
  //
  // If we arent' on Internet Explorers before 5,
  // use the DOM way of doing this
  //
  if ((_agent.kind != "ie") || (_agent.version >= 5))
  {
    var element = doc.getElementById(id);

    // IE's implementation of getElementById() is buggy.  If
    // the page contains an anchor which has the same name
    // as the requested id, IE will return the anchor, even
    // if the anchor's id attribute is not set.  So, make
    // sure that we actually get back an element with the
    // correct id.
    if ((element == null) || (element.id == id))
      return element;

    // If we get here, that means that IE has probably returned
    // an anchor instead of the desired element.  Let's scan
    // the entire DOM tree to find the element we want.
    return _findElementById(doc, id);
  }
  else
  {
    return doc.all[id];
  }
}

// A recursive method which searches the entire DOM tree
// to find the element with the specified ID
function _findElementById(
  node,
  id
  )
{
  // Check to see if the current node is the node
  // that we are looking for
  if (node.id == id)
    return node;

  // Check all children of the current node
  if (node.childNodes)
  {
    var childNodes = node.childNodes;
    for (var i = 0; i < childNodes.length; i++)
    {
      var foundNode = _findElementById(childNodes.item(i), id);
      if (foundNode != null)
        return foundNode;
    }
  }

  return null;
}

// Returns '?' or '&' depending on whether the
// baseURL already contains a query string
function _getQuerySeparator(baseURL)
{
  var lastChar = baseURL.charAt(baseURL.length - 1);
  if ((lastChar == '&') || (lastChar == '?'))
    return "";

  return (baseURL.indexOf('?') >= 0) ? '&' : '?';
}

/**
 * Adds a parameter to an existing URL, replacing the parameter if
 * it already exists
 */
function _addParameter(
  baseURL,
  paramName,
  paramValue
  )
{
  // check if we have parameters
  var queryIndex = baseURL.indexOf('?');

  if (queryIndex == -1)
  {
    // no parameters, so append to parameters
    return baseURL + '?' + paramName + '=' + paramValue;
  }
  else
  {
    // check if the parameter already exists
    var paramIndex = baseURL.indexOf(paramName + '=', queryIndex + 1);

    if (paramIndex == -1)
    {
      // parameter isn't in the URL
      return baseURL + '&' + paramName + '=' + paramValue;
    }
    else
    {
      //
      // replace the value of the parameter
      //
      var valueIndex = paramIndex + paramName.length + 1;

      // get the URL + the parameter
      var newString = baseURL.substring(0, valueIndex);

      // append the new value
      newString += paramValue;

      var lastIndex = baseURL.indexOf('&', valueIndex);

      if (lastIndex != -1)
      {
        // append the rest of the string after the replaced value
        newString += baseURL.substring(lastIndex, paramName.length);
      }

      return newString;
    }
  }
}

/**
 * Adds a parameter to the parameters object for form submission
 */
function _addFormParameter(
  parameters,
  paramName,
  paramValue
  )
{
  // Always create a new object, since we don't want to mess with
  // the caller's parameters
  var newParameters = new Object();

  // Copy over existing parameters
  if (parameters)
  {
    for (var name in parameters)
      newParameters[name] = parameters[name];
  }

  // Now set the new parameter value
  newParameters[paramName] = paramValue;

  return newParameters;
}

function _firePCUpdateMaster(
  firer,
  tableName,
  baseURL,
  targetID
  )
{
  var selectedVarName = tableName + '_dt';

  // get the name of the selected detail
  var selectedDetailName = window[selectedVarName];

  // we only need to change if these are different
  if (selectedDetailName != firer.id)
  {
    // save our id as the id of the selected detail
    window[selectedVarName] = firer.id;

    // if there is a selected detail, change its icon to unselected
    if (selectedDetailName)
    {
      // get the detail element that is selected
      var selectedDetail = _getElementById(document, selectedDetailName);

      if (selectedDetail)
      {
        // change our icon to selected
        // =-= bts temp hack of URL
        _updateDetailIcon(selectedDetail, '/marlin/cabo/images/cache/c-sdtl.gif');
      }
    }


    // change our icon to selected
    // =-= bts temp hack of URL
    _updateDetailIcon(firer, '/marlin/cabo/images/cache/c-dtl.gif');

    // actual fire the partial change
    _firePartialChange(baseURL, targetID);
  }
}

function _updateDetailIcon(
  sourceElement,
  iconURL
  )
{
  // do this the DOM way
  sourceElement.firstChild.src = iconURL;
}

// Fires a partial change event to the specified URL
function _firePartialChange(baseURL)
{
    var srcURL = _addParameter(baseURL,
                               _getPartialParameter(),
                               "true");

    // For now, we just use a single shared iframe "_pprIFrame".
    var iframe = _getElementById(document, '_pprIFrame');

    // Before we fire, update the request count
    _pprRequestCount++;

    // and block all mouse clicks until the update is done
    _pprStartBlocking(window);

    if (_agent.isIE)
    {
      // iframe.src = srcURL;
      //iframe.ownerDocument.location.replace(srcURL);
      //iframe.document.location.replace(srcURL);

      // ie 5.5 way of doing this
      iframe.contentWindow.location.replace(srcURL);
    }
    else
    {
      // do this a slightly more confusing way for Mozilla
      //dump(iframe.contentDocument.location);

      iframe.contentDocument.location.replace(srcURL);
    }
}

// Fires a partial page request via form submission.
// The args are the same as submitForm().  The
// partialTargets are passed in as parameters
function _submitPartialChange(
  form,
  doValidate,
  parameters
  )
{
  // Get the actual form object
  if ((typeof form) == "string")
    form = document[form];

  if (!form)
    return false;

  // Tack on the "partial" event parameter parameter
  parameters = _addFormParameter(parameters, _getPartialParameter(), "true");

  // We need to set the target of the form to be the PPR iframe.
  // Save the old target and set the new one.
  var oldTarget = form.target;
  form.target = "_pprIFrame";

  // Before we fire, update the request count
  _pprRequestCount++;

  var delta = 0;

  // IE only adds to the history list if something has already been done on
  // this page, so until there has been some action, we don't
  // increment/decrement at all.
  if ((!_agent.isIE) || parent._pprSomeAction)
  {
    delta = 1;
  }
  _pprSubmitCount += delta;

  parent._pprSomeAction = true;

  // block all mouse clicks until the submit is done
  _pprStartBlocking(window);

  // Submit the form
  var submitted = submitForm(form, doValidate, parameters);

  // If the form wasn't actually submitted, update the ref count
  if (!submitted)
  {
    _pprStopBlocking(window);
    _pprRequestCount--;
    _pprSubmitCount -= delta;
  }

  // Reset the form target
  form.target = oldTarget;
}

/**
 * Returns the encoded value of the "partial" event
 * parameter.
 */
function _getPartialParameter()
{
  // Check to see we've got a variable which defines the partial parameter
  if (window._pprPartialParam)
    return window._pprPartialParam;

  return "partial";
}

/**
 * Equivalent of targetItem.outerHTML = sourceItem.outerHTML for platforms
 * that don't support outerHTML, but do support innerHTML, or platforms that
 * don't suppoer outerHTML on all elements
 */
function _setOuterHTML(
  targetDocument,
  targetItem,
  sourceItem
  )
{
  var sourceTagName = sourceItem.tagName;

  if (_agent.isIE)
  {
    var useOuter = true;

    // is this a special IE leaf item that doesn't support outerHTML
    var isLeafItem = ((sourceTagName == "TD")      ||
                      (sourceTagName == "TH")      ||
                      (sourceTagName == "CAPTION"));

    // is this a special IE container item, that doesn't support
    // inner or outerHTML
    var isContainerItem = !isLeafItem &&
                          ((sourceTagName == "COL")      ||
                           (sourceTagName == "COLGROUP") ||
                           (sourceTagName == "TR")       ||
                           (sourceTagName == "TFOOT")    ||
                           (sourceTagName == "THEAD")    ||
                           (sourceTagName == "TBODY"));

    if (isLeafItem || isContainerItem)
    {
      // create an element with the correct attributes
      var newTargetItem = targetDocument.createElement(sourceTagName);
      // mergeAttributes, including the id
      newTargetItem.mergeAttributes(sourceItem, false);

      // copy over the content
      if (isLeafItem)
      {
        newTargetItem.innerHTML = sourceItem.innerHTML;
      }
      else
      {
        if (isContainerItem)
        {
          //
          // add all of the cloned child elements
          // firstChild gets the comments as elements. we will
          // need to filter those out.
          //
          var currCell = sourceItem.firstChild;

          while (currCell != null)
          {
            // add the cloned cell.
            // filter out the comment tag.
            while(currCell != null && currCell.tagName == "!")
            {
              currCell = currCell.nextSibling;
            }

            newTargetItem.appendChild(_setOuterHTML(targetDocument,
                                                    null,
                                                    currCell));


            currCell = currCell.nextSibling;

          }
        }
      }

      // replace the current child with the new child
      if (targetItem)
      {
        targetItem.parentNode.replaceChild(newTargetItem, targetItem);
      }
      else
      {
        targetItem = newTargetItem;
      }

      useOuter = false;
    }

    if (useOuter)
    {
      targetItem.outerHTML = sourceItem.outerHTML;
    }
  }
  else
  {
    // create the new Target item using a namespace to create this item
    // screws up Mozilla!
    var newTargetItem;

    // treat tr differently, since there is a Mozilla bug regarding innerHTML
    // and tr. It seems to strip out all the tds.
    if (sourceTagName != 'TR')
    {
      newTargetItem = targetDocument.createElement(sourceTagName);

      // Mozilla has a bug with copying innerHTML for the select element (#100175)
      // and it also has a problem when we use cloneNode (the scripts that
      // are in the iframe that get eval'd sometimes don't seem to get eval'd
      // correctly, and we get a missing function error). So, this is what
      // we came up with to work around that.
      if (sourceTagName == 'SELECT')
      {
        // the multiple attribute does not get copied over when we copy over
        // the source attributes, so do it here. This way when we set the
        // selected options below, multiple selections will be allowed.
        if (sourceItem.multiple)
        {
          newTargetItem.multiple = sourceItem.multiple;
        }
        for (var i = 0; i< sourceItem.options.length; i++)
        {
          var sourceOption = sourceItem.options[i];
          var newOption = new Option();

          newOption.value = sourceOption.value;
          newOption.text = sourceOption.text;
          newOption.selected = sourceOption.selected;


          newTargetItem.options[i] = newOption;
        }


      }
      else
      {
        // copy over the inner html
        var sourceInnerHTML = sourceItem.innerHTML;

        if ((sourceInnerHTML != null) && (sourceInnerHTML.length > 0))
        {

          newTargetItem.innerHTML = sourceItem.innerHTML;

        }

      }

      //
      // set the attributes of the source node
      //
      var sourceAttrs = sourceItem.attributes;

      for (var i = 0; i < sourceAttrs.length; i++)
      {
        newTargetItem.setAttribute(sourceAttrs[i].name, sourceAttrs[i].value);
      }


    }
    else
    {
      // for tr, use the importNode function. I'm limiting its
      // use to tr in case it is buggy, and to reduce the cases
      // where it is used. It might be a good thing to use in
      // the future. =-=jmw
      newTargetItem = targetDocument.importNode(sourceItem, true);

    }



    // Replace the current child with the new child.
    // We would like replaceChild(), but this causes problems on
    // Mozilla.  For example, in the PPR validation demo, if we use
    // replaceChild(), the validation demo contents disappear!  Using
    // insertBefore() followed by removeChild() seems to work just
    // fine though...
    // targetItem.parentNode.replaceChild(newTargetItem, targetItem);

    targetItem.parentNode.insertBefore(newTargetItem, targetItem);
    targetItem.parentNode.removeChild(targetItem);


  }

  return targetItem;
}

/**
 * Unload event handler for the partial iframe
 */
function _partialUnload()
{
  // We try to detect back events and jump back to the last
  // fully rendered page.  We consider an unload to be the
  // result of a back or refresh event if the _pprRequestCount
  // has not been incremented.  Note: If the parent window has
  // been unloaded (_pprUnloaded), this must be a refresh, so
  // do nothing.
  if ((parent._pprRequestCount <= 0) && !parent._pprUnloaded)
  {
    // This should be a no-op, but just in case...
    _pprStopBlocking(parent);

    if (!(_agent.isIE) && (parent.document.referrer != null))
    {
      // If we have a referrer, it's safer to jump to it than to try to
      // figure out how deep we are in the history list.
      // Although Microsoft claims that history.go takes either an integer or a
      // string, it appears that only the integer argument works.
      parent.history.go(parent.document.referrer);
    }
    else
    {
      var backOffset = -1;

      if (_agent.isIE)
      {
        // I'm sure this will come as a big surprise, but IE's history mechanism
        // is busted. It never seems to grab the first submit unless some other
        // action has taken place...
        if (parent._pprSomeAction)
        {
          backOffset = -(parent._pprSubmitCount);
        }
      }
      else if (parent._pprSubmitCount && (parent._pprSubmitCount > 0))
      {
        // On Mozilla, the submitCount will be accurate
        backOffset -= parent._pprSubmitCount;
      }

      // we're about to jump back, reset the submit (page) count.
      parent._pprSubmitCount = 0;
      parent._pprSomeAction = false;

      // Only jump back if the offset is actually negative
      if (backOffset < 0)
      {
        parent.history.go(backOffset);
      }
    }
  }
}

function _partialRedirect(URL)
{
  // ER #2603173
  // If an error occurred during a partial operation, the client
  // can call the method PartialPageUtils.forceRedirectURL(context, url)
  // with an URL to which we can redirect to show the error.
  if (URL && (parent._pprRequestCount > 0))
  {
    if (((typeof URL) == "string") && (URL.length > 0))
    {
      // We're going to an error page, make sure we clear all
      // tracking variables before the onUnload gets called.
      parent._pprRequestCount--;
      parent._pprSubmitCount = 0;
      parent._pprSomeAction = false;
      parent.location.href = URL;
      _pprStopBlocking(parent);
    }
  }
}


/**
 * function used to store the global variables
 * needed to load the libraries for IE
 */
function _pprLibraryStore(numberOfLibraries)
{
  // keep track of which libraries are loaded
  this.loadedStatus = new Array(numberOfLibraries);

  for (var i=0; i < numberOfLibraries; i++)
    this.loadedStatus[i] = false;

  // keep track of the total number of libraries to load
  this.total = numberOfLibraries;

  // an array to store each library script
  this.allLibraries = new Array(numberOfLibraries);
}
// stores the variables needed to load the libraries for IE
var _pprLibStore;

/**
 * This callback function gets called each time
 * a library is downloaded. This is for IE only.
 * index - an index which indicates which library is being loaded.
 *     s - the library contents.
 * Store the library scripts in an array. When they are all downloaded
 * execute the contents of each library.
 */
function _pprExecScript(index, s)
{
  if (_pprLibStore && _pprLibStore.allLibraries != (void 0))
  {
    _pprLibStore.allLibraries[index] = s;
    _pprLibStore.loadedStatus[index] = true;

    // Check to see if all the libraries are loaded. Start at the end
    // of the array, since those will most likely be the last loaded, so
    // we can return quicker.
    for (var index = _pprLibStore.total-1; index >= 0; index--)
    {
      // well, we see that there is a library not loaded, so return. When
      // they are all loaded, then we'll pass through and execute them.
      if (!_pprLibStore.loadedStatus[index])
        return;
    }

    // all the libraries are loaded, so loop through each and evaluate it.
    for (var i=0; i < _pprLibStore.total; i++)
    {
     parent.window.execScript(_pprLibStore.allLibraries[i]);
    }
    _pprLibStore = null;

  }
}

/**
 * returns an array with the libraries that need to be loaded due to the
 * PPR request and are not already cached.
 */
function _createToLoadArray()
{
  var toLoadArray = new Array();
  var toLoadIndex = 0;

  if (window["_pprLibraries"] != (void 0))
  {
    // loop through each library in _pprLibraries
    // if it is not in the cached libraries list, then
    // add it to an array which indicates that the library needs to be loaded.
    for (var i=0; i < _pprLibraries.length; i++)
    {

      if ((parent._cachedLibs == null)
          || (parent._cachedLibs.indexOf(_pprLibraries[i]) == -1))
      {
        toLoadArray[toLoadIndex++] = _pprLibraries[i];
      }
    }
  }

  return toLoadArray;
}

function _addLibraryToCache(libraryName)
{
  // always load ScriptEval, so don't put it in the _cachedLibs list.
  if ((libraryName.indexOf("ScriptEval")) == -1)
  {
    if (parent._cachedLibs == null)
      //=-=jmw does not work in some cases in IE for some unknown reason
      // parent._cachedLibs = new String(libraryName);
      parent._cachedLibs = "" + libraryName;
    else
      parent._cachedLibs += "," + libraryName;
  }
}

// Given the targetDocument and the array of libraries to load,
// download the libraries using IE's startDownload function.
function _loadScriptLibrariesIE(targetDocument, toLoadArray)
{
  if (toLoadArray == null) return;

  var downloadElement   = _getElementById(targetDocument, "_uixDownload");

  if (downloadElement == null) return;

  var numLibraries = toLoadArray.length;

  // initialize the global variables that are needed to load the libraries
  // in the callback.
  _pprLibStore = new _pprLibraryStore(numLibraries);

  // For each library, start the download and set up the callback
  // which will get called when the download is complete.
  for (var i = 0; i < numLibraries; i++)
  {
    var scriptStatement = "_pprExecScript(" + i + ", s);"
    // create a callback function that takes "s" as an argument
    // (this will be the script's content in string form) and
    // takes the scriptStatement as its body. This callback
    // function will then turn around and call _pprExecScript(i,s),
    // where i is the library index and s is the library string
    var downloadCallback = new Function("s",scriptStatement );
    downloadElement.startDownload(toLoadArray[i], downloadCallback);

    _addLibraryToCache(toLoadArray[i]);

  }
}

/**
 * Download the libraries that are defined in the iframe
 * into the targetDocument. The method for Gecko is
 * to create a script element in the targetDocument for
 * each library.
 */
function _loadScriptLibrariesGecko(targetDocument, toLoadArray)
{
  // we first walk through the list of library paths that are stored in
  // the _pprLibraries variable in the iframe.
  // As we get each path, we create a script element
  // with the library path as the src and add it to the target document.

  var iFrameElement   = _getElementById(targetDocument, "_pprIFrame");

  if (iFrameElement)
  {
    for (var i = 0; (i < toLoadArray.length); i++)
    {
      // first check to see if this library is already loaded.
      // if so, don't bother to load it again. However, always load
      // ScriptEval

       var newScriptElement= targetDocument.createElement("script");
       newScriptElement.setAttribute('src', toLoadArray[i]);

       iFrameElement.parentNode.insertBefore(newScriptElement, iFrameElement);

      _addLibraryToCache(toLoadArray[i]);
    }
  }
}


/**
 * Download the libraries that are defined in the iframe
 * into the targetDocument. The last library will
 * always be ScriptEval.js, which will evaluate the
 * scripts defined in the _pprScripts variable.
 */
function _loadScriptLibraries(targetDocument)
{
  if (window["_pprLibraries"] != (void 0))
  {
    var toLoadArray = _createToLoadArray();

    if (toLoadArray.length > 0)
    {
      if (_agent.isIE)
      {
        _loadScriptLibrariesIE(targetDocument, toLoadArray);
      }
      else
      {
        _loadScriptLibrariesGecko(targetDocument, toLoadArray);
      }
    }
  }
}


/**
 * Onload event handler for the partial iframe
 */
function _partialChange(navigationFormName)
{
  // If we don't have an outstanding request, don't do
  // anything.  Technically we shouldn't see this case,
  // but this might come up if our back button/history
  // handling code fails.
  if (parent._pprRequestCount <= 0)
    return;

  // Update the request count
  parent._pprRequestCount--;
  parent._pprSomeAction = true;

  // We need to do some pre-processing... Fix up links
  if (navigationFormName)
    _fixAllLinks(navigationFormName, parent);

  // use source document, since Mozilla claims that
  // the document object no longer exists
  // when we call write and close at the end
  // of this method
  var sourceDocument = document;
  var targetDocument = parent.document;

  var oldActiveElement = _getParentActiveElement();
  var newActiveElement = null;

  // We want to keep track of whether we are restoring the focus
  // to the actual component which should have the focus, or just
  // to the first focusable component that we can find.
  var isFirstFocusable = false;

  for (var i = 0; i < _pprTargets.length; i++)
  {

    var targetID = _pprTargets[i];

    //
    // copy over the content
    //
    var sourceItem = _getElementById(sourceDocument, targetID);
    var targetItem = _getElementById(targetDocument, targetID);

    if (sourceItem && targetItem)
    {
      // If the old activeElement is a descendent of the node that
      // is being replaced, we'll need to restore the focus after
      // we are done replacing content.
      var restoreFocus = _isDescendent(oldActiveElement, targetItem);

      _setOuterHTML(targetDocument, targetItem, sourceItem);

      // If that activeElement was in the subtree that was just
      // replaced, figure out what the new activeElement should be.
      if ((restoreFocus) && (newActiveElement == null))
      {
        // Make sure we have a current reference to the target item.
        // It seems that the call _setOuterHTML() can cause our
        // targetItem reference to become invalid.  The end result
        // is that we are not able to successfully traverse the
        // subtree under targetItem.  Getting the targetItem from
        // the document after _setOuterHTML() has been called fixes
        // this problem.
        targetItem = _getElementById(targetDocument, targetItem.id);

        // Get the new element to set the focus to
        newActiveElement = _getNewActiveElement(targetDocument,
                                                targetItem,
                                                oldActiveElement);

        if (newActiveElement == null)
        {
          newActiveElement = _getFirstFocusable(targetItem);
          if (newActiveElement != null)
            isFirstFocusable = true;
        }
        // if we're here, we'll take care of resetting the focus,
        // no need for the unblock to do it.
        parent._pprEventElement = null;
      }
    }
  }
  // Load the script libraries into target document
  _loadScriptLibraries(targetDocument);

  _saveScripts(targetDocument);

  // unblock
  _pprStopBlocking(parent);

  // Restore focus to new active element
  var req = _getRequestedFocusNode();
  if (req != null)
    newActiveElement = req;

  _restoreFocus(newActiveElement, isFirstFocusable, targetDocument);
  _setRequestedFocusNode(null, null, false);

  // Make sure form actions are up to date
  _updateFormActions(sourceDocument, targetDocument);

  // clear out the iframe content
// =-=ags For now, we avoid overwriting the iframe content, as the
//        additional write essentially breaks the history list on IE.
//        We should, however, investigate whether there is some way
//        to free up the memory used by the iframe without breaking
//        back button behavior
//  sourceDocument.write();
//  sourceDocument.close();
}

/* If the UIX facility needs to set focus to a particualr node after a PPR
 * update, calling this function saves off the data needed to find that node
 *
 * Args:
 *    doc : The document that the node lives in
 * nodeid : The id attached to the desired node
 *   next : If true, we'll try to focus on the node following the one above,
 *          otherwise, we'll try to focus on the requested node.
 */
function _setRequestedFocusNode(doc, nodeid, next)
{
  // we only allow one outstanding focus request
  top._UixFocusRequestDoc = doc;
  top._UixFocusRequestID = nodeid;
  top._UixFocusRequestNext = (next == true);
}


/* If a request was made to focus on a particular node, this function will
 * attempt to get that node.
 */
function _getRequestedFocusNode()
{
  if ((top._UixFocusRequestDoc != null)
      && (top._UixFocusRequestID != null))
  {
    var element = _getElementById(top._UixFocusRequestDoc,
                                  top._UixFocusRequestID);
    if (!element)
      return null;

    if (top._UixFocusRequestNext)
    {
      // If "next" was set, the caller doesn't want this node, but the next
      // one. Try to find something that'll accept focus.
      for (var next = element.nextSibling;
           next != null;
           next = next.nextSibling)
      {
        if (_isFocusable(next)
            // we actually DO want to "tab" to links
            || ((_agent.isIE) && (next.nodeName.toLowerCase() == 'a')))
        {
          element = next;
          break;
        }
      }
    }
    return element;
  }
  return null;
}

/**
 * Onload event handler for the partial iframe for full page response.
 * Takes two parameters: the onload and onunload handlers for the
 * full page.
 */
function _fullChange()
{
  // Update the request count
  if (parent._pprRequestCount > 0)
  {

    parent._pprRequestCount--;

    // Before we get the contents, insert a script which disables
    // document.write().  That way, when we write out the contents
    // to the parent window, we won't duplicate calls to document.write()
    // that have already been performed.
    var disableWriteItem = _getElementById(document, "_pprDisableWrite");
    disableWriteItem.text = "var _pprDocumentWrite = document.write;" +
                            "var _pprDocumentWriteln = document.writeln;" +
                            "document.write = new Function('return;');" +
                            "document.writeln = new Function('return;');";

    // Insert a script which re-enables document.write() after we've
    // finished writing out the contents to the parent window.
    var enableWriteItem = _getElementById(document, "_pprEnableWrite");
    enableWriteItem.text = "document.write = _pprDocumentWrite;" +
                           "document.writeln = _pprDocumentWriteln";

    // We also need to replace the onload/onunload event handlers
    // before we get the contents for the parent window.
    var bodyItem = document.body;

    // Save the old event handlers
    var iframeOnload = bodyItem.getAttribute("onload");
    var iframeOnunload = bodyItem.getAttribute("onunload");

    // Set up the new event handlers
    bodyItem.setAttribute("onload",
      _getCommentedScript(document, ("_pprFullOnload")));
    bodyItem.setAttribute("onunload",
      _getCommentedScript(document, ("_pprFullOnunload")));


    // Get the full content of the iframe document.  This should now have
    // the correct uncommented version of our partial page scripts, plus
    // scripts to disable and re-enable document.write().
    var content = _getDocumentContent();

    // We do not want the _fullChange script to be called again.
    // It will cause a javascript error if it is. Therefore,
    // we need to clear it out.
    var pattern =
      new RegExp("<script id=_pprFullChange>_fullChange\\(\\)</script>","i");
    content = content.replace(pattern, "");

    // Reset the event handlers
    bodyItem.setAttribute("onload", iframeOnload);
    bodyItem.setAttribute("onunload", iframeOnunload);

    // Write the full content to the parent window

    var targetDocument = parent.document;

    if (_agent.isIE)
    {
       // Explicitly set the charset on the target document
       // before writing contents - see bug 3296281
       var sourceCharset = document.charset;
       targetDocument.open();
       targetDocument.charset = sourceCharset;
    }

    targetDocument.write(content);
    targetDocument.close();

    // the scripts for the page will be loaded onLoad, see _checkLoad

  }
}

// Called by _partialChange() to ensure that the form action
// attributes are update to date.
function _updateFormActions(sourceDocument, targetDocument)
{
  var sourceForms = sourceDocument.forms;

  // Loop through all of the forms in the iframe, checking
  // the action attribute against the corresponding forms
  // in the parent window.  If any action attributes are
  // out of date, set the new action on the form in the
  // parent window.
  for (var i = 0; i < sourceForms.length; i++)
  {
    var sourceForm = sourceForms[i];

    // The partial page response may include forms which
    // have not had their contents updated.  We only care
    // about forms that contain partial targets...
    if (sourceForm.hasChildNodes())
    {
      // Get the source form's name and action
      var sourceName = sourceForm.name;
      var sourceAction = sourceForm.action;

      // Locate the target form
      var targetForm = targetDocument.forms[sourceName];

      if (targetForm)
      {
        var targetAction = targetForm.action;

        // If the target form's action is out of date,
        // set it to the new action.
        if (targetAction != sourceAction)
          targetForm.action = sourceAction;
      }
    }
  }
}

// Called by _partialChange to get the parent window's
// active element.
function _getParentActiveElement()
{
  // This is more complicated then it should be.  We should be
  // able to access the activeElement through parent.activeElement,
  // but when we do that IE gives us a non-readable node - and we
  // we can't get at any properties that we need, like id, parentNode,
  // etc...  This seems to be some sort of security restriction - it
  // is only a problem when we try to access the parent window's
  // active element from a child iframe.  This isn't a problem
  // if we access the local window's activeElement.
  //
  // As usual, we have a totally sick work around.  Since the parent
  // window can get at its own activeElement, we execute a script
  // in the parent window which saves away the active element in
  // a well known variable.  Then, we can access the activeElement
  // and all of its properties through this variable.
  //
  // Of course, none of this actually works on Mozilla, where
  // activeElement is not supported.

  if (parent.document.activeElement)
  {
    _eval(parent, "_saveActiveElement()");
    return parent._pprActiveElement;
  }

  return null;
}

// This function is executed by _getParentActiveElement() to
// save away the activeElement in a well known location where
// it can be accessed by the PPR iframe
function _saveActiveElement()
{
  if (window._pprEventElement)
    window._pprActiveElement = window._pprEventElement;
  else if (document.activeElement)
    window._pprActiveElement = document.activeElement;
  else
    window._pprActiveElement = null;
}


// Finds the node under the specified target item
// to which the focus should be restored.
// If the targetItem contains an element with the same
// id or name as the old activeElement, we set
// the focus to this item.  Otherwise we set the focus
// to the first focusable item.
function _getNewActiveElement(targetDocument, targetItem, oldActiveElement)
{
  // First check to see if we have a new element with the same
  // ID as the old active element.
  if (oldActiveElement.id)
  {
    var newActiveElement = _getElementById(targetDocument,
                                           oldActiveElement.id);
    if (_isFocusable(newActiveElement))
      return newActiveElement;
  }

  // =-=ags We should look for nodes with the same name

  return null;
}

// Returns the first focusable node under the specified node
function _getFirstFocusable(node)
{
  if ((node == null) || _isFocusable(node))
    return node;

  if (node.hasChildNodes)
  {
    var children = node.childNodes;
    for (var i = 0; i < children.length; i++)
    {
      var child = children[i];
      var firstFocusable = _getFirstFocusable(child);
      if (firstFocusable != null)
        return firstFocusable;
    }
  }

  return null;
}

// Restores the focus to the specified node
function _restoreFocus(node, isFirstFocusable, doc)
{
  if (node == null)
    return;

  // If we are in a scrolled DIV, restoring the focus to the
  // first focusable node may cause the DIV to scroll back to 0,0.
  // So, for now we just avoid restoring the focus in this situation.
  // In the future we should see less cases where scrolling occurs,
  // since we should do a better job locating the correct node to
  // receive the focus.
  var divNode = _getAncestorByName(node, "DIV");
  if (!divNode)
  {
    _pprFocus(node, doc);
  }
  else
  {
    var scrollTop = divNode.scrollTop;
    var scrollLeft = divNode.scrollLeft;

    // If we aren't scrolled at all, or if we are restoring the
    // focus to the correct focusable owner (and not just the
    // first focusable node), then restore the focus.  Otherwise,
    // we do nothing, in order to avoid unnecessary scrolling.
    if (((scrollTop == 0) && (scrollLeft == 0)) || !isFirstFocusable)
    {
      _pprFocus(node, doc);
    }
  }

  // Bug #2753958: IE doesn't seem to want to re-set the focus when we're
  // done with a PPR update if the input element happens to be enclosed
  // within a table. However, if we make a second request, the focus is set
  // correctly. This is pretty hacky, but it's limited to the one
  // interesting case. We can live with this.
  // =-= Check to see if this case is even needed anymore after
  //     the introduction of _pprFocus.
  if ((_agent.isIE)
      && (node.tagName == 'INPUT')
      && (_getAncestorByName(node, 'TABLE')))
  {
    _pprFocus(node, doc);
  }
}

// Returns an ancestor with the specified name
function _getAncestorByName(
  node,
  ancestorName
  )
{
  ancestorName = ancestorName.toUpperCase();

  while (node)
  {
    if (ancestorName == node.nodeName)
      return node;

    node = node.parentNode;
  }

  return null;
}

// Tests whether one node is a descendent of another
function _isDescendent(
  node,
  ancestorNode
  )
{
  if (node == null)
    return false;

  while (node.parentNode)
  {
    if (node == ancestorNode)
      return true;

    node = node.parentNode;
  }

  return false;
}

// Tests whether the specified node is focusable
function _isFocusable(node)
{
  if (node == null)
    return false;

  var name = node.nodeName.toLowerCase();

  // Links that have a destination are generally focusable
  if (('a' == name) && (node.href))
  {
    // We need to be careful on IE - it seems that
    // IE has problems setting the focus to links
    // which contain a single image.  We see this when
    // IE tries to set the focus to the link around the
    // previous icon in the table.  Actually, this does
    // not seem to be a problem if the link has its
    // id set, so we first check for that.

    // If we're not on IE, or if the link has an id,
    // the link should be focusable
    if (!_agent.isIE || (node.id))
      return true;

    // If we're on IE, we only consider the link to be
    // focusable if it has something other than a single
    // image for its contents.
    var children = node.childNodes;
    if ((children) && (children.length == 1))
    {
      var childName = children[0].nodeName;
      if ('img' == childName.toLowerCase())
        return false;
    }

    return true;
  }

  // Blow off any disabled elements
  if (node.disabled)
    return false;

  // Input elements are also usually focusable
  if ('input' == name)
  {
    // But don't set the focus to hidden fields
    return (node.type != 'hidden');
  }

  // Catch everything else here...
  return (('select' == name) ||
          ('button' == name) ||
          ('textarea' == name));
}

// The partial page response includes a single script element
// which contains all of the code that needs to be executed when
// the partial page changes are applied.  Oddly enough, the code
// is actually commented out to prevent execution within the
// iframe context.  We need to pull the code out from the comment
// braces (/* */) and execute the code in the parent window's
// context.  This function returns the uncommented script code
// that needs to be executed in the parent window.
function _getCommentedScript(doc, id)
{
  var scriptItem = _getElementById(doc, id);

  if (scriptItem != null)
  {
    var scriptContents = scriptItem.text;

    // Strip off the start/end comments
    var start = 0;
    var end = scriptContents.length - 1;

    while (start < end)
    {
      if (scriptContents.charAt(start) == '*')
        break;

      start++;
    }

    while (end > start)
    {
      if (scriptContents.charAt(end) == '*')
        break;

      end--;
    }

    // Rip out the actual script contents from within the comments
    return scriptContents.substring(start + 1, end);
  }

  return null;
}

// Evaluates the specified code in the target window
function _eval(targetWindow, code)
{
  if (code == null)
    return;

  // For IE, we use window.execScript().  For Mozilla, we use
  // window.eval().  It would be nice if we could use eval() on
  // IE too, but IE's implementation of eval() always executes
  // the script in the current context, even if some other
  // window is specified.
  if (_agent.isIE)
    targetWindow.execScript(code);
  else
    targetWindow.eval(code);
}

// Returns the full content (outer HTML) of the current document
function _getDocumentContent()
{
  // Would be nice if we could also include the doctype!

  if (_agent.isIE)
    return document.documentElement.outerHTML;

  // We use innerHTML to get the content on Mozilla, so we
  // need to build up the <html> element manually
  var content = "<html"

  // We need to explicitly add in any attrs
  var attrs = document.documentElement.attributes;
  for (var i = 0; i < attrs.length; i++)
  {
    content += " ";
    content += attrs[i].name;
    content += "=\""
    content += attrs[i].value;
    content += "\"";
  }

  content += ">";

  // Add in the inner content
  content += document.documentElement.innerHTML;

  // Close up the content
  content += "</html>";

  return content;
}

//
// Transform all links on a page to (conditionally)
// perform a form submit (using the given form).
// The targetWindow is the window in which the links
// will appear.  If this is called in response to a
// partial page update, the targetWindow is the
// parent window - and not the current (iframe's)
// window.
//
function _fixAllLinks(formName, targetWindow, exclude)
{
  // Save off the initial state of this form.
  _initialFormState = _getFormState(formName, exclude);
  _initialFormStateName = formName;
  if (exclude != (void 0))
    _initialFormExclude = exclude;

  // If we are collecting state for a different window,
  // copy the state over to the target window
  if (window != targetWindow)
  {
    if (targetWindow._initialFormState == null)
      targetWindow._initialFormState = new Object();

    var sourceWindowState = _initialFormState;
    var targetWindowState = targetWindow._initialFormState;

    for (var key in sourceWindowState)
      targetWindowState[key] = sourceWindowState[key];
  }

  var links = document.links;
  var currHrefWithAnchor = targetWindow.location.href + '#';
  var iframeWithAnchor = location.href + '#';

  // Look through all of the links
  for (var i = 0; i < links.length; i++)
  {
    var href = links[i].href;
    // If the link is actually pointing to anchor on this page, or
    // to javascript: or mailto:, don't mess with it!
    // Bug #3369822: A destination of '#' in the iFrame comes back as the full
    // submitted URL, so have to check against it too.
    if (!href
        || (href.substr(0, iframeWithAnchor.length) == iframeWithAnchor)
        || (href.substr(0, currHrefWithAnchor.length) == currHrefWithAnchor)
        || (href.substr(0, 11).toLowerCase() == "javascript:")
        || (href.substr(0, 7).toLowerCase() == "mailto:")
        || (href.indexOf("_noSv=M") >= 0))
    {
      continue;
    }

    // Similarly, any link with a target frame should be ignored
    if (links[i].target)
    {
      continue;
    }

    // Encode single-quotes correctly
    var hrefArr = href.split("'");
    href = hrefArr[0];
    for (var j = 1; j < hrefArr.length; j++)
      href = href + "\\'" + hrefArr[j];

    // Bug 2807885: bizarrely, setting "href" actually decodes
    // the href - at least it does on Mozilla and IE.  (It doesn't
    // on Netscape 4.)
    if (!_agent.isNav)
      href = escape(href);

    links[i].href = "javascript:_submitNav('" + formName + "','" + href + "')";
  }
}


//
// Store the current value of all elements in a form.
// Ignores all hidden fields.
//
function _getFormState(formName, exclude)
{
  var state = new Object();
  var frm = document[formName];

  for (var i = 0; i < frm.length; i++)
  {
    var name = frm.elements[i].name;
    if (name)
    {
      var element = frm[name];
      if (element)
      {
        if ((exclude != (void 0)) && (exclude[name] != (void 0)))
          continue;

        // Skip over hidden values
        if (!element.type || (element.type != 'hidden'))
          state[name] = _getValue(element);
      }
    }
  }

  return state;
}

/**
 * Determines if the "navigation form" is dirty, returning true if
 * the form is dirty.  This function can only be called if
 * the "navigationFormName" attribute was set on the UIX body component.
 *
 * @return true if the navigation form is dirty
 */
function isNavDirty()
{
  var dirty    = false;

  if (_navDirty)
    dirty = true;
  else
  {
    // Get the current state of the form
    var newState = _getFormState(_initialFormStateName, _initialFormExclude);
    // Compare it to the old state
    for (var key in newState)
    {
      if (newState[key] != _initialFormState[key])
      {
        dirty = true;
        break;
      }
    }
  }

  return dirty;
}


//
// Add a name to the list of excluded names
//
function _addNavExclude(exclude)
{
  if (_initialFormExclude != (void 0))
    _initialFormExclude[exclude] = 1;
  else
  {
    _initialFormExclude = new Object();
    _initialFormExclude[exclude] = 1;
  }
}

//
// Submit a form navigation.  Note that we only bother submitting if
// the form is "dirty"; this relies on fixAllLinks() having been called
// to store the state of that form.
//
//
function _submitNav(formName, uri)
{
  // The form is "dirty": send a "navigate" event, and store
  // off the URL that we would have navigated to
  if (isNavDirty())
  {
    var onNav = window["_onNavigate"];
    if ((onNav == (void 0)) || !(onNav(formName, uri) == false))
      submitForm(formName, 0, {'event':'navigate','uri':uri});
  }
  // Nope, not dirty: just navigate to the page
  else
    document.location.href = uri;
}


/**
 * Called to identify the input field from an event
 * This is called not only below, but also from LovInput.js.
 */
function _getInputField(event)
{
  var input = (void 0);
  var src = (void 0);

  if (window.event)
  {
    kc = window.event.keyCode;
    src = window.event.srcElement;
  }
  else if (event)
  {
    kc = event.which;
    src = event.target;
  }

  if (src != (void 0)
      && (src.tagName == "INPUT" ||
          src.tagName == "TEXTAREA" ))
    input = src;

  return input;
}

/**
 * Called when a field receives focus.
 * Prepares for a later reset of this field by saving its current value.
 */
function _enterField(
  event
  )
{
  var input;
  var src;
  var retv = true;

  var input = _getInputField(event);

  if (input != (void 0))
  {
    input.form._mayResetByInput = false;

    if (input != window._validating)
    {
      // save the last valid value for later restoration
      input._validValue = input.value;
    }
    retv = false;
  }

  return retv;
}

/**
 * Resets the form input to its last valid value.
 * This function is called from the onKeyDown for a form input.
 * If called twice in succession for the same form, with the
 * escape keycode both times, this function will reset the form.
 */
function _resetOnEscape(event)
{
  var kc;
  var input = _getInputField(event);

  if (input != (void 0))
  {
    var form = input.form;

    if (kc == 27)  // escape keycode
    {
      // reset the form input to its last valid value
      // providing there is no selection (consistent with IE)

      var hasSelection = false;

      if ((input.selectionStart != (void 0)) &&
          (input.selectionEnd   != (void 0)))
      {
        hasSelection = (input.selectionStart != input.selectionEnd);
      }
      else if (document.selection)
      {
        hasSelection = (document.selection.createRange().text.length != 0);
      }

      if (!hasSelection)
      {
        // always reset the field
        input.value = input._validValue;

        // determine if a full form reset is required
        if (form._mayResetByInput == true)
        {
          // reset the form
          // unset the flag for form reset
          form.reset();
          form._mayResetByInput = false;
        }
        else
        {
          // set the flag for form reset
          form._mayResetByInput = true;
        }
      }

      // consume this event to prevent any browser behavior from kicking in
      return false;
    }
    else // any keycode other than escape
    {
      // unset the flag for form reset
      // since some other key was pressed
      form._mayResetByInput = false;
    }
  }
  return true;
}


/**
 * Called by the load handler of each document body to prepare event handlers
 * for forms and fix links to submit the navigation form.
 */
function _checkLoad(
  event,
  navFormName,
  excludeList
  )
{
  // first restore page state
  restorePartialPageState();

  for (var i = 0; i < document.forms.length; i++)
  {
    var form = document.forms[i];

    // Note: event listener functions must already be defined above
    //       no forward references
    if (form.addEventListener) // DOM events
    {
      form.addEventListener('focus', _enterField, true);
      form.addEventListener('keydown', _resetOnEscape, true);
    }
    else if (form.attachEvent) // IE5 events
    {
      form.attachEvent('onfocusin', _enterField);
      form.attachEvent('onkeydown', _resetOnEscape);
    }
  }

  if (navFormName != (void 0))
  {
    var exclude = new Object();

    if (excludeList != (void 0))
    {
      for (var j = 0; j < excludeList.length; j++)
        exclude[excludeList[j]] = 1;
    }

    _fixAllLinks(navFormName, window, exclude);
  }

  // If we're inside a frameset, and the top frame wants
  // reloads blocked, install a _noReload handler.
  // chain _monitor also if that is already set.
  if ((self != top) && top["_blockReload"])
  {
    // If _monitor is already set on document.onkeydown, then
    // chain _noReload and _monitor by calling another function which
    // calls both: _monitorNoReload.
    // _monitor is set on IE only.
    if ((_agent.isIE)
        && (document.onkeydown != null)
        && (((document.onkeydown).toString().indexOf('_monitor')) > 0))
    {
      document.onkeydown = _monitorNoReload;
    }
    else
    {
      document.onkeydown = _noReload;
    }
  }

  // Set initialFocus if necessary
  if ((!_agent.isNav) && (_initialFocusID != null))
  {
    var myElement = _getElementById(document, _initialFocusID);
    if (myElement && myElement.focus)
    {
      myElement.focus();
      if (myElement.type == 'text')
        myElement.select();
    }
  }

  if (!_agent.isNav)
    _loadScriptLibraries(document);

}


//
// Event handle that blocks keys that lead to a page reloading.
//
function _noReload(e)
{
  if (!e) e=window.event;
  var kc=e.keyCode;
  // F5 and Ctrl-R
  if ((kc==116)||(kc==82 && e.ctrlKey))
  {
    if (e.preventDefault) e.preventDefault();
    e.keyCode=0;
    return false;
  }
}

//
// Event handle that monitors access keys and
// blocks keys that lead to a page reloading.
// used for our dialogs.
//
function _monitorNoReload(e)
{
  if (_agent.isIE)
    _monitor(e);
  return _noReload(e);
}


//
// Deliver a client event with the specified type, source and parameters
// to the handler body.
//
function _handleClientEvent(type, source, params, handlerBody)
{
  var event = new Object();
  event.type = type;
  event.source = source;
  event.params = params;
  var func = new Function("event", handlerBody);
  return func(event);
}


//
// APIs dealing with cookies.  We currently have no supported
// public functions.  _getCookie() and _setCookie() are good candidates.
//

function _getCookie(name)
{
  var dc = document.cookie;

  var value = "";
  var prefix = name + "=";
  if (dc)
  {
    // Look for the cookie name in in the middle.
    var begin = dc.indexOf("; " + prefix);

    if (begin < 0)
    {
      // Not there: look for it at the beginning
      begin = dc.indexOf(prefix);
      if (begin > 0)
        begin = -1;
    }
    else
      // Found it - now skip over the colon and space
      begin += 2;

    if (begin >= 0)
    {
      var end = dc.indexOf(";", begin);
      if (end < 0)
        end = dc.length;

      value = unescape(dc.substring(begin + name.length + 1, end));
    }
  }

  return value;
}

//
// Sets a cookie value.
// This function isn't especially general (yet) as it doesn't
// allow overriding the domain, path, or expiry.
//
function _setCookie(name, value)
{
  // Compute the domain to scope as widely as is legit
  // by scoping off.
  // =-=AEW "localhost" just doesn't work.  I don't know how to
  // deal with this...  It'll confuse developers, even though
  // it will never have any real impact.
  var domain = window.location.host;

  /* =-=AEW The current Oracle Cookie design guidelines require
     cookies to _not_ be scoped widely.  So don't!
  var periodIndex = domain.indexOf(".");
  if ((periodIndex >= 0) &&
      (domain.lastIndexOf(".") != periodIndex))
  {
    // But don't scope off anything that's entirely a number,
    // since then we're probably dealing with an IP address
    var startOfDomain = domain.substr(0, periodIndex);
    if (!(((startOfDomain * startOfDomain) == 0) ||
          ((startOfDomain / startOfDomain) == 1)))
      domain = domain.substr(periodIndex);
  }
  */

  var colonIndex = domain.indexOf(":");
  if (colonIndex >= 0)
    domain = domain.substr(0, colonIndex);

  // Expire 10 years after today
  var expires = new Date();
  expires.setFullYear(expires.getFullYear() + 10);

  // And here's the cookie:
  // (Reordering the parameters seemed to break some browsers)
  var curCookie = name + "=" + value +
      "; path=/;domain=" + domain + "; expires=" + expires.toGMTString();

  document.cookie = curCookie;
}


//
// Set a single value in the UIX cookie
//
function _setUIXCookie(index, value)
{
  var arry = _getUIXCookie();
  arry[index] = value;

  // Rebuild the encoded value
  var encodedValue = arry[0];
  for (var i = 1; i < arry.length; i++)
  {
    encodedValue = encodedValue + "^" + arry[i];
  }

  _setCookie("oracle.uix", encodedValue);
}


//
// Extract the decoded form of the UIX cookie
//
function _getUIXCookie()
{
  var encodedValue = _getCookie("oracle.uix");
  var arry;
  if (encodedValue)
    arry = encodedValue.split("^");
  else
    arry = new Array("0", "", "");

  return arry;
}


//
// Defaults the time zone
//
function _defaultTZ()
{
  var tz = _getUIXCookie()[2];

  // If the time zone has already been set, then bail:  however,
  // time zones that start with "GMT" are inherently unreliable,
  // because they won't track daylight savings.  For such time zones,
  // always replace the value.
  if (tz && (tz.indexOf("GMT") != 0))
  {
    return;
  }

  // Set the correct time zone.
  _setUIXCookie(2, _getTimeZoneID());
}


//
// Compute the time zone ID, of form GMT+-XX:YY
//
function _getTimeZoneID()
{
  // Get the time zone offset, then flip the sign, as this
  // is opposite in meaning to the time zone ID
  var tzOffset = -(new Date()).getTimezoneOffset();
  var newTZ;

  // Build up the name of the time zone
  if (tzOffset > 0)
    newTZ = "GMT+";
  else
  {
    newTZ = "GMT-";
    tzOffset = -tzOffset;
  }

  var minutes = "" + tzOffset % 60;
  if (minutes.length == 1)
    minutes = "0" + minutes;
  return (newTZ + (Math.floor(tzOffset / 60)) + ":" + minutes);
}


/**
 * _monitor: monitor the document's onkeydown presses.
 * Activate the uix button that has the access key that matches the
 * key that is pressed along with the ALT key (ALT+accessKey).
 * This is called for IE, not on Mac, only.
 *
 * Background: we want the accesskey to work on our UIX
 * buttons (which are links wrapped around an image)
 * like it does for a <button> html element, and that is
 * it activates the button rather than sets focus like it does for a link.
 * (This already works as expected in Mozilla)
 * This javascript code must know that the element is our uix button.
 * In ButtonRenderer, we render a bogus attribute for this purpose,
 * called "uixbtn".
 * We also register document.onkeydown=_monitor on the page.
 */
function _monitor(e)
{

  // when the user holds down only the accessKey, this triggers a mouse
  // down event over and over with a keyCode of 18. So...
  // we make sure not to call the _findAccessKey function if
  // the user is holding down just the access key.

  var windowEvent = window.event;

  if ((windowEvent.altKey == true) && (windowEvent.ctrlKey == false) &&
      (windowEvent.keyCode != null) && (windowEvent.keyCode != 18 )
      && (!windowEvent.repeat) )
  {
    var keyPressed = String.fromCharCode(window.event.keyCode);

    var accessKeyNode = _getNodeWithAccessKey(document, keyPressed);

    // if the element is a UIX Button, make it act like an accelerator

    if (accessKeyNode != null && (accessKeyNode.getAttribute("uixbtn") != null))
    {

      // now I need to find the node to click...

      // if label, get the "for" attribute value.
      // That is the id of the element to click
      if (accessKeyNode.htmlFor)
      {
          var id=accessKeyNode.htmlFor;
          accessKeyNode = (id != null)
                          ? window.document.getElementById(id)
                          : null;
      }

      if (accessKeyNode != null)
      {
        // IE acts like it sets the focus before it triggers the access key,
        // so we will do the same.

        accessKeyNode.focus();
        accessKeyNode.click();
      }

    } // end if (accessKeyNode != null)
  }
  return true;
}

/**
 * This function is passed a DOM Node object and a keyboard key
 * that was pressed at the same time as the ALT key.
 * return the node with the access key, otherwise null.
 */
function _getNodeWithAccessKey(node, keyPressed)
{
  var keyPressedUpper = keyPressed.toUpperCase();
  var keyPressedLower = keyPressed.toLowerCase();

  var accessKeyObject =
  {
    activeFound:false,
    firstAccessKeyNode:null,
    accessKeyNode:null
  }

  // start looking through the DOM Node tree from the beginning
  // for the access key the user pressed.
  // The node returned will be the first one
  // found starting from the current active element, otherwise we will use the
  // first one found in the DOM tree regardless of the current active element.

  accessKeyObject = _findAccessKey(document,
                                   accessKeyObject,
                                   keyPressedUpper,
                                   keyPressedLower);

  var accessKeyNode = accessKeyObject.accessKeyNode;
  var firstAccessKeyNode = accessKeyObject.firstAccessKeyNode;

  // if we didn't find accessKeyNode in the tree from the active node
  // to the end of the tree,
  // just use the first element we found with the correct access key.

  if ( (accessKeyNode == null) && (firstAccessKeyNode != null) )
  {
    accessKeyNode = firstAccessKeyNode;
  }
  return accessKeyNode;
}

/**
 * This function is passed a DOM Node object, a local accessKeyObject,
 * and the keyPressed in upper and lower cases.
 * This function checks to see if the node
 * represents an HTML tag; i.e., if the node is an Element object. It
 * recursively calls itself on each of the children of the node, testing
 * them in the same way. If you invoke this function by passing it the
 * Document object, it traverses the entire DOM tree.

 * Find the element with the access key.
 * first, store the first element with the access key encountered in
 * the accessKeyObject.firstAccessKeyNode.
 * Then, find the first element with the access key
 * that is found after the focus element,
 * since this is how it works on IE for <button></button>
 * return accessKeyObject if the access key node found, otherwise return null.
 */
function _findAccessKey(node, accessKeyObject, keyPressedUpper, keyPressedLower)
{
  // check all nodes of type ELEMENT_NODE
  if (node.nodeType == 1 /*Node.ELEMENT_NODE*/)
  {
    // check if the _keyPressed  is the access key
    if ( (node.accessKey == keyPressedUpper) ||
          (node.accessKey == keyPressedLower))
    {
      if (accessKeyObject.activeFound == true)
      {
        // Great! Found accessKey on an element after the focus, so return
        // the node. The recursion will unwind and pass out this node to
        // the original call.
        accessKeyObject.accessKeyNode = node;
        return accessKeyObject;
      }
      else if (accessKeyObject.firstAccessKeyNode == null)
      {
        // the focus hasn't been found yet, but store away this node
        // in the global variable, since it is the first one found
        accessKeyObject.firstAccessKeyNode = node;
      }
    }
    // If the focus is on an element with the access key, IE activates the NEXT
    // element, which is why I set the _activeFound flag after checking the
    // access key.
    if (node == document.activeElement)
    {
      accessKeyObject.activeFound = true;
    }
  } // end check if it is an element node

  // Now get all children of n
  // Loop through the children
  // Recurse on each one

  var children = node.childNodes;
  for(var i=0; i < children.length; i++)
  {
    var accessKeyObject =
    _findAccessKey( children[i],
                    accessKeyObject,
                    keyPressedUpper,
                    keyPressedLower);

     if (accessKeyObject.accessKeyNode != null)
     {
      return accessKeyObject; // stop looping.
     }
  }
  return accessKeyObject;
}


//
// Returns true if the value contains nothing but strings
//
function _isEmpty(value)
{
  var s = "" + value;
  var i = 0;
  while (i < s.length)
  {
    if (s.charAt(i) != ' ')
      return false;
    i++;
  }

  return true;
}

//
// Returns true if the current document reads left to right.
//
function _isLTR()
{
  return document.documentElement["dir"].toUpperCase() == "LTR";
}

//
// _pprConsumeClick: Helps implement blocking. This function just consumes
//                   every click that falls within the body.
//
function _pprConsumeClick(event)
{
  if (_agent.isIE)
  {
    var body = document.body;
    if ((event.x < body.offsetLeft) || (event.y < body.offsetTop)
        || (event.x > body.offsetWidth) || (event.y > body.offsetHeight))
    {

      // OK, we've caught an event outside the body of the document. Assume
      // that the user is clicking somewhere on the menu bar, or anther window.
      // At this point, we release the mouse and continue (that's better than
      // keeping the user in limbo).

      _pprStopBlocking(window);
    }
  }
  // Mozilla is much better about this, we only catch the clicks within
  // the div element expanded to the full size of the body, so every click
  // gets blocked.
  return false;
}

//
// _pprStartBlocking: Starts consuming every click (to implement blocking)
//
function _pprStartBlocking(win)
{
  // In order to force the nice impatient user to allow a PPR update to
  // complete, we block all mouse clicks between the start of a PPR update, and
  // the end. We do this by building a dummy DIV element and having it grab all
  // clicks. On Mozilla, we just expand it to cover the entire body as a glass
  // frame. On IE, we leave the DIV at zero size, but route every click to it.
  if (!win._pprBlocking)
  {
    var body = win.document.body;
    var divElement = _getElementById(win.document, _pprdivElementName);

    if (divElement)
    {
      if (_agent.isGecko)
      {
        var w = body.offsetWidth + 16;
        var h = body.offsetHeight;
        // expand the div element to cover the entire page.
        divElement.style.height = h;

        // add in 16 pixels to cover the scrollbar area
        divElement.style.width = w;

        // If the user clicks the stop button, then we'll be stuck blocking.
        // So we don't hang, this timeout will clear the block in eight
        // seconds whether we've finished or not, but first we clear any
        // previously existing timeout.
        if (top._pprBlockingTimeout != null)
        {
          top.clearTimeout(top._pprBlockingTimeout);
        }
        top._pprBlockingTimeout = top.setTimeout("_pprStopBlocking(window);",
                                                 8000);
      }
      else if (_agent.isIE)
      {
        // DIV captures every event
        divElement.setCapture();

        // save off the element we'll return focus to
        _pprEventElement = window.document.activeElement;

        // catch keyPress events too...
        if (win._pprEventElement)
          divElement.focus();

        // save current cursor and display a wait cursor
        top._pprSavedCursor = body.style.cursor;
        body.style.cursor = "wait";
      }
      win._pprBlocking = true;
    }
  }
}

//
// _pprStopBlocking: Finishes up the blocking, releases the page back
//                   to normal processing
//
function _pprStopBlocking(win)
{
  if (win._pprBlocking)
  {
    var divElement = _getElementById(win.document, _pprdivElementName);
    if (_agent.isGecko)
    {
      // just resize to zero
      divElement.style.width = 0;
      divElement.style.height = 0;

      // If we've set a timeout, clear it now.
      if (top._pprBlockingTimeout != null)
      {
        top.clearTimeout(top._pprBlockingTimeout);
        top._pprBlockingTimeout == null;
      }
      // Mozilla doesn't support setting focus to a DIV. Have to find a way to
      // eat the keyPress events.
    }
    else if (_agent.isIE)
    {
      // Stop capturing mouse and keyPress events, and reset the cursor to
      // whatever it was before.
      divElement.releaseCapture();
      if (win._pprEventElement)
        win._pprEventElement.focus();
      win.document.body.style.cursor = top._pprSavedCursor;
    }
    win._pprEventElement = null;
  }
  win._pprBlocking = false;
}

// handle the onClick or onBlur for an IE SELECT element
// Returns true if the user has finally made a selection, and is ready to go.
function _pprChoiceAction()
{
  // this function is only needed to handle IE's weird select element
  if (!_agent.isIE)
    return true;

  var rv = false;

  // This gets called as both onClick and onBlur, but both really only want
  // to submit the event if a change has been made.
  if ((!top._pprBlocking) && (_pprChoiceChanged))
  {
    // clear the choice tracker
    _pprChoiceChanged = false;
    rv = true;
  }
  return rv;
}

// handle the onChange for an IE SELECT element
function _pprChoiceChangeEvent(event)
{
  if (!_agent.isIE)
    return true;

  // Just remember the fact that a change has occurred.
  if (!top._pprBlocking)
    _pprChoiceChanged = true;

  return true;
}

// Get the keycode from an event
function _getKC(event)
{
  if (window.event)
    return window.event.keyCode;
  else if (event)
    return event.which;
  return -1;
}


// Returns true if a form has been submitted a "short" time before newDate
function _recentSubmit(newDate)
{
  if (_lastDateSubmitted)
  {
    var diff = newDate - _lastDateSubmitted;
    if ((diff >= 0) && (diff < 200))
      return true;
  }
  return false;
}

// Returns true if a form has been reset a "short" time before newDate
function _recentReset(newDate)
{
  if (_lastDateReset)
  {
    var diff = newDate - _lastDateReset;
    if ((diff >= 0) && (diff < 200))
      return true;
  }
  return false;
}

/*
 * After updates we often can't just set focus to a node, it has to be prepared
 * in a browser specific way (different idosyncracies cause poor focus
 * behavior).
 */
function _pprFocus(node, doc)
{
  if (_agent.isIE)
  {
    // On IE, if a node has focus and we update it, then setting focus to it
    // seems to have no effect. Setting the focus to another node, then back to
    // the target seems to work correctly. Here we set the focus to a hidden
    // field.
    var divnode = _getElementById(doc, _pprdivElementName);
    if ((divnode) && (divnode["focus"]))
      divnode.focus();

    // IE often needs a bit more coaxing to really set the focus
    node.setActive();
  }
  node.focus();
}

// For PPR Back issue:
// save the page's contents into a hidden, disabled field.
// This function gets called when the beforeunload event fires, which is
// an IE-only event.
function _savePageStateIE()
{
  // the PPR Back Button support only works for IE.
  if (!_agent.isIE)
    return;

  var theSpan = _getElementById(document, "_pprPageContent");
  if (theSpan == null)
    return;

  var savePageField = _getElementById(document, "_pprSavePage");
  if (savePageField == null)
    return;

  savePageField.value = theSpan.outerHTML;

}

// For PPR Back issue:
// save the scripts and libraries with every partial page response
// so that the scripts and libraries can be re-loaded when the user
// comes back to the page after navigating off of it. (this is called
// from _partialChange())
function _saveScripts(targetDocument)
{
  // the PPR Back Button support only works for IE.
  if (!_agent.isIE)
    return;

  // save inline scripts by appending them to the previous saved scripts.
  var saveScriptElement= _getElementById(targetDocument, "_pprSaveScript");

  if (saveScriptElement != null)
  {
    var _pprScripts = _getCommentedScript(document, "_pprScripts");
    saveScriptElement.value =
      saveScriptElement.value + " " + _pprScripts;
  }

  // save the library by appending to the already saved libraries
  // if it isn't already there.
  var saveLibrariesElement = _getElementById(targetDocument, "_pprSaveLib");

  if (saveLibrariesElement != null && (window["_pprLibraries"] != (void 0)))
  {

    for (var i = 0; (i < _pprLibraries.length); i++)
    {

      if (saveLibrariesElement.value.indexOf(_pprLibraries[i]) == -1)
      {
        if (saveLibrariesElement.value != "")
         saveLibrariesElement.value += "," + _pprLibraries[i];
        else
          saveLibrariesElement.value += _pprLibraries[i];
      }
    }
  }
}


// For PPR Back issue:
// restore the page state when the user comes back to the page
// after navigating off of it.
// This includes copying the saved html, loading the saved libraries,
// and eval'ing the inline scripts (done from ScriptEval.js).
// This function is called from _checkLoad
function restorePartialPageState()
{
  // the PPR Back Button support only works for IE.
  if (!_agent.isIE)
    return;

  // get the saved page state from the hidden, disabled field
  // and write it to the page
  var savePageField   = _getElementById(document, "_pprSavePage");

  // if the field doesn't exist or if there was nothing saved in it, return.
  if (savePageField == null || savePageField.value == "")
    return;

  var theSpan = _getElementById(document, "_pprPageContent");

  if (theSpan == null)
    return;

  theSpan.outerHTML = savePageField.value;

  // next, get the libraries out of the saved field, and parse each one,
  // and reload.
  var saveLibrariesElement = _getElementById(document, "_pprSaveLib");
  if (saveLibrariesElement != null && saveLibrariesElement.value != "")
  {
    var libraries = saveLibrariesElement.value.split(",");
    _loadScriptLibrariesIE(document, libraries);
  }

  // Note: The inline javascript is eval'd in ScriptEval.js, so that it
  // is guaranteed that the libraries are all loaded first.
  // set this variable so that the saved script will be loaded, as opposed
  // to the script that is in _pprScripts, which gets loaded with every
  // ppr response.
  _pprBackRestoreInlineScripts = true;
}


