/*
** Copyright (c) Oracle Corporation 2000. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/

function _wmlPatternFormat(
  value
  )
{
  return value;
}


function _wmlPatternParse(
  parseString
  )
{
  var formatPattern = this._pattern;
  //alert(formatPattern);
  var formatLength  = formatPattern.length;
  var parseLength   = parseString.length;
  var outString     = "";
  
  var parseIndex     = 0;
  var isEscaped      = false;
  var minRepeatCount = 1;
  var maxRepeatCount = 1;
    
  for (var formatIndex = 0; formatIndex < formatLength; formatIndex++)
  {
    var currFormatChar = formatPattern.charAt(formatIndex);
    
    if (currFormatChar != "\\")
    {       
      if (isEscaped)
      {        
        if (parseString.charAt(parseIndex) != currFormatChar)
        {
          return (void 0);
        }
        else
        {
          isEscaped = false;
          minRepeatCount = 1;
          maxRepeatCount = 1;
          parseIndex++;
        }
      }
      else
      {
        // check for the infinite repeat character
        if (currFormatChar == "*")
        {          
          minRepeatCount = 0;
          maxRepeatCount = Infinity;
        }
        else
        {
          // check for the maximum repeat character
          if (isDigit(currFormatChar))
          {
            minRepeatCount = 0;
            maxRepeatCount = parseDigit(currFormatChar);
          }
          else
          {
            var lastParseIndex = parseIndex + minRepeatCount;
            
            // there aren't enough characters for the length
            if (lastParseIndex > parseLength)
              return (void 0);
            
            //
            // Check all of the required format items
            //
            while (parseIndex < lastParseIndex)
            {
              var parseChar = parseString.charAt(parseIndex);
              
              if (!_testWMLChar(currFormatChar, parseChar))
              {
                // character test failed
                // alert("Failed:" + currFormatChar + "(" + parseChar + ")");
                return (void 0);
              }
              
              // add this character to the parsed output
              outString += parseChar;
  
              parseIndex++;
            }
            
            //
            // Check all of the optional format items
            //
            var maxParseIndex = Math.min(parseLength,
                                         lastParseIndex + (maxRepeatCount -
                                                           minRepeatCount));
                                                     
            while (parseIndex < maxParseIndex)
            {
              var parseChar = parseString.charAt(parseIndex);

              if (!_testWMLChar(currFormatChar,
                                parseString.charAt(parseIndex)))
              {
                break;
              }
  
              // add this character to the parsed output
              outString += parseChar;

              parseIndex++;
            }
            
            // reset the repeat counts for next time
            minRepeatCount = 1;
            maxRepeatCount = 1;
          }
        }
      }
    }
    else
    {
      isEscaped = true;
    }
  }
  
  // only return the outstring if we have consumed all of the input
  // characters
  if (parseIndex == parseLength)
  {
    return outString;
  }
  else
  {
    return (void 0);
  }
}

function _testWMLChar(
  formatChar,
  parseChar
  )
{
  if (formatChar == "A")
  {
    // uppercase letter or punctuation
    return !isDigit(parseChar) && isNotLowerCase(parseChar);
  }
  else if (formatChar == "a")
  {
    // lowercase letter or punctuation
    return !isDigit(parseChar) && isNotUpperCase(parseChar);
  }
  else if (formatChar == "N")
  {
    // Number
    return isDigit(parseChar);
  }
  else if (formatChar == "X")
  {
    // any uppercase
    return isUpperCase(parseChar);
  }
  else if (formatChar == "x")
  {
    return isLowerCase(parseChar);
  }
  else if (formatChar == "M")
  {
    // any character, assume upper case
    return true;
  }
  else if (formatChar == "m")
  {
    // any character, assume lower case
    return true;
  }
  else
  {
    // unknown format character
    // alert("Unknown format character:" + formatChar);
    return false;
  }
}

function WMLPatternFormat(
  pattern
  )
{
  this._class   = "WMLPatternFormat";
  this._pattern = pattern;
}


WMLPatternFormat.prototype = new Format();
WMLPatternFormat.prototype.format = _wmlPatternFormat;
WMLPatternFormat.prototype.parse  = _wmlPatternParse;