/*
** Copyright (c) Oracle Corporation 2000-2002. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/

function _decimalFormat(
  number
  )
{
  return "" + number;
}


function _decimalParse(
  numberString
  )
{
  if (!numberString)
    return (void 0);

  // Get LocaleSymbols (from Locale.js)
  var symbols = getLocaleSymbols();
  if (symbols)
  {
    // We don't want leading or trailing grouping separators
    var grouping = symbols.getGroupingSeparator();
    if ((numberString.indexOf(grouping) == 0) ||
        (numberString.lastIndexOf(grouping) ==  (numberString.length - 1)))
      return (void 0);

    // Remove the thousands separator - which Javascript doesn't want to see
    var thousands = new RegExp("\\" + grouping, "g");
    numberString = numberString.replace(thousands, "");
    // Then change the decimal separator into a period, the only
    // decimal separator allowed by JS
    var decimal = new RegExp("\\" + symbols.getDecimalSeparator(),  "g");
    numberString = numberString.replace(decimal, ".");
  }

  // First, reject anything that's all spaces
  var i = numberString.length - 1;
  while (i >= 0)
  {
    if (numberString.charAt(i) != ' ')
      break;
    i--;
  }

  if (i >= 0)
  {
    // OK; it's non-empty.  Now, disallow exponential
    // notation, and then use some JS magic to exclude
    // non-numbers
    if ((numberString.indexOf('e') < 0) &&
        (numberString.indexOf('E') < 0) &&
        (((numberString * numberString) == 0) ||
         ((numberString / numberString) == 1)))
    {
      var result = parseFloat(numberString);
      if (!isNaN(result))
      {
        var integerDigits = numberString.length;
        var fractionDigits = 0;

        var sepIndex = numberString.lastIndexOf('.');
        if (sepIndex != -1)
        {
          integerDigits = sepIndex;
          fractionDigits = numberString.length - sepIndex -1;
        }
        
        var messageKey;
        
        if ((this._maxValue != (void 0)) &&
            (result  > this._maxValue))
        {
          messageKey = DecimalFormat.LV;
        }
        else if ((this._minValue != (void 0)) &&
                 (result  < this._minValue))
        {
          messageKey = DecimalFormat.MV;
        }
        else if ((this._maxPrecision != (void 0)) &&
                 (integerDigits  > this._maxPrecision))
        {
          messageKey = DecimalFormat.LID;
        }
        else if ((this._maxScale != (void 0)) &&
                 (fractionDigits  > this._maxScale))
        {
          messageKey = DecimalFormat.LFD;
        }

        if (messageKey != (void 0))
        {
          var messages = this._messages;
          
          if ((messages == (void 0)) ||
              (messages[messageKey] == (void 0)))
            return (void 0);  // default error format
          else
            return new ParseException(messages[messageKey]);
        }
        
        return result;
      }
    }
  }

  return (void 0);
}

function DecimalFormat(
  messages,
  maxPrecision,
  maxScale,
  maxValue,
  minValue)
{
  this._messages = messages;
  this._maxPrecision = maxPrecision;
  this._maxScale = maxScale;
  this._maxValue = maxValue;
  this._minValue = minValue;

  // for debugging
  this._class = "DecimalFormat";
}

DecimalFormat.prototype = new Format();
DecimalFormat.prototype.format = _decimalFormat;
DecimalFormat.prototype.parse  = _decimalParse;

// Less fraction digits
DecimalFormat.LFD = 'LFD';
// Less integer digits
DecimalFormat.LID   = 'LID';
// Less value
DecimalFormat.LV        = 'LV';
// More value
DecimalFormat.MV        = 'MV';


function _regExpFormat(
  value
  )
{
  return value;
}


function _regExpParse(
  parseString
  )
{
  var matchArr = parseString.match(this._pattern);
  
  if ((matchArr != (void 0)) && (matchArr[0] == parseString))
  {
    return parseString;
  }
  else
  {
    return (void 0);
  }
}


function RegExpFormat(
  pattern
  )
{
  this._class = "RegExpFormat";
  this._pattern = pattern;
}


RegExpFormat.prototype = new Format();
RegExpFormat.prototype.format = _regExpFormat;
RegExpFormat.prototype.parse  = _regExpParse;
