
// DiagrammerLib
//-----------------------------------------------------------------------------
function GraphView(source, mainImageID, form,
                   graphBounds,
                   thumbWidth, thumbHeight,
                   viewClip)
{
  this.source = source;
  this.imageID = mainImageID;
  this.graphBounds = graphBounds;
  this.viewClip = viewClip;

  this.formName = form;
  this.thumbWidth = thumbWidth;
  this.thumbHeight = thumbHeight;
}

GraphView.prototype.submit = _submit;
GraphView.prototype.move = _move;
GraphView.prototype.moveRel = _moveRel;
GraphView.prototype.thumbClick = _thumbClick;
GraphView.prototype.nodeClick = _nodeClick;
GraphView.prototype.canvasClick = _canvasClick;

function _canvasClick(eventObj)
{
  var imgTag = document.getElementById(this.imageID);
  var pt = _getClickXY(imgTag, eventObj);
  var vc = this.viewClip;

  var p = new Object();
  p.event = "canvas";
  p.x = pt.x + vc.x;
  p.y = pt.y + vc.y;
  this.submit(p);
}

function _nodeClick(id,part)
{
  var p = new Object();
  p.event = "click";
  p.part = part;
  p.partID = id;
  this.submit(p);
}

function _moveRel(x,y)
{
  var vc = this.viewClip;
  var xx = (vc.x + vc.width / 2) + (vc.width * x / 2);
  var yy = (vc.y + vc.height / 2) + (vc.height * y / 2);
  this.move(xx, yy);
}

/**
 * @param x, y the center of the new view port
 */
function _move(x,y)
{
  var p = new Object();
  p.event = "pan";
  p.x = x;
  p.y = y;
  this.submit(p);
}

function _submit(p)
{
  p.source = this.source;
  submitForm(this.formName,0,p);
}

function _thumbClick(tag, eventObj)
{
  var pt = _getClickXY(tag, eventObj);
  var xx = pt.x;
  var yy = pt.y;

  //alert(eventObj.clientX);

  var gb = this.graphBounds;

  var x = (xx * gb.width / this.thumbWidth) + gb.x;
  var y = (yy * gb.height / this.thumbHeight) + gb.y;
  this.move(Math.round(x), Math.round(y));
}

function _getClickXY(tag, eventObj)
{
  var pt = _getLocation(tag);
  pt.x = eventObj.clientX - pt.x;
  pt.y = eventObj.clientY - pt.y;
  return pt;
}

function _getLocation(pos)
{
 var oCurrentNode=pos;
 var rootx=0;
 var rooty=0;
 while(oCurrentNode.tagName!="BODY") {
   // alert(oCurrentNode.tagName+" y:"+rooty);
      rootx+=oCurrentNode.offsetLeft;
      rooty+=oCurrentNode.offsetTop;
      oCurrentNode=oCurrentNode.offsetParent;
 }

 return {x:rootx,y:rooty};
}

// ColorFieldFormat
//-----------------------------------------------------------------------------
/*
** Copyright (c) Oracle Corporation 2000. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/
function _getColorFieldFormat(
  colorField)
{
  var name = colorField.name;
  if (name && _cfs)
  {
    var format = _cfs[name];
    var trans  = _cfts[name];
    if (format || trans)
      return new RGBColorFormat(format, trans);
  }

  return new RGBColorFormat();
}

function _fixCFF(
  colorField)
{
  var format = _getColorFieldFormat(colorField);

  if (colorField.value != "")
  {
    var value = format.parse(colorField.value);

    if (value != (void 0))
      colorField.value = format.format(value);
  }
}


// WMLPatternFormat
//-----------------------------------------------------------------------------
/*
** Copyright (c) Oracle Corporation 2000. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/

function _wmlPatternFormat(
  value
  )
{
  return value;
}


function _wmlPatternParse(
  parseString
  )
{
  var formatPattern = this._pattern;
  //alert(formatPattern);
  var formatLength  = formatPattern.length;
  var parseLength   = parseString.length;
  var outString     = "";
  
  var parseIndex     = 0;
  var isEscaped      = false;
  var minRepeatCount = 1;
  var maxRepeatCount = 1;
    
  for (var formatIndex = 0; formatIndex < formatLength; formatIndex++)
  {
    var currFormatChar = formatPattern.charAt(formatIndex);
    
    if (currFormatChar != "\\")
    {       
      if (isEscaped)
      {        
        if (parseString.charAt(parseIndex) != currFormatChar)
        {
          return (void 0);
        }
        else
        {
          isEscaped = false;
          minRepeatCount = 1;
          maxRepeatCount = 1;
          parseIndex++;
        }
      }
      else
      {
        // check for the infinite repeat character
        if (currFormatChar == "*")
        {          
          minRepeatCount = 0;
          maxRepeatCount = Infinity;
        }
        else
        {
          // check for the maximum repeat character
          if (isDigit(currFormatChar))
          {
            minRepeatCount = 0;
            maxRepeatCount = parseDigit(currFormatChar);
          }
          else
          {
            var lastParseIndex = parseIndex + minRepeatCount;
            
            // there aren't enough characters for the length
            if (lastParseIndex > parseLength)
              return (void 0);
            
            //
            // Check all of the required format items
            //
            while (parseIndex < lastParseIndex)
            {
              var parseChar = parseString.charAt(parseIndex);
              
              if (!_testWMLChar(currFormatChar, parseChar))
              {
                // character test failed
                // alert("Failed:" + currFormatChar + "(" + parseChar + ")");
                return (void 0);
              }
              
              // add this character to the parsed output
              outString += parseChar;
  
              parseIndex++;
            }
            
            //
            // Check all of the optional format items
            //
            var maxParseIndex = Math.min(parseLength,
                                         lastParseIndex + (maxRepeatCount -
                                                           minRepeatCount));
                                                     
            while (parseIndex < maxParseIndex)
            {
              var parseChar = parseString.charAt(parseIndex);

              if (!_testWMLChar(currFormatChar,
                                parseString.charAt(parseIndex)))
              {
                break;
              }
  
              // add this character to the parsed output
              outString += parseChar;

              parseIndex++;
            }
            
            // reset the repeat counts for next time
            minRepeatCount = 1;
            maxRepeatCount = 1;
          }
        }
      }
    }
    else
    {
      isEscaped = true;
    }
  }
  
  // only return the outstring if we have consumed all of the input
  // characters
  if (parseIndex == parseLength)
  {
    return outString;
  }
  else
  {
    return (void 0);
  }
}

function _testWMLChar(
  formatChar,
  parseChar
  )
{
  if (formatChar == "A")
  {
    // uppercase letter or punctuation
    return !isDigit(parseChar) && isNotLowerCase(parseChar);
  }
  else if (formatChar == "a")
  {
    // lowercase letter or punctuation
    return !isDigit(parseChar) && isNotUpperCase(parseChar);
  }
  else if (formatChar == "N")
  {
    // Number
    return isDigit(parseChar);
  }
  else if (formatChar == "X")
  {
    // any uppercase
    return isUpperCase(parseChar);
  }
  else if (formatChar == "x")
  {
    return isLowerCase(parseChar);
  }
  else if (formatChar == "M")
  {
    // any character, assume upper case
    return true;
  }
  else if (formatChar == "m")
  {
    // any character, assume lower case
    return true;
  }
  else
  {
    // unknown format character
    // alert("Unknown format character:" + formatChar);
    return false;
  }
}

function WMLPatternFormat(
  pattern
  )
{
  this._class   = "WMLPatternFormat";
  this._pattern = pattern;
}


WMLPatternFormat.prototype = new Format();
WMLPatternFormat.prototype.format = _wmlPatternFormat;
WMLPatternFormat.prototype.parse  = _wmlPatternParse;
// Shuttle
//-----------------------------------------------------------------------------

var _shuttle_no_name = "You must supply the shuttle's name to create a proxy";
var _shuttle_no_form_name_provided = "A form name must be provided";
var _shuttle_no_form_available = "This shuttle is not in a form";


//========================================================================
//
// (Public) ShuttleProxy Object
//

// ShuttleProxy instances can be used to determine information about
// a shuttle at runtime and gain access to the data it contains.
// A ShuttleProxy can be constructed from just the name of the shuttle,
// or the shuttle name and its form name. If the form name is not
// supplied, it will be determined at runtime.


// Constructor
function ShuttleProxy(
  shuttleName,
  formName
  )
{
  if (shuttleName == (void 0))
  {
    alert(_shuttle_no_name);
    this.shuttleName = "";
    this.formName = "";
    return;
  }

  this.shuttleName = shuttleName;

  this.formName = "";
  // get the form name
  if (formName == (void 0))
  {
    // find the first form with this shuttle
    var formsLength = document.forms.length;
    var listName = shuttleName + ":leading";

    for (var i = 0; i < formsLength; i++)
    {
      if (document.forms[i][listName] != (void 0))
      {
        this.formName = document.forms[i].name;
        break;
      }
    }
    if(this.formName == "")
    {
      alert(shuttle_no_form_available);
      return;
    }
  }
  else
  {
    this.formName = formName;
  }
}

//
// Public prototype method hookups
//

//
// Calling getItems(boolean leadingList) on a ShuttleProxy will return an
// array of the items in the given list of the shuttle
//
ShuttleProxy.prototype.getItems = _getItems;

//
// Calling getSelectedItems(boolean leadingList) on a ShuttleProxy will
// return an array of the selected items in the given list of the shuttle
//
ShuttleProxy.prototype.getSelectedItems = _getSelectedItems;

//
// Calling getItemCount(boolean leadingList) on a ShuttleProxy will
// return the number of items in the given list of the shuttle
//
ShuttleProxy.prototype.getItemCount = _getItemCount;

//
// Calling getSelectedItemCount(boolean leadingList) on a ShuttleProxy will
// return the number of selected items in the given list of the shuttle
//
ShuttleProxy.prototype.getSelectedItemCount = _getSelectedItemCount;

//
// Calling addItem(boolean leadingList, int index, string text, string value, string description)
// on a ShuttleProxy will insert a new option into the given list at the
// specified index with the specified text, value, and description.
//
ShuttleProxy.prototype.addItem = _addItem;

//
// Calling deleteItemByValue(boolean leadingList, string value)
// on a ShuttleProxy will delete the option with the given value
// from the given list.
//
ShuttleProxy.prototype.deleteItemByValue = _deleteItemByValue;

//
// Calling deleteSelectedItems(boolean leadingList)
// on a ShuttleProxy will delete all selected items from the given list
//
ShuttleProxy.prototype.deleteSelectedItems = _deleteSelectedItems;

//
// Calling move(boolean fromLeadingList, boolean allItems) on a ShuttleProxy
// will move the selected items, or all items depending on allItems parameter,
// from the given list to the other list.
//
ShuttleProxy.prototype.move = _move;

//
// Calling reorderList(boolean down, boolean allTheWay, boolean leadingList) on
// a ShuttleProxy will move the selected items in the second list of the proxy
//in the direction specified.  If allTheWay is true it will move the items all
// the way to the top or bottom.  Otherwise the items move one slot.
//
ShuttleProxy.prototype.reorderList = _reorderList;


//
// Calling reset will reset the shuttle to its initial state.
//
ShuttleProxy.prototype.reset = _reset;


/*===========================================================================*/
/*-----------------------------------------------------------------------
 *PRIVATE SHUTTLE METHODS
 *-----------------------------------------------------------------------*/


/*
 * _remove
 *
 * This function removes the number of elements specified
 * by deleteCount from an array starting at the index given
 * by start
 */

function _remove( array, start, deleteCount )
{

  var len = array.length;

  if (deleteCount > len)
    return;

  for ( var i = start; i < len ; i++)
  {

    if ( i < len - deleteCount )
      array[i] = array[ i + deleteCount];
    else
      array[ i ] = void 0;
  }

  array.length = len - deleteCount;

}


/*
 * _displayDesc
 *
 * Displays the description in the description area below the list.
 *
 */
function _displayDesc(
  listName,
  formName
  )
{


  if(formName == (void 0))
  {
    alert(_shuttle_no_form_name_provided);
    return;
  }


  if(formName.length == 0)
  {
    alert(shuttle_no_form_available);
    return;
  }

  // the textInput where descriptions are displayed
  var descArea = document.forms[formName].elements[ listName + ':desc'];

  if( descArea == void(0))
  {
    return;
  }


  // the array of descriptions
  var descArray  =  _getDescArray( listName );

  if( descArray == (void 0) || descArray.length == 0)
  {
    return;
  }

  //get the indexes of the selected items
  var selItems = _getSelectedIndexes(formName, listName);

  //if no items are selected, return
  if(selItems.length == 0)
  {
    descArea.value = "";
    _setSelected( listName, selItems );
    return;
  }

  // get the last description selected
  var selOptDesc = _getSelectedDesc( listName, descArray, selItems );

  // set the value of the description area to be the last selected item
  descArea.value = selOptDesc;

  // set which items are currently selected
  _setSelected( listName, selItems );


}


/*
 * _getDescArray
 *
 * This function gets the description array
 */

function _getDescArray
(
  listName
)
{

  var descArray = window[listName.replace(':','_') + '_desc'];
  return descArray;

}

/*
 * _getSelectedDesc
 *
 * This function gets the last selected description. If a
 * user selects items using the control key, and they select
 * the item at index 4, then 10, and then 6, the descriptions
 * for 4, then 10, then 6 should be displayed.
 *
 * There is a local variable where what was previously selected
 * is kept and this is compared to what is currently selected.
 * These are compared to determine the last item selected.
 */

function _getSelectedDesc
(
  listName,
  descArray,
  selItems
)
{

  // get the array of the indexes of previously selected items
  var prevSelArray = _getSelectedArray( listName );

  // if only one item is currently selected return its description
  if ( selItems.length == 1 )
    return descArray[selItems[0]];


  // if the difference between the number of items
  // previously selected and currently selected is
  // not equal to one return no description, otherwise
  // it is unclear which description to display
  if ( selItems.length - prevSelArray.length != 1 )
    return "";


  // find the index now selected that was not
  // previously selected and return the description
  // for the item at that index.
  for ( var i = 0; i < selItems.length; i++ )
  {
    if ( i >= prevSelArray.length || prevSelArray[i] != selItems[i] )
      return descArray[selItems[i]];
  }

  // return an empty string if all else fails
  return "";
}

/*
 * _getSelectedArray
 *
 * This function gets the array of indexes of
 * previously selected items
 *
 */


function _getSelectedArray
(
  listName
)
{
  var selected = window[listName.replace(':','_') + '_sel'];
  return selected;
}

/*
 * _setSelected
 *
 * This function sets the array of indexes of
 * previously selected items
 *
 */

function _setSelected
(
  listName,
  selected
)
{

  var selectedArray = _getSelectedArray( listName );

  if ( selectedArray != (void 0) )
  {
    var len = selectedArray.length;
    _remove( selectedArray, 0, len);

    for ( var i = 0; i < selected.length; i++ )
    {
      selectedArray[i] = selected[i];
    }

  }

}


/*
 * _addDescAtIndex
 *
 * This function adds a description at a given index.
 */

function _addDescAtIndex
(
  descArray,
  addedDesc,
  index
)
{


  if ( descArray != (void 0 ) )
  {
    var len = descArray.length;

    for ( var i = len - 1 ; i >= index; i-- )
    {
      descArray[i + 1] = descArray[i];
    }

    descArray[index] = addedDesc;
    descArray.length = len + 1;
  }
}

/*
 * _deleteDescAtIndex
 *
 * This function removes a description at a given index.
 */

function _deleteDescAtIndex
(
  descArray,
  index
)
{
  if ( descArray != (void 0 ))
    _remove(descArray, index, 1);
}

/*
 * _deleteDescAtIndexes
 *
 * This function removes descriptions at given indexes.
 */

function _deleteDescAtIndexes
(
  descArray,
  indexes
)
{
  if ( descArray != (void 0 ))
  {
    for ( var i = indexes.length - 1; i >= 0; i--)
    {
      _remove(descArray, indexes[i], 1);
    }
  }
}


/*
 * _deleteDescAtIndexes
 *
 * Sets the textInput areas displaying descriptions to
 * the empty string.
 */

function _clearDescAreas(
  formName,
  list1,
  list2
)
{
  // move descriptions and clear description area
  var descArea1 = document.forms[formName].elements[ list1 + ':desc'];
  var descArea2 = document.forms[formName].elements[ list2 + ':desc'];

  if( descArea1 != void(0))
  {
    descArea1.value = "";
  }

  if( descArea2 != void(0))
  {
    descArea2.value = "";
  }
}



/*
 * _moveItems
 *
 * This function moves the selected items in the 'from' list to the
 * 'to' list.  If no formName is supplied, the form is found when
 * this is called.  The items are inserted in the 'to' list
 * at the bottom. The 'from' and 'to' parameters should be the
 * list names(i.e.  "<shuttleName>:leading" or "<shuttleName>:trailing")
 */
function _moveItems(
  from,
  to,
  formName
  )
{
  //get the formName if needed
  if(formName == (void 0))
  {
    formName = _findFormNameContaining(from);
  }

  if(formName.length == 0)
  {
    alert(shuttle_no_form_available);
    return;
  }

  //store the from and to lists
  var fromList = document.forms[formName].elements[from];
  var toList = document.forms[formName].elements[to];

  if ( fromList == (void 0 ) || toList == (void 0 ))
    return;


  //get all the indexes of the selected items
  var selItems = _getSelectedIndexes(formName, from);

  //if no items are selected, return with alert.
  if(selItems.length == 0)
  {
    if (_shuttle_no_items_selected.length > 0)
      alert(_shuttle_no_items_selected);
      
    return;
  }


  var fromDescArray = _getDescArray(from);
  var toDescArray = _getDescArray(to);

  //set no selection on toList so it will only have new items selected.
  toList.selectedIndex = -1;

  //get the index in the toList to start inserting at.  Length-1 because of
  //bars.
  var insertAt = toList.length-1;

  //save bar text so you know how long it should be
  var barText = toList.options[insertAt].text;

  //insert the items at the end of the toList
  for(var i=0; i<selItems.length; i++)
  {
    var oText = fromList.options[selItems[i]].text;
    var oValue = fromList.options[selItems[i]].value;

    if(i == 0)
    { //replace the bars
      toList.options[insertAt].text = oText;
      toList.options[insertAt].value = oValue;
    }
    else
    {  //have to make new item
      toList.options[insertAt] = new Option(oText, oValue, false, false);
    }

    if ( toDescArray != (void 0) && fromDescArray != (void 0) )
      toDescArray[insertAt] = fromDescArray[selItems[i]];

    toList.options[insertAt].selected = true;
    insertAt++;
  }

  //insert a new bar at bottom of toList
  toList.options[insertAt] = new Option(barText, "", false, false);
  toList.options[insertAt].selected = false;

  //remove items from fromList.  do this backward to maintain indices
  for( var i=selItems.length-1; i >= 0; i--)
  {
    if ( fromDescArray != (void 0) )
      _remove( fromDescArray, selItems[i], 1 );
    fromList.options[selItems[i]] = null;
  }

  //make no selected on fromList
  fromList.selectedIndex = -1;

  _clearDescAreas( formName, from);
  _displayDesc( to, formName );

  //make the new lists for submitting.
  _makeList(formName, from);
  _makeList(formName, to);

  // Mark the page as "dirty"
  _navDirty = true;
}

/*
 * _moveAllItems
 *
 * This function moves all the items in the 'from' list to the
 * 'to' list.  If no formName is supplied, the form is found when
 * this is called.  The items are inserted in the 'to' list
 * at the bottom. The 'from' and 'to' parameters should be the
 * list names(i.e.  "<shuttleName>:leading" or "<shuttleName>:trailing")
 */
function _moveAllItems(
  from,
  to,
  formName
  )
{
  //get the formName is needed
  if(formName == (void 0))
  {
    formName = _findFormNameContaining(from);
  }

  //get the lists
  var fromList = document.forms[formName].elements[from];
  var toList = document.forms[formName].elements[to];

  //save the bar text for later use.
  var barText =
    toList.options[document.forms[formName].elements[to].length-1].text

  //get the index to start inserting at in the toList.  length-1 because of
  //bars
  var insertAt = toList.length-1;
  var fromDescArray = _getDescArray(from);
  var toDescArray = _getDescArray(to);

  //move the items
  if (fromList.length > 1)
  {
    //move all but the last (bars).
    var initialLength = fromList.length
    for(var i=0; i<initialLength-1; i++)
    {
      var oText = fromList.options[0].text;
      var oValue = fromList.options[0].value;
      fromList.options[0] = null;
      if(i == 0)
      { //replace the bars
        toList.options[insertAt].text = oText;
        toList.options[insertAt].value = oValue;
      }
      else
      { //make new option
        toList.options[insertAt] = new Option (oText,oValue,false,false);

      }

      if ( toDescArray != (void 0) && fromDescArray != (void 0) )
        toDescArray[insertAt] = fromDescArray[i];

      insertAt++;
    }

    //insert a new bar:
    toList.options[insertAt] = new Option(barText, "", false, false);
    toList.options[insertAt].selected = false;

    if ( fromDescArray != (void 0) )
    {
      var len = fromDescArray.length;
      _remove(fromDescArray, 0, len);
    }

    //set no selection on both lists
    fromList.selectedIndex = -1;
    toList.selectedIndex = -1;

    _clearDescAreas( formName, from, to );


    //make the lists for submission
    _makeList(formName, from);
    _makeList(formName, to);

    // Mark the page as "dirty"
    _navDirty = true;
  }
  else if (_shuttle_no_items.length > 0)
  {
    alert(_shuttle_no_items);
  }
}

/*
 * _orderList
 *
 * This function reorders the given list by shifting the selections in
 * the given direction.  If no formName is supplied, the form is found when
 * this is called. The 'list' parameter should be the
 * list name(i.e.  "<shuttleName>:leading" or "<shuttleName>:trailing")
 */
function _orderList(
  down,
  list,
  formName
  )
{
  //get the formName if needed
  if(formName == (void 0))
  {
    formName = _findFormNameContaining(list);
  }

  //get the actual list
  var colList = document.forms[formName].elements[list];

  //get all the selected item indexes
  var selItems = _getSelectedIndexes(formName, list);

  //if no items are selected, return with alert.
  if(selItems.length == 0)
  {
    if (_shuttle_no_items_selected.length > 0)
      alert(_shuttle_no_items_selected);
      
    return;
  }

  var descArray = _getDescArray(list);

  // Start with the last selected index and move up, working by blocks
  var processed = selItems.length - 1;
  while (processed >= 0)
  {
    var lastInBlock = selItems[processed];
    var firstInBlock = lastInBlock;

    var tempIndex = processed;


    // find the first index in that block
    while ((tempIndex > 0) && ((selItems[tempIndex] -
                                selItems[tempIndex - 1]) == 1))
    {
      tempIndex--;
      firstInBlock--;
    }

    if (down == 0)
    {
      // move this block up
      // if we are at the top, do nothing
      if(firstInBlock != 0)
      {
        //get the text and value of the one space above the block
        var oText = colList.options[firstInBlock-1].text;
        var oValue = colList.options[firstInBlock-1].value;

        if ( descArray != (void 0) )
          var dValue = descArray[firstInBlock - 1];

        //move the block up one at a time
        for (var i = firstInBlock; i <= lastInBlock; i++)
        {
          colList.options[i-1].text = colList.options[i].text;
          colList.options[i-1].value = colList.options[i].value;
          colList.options[i-1].selected = true;

          if ( descArray != (void 0) )
            descArray[i-1] = descArray[i];
        }

         //put the info of the slot above the selection below it
        colList.options[lastInBlock].text = oText;
        colList.options[lastInBlock].value = oValue;
        colList.options[lastInBlock].selected = false;

        if ( descArray != (void 0) )
          descArray[lastInBlock] = dValue;

        // Mark the page as "dirty"
        _navDirty = true;
      }
    }
    else
    {
      // move this block down
      // if we are at the bottom, do nothing
      if(lastInBlock != colList.length-2)
      {
        //get the text and value of the one space below the block
        var oText = colList.options[lastInBlock+1].text;
        var oValue = colList.options[lastInBlock+1].value;

        if ( descArray != (void 0) )
          var dValue = descArray[lastInBlock+1];

         //move the block down one at a time
        for (var i = lastInBlock; i >= firstInBlock; i--)
        {
          colList.options[i+1].text = colList.options[i].text;
          colList.options[i+1].value = colList.options[i].value;
          colList.options[i+1].selected = true;

          if ( descArray != (void 0) )
            descArray[i+1] = descArray[i];
        }

         //put the info of the slot below the selection above it
        colList.options[firstInBlock].text = oText;
        colList.options[firstInBlock].value = oValue;
        colList.options[firstInBlock].selected = false;

        if ( descArray != (void 0) )
          descArray[firstInBlock] = dValue;

        // Mark the page as "dirty"
        _navDirty = true;
      }
    }

    processed = tempIndex - 1;
  }

  _displayDesc( list, formName );

  //make the list for submission
  _makeList(formName, list);
}

/*
 * _orderTopBottomList
 *
 * This function reorders the given list by shifting the selections all the way
 * in the given direction.  If no formName is supplied, the form is found when
 * this is called. The 'list' parameter should be the
 * list name(i.e.  "<shuttleName>:leading" or "<shuttleName>:trailing")
 */
function _orderTopBottomList(
  down,
  list,
  formName
  )
{
  //get the formname if needed
  if(formName == (void 0))
  {
    formName = _findFormNameContaining(list);
  }

  //get the actual list
  var colList = document.forms[formName].elements[list];

  //get all the indexes of the items selected in the list
  var selItems = _getSelectedIndexes(formName, list);

  //if no items are selected, return with alert.
  if(selItems.length == 0)
  {
    if (_shuttle_no_items_selected.length > 0)
      alert(_shuttle_no_items_selected);
      
    return;
  }

  var descArray = _getDescArray(list);
  var moveDescArray = new Array();
  var selDescArray = new Array();

  var moveItemsText = new Array();
  var moveItemsValue = new Array();
  var moveItemsIndex = 0;
  if(down == 0)
  {
    //get an array of all the items we will have to displace in order
    var selItemsIndex = 0;
    var moveItemsIndex = 0;
    for(var colListIndex=0;
        colListIndex < selItems[selItems.length - 1];
        colListIndex++)
    {
      if(colListIndex != selItems[selItemsIndex])
      {
        moveItemsText[moveItemsIndex] = colList.options[colListIndex].text;
        moveItemsValue[moveItemsIndex] = colList.options[colListIndex].value;

        if (  descArray != (void 0) )
          moveDescArray[moveItemsIndex] = descArray[colListIndex];

        moveItemsIndex++
      }
      else
      {

        if ( descArray != (void 0) )
          selDescArray[selItemsIndex] = descArray[colListIndex];

        selItemsIndex++;

      }
    }

    if ( descArray != (void 0) )
      selDescArray[selItemsIndex] = descArray[colListIndex];


    //place items to move toward top of col
    for(var i = 0; i < selItems.length; i++)
    {
      colList.options[i].text = colList.options[selItems[i]].text;
      colList.options[i].value = colList.options[selItems[i]].value;
      colList.options[i].selected = true;

      if ( descArray != (void 0) )
        descArray[i] = selDescArray[i];
    }

    //place displaced items below
    for(var j = 0; j < moveItemsText.length; j++)
    {
      colList.options[i].text = moveItemsText[j];
      colList.options[i].value = moveItemsValue[j];
      colList.options[i].selected = false;

      if ( descArray != (void 0) )
        descArray[i] = moveDescArray[j];
      i++
    }



  }
  else
  {
    //get an array of all the items we will have to displace in order
    var selItemsIndex = 1;
    var moveItemsIndex = 0;

    if ( descArray != (void 0) )
      selDescArray[0] = descArray[selItems[0]];

    for(var colItemsIndex=selItems[0]+1;
        colItemsIndex <= colList.length-2;
        colItemsIndex++)
    {
      if((selItemsIndex == selItems.length) ||
         (colItemsIndex != selItems[selItemsIndex]))
      {
        moveItemsText[moveItemsIndex] = colList.options[colItemsIndex].text;
        moveItemsValue[moveItemsIndex] = colList.options[colItemsIndex].value;

        if ( descArray != (void 0) )
          moveDescArray[moveItemsIndex] = descArray[colItemsIndex];

        moveItemsIndex++;
      }
      else
      {
        if ( descArray != (void 0) )
          selDescArray[selItemsIndex] = descArray[colItemsIndex];

        selItemsIndex++;
      }
    }


    //place items to move toward bottom of col
    var j = colList.length - 2;
    for(var i = selItems.length-1; i >= 0; i--)
    {
      colList.options[j].text = colList.options[selItems[i]].text;
      colList.options[j].value = colList.options[selItems[i]].value;
      colList.options[j].selected = true;

      if ( descArray != (void 0) )
        descArray[j] = selDescArray[i];
      j--;
    }


    //place displaced items above
    for(var i = moveItemsText.length-1; i >= 0; i--)
    {
      colList.options[j].text = moveItemsText[i];
      colList.options[j].value = moveItemsValue[i];
      colList.options[j].selected = false;

      if ( descArray != (void 0) )
        descArray[j] = moveDescArray[i];
      j--
    }
  }

  _displayDesc( list, formName );

  //make the list for submission
  _makeList(formName, list);

  // Mark the page as "dirty"
  _navDirty = true;
}

// helper functions
function _getSelectedIndexes(
  formName,
  listName
  )
{
  var colList = document.forms[formName].elements[listName];
  var selItems = new Array();
  var selItemsIndex=0;
  //minus 1 for bars
  for(var colListIndex=0; colListIndex<colList.length-1; colListIndex++)
  {
    if(colList.options[colListIndex].selected)
    {
      selItems[selItemsIndex] = colListIndex;
      selItemsIndex++;
    }
  }

  return selItems;
}

function _findFormNameContaining(
  element
  )
{
  var formsLength = document.forms.length;

  for (var i = 0; i < formsLength; i++)
  {
    if (document.forms[i][element] != (void 0))
    {
      return document.forms[i].name;
    }
  }
  return "";
}

function _makeList(
  formName,
  listName
  )
{
  var list = document.forms[formName].elements[listName];
  
  if ( list == null )
    return;
    
  
  var val = "";  
  for(var i=0; i< list.length - 1; i++)
  {
    if(list.options[i].value.length > 0)
    {
      val = val +
            _trimString(list.options[i].value)
            + ';';
    }
    else
    {
      val = val +
            _trimString(list.options[i].text)
            + ';';
    }
  }
  document.forms[formName].elements[listName+':items'].value = val;
}

function _trimString (
  str
  )
{
  var j = str.length - 1;
  if(str.charAt(j) != ' ')
  {
    return str;
  }
  while ((str.charAt(j) == ' ') && (j > 0))
  {
    j = j - 1;
  }
  str = str.substring(0, j+1);
  return str;
}

function _getListName(
  shuttleName,
  leadingList
  )
{
  var theListName = (leadingList) ? shuttleName + ":leading" :
                                  shuttleName + ":trailing";
  return theListName;
}


/**
* Reset items to their original values
* 
*/
function _resetItems(
  shuttleName,
  formName)
{
  
  //get the formName if needed
  if(formName == (void 0))
  {
    formName = _findFormNameContaining(from);
  }

  if(formName.length == 0)
  {
    alert(shuttle_no_form_available);
    return;
  }

  // get list names
  leadingListName = _getListName( shuttleName, true);
  trailingListName = _getListName( shuttleName, false);

  // get current lists  
  var leadingList  = document.forms[formName].elements[leadingListName];
  var trailingList = document.forms[formName].elements[trailingListName];

  // get original lists  
  var origLists = _getOriginalLists(shuttleName, formName);
  var origLeadingList  = origLists.leading;
  var origTrailingList = origLists.trailing;  
      
  // get original description arrays
  var origLeadingDescArray =  _getDescArray(leadingListName);
  var origTrailingDescArray = _getDescArray(trailingListName);
  
  // reset values of lists
  _resetToOriginalList( origLeadingList, origLeadingDescArray, leadingList );  
  _resetToOriginalList( origTrailingList, origTrailingDescArray, trailingList );
  
  //make the new lists for submitting.
  _makeList(formName, leadingListName);
  _makeList(formName, trailingListName);  
  
  // return that no reload necessary
  return false;
}



/*
 * _getOriginalLists
 *
 * This function gets a copy of the original lists list
 */
function _getOriginalLists
(
  shuttleName,  
  formName
)
{

  var originalLists = window['_' + formName + '_' + shuttleName + '_orig'];
  return originalLists;

}

/** 
 * Given the original list info,
 * reset the list and description info
 *
 */
function _resetToOriginalList
(
  origList,
  descArray,
  list
)
{

  // If the original list or new list are null, return
  if ( origList == (void 0) || list == (void 0) )
    return;

  // reset selectedIndex
  list.selectedIndex = origList.selectedIndex;
  
  var i = 0;
  
  for( ;i < origList.options.length;i++) 
  {
    var oText            = origList.options[i].text;
    var oValue           = origList.options[i].value;
    var oDefaultSelected = origList.options[i].defaultSelected;
    var oSelected        = origList.options[i].selected;
    
    // =-= gc Replacing values for options that already exist 
    // doesn't work properly on Mozilla 1.3. If you
    // uncomment the following it breaks on mozilla, but if you then 
    // add "alert('hi');" at the end of this method, it works. Wacky.
    /*
    if(list.options[i] != (void 0 ))
    { 
      list.options[i].text = oText;
      list.options[i].value = oValue;
      list.options[i].defaultSelected = oDefaultSelected;
      list.options[i].selected = oSelected;
    }
    else
    */
    { 
      // make new option
      list.options[i] = new Option(oText, oValue, 
                                   oDefaultSelected, oSelected);      
      
      // Netscape 4 bug, selection using constructor not working,
      // Remove next 2 lines when we desupport Netscape 4.
      list.options[i].defaultSelected = oDefaultSelected;     
      list.options[i].selected = oSelected;        
    }

    // reset description
    if ( descArray != (void 0 ))
      descArray[i] = origList.descriptions[i];    
        
  }
  
  
    
  var max = list.options.length - 1;

  // nulling out extra options from high end of array 
  // because doing otherwise not working.
  while ( max >= i )
  { 
    if ( descArray != (void 0 )) 
      descArray[max] = null;    
      
    list.options[max] = null;
    max--;
  }
  
  
 }


 /**
  * _copyLists
  *
  * Copy the lists and descriptions info to an object
  * 
  *
  */
 function _copyLists( shuttleName, formName )
 {

   //get the formName if needed
   if(formName == (void 0))
   {
     formName = _findFormNameContaining(from);
   }

   if(formName.length == 0)
   {
     alert(shuttle_no_form_available);
     return;
   }

   var origLists = new Object();
   origLists.leading = _copyList( _getListName( shuttleName, true), formName);
   origLists.trailing = _copyList( _getListName( shuttleName, false), formName);


   return origLists;
 }                 


 /**
  * _copyList
  *
  * copy the values in a single list
  */
 function _copyList( listName, formName )
 {  
   if ( formName == (void 0 ) || listName == (void 0))
     return;
     
   var origList = document.forms[formName].elements[listName];
      
   if ( origList == null)
     return;
   
   var origDescs = _getDescArray(listName);

   var copyList = new Object();

 
   copyList.selectedIndex = origList.selectedIndex;
   copyList.options = new Array();
   copyList.descriptions = new Array();

   for ( var i = 0; i < origList.options.length; i++ )
   {
     copyList.options[i] = new Option( origList.options[i].text,
                                       origList.options[i].value,
                                       origList.options[i].defaultSelected,
                                       origList.options[i].selected);
     
     // Netscape 4 bug, selection using constructor not working,
     // Remove next 2 lines when we desupport Netscape 4.
     copyList.options[i].defaultSelected = origList.options[i].defaultSelected;     
     copyList.options[i].selected = origList.options[i].selected;                                    

     if (origDescs != null )
       copyList.descriptions[i] = origDescs[i];                                     
   }


   return copyList;
 }

//------------------------------------------------------------------------

//
// METHOD IMPLEMENTATIONS FOR PROXY
//




function _reset()
{
  
  _resetItems( this.shuttleName, this.formName);
}





function _move(
  fromLeadingList,
  allItems
  )
{
  //default parameters not given
  if(allItems == (void 0))
  {
    allItems = false;
  }
  if(fromLeadingList == (void 0))
  {
    fromLeadingList = true;
  }

  //get the list names
  var fromListName = _getListName(this.shuttleName, fromLeadingList);
  var toListName = _getListName(this.shuttleName, !fromLeadingList);

  //move the items
  if(allItems)
  {
    _moveAllItems(fromListName, toListName, this.formName);
  }
  else
  {
    _moveItems(fromListName, toListName, this.formName);
  }
}

function _reorderList(
  down,
  allTheWay,
  leadingList
)
{
  //default params not given
  if(leadingList == (void 0))
  {
    leadingList = true;
  }
  if(allTheWay == (void 0))
  {
    allTheWay = false;
  }
  if(down == (void 0))
  {
    down = false;
  }

  //get the listName
  var listName = _getListName(this.shuttleName, leadingList);



  //reorder the list
  if(!allTheWay)
  {
    _orderList(down, listName, this.formName);
  }
  else
  {
    _orderTopBottomList(down, listName, this.formName);
  }
}


function _getItems(
  leadingList
  )
{
  //default params not given
  if(leadingList == (void 0))
  {
    leadingList = true;
  }

  //get the list name
  var listName = _getListName(this.shuttleName, leadingList);


  var list = document.forms[this.formName].elements[listName];

  var items = new Array();
  //length - 1 because of bars
  for(var i=0; i<list.length-1; i++)
  {
    items[i] = list.options[i];
  }

  return items;
}

function _getSelectedItems(
  leadingList
  )
{
  //default params not given
  if(leadingList == (void 0))
  {
    leadingList = true;
  }

  var listName = _getListName(this.shuttleName, leadingList);


  var list = document.forms[this.formName].elements[listName];

  var items = new Array();
  var j = 0;
  //length - 1 because of bars
  for(var i=0; i<list.length-1; i++)
  {
    if(list.options[i].selected)
    {
      items[j] = list.options[i];
      j++;
    }
  }

  return items;
}

function _getItemCount(
  leadingList
  )
{
  //default params not given
  if(leadingList == (void 0))
  {
    leadingList = true;
  }

  var listName = _getListName(this.shuttleName, leadingList);


  //minus 1 for bars
  return document.forms[this.formName].elements[listName].length - 1;
}

function _getSelectedItemCount(
  leadingList
)
{
  //default params not given
  if(leadingList == (void 0))
  {
    leadingList = true;
  }

  var listName = _getListName(this.shuttleName, leadingList);


  var list = document.forms[this.formName].elements[listName];

  var j = 0;
  //length - 1 because of bars
  for(var i=0; i<list.length-1; i++)
  {
    if(list.options[i].selected)
    {
       j++;
    }
  }

  return j;
}

function _addItem(
  leadingList,
  index,
  text,
  value,
  description
  )
{
  //default params not given
  if(value == (void 0))
  {
    value = "";
  }
  if(text == (void 0))
  {
    text = "";
  }
  if(description == (void 0))
    {
      description = "";
  }
  if(leadingList == (void 0))
  {
    leadingList = true;
  }


  //get the list
  var listName = _getListName(this.shuttleName, leadingList);



  //get an appropriate index
  if(index == (void 0))
  { //minus 1 for bars
    index = document.forms[this.formName].elements[listName].length - 1;
  }

  if(index < 0)
  {
    index = 0;
  }

  //minus 1 for bars
  if(index > document.forms[this.formName].elements[listName].length - 1)
  {
    index = document.forms[this.formName].elements[listName].length - 1;
  }

  //first move all items at that index and below down to make room for this
  //new item.
  var theList = document.forms[this.formName].elements[listName];

  //make a new option for the bars
  theList.options[theList.length] =
       new Option(theList.options[theList.length-1].text,
                  theList.options[theList.length-1].value,
                  false,
                  false);

  //move items down
  for(var i = theList.length - 1; i > index; i--)
  {
    theList.options[i].text = theList.options[i-1].text;
    theList.options[i].value = theList.options[i-1].value;
    theList.options[i].selected = theList.options[i-1].selected;
  }

  //insert the new item
  theList.options[index].text = text;
  theList.options[index].value = value;
  theList.options[index].selected = false;

  // add description
  var descArray = _getDescArray(listName);
  _addDescAtIndex( descArray, description, index);

  _makeList(this.formName, listName);

  // Mark the page as "dirty"
  _navDirty = true;
}

function _deleteItemByValue(
  leadingList,
  value
  )
{
  if(value == (void 0))
  {
    return;
  }


  //get the list
  var listName = _getListName(this.shuttleName, leadingList);
  var theList = document.forms[this.formName].elements[listName];

  for(var i=0; i<theList.length-1; i++)
  {
    var val = theList.options[i].value;
    if(val == value)
    {
      var descArray = _getDescArray( listName );
      _deleteDescAtIndex( descArray, i);
      _clearDescAreas(this.formName, listName);

      theList.options[i] = null;
      _makeList(this.formName, listName);

      // Mark the page as "dirty"
      _navDirty = true;

      return;
    }
  }

}

function _deleteSelectedItems(
  leadingList
  )
{
  if(leadingList == (void 0))
  {
    leadingList = true;
  }


  //get the list
  var listName = _getListName(this.shuttleName, leadingList);
  var theList = document.forms[this.formName].elements[listName];

  var selIndexes = _getSelectedIndexes(this.formName,listName);

  for(var i = selIndexes.length; i >=0; i--)
  {
    theList.options[selIndexes[i]] = null;
  }

  var descArray = _getDescArray(listName);

  _deleteDescAtIndexes( descArray, selIndexes);

  _clearDescAreas(this.formName, listName);

  _makeList(this.formName, listName);

   // Mark the page as "dirty"
   _navDirty = true;
}


// ColorFormat
//-----------------------------------------------------------------------------
/*
** Copyright (c) Oracle Corporation 2000. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/


// External variables used:
//  _cfTrans    - the localized text for transparent colors

var _cfTrans;

function _rgbColorFormat(
  formatColor)
{
  // return localized transparent text for transparent color
  if (formatColor.alpha == 0)
  {
    if (this._allowsTransparent)
      return _cfTrans;
    else
      return (void 0);
  }

  var stringHolder = new Object();
  stringHolder.value ="";
  
  var pattern = this._pattern;
  if (typeof pattern != "string")
    pattern = pattern[0];
    
  _cfoDoClumping(pattern,
              _cfoSubformat,
              formatColor,
              stringHolder);
  
  return stringHolder.value;
}


/**
 * Parses a String into a Color using the current object's pattern.  If the
 * parsing fails, undefined will be returned.
 */
function _rgbColorParse(
  parseString)
{
  // return transparent color for localized transparent text
  if (this._allowsTransparent && _cfTrans == parseString)
    return new Color(0,0,0,0);
    
  var pattern = this._pattern;
  if (typeof pattern == "string")
  {
    return _rgbColorParseImpl(parseString,
                              pattern);
  }
  else
  { 
    var i;
    for (i = 0; i < pattern.length; i++)
    {
      var color = _rgbColorParseImpl(parseString,
                                     pattern[i]);
      if (color != (void 0))
        return color;
    }
  }
}

function _rgbColorParseImpl(
  parseString,
  parsePattern)
{
  var parseContext = new Object();
  parseContext.currIndex = 0;
  parseContext.parseString = parseString;
  parseContext.parseException = new ParseException();

  var parsedColor = new Color(0x00, 0x00, 0x00);

  // parse the color
  if (_cfoDoClumping(parsePattern,
                  _cfoSubParse,
                  parseContext,
                  parsedColor))
  {
    if (parseString.length != parseContext.currIndex)
    {
      return (void 0);
    }

    return parsedColor;
  }
  else
  {
    // failure
    return (void 0);
  }
}


/**
 * Clump up similar runs of pattern characters from the format patter and
 * call the subfunction for each result.  Return whether the clumping
 * succeeded.
 */
function _cfoDoClumping(
  formatPattern,
  subFunction,
  param,
  outValue
  )
{  
  var formatLength = formatPattern.length;
  var inQuote      = false;
  var kindCount    = 0;
  var lastChar     = void 0;
  var startIndex   = 0;
  
  for (var i = 0; i < formatLength; i++)
  {
    var currChar = formatPattern.charAt(i);
    
    if (inQuote)
    {
      if (currChar == "\'")
      {
        inQuote = false;
        
        // handle to single quotes in a row as escaping the quote
        // by not skipping it when outputting
        if (kindCount != 1)
        {
          startIndex++;
          kindCount--;
        }

        // output the quoted text
        if (!subFunction(formatPattern,
                         "\'",
                         startIndex,
                         kindCount,
                         param,
                         outValue))
        {
          // failure
          // alert("failure at " + startIndex + " with " + lastChar);
          return false;
        }
        
        kindCount = 0;
        lastChar  = void 0;
      }
      else
      {
        // keep adding characters to the escaped String
        kindCount++;
      }
    }
    else
    {
      // the characters that we are collecting have changed
      if (currChar != lastChar)
      {
        if (kindCount != 0)
        {       
          // output the previously collected string
          if (!subFunction(formatPattern,
                           lastChar,
                           startIndex,
                           kindCount,
                           param,
                           outValue))
          {
            // failure
            //alert("failure at " + startIndex + " with " + lastChar);
            return false;
          }
          
          kindCount = 0;
          lastChar  = void 0;
        }
        
        if (currChar == '\'')
        {
          inQuote = true;
        }
  
        startIndex = i;      
        lastChar = currChar;
      }

      // keep collecting this kind of character together
      kindCount++;
    }   
  }
  
  // output any left over substring being collected
  if (kindCount != 0)
  {
    if (!subFunction(formatPattern,
                     lastChar,
                     startIndex,
                     kindCount,
                     param,
                     outValue))
    {
      // failure
      //alert("failure at " + startIndex + " with " + lastChar);
      return false;
    }
  }
  
  // success
  return true;
}


/**
 * Format a clump of pattern elements using the specified time.
 */
function _cfoSubformat(
  inString,
  formatType,
  startIndex,
  charCount,
  color,
  stringHolder
  )
{    
  // string to append to the toString
  var appendString = null;
  
  if ((formatType >= 'A') && (formatType <= 'Z') ||
      (formatType >= 'a') && (formatType <= 'z'))
  {
    switch (formatType)
    {
      case 'r': // decimal red component (0-255)
        appendString = _cfoGetPaddedNumber(color.red, charCount, 3, 10);
        break;
      
      case 'g': // decimal green component (0-255)
        appendString = _cfoGetPaddedNumber(color.green, charCount, 3, 10);
        break;
      
      case 'b': // decimal blue component (0-255)
        appendString = _cfoGetPaddedNumber(color.blue, charCount, 3, 10);
        break;
      
      case 'a': // decimal alpha component (0-255)
        appendString = _cfoGetPaddedNumber(color.alpha, charCount, 3, 10);
        break;
      
      case 'R': // hex red component (0x00-0xff)
        appendString = 
          _cfoGetPaddedNumber(color.red, charCount, 2, 16).toUpperCase();
        break;
      
      case 'G': // hex green component (0x00-0xff)
        appendString = 
          _cfoGetPaddedNumber(color.green, charCount, 2, 16).toUpperCase();
        break;
      
      case 'B': // hex blue component (0x00-0xff)
        appendString = 
          _cfoGetPaddedNumber(color.blue, charCount, 2, 16).toUpperCase();
        break;
      
      case 'A': // hex alpha component (0x00-0xff)
        appendString = 
          _cfoGetPaddedNumber(color.alpha, charCount, 2, 16).toUpperCase();
        break;
      
      default:
        // do nothing rather than throw an exception
        appendString = "";
    }
  }
  else
  {
    // all other results are literal
    appendString = inString.substring(startIndex, startIndex + charCount);
  }
  
  stringHolder.value += appendString;
  
  // formatting should never fail
  return true;
}


/**
 * Parse a substring using a clump of format elements.
 */
function _cfoSubParse(
  inString,
  formatType,
  startIndex,
  charCount,
  parseContext,
  parsedColor
  )
{
  // Start index of the string being parsed (as opposed
  // to startIndex, which is the index on the format mask)
  var inStartIndex = parseContext.currIndex;


  if ((formatType >= 'A') && (formatType <= 'Z') ||
      (formatType >= 'a') && (formatType <= 'z'))
  {
    switch (formatType)
    {
      case 'r': // decimal red component (0-255)
        parsedColor.red = _cfoAccumulateNumber(parseContext, charCount, 3, 10);
        if (parsedColor.red == (void 0))
        {
          return false;
        }
        break;
      
      case 'g': // decimal green component (0-255)
        parsedColor.green = _cfoAccumulateNumber(parseContext, charCount, 3, 10);
        if (parsedColor.green == (void 0))
        {
          return false;
        }
        break;
      
      case 'b': // decimal blue component (0-255)
        parsedColor.blue = _cfoAccumulateNumber(parseContext, charCount, 3, 10);
        if (parsedColor.blue == (void 0))
        {
          return false;
        }
        break;
      
      case 'a': // decimal alpha component (0-255)
        parsedColor.alpha = _cfoAccumulateNumber(parseContext, charCount, 3, 10);
        if (parsedColor.alpha == (void 0))
        {
          return false;
        }
        break;
      
      case 'R': // hex red component (0x00-0xff)
        parsedColor.red = _cfoAccumulateNumber(parseContext, charCount, 2, 16);
        if (parsedColor.red == (void 0))
        {
          return false;
        }
        break;
      
      case 'G': // hex green component (0x00-0xff)
        parsedColor.green = _cfoAccumulateNumber(parseContext, charCount, 2, 16);
        if (parsedColor.green == (void 0))
        {
          return false;
        }
        break;
      
      case 'B': // hex blue component (0x00-0xff)
        parsedColor.blue = _cfoAccumulateNumber(parseContext, charCount, 2, 16);
        if (parsedColor.blue == (void 0))
        {
          return false;
        }
        break;
      
      case 'A': // hex alpha component (0x00-0xff)
        parsedColor.alpha = _cfoAccumulateNumber(parseContext, charCount, 2, 16);
        if (parsedColor.alpha == (void 0))
        {
          return false;
        }
        break;
      
      default:
    }
  }
  else
  {
    // consume constants
    return _cfoMatchText(parseContext,
                      inString.substring(startIndex, startIndex + charCount));
  }
  
  // match succeeded
  return true;
}


/**
 * Match the specified text in a case insensitive manner,
 * returning true and updating the
 * <code>parseContext</code> if the match succeeded.
 */
function _cfoMatchText(
  parseContext,
  text
  )
{
  // if no text to match then match will fail
  if (!text)
    return false;

  // get the length of the text to match
  var textLength  = text.length;

  var currIndex   = parseContext.currIndex;
  var parseString = parseContext.parseString;
  
  // determine whether we have enough of the parseString left to match
  if (textLength > parseString.length - currIndex)
  {
    return false;
  }

  var parseText = parseString.substring(currIndex, currIndex + textLength);
  
  if (parseText != text)
    return false;
    
  // update the current parseContext
  parseContext.currIndex += textLength;
  
  return true;
}
 

/**
 * Accumlates and returns a number at this location or undefined, if
 * there is no number.
 */
function _cfoAccumulateNumber(
  parseContext,
  minDigits,
  maxDigits,
  base)
{
  var startIndex  = parseContext.currIndex;
  var currIndex   = startIndex;
  var parseString = parseContext.parseString;
  var parseLength = parseString.length;
  if (parseLength > currIndex + maxDigits)
    parseLength = currIndex + maxDigits;

  var currValue = 0;

  // gather up all of the digits
  while (currIndex < parseLength)
  {
    var currDigit = parseInt(parseString.charAt(currIndex), base);

    if (!isNaN(currDigit))
    {
      // add on the digit and shift over the results
      currValue *= base;
      currValue += currDigit;

      currIndex++;
    }
    else
    {
      break;
    }
  }

  if (startIndex != currIndex && 
      (currIndex - startIndex) >= minDigits)
  {
    // update the current parseContext
    parseContext.currIndex = currIndex;

    // return the numeric version
    return currValue;
  }
  else
  {
    // no number at this location
    return (void 0);
  }
}


/**
 * Pad out a number with leading 0's to meet the minDigits digits or
 * truncate to meet the maxDigits.
 */
function _cfoGetPaddedNumber(
  number,
  minDigits,
  maxDigits,
  base)
{  
  var stringNumber = number.toString(base);
  
  //
  // pad out any number strings that are too short
  //
  if (minDigits != (void 0))
  {    
    var addedDigits = minDigits - stringNumber.length;
  
    while (addedDigits > 0)
    {
      stringNumber = "0" + stringNumber;
      addedDigits--;
    }
  }
  
  //
  // truncate any number strings that are too long
  //
  if (maxDigits != (void 0))
  {
    var extraDigits = stringNumber.length - maxDigits;
    
    if (extraDigits > 0)
    {
      stringNumber = stringNumber.substring(extraDigits,
                                            extraDigits + maxDigits);
    }
  }
  
  return stringNumber;
}

/**
 * Construct a RGBColorFormat with the specifed color pattern.
 */
function RGBColorFormat(
  pattern,
  allowsTransparent)
{
  // for debugging
  this._class = "RGBColorFormat";
  this._allowsTransparent = allowsTransparent;
  
  if (pattern != (void 0))
  {
    if (typeof(pattern) == "string" ) 
      pattern = [pattern];
  }

  this._pattern = pattern;
}

RGBColorFormat.prototype = new Format();
RGBColorFormat.prototype.format = _rgbColorFormat;
RGBColorFormat.prototype.parse  = _rgbColorParse;

function Color(
  red,
  green,
  blue,
  alpha)
{
  // for debugging
  this._class = "Color";
  
  if (alpha == (void 0))
    alpha = 0xff;

  this.red   = (red & 0xff);
  this.green = (green & 0xff);
  this.blue  = (blue & 0xff);
  this.alpha = (alpha & 0xff);
}

function _Color_toString()
{
  return "rgba(" + this.red + 
         "," + this.green + 
         "," + this.blue + 
         "," + this.alpha + ")";
}

Color.prototype.toString = _Color_toString;


// ColorField
//-----------------------------------------------------------------------------
/*
** Copyright (c) Oracle Corporation 2000. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/

// External variables used:
//  _cfTransIconURL  - the transparent color swatch icon
//  _cfOpaqueIconURL - the opaque color swatch icon
//  _cfBgColor       - the background color of the page

var _cfBus = new Object();
var _cfTransIconURL;
var _cfOpaqueIconURL;
var _cfBgColor;

/*
 * Update the color field swatch.
 */
function _cfsw(
  colorField)
{
  var cff = _getColorFieldFormat(colorField);
  var swatchID = colorField.name + "$sw";
  var value = (void 0);
  var swatch = _getElementById(document, swatchID);

  if (swatch != (void 0))
  {
    if (colorField.value != "")
    {
      value = cff.parse(colorField.value);
    }

    if (value != (void 0))
    {
      if (value.alpha == 0)
      {
        swatch.style.backgroundColor = (void 0);
        swatch.src = _cfTransIconURL;
        swatch.alt = _cfTrans;
      }
      else
      {
        swatch.style.backgroundColor = 
          new RGBColorFormat("#RRGGBB").format(value);
        swatch.src = _cfOpaqueIconURL;
        swatch.alt = cff.format(value);
      }
    }
    else
    {
      swatch.style.backgroundColor = _cfBgColor;
      swatch.src = _cfOpaqueIconURL;
      swatch.alt = (void 0);
    }

    // make sure tooltips are updated correctly
    // on Mozilla
    if (_agent.isGecko)
      swatch.title = swatch.alt;
  }
}

function _returnColorPickerValue(
  closingWindow,
  event
  )
{
  // extract the return value
  var newColor = closingWindow.returnValue;
  
  var colorField = closingWindow._colorField; 

  // See below!
  if (colorField == (void 0))
  {
    colorField = _savedColorField1879034;
  }

  _cfUpdate(colorField, newColor);
}

/*
 * ColorField Bus Select (used by ColorPalette)
 */
function _cfbs(
  event)
{
  _cfUpdate(_cfBus[event.source], event.params.value);
}

function _cfUpdate(
  colorField,
  newColor)
{
  if (colorField != (void 0) && newColor != (void 0))
  {
    //
    // get the format to use to format the result
    //
    var cff = _getColorFieldFormat(colorField);
    
    // update the contents of the text field
    // or update hidden input for compact color field
    var isVisible = (colorField.type != 'hidden');
    var oldValue = colorField.value;
    var newValue = cff.format(newColor);

    if (newValue != (void 0) &&
        newValue != colorField.value)
    {
      // bug 2898744
      // also trigger onchange for compact mode
      // with only color swatch displayed
      // bug 2275982
      // trigger onchange programatically
      if (colorField.onchange != (void 0))
      {
        // The IE event delivery mechanism for built-in events like onchange
        // automatically sets the window.event property for inspection by
        // event handlers.
        // When we synthesize this onchange event here, we must ensure that
        // window.event is as close as possible to the value of window.event
        // that would be visible to the onchange handler if it had been triggered 
        // by user interaction, like moving focus away from the field.
        // Updating window.event directly triggers a JS error in IE, so we must
        // use the built-in event delivery mechanism instead.
        // Here, we attach a handler to the IE-specific propertychange event,
        // which will fire when any property is changed, not just the value.
        // In our handler, we only respond to a change in the value property,
        // by detaching the handler and delivering the onchange event.
        // Now, the value for window.event during the onchange handler execution
        // is the property change event but it has the right values for source,
        // target, etc.
        // In all cases, the ordering for updates to the color field are as follows
        // 1. input value updated
        // 2. color swatch synchronized with new value (if present)
        // 3. onchange handler fires (if present)
        if (_agent.isIE)
        {
          // attach the value change listener
          colorField.onpropertychange = function()
          {
            var event = window.event;
            if (event.propertyName == 'value')
            {
              // detach the value change listener
              colorField.onpropertychange = function(){};
              
              // update swatch first to make sure it has correct
              // value before propertychange event delivery
              _cfsw(colorField);
       
              colorField.onchange(event);
            }
          }

          colorField.value = newValue;
        }
        else
        {
          colorField.value = newValue;

          // swatch not supported on NS47
          if (!_agent.isNav)
            _cfsw(colorField);
        
          var event = new Object();
          event.type = 'change';
          event.target = colorField;
          colorField.onchange(event);
        }
      }
      else // no onchange handler
      {
        colorField.value = newValue;

        // swatch not supported on NS47
        if (!_agent.isNav)
          _cfsw(colorField);
      }
    }
        
    if (isVisible)
    {
      colorField.select();
      colorField.focus();
    }
  }
}

/**
 * Private function for launching the color picker
 */
function _lcp(
  formName,
  nameInForm,
  destination
  )
{
  var colorField = document.forms[formName][nameInForm];

  // default the destination to the color picker dialog destination
  if (!destination)
  {
    destination = _jspDir + _getQuerySeparator(_jspDir) + "_t=fred&_red=cp";
  }
  else
  {
    // since we need to redirect, replace the last portion of the URL with
    // the redirect JSP
    var endOfUrl = destination.lastIndexOf('?');
    var urlArgs  = "";
  
    if (endOfUrl == -1)
    {
      endOfUrl = destination.length;
    }
    else
    {
      urlArgs = destination.substr(endOfUrl + 1);
    }

    var newDest = _jspDir + _getQuerySeparator(_jspDir);
    newDest += urlArgs;

    // add the correct first param separator
    newDest += _getQuerySeparator(newDest);
    newDest += "_t=fred";

    var redirect = destination.substring(0, endOfUrl);

    destination = newDest;

    // add in the redirect
    destination += "&redirect=" + escape(redirect);
  }
  
  // add the current color text
  if (colorField.value != "")
  {
    var format = _getColorFieldFormat(colorField);
    var color = format.parse(colorField.value);

    if (color != (void 0))
    {
      destination += "&value=";

      // escape due to '#' in color code
      if (color.alpha == 0)
        destination += escape("#trans");
      else
        destination += escape(new RGBColorFormat("#RRGGBB").format(color));
    }
  }

  var patterns = _cfs[nameInForm];
  if (patterns != (void 0))
  {
    var pattern = (typeof patterns == "string") ? patterns : patterns[0];

    destination += "&pattern=" + escape(pattern);
  }

  var allowsTransparent = _cfts[nameInForm];
  if (allowsTransparent != (void 0))
  {
    destination += "&allowsTransparent=" + allowsTransparent;
  }

  if (_configName.length > 0)
  {
    destination += "&configName=" + escape(_configName);
  }

  // add the locale
  destination += "&loc=" + _locale;
    
  // and the character set encoding
  if (window["_enc"])
  {
    destination += "&enc=" + _enc;
  }

  // add the context URI
  if (window["_contextURI"])
  {
    destination += "&contextURI=" + escape(_contextURI);
  }

  var colorDialog = openWindow(self,
                             destination,
                             'colorDialog',
                             {width:430, height:230},
                             true,
                             void 0,
                             _returnColorPickerValue);
  
  // save the date field on the calendar window for access
  // from event handler
  colorDialog._colorField = colorField;

  // And, for bug 1879034, stash it on a JS variable.  It
  // seems that IE sometimes has already blown away the values
  // on "colorDialog"!
  _savedColorField1879034 = colorField;  
}

var _savedColorField1879034;
