
// Locale
//-----------------------------------------------------------------------------
/*
** Copyright (c) Oracle Corporation 2000. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/

var _digits;
var _decimalSep;
var _groupingSep;

/**
 * Returns true if the character is a digit.
 */
function isDigit(
  digitChar
  )
{  
  return (_getDigits()[digitChar] != (void 0));
}


/**
 * Returns an Object containing the digit value for all numeric characters
 * and undefined for non-numeric characters
 */
function _getDigits()
{
  if (_digits == (void 0))
  {
    // starts of 10 digit unicode ranges
    var digitStarts = [
                        0x0030, // ISO-LATIN-1 digits ('0' through '9')
                        0x0660, // Arabic-Indic digits
                        0x06F0, // Extended Arabic-Indic digits
                        0x0966, // Devanagari digits
                        0x09E6, // Bengali digits
                        0x0A66, // Gurmukhi digits
                        0x0AE6, // Gujarati digits
                        0x0B66, // Oriya digits
                        0x0BE7, // Tamil digits
                        0x0C66, // Telugu digits
                        0x0CE6, // Kannada digits
                        0x0D66, // Malayalam digits
                        0x0E50, // Thai digits
                        0x0ED0, // Lao digits
                        0x0F20, // Tibetan digits
                        0xFF10  // Fullwidth digits
                      ];
    
    _digits = new Object();
    
    for (var i = 0; i < digitStarts.length; i++)
    {
      for (var offset = 0; offset < 10; offset++)
      {
        // get the string value of the current unicode character
        var currKey = String.fromCharCode(digitStarts[i] + offset);
        
        // store the digit value of this character
        _digits[currKey] = offset;
      }
    }
  }
  
  return _digits;
}


/**
 * Returns thenumeric value of a digit character or Nan if the
 * character isn't a digit.
 */
function parseDigit(
  digitChar
  )
{  
  var value = _getDigits()[digitChar];
  
  if (value == (void 0))
  {
    return NaN;
  }
  else
  {
    return value;
  }
}


/**
 * Returns true if a character isn't a lowercase character or
 * might not be a lowercase character.
 */
function isNotLowerCase()
{
  var charCode = alphaChar.charCodeAt(0);

  if (charCode > 0xFF)
  {
    // be lenient for non-ISO-Latin1
    return true;
  }
  else
  {    
    return !_isLowerCaseStrict(alphaChar);
  }
}


/**
 * Returns true if a character is a lowercase character or
 * might be a lowercase character.
 */
function isLowerCase(
  alphaChar
  )
{
  var charCode = alphaChar.charCodeAt(0);

  if (charCode > 0xFF)
  {
    // be lenient for non-ISO-Latin1
    return !isDigit(alphaChar);
  }
  else
  {    
    return _isLowerCaseStrict(alphaChar);
  }
}


function _isLowerCaseStrict(
  alphaChar
  )
{
  var charCode = alphaChar.charCodeAt(0);

  return (((charCode >= 0x61) && (charCode <= 0x7A)) || // "a-z"
          ((charCode >= 0xDF) && (charCode <= 0xFF)));  // iso-latin1 lowercase
}


/**
 * Returns true if a character is an uppercase character or might be an
 * uppercase character.
 */
function isUpperCase(
  alphaChar
  )
{
  var charCode = alphaChar.charCodeAt(0);
  
  if (charCode > 0xFF)
  {
    // be lenient for non-IS-Latin1
    return !isDigit(alphaChar);
  }
  else
  {
    return _isUpperCaseStrict(alphaChar);
  }
}


/**
 * Returns true if a character isn't an uppercase character or might not be an
 * uppercase character.
 */
function isNotUpperCase(
  alphaChar
  )
{
  var charCode = alphaChar.charCodeAt(0);
  
  if (charCode > 0xFF)
  {
    // be lenient for non-IS-Latin1
    return true;
  }
  else
  {
    return !_isUpperCaseStrict(alphaChar);
  }
}


/**
 * Returns true if a character is an uppercase character.
 */
function _isUpperCaseStrict(
  alphaChar
  )
{
  var charCode = alphaChar.charCodeAt(0);
  
  return (((charCode >= 0x41) && (charCode <= 0x5A)) || // "A-Z"
          ((charCode >= 0xC0) && (charCode <= 0xDe)));  // iso-latin1 lowercase
}


/**
 * Returns true if a character is a latter.
 */
function isLetter(
  alphaChar
  )
{
  // =-= bts not technically correct but hopefully OK for ISO-Latin1
  return isLowerCase(alphaChar) | isUpperCase(alphaChar);
}


function getUserLanguage()
{
  var language = _locale;

  if (language == (void 0))
  {
    // try this the IE way
    language =  window.navigator.userLanguage;
    
    if (language == (void 0))
    {
      // try this the Netscape way
      language = window.navigator.language;
    }
  }
  
  // return language;
  return language;
}


function getJavaLanguage(
  javascriptLang
  )
{
  // default to the user language if no language is passed in
  if (javascriptLang == (void 0))
  {
    javascriptLang = getUserLanguage();
  }
      
  // look for first dash, the territory appears after the dash
  var territoryIndex = javascriptLang.indexOf("-", 0);
  
  // no dash found, so the name is just a language;
  if (territoryIndex == -1)
    return javascriptLang;
  
  var inLength = javascriptLang.length;
  var javaLang = javascriptLang.substring(0, territoryIndex);
  
  javaLang += "_";
  
  territoryIndex++;
  
  var variantIndex = javascriptLang.indexOf("-", territoryIndex);
  
  if (variantIndex == -1)
  {
    // we have no variant
    variantIndex = inLength;
  }
  
  var territoryString = javascriptLang.substring(territoryIndex,
                                                 variantIndex);
                                                 
  javaLang += territoryString.toUpperCase();
  
  // we have a variant, so add it
  if (variantIndex != inLength)
  {
    javaLang += "_";
    javaLang += javascriptLang.substring(variantIndex + 1,
                                         inLength);
  }
    
  return javaLang;
}
 

function getLocaleSymbols(
  jsLocaleString
  )
{  
  var suffix = getJavaLanguage(jsLocaleString);
    
  //
  // look for our localeSymbols, from most specific to least
  // specific.  Unfortunately, this will only work if the
  // less specific library has already been loaded.
  //
  while(true)
  {    
    var localeSymbols = window["LocaleSymbols_" + suffix];
    
    if (localeSymbols != (void 0))
    {
      return localeSymbols;
    }
    else
    {
      var previousIndex = suffix.lastIndexOf("_");
      
      if (previousIndex != -1)
      {
        suffix = suffix.substring(0, previousIndex);
      }
      else
      {
        break;
      }
    }
  }
}


function _getEras()
{
  return this.getLocaleElements()["Eras"];
}

function _getMonths()
{
  return this.getLocaleElements()["MonthNames"];
}

function _getShortMonths()
{
  return this.getLocaleElements()["MonthAbbreviations"];
}

function _getWeekdays()
{
  return this.getLocaleElements()["DayNames"];
}

function _getShortWeekdays()
{
  return this.getLocaleElements()["DayAbbreviations"];
}

function _getAmPmStrings()
{
  return this.getLocaleElements()["AmPmMarkers"];
}

function _getZoneStrings()
{
  return this.getLocaleElements()["zoneStrings"];
}

function _getLocalPatternChars()
{
  return this.getLocaleElements()["localPatternChars"];
}


function _getDecimalSeparator()
{
  if (_decimalSep != (void 0))
    return _decimalSep;

  return this.getLocaleElements()["NumberElements"][0];
}

function _getGroupingSeparator()
{
  if (_groupingSep != (void 0))
    return _groupingSep;

  return this.getLocaleElements()["NumberElements"][1];
}

function _getPatternSeparator()
{
  return this.getLocaleElements()["NumberElements"][2];
}

function _getPercent()
{
  return this.getLocaleElements()["NumberElements"][3];
}

function _getZeroDigit()
{
  return this.getLocaleElements()["NumberElements"][4];
}

function _getDigit()
{
  return this.getLocaleElements()["NumberElements"][5];
}

function _getMinusSign()
{
  return this.getLocaleElements()["NumberElements"][6];
}

function _getExponential()
{
  return this.getLocaleElements()["NumberElements"][7];
}

function _getPerMill()
{
  return this.getLocaleElements()["NumberElements"][8];
}

function _getInfinity()
{
  return this.getLocaleElements()["NumberElements"][9];
}

function _getNaN()
{
  return this.getLocaleElements()["NumberElements"][10];
}

function _getCurrencySymbol()
{
  return this.getLocaleElements()["CurrencyElements"][0];
}

function _getInternationalCurrencySymbol()
{
  return this.getLocaleElements()["CurrencyElements"][1];
}

function _getMonetaryDecimalSeparator()
{
  var separator = this.getLocaleElements()["CurrencyElements"][2];

  // if the resource data specified the empty string as the monetary decimal
  // separator, that means we should just use the regular separator as the
  // monetary separator
  if (separator.length != 0)
  {
    return separator;
  }
  else
  {
    return this.getDecimalSeparator();
  }
}

function _getLocaleElements()
{
  return this["LocaleElements"];
}

function _getFullTimePatternString()
{
  return this.getLocaleElements()["DateTimePatterns"][0];
}

function _getLongTimePatternString()
{
  return this.getLocaleElements()["DateTimePatterns"][1];
}

function _getMediumTimePatternString()
{
  return this.getLocaleElements()["DateTimePatterns"][2];
}

function _getShortTimePatternString()
{
  return this.getLocaleElements()["DateTimePatterns"][3];
}

function _getFullDatePatternString()
{
  return this.getLocaleElements()["DateTimePatterns"][4];
}

function _getLongDatePatternString()
{
  return this.getLocaleElements()["DateTimePatterns"][5];
}

function _getMediumDatePatternString()
{
  return this.getLocaleElements()["DateTimePatterns"][6];
}

function _getShortDatePatternString()
{
  return this.getLocaleElements()["DateTimePatterns"][7];
}

function _getDateTimeFormatString()
{
  return this.getLocaleElements()["DateTimePatterns"][8];
}


function LocaleSymbols(
  localeElements
  )
{
  this["LocaleElements"] = localeElements;
}

LocaleSymbols.prototype.getFullTimePatternString = _getFullTimePatternString;
LocaleSymbols.prototype.getLongTimePatternString = _getLongTimePatternString;
LocaleSymbols.prototype.getMediumTimePatternString = _getMediumTimePatternString;
LocaleSymbols.prototype.getShortTimePatternString = _getShortTimePatternString;
LocaleSymbols.prototype.getFullDatePatternString = _getFullDatePatternString;
LocaleSymbols.prototype.getLongDatePatternString = _getLongDatePatternString;
LocaleSymbols.prototype.getMediumDatePatternString = _getMediumDatePatternString;
LocaleSymbols.prototype.getShortDatePatternString = _getShortDatePatternString;
LocaleSymbols.prototype.getDateTimeFormatString = _getDateTimeFormatString;

LocaleSymbols.prototype.getEras = _getEras;
LocaleSymbols.prototype.getMonths = _getMonths;
LocaleSymbols.prototype.getShortMonths = _getShortMonths;
LocaleSymbols.prototype.getWeekdays = _getWeekdays;
LocaleSymbols.prototype.getShortWeekdays = _getShortWeekdays;
LocaleSymbols.prototype.getAmPmStrings = _getAmPmStrings;
LocaleSymbols.prototype.getZoneStrings = _getZoneStrings;
LocaleSymbols.prototype.getLocalPatternChars = _getLocalPatternChars;

LocaleSymbols.prototype.getDecimalSeparator = _getDecimalSeparator;
LocaleSymbols.prototype.getGroupingSeparator = _getGroupingSeparator;
LocaleSymbols.prototype.getPatternSeparator = _getPatternSeparator;
LocaleSymbols.prototype.getPercent = _getPercent;
LocaleSymbols.prototype.getZeroDigit = _getZeroDigit;
LocaleSymbols.prototype.getDigit = _getDigit;
LocaleSymbols.prototype.getMinusSign = _getMinusSign;
LocaleSymbols.prototype.getExponential = _getExponential;
LocaleSymbols.prototype.getPerMill = _getPerMill;
LocaleSymbols.prototype.getInfinity = _getInfinity;
LocaleSymbols.prototype.getNaN = _getNaN;
LocaleSymbols.prototype.getCurrencySymbol = _getCurrencySymbol;
LocaleSymbols.prototype.getInternationalCurrencySymbol = _getInternationalCurrencySymbol;
LocaleSymbols.prototype.getMonetaryDecimalSeparator = _getMonetaryDecimalSeparator;
LocaleSymbols.prototype.getLocaleElements = _getLocaleElements;

function _formatValidate(
  value
  )
{  
  var result = this.parse(value);
    
  if (result == (void 0))
  {
    return "Failed:" + this._class + " for value:" + value;
  }
  else if (_instanceof(result, ParseException))
  {
    return result;
  }
  else
  {
    return (void 0);
  }
}


function Format()
{
  // for debugging
  this._class = "Format";
}

Format.prototype.format = (void 0);
Format.prototype.parse  = (void 0);
Format.prototype.validate = _formatValidate;

/**
 * Conntruct a ParseException
 */
function ParseException(
  parseString
  )
{
  this.parseString = parseString;
}

ParseException.prototype.errorOffset = (void 0);
ParseException.prototype.parseString = (void 0);

// CharSets
//-----------------------------------------------------------------------------
/*
** Copyright (c) Oracle Corporation 2001-2002. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/
function _noopFormat(
  value
  )
{
  return value;
}


function _cjkParse(
  parseString
  )
{
  var i = 0;
  var length = this._length;

  while (i < parseString.length)
  { 
    var ch = parseString.charCodeAt(i);
    if (ch < 0x80) length--;
    else length -= 2;
   
    if (length < 0)
    {
      _setFailedPos(i);
      return (void 0);
    }

    i++;
  }

  return parseString;
}


function CjkFormat(
  length
  )
{
  this._class = "CjkFormat";
  this._length = length;
}


CjkFormat.prototype = new Format();
CjkFormat.prototype.format = _noopFormat;
CjkFormat.prototype.parse  = _cjkParse;


function _utf8Format(
  value
  )
{
  return value;
}


function _utf8Parse(
  parseString
  )
{
  var i = 0;
  var length = this._length;

  while (i < parseString.length)
  { 
    var ch = parseString.charCodeAt(i);
    if (ch < 0x80) length--;
    else if (ch < 0x800) length -= 2;
    else
    {
      // Surrogates...
      if ((ch & 0xFC00) == 0xD800)
        length -= 4;
      else
        length -= 3;
    }

    if (length < 0)
    {
      _setFailedPos(i);
      return (void 0);
    }

    i++;
  }

  return parseString;
}


function Utf8Format(
  length
  )
{
  this._class = "Utf8Format";
  this._length = length;
}


Utf8Format.prototype = new Format();
Utf8Format.prototype.format = _noopFormat;
Utf8Format.prototype.parse  = _utf8Parse;



function _sbParse(
  parseString
  )
{
  if (this._length < parseString.length)
  {
    _setFailedPos(this._length);
    return (void 0);
  }

  return parseString;
}


function SBFormat(
  length
  )
{
  this._class = "SBFormat";
  this._length = length;
}


SBFormat.prototype = new Format();
SBFormat.prototype.format = _noopFormat;
SBFormat.prototype.parse  = _sbParse;

function _setFailedPos(i)
{
  window["_failedPos"] = i;
}
// DateFieldFormat
//-----------------------------------------------------------------------------
/*
** Copyright (c) Oracle Corporation 2000. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/
function _getDateFieldFormat(dateField)
{
  var name = dateField.name;
  if (name && _dfs)
  {
    var format = _dfs[name];
    if (format)
      return new SimpleDateFormat(format);
  }

  return new SimpleDateFormat();
}

function _fixDFF(dateField)
{
  var format = _getDateFieldFormat(dateField);

  if (dateField.value != "")
  {
    var value = format.parse(dateField.value);

    if (value != (void 0))
      dateField.value = format.format(value);
  }
}


// DateField
//-----------------------------------------------------------------------------
/*
** Copyright (c) Oracle Corporation 2000. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/

// _dfsv(): Date Field Set Value function.  
function _dfsv(
  dateField,
  newValue
  )
{
  // Make sure we have valid values
  if ((dateField == (void 0)) || (newValue == (void 0)))
    return;

  // Add back in the time
  // offset,since we don't want to overwrite the user's time when 
  // they pick a new date from the calendar.
  newValue += _dfgt(dateField);
  var newDate = new Date(newValue);

  //
  // get the format to use to format the result
  //
  var format = _getDateFieldFormat(dateField);
    
  // update the contents of the text field
  var oldValue = dateField.value;
  var newValue = format.format(newDate);

  if (dateField.value != newValue)
  {
    // bug 2275982
    // trigger onchange programatically
    if (dateField.onchange != (void 0))
    {
      // The IE event delivery mechanism for built-in events like onchange
      // automatically sets the window.event property for inspection by
      // event handlers.
      // When we synthesize this onchange event here, we must ensure that
      // window.event is as close as possible to the value of window.event
      // that would be visible to the onchange handler if it had been triggered 
      // by user interaction, like moving focus away from the field.
      // Updating window.event directly triggers a JS error in IE, so we must
      // use the built-in event delivery mechanism instead.
      // Here, we attach a handler to the IE-specific propertychange event,
      // which will fire when any property is changed, not just the value.
      // In our handler, we only respond to a change in the value property,
      // by detaching the handler and delivering the onchange event.
      // Now, the value for window.event during the onchange handler execution
      // is the property change event but it has the right values for source,
      // target, etc.
      if (_agent.isIE)
      {
        // attach the value change listener
        dateField.onpropertychange = function()
        {
          var event = window.event;
          if (event.propertyName == 'value')
          {
            // detach the value change listener
            dateField.onpropertychange = function(){};
              
            dateField.onchange(event);
          }
        }

        dateField.value = newValue;
      }
      else
      {
        dateField.value = newValue;

        var event = new Object();
        event.type = 'change';
        event.target = dateField;
        dateField.onchange(event);
      }
    }
    else // no onchange handler
    {
      dateField.value = newValue;
    }
  }

  dateField.select();
  dateField.focus();
}

function _returnCalendarValue(
  closingWindow,
  event
  )
{
  // extract the return value
  var newValue = closingWindow.returnValue;
  
  if (newValue != (void 0))
  {
    var dateField = closingWindow._dateField; 
    // See below!
    if (dateField == (void 0))
    {
      dateField = _savedField1879034;
    }

    _dfsv(dateField, newValue);
  }
}

/**
 * Provate function for launching the date picker
 */
function _ldp(
  formName,
  nameInForm,
  minValue,
  maxValue,
  destination
  )
{
  var dateField = document.forms[formName][nameInForm];
  var oldValue = _dfgv(dateField);

  if (!oldValue)
  {
    // if the parse failed, then create a new Date object.
    oldValue = new Date();
  }


  // default the destination to the calendar dialog destination
  if (!destination)
  {
    destination = _jspDir + _getQuerySeparator(_jspDir);
    destination += "_t=fred&_red=cd";
  }
  else
  {
    // since we need to redirect, replace the last portion of the URL with
    // the redirect JSP
    var endOfUrl = destination.lastIndexOf('?');
    var urlArgs  = "";
  
    if (endOfUrl == -1)
    {
      endOfUrl = destination.length;
    }
    else
    {
      urlArgs = destination.substr(endOfUrl + 1);
    }

    var startOfLastPart = destination.lastIndexOf('/', endOfUrl);
  
    var newDest = destination.substring(0, startOfLastPart + 1);
    newDest += _jspDir + _getQuerySeparator(_jspDir);
    newDest += urlArgs;

    // add the correct first param separator
    newDest += _getQuerySeparator(newDest);
    newDest += "_t=fred";

    var redirect = destination.substring(startOfLastPart + 1, endOfUrl);

    destination = newDest;

    // add in the redirect
    destination += "&redirect=" + escape(redirect);
  }
  
  // add the current time in Millis since 1970
  destination += "&value=" + oldValue.getTime();
  
  if (_configName.length > 0)
  {
    destination += "&configName=" + escape(_configName);
  }

  // add the locale
  destination += "&loc=" + _locale;
    
  // and the character set encoding
  if (window["_enc"])
  {
    destination += "&enc=" + _enc;
  }

  // add the context URI
  if (window["_contextURI"])
  {
    destination += "&contextURI=" + escape(_contextURI);
  }

  // supply the client's timezone
  destination += "&tzOffset=" + oldValue.getTimezoneOffset();
 
  //
  // add on min and max value attributes
  //
  if (minValue != (void 0))
  {
    destination += "&minValue=" + minValue; 
  }
  
  if (maxValue != (void 0))
  {
    destination += "&maxValue=" + maxValue; 
  }

  // Open the window;  we used to name it "calendar", but
  // that's a common enough name that we hit bug 2807778
  var calWindow = openWindow(self,
                             destination,
                             'uix_2807778',
                             {width:350, height:370},
                             true,
                             void 0,
                             _returnCalendarValue);
  
  // save the date field on the calendar window for access
  // from event handler
  calWindow._dateField = dateField;

  // And, for bug 1879034, stash it on a JS variable.  It
  // seems that IE sometimes has already blown away the values
  // on "calWindow"!
  _savedField1879034 = dateField;  
}

// _dfgv(): Date Field Get Value function
// Returns the value of the dateField as a Date object
function _dfgv(dateField)
{
  if (dateField.value != "")
    return _getDateFieldFormat(dateField).parse(dateField.value);

  return null;
}

// _dfgt(): Date Field Get Time function
// Returns the time-only portion of the date field in ms.
function _dfgt(dateField)
{
  var oldValue = _dfgv(dateField);

  // If the date field doesn't have a value, use the
  // current time.
  if (!oldValue)
    oldValue = new Date();

  // get just the time portion in milliseconds from the date field.
  // First, get just the date portion of the oldValue 
  // (the value in the date field). 
  // Then subtract the date-only date from the date-time date to get
  // the time only portion. We'll add this time back in after the
  // user picks a date from the calendar. This way the time 
  // in the field, if any, will be preserved, and empty fields
  // will default to the current time.
  var oldValueDateOnly = new Date ( oldValue.getFullYear(), 
                                    oldValue.getMonth(), 
                                    oldValue.getDate());
      
  // get only the time portion of the date in the field.
  return oldValue-oldValueDateOnly;
}

/**
 * _dbb(): Date Field Blur handler
 *
 * Parameters:
 * - dateField is the date field object
 * - lenient indicates whether to fix formatting
 * - calendarID is the ID of the inline calendar 
 *     associated with dateField
 */
function _dfb(dateField, lenient, calendarID)
{
  if (lenient)
    _fixDFF(dateField);

//  if (calendarID != (void 0))
//    _calActiveDateFields[calendarID] = null;

}

/**
 * _dbf(): Date Field Focus handler
 *
 * Parameters:
 * - dateField is the date field object
 * - calendarID is the ID of the inline calendar 
 *     associated with dateField
 */
function _dff(dateField, calendarID)
{
  _dfa(dateField, calendarID);
}

/**
 * _dba(): Date Field Activate
 * 
 * Makes the specified dateField the "active" dateField
 * for the calendar
 *
 * Parameters:
 * - dateField is the date field object or id
 * - calendarID is the ID of the inline calendar 
 *     associated with dateField
 */
function _dfa(dateField, calendarID)
{
  if (calendarID != (void 0))
  {
    if (window._calActiveDateFields === (void 0))
      window._calActiveDateFields = new Object();

    if (typeof(dateField) == "string")
    {
      dateField = _getElementById(document, dateField);
    }

    window._calActiveDateFields[calendarID] = dateField;
  }
}

/**
 * _calsd(): Calendar Select Date function.
 *
 * Parameters:
 * - source is the id of the calendar
 * - value is the date value to select
 */
function _calsd(source, value)
{

  if (window._calActiveDateFields != (void 0))
  {
    var dateField = window._calActiveDateFields[source];

    if (dateField)
      _dfsv(dateField, value);
  }

  return false;
}

function _updateCal(choice,url,p)
{

  url += ('&scrolledValue='+choice.options[choice.selectedIndex].value);
  if (p) 
    _firePartialChange(url);
  else 
    document.location.href=url;
}


function _doCancel()
{
  top.returnValue = (void 0);
  top.close();
  return false;
}

function _selectDate(dateTime)
{
  var dstDate = new Date(dateTime);
  var hours = dstDate.getHours();
  
  if (hours >= 12)
  {
    hours = 24;
  }
  
  dstDate.setHours(hours);
  top.returnValue = dstDate.getTime();
  top._unloadUIXDialog(window.event);
  top.close();return false;
}


var _savedField1879034;

// LovInput
//-----------------------------------------------------------------------------
/*
** Copyright (c) Oracle Corporation 2001-2003. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/

/**
 * LovInput.js: Javascript support library for the ListOfValues template
 *
 * There are no public functions in this library. Any function may be
 * altered or deleted by the Cabo group at any release. Developers should
 * not directly call any of these functions, or use any variable defined here.
 *
 * There are three interfaces that will be supported: the callback functions
 * onLovValidate, onLovInit and onLovSelect.
 *
 * There is, also, an LovOpenWindowAction clientAction that will generate a
 * call into the _LovInputOLW function.
 *
 * @version $Name:  $ ($Revision: 1.66 $) $Date: 2004/02/19 01:27:56 $
 * @author Bud Osterberg (Bud.Osterberg@Oracle.com)
 */

////
////  Reserved Names
////
//  The following names are used by this library, and the lovTemplate, and
// should not be used by the developer on any pages using this library.
//
// _LovEV, _LovFL, _LovFR, _LovHR, _LovIC, _LovLD, _LovLI,
// _LovNM, _LovPT, _LovSC, _LovSF, _LovSL, _LovSR, _LovST, _LovWN
//

////
////  Globals
////
// EV: EVent     - Key string used to save/fetch the event parameter
var _LovEV = "event";
// FL: FieLd     - Key string used to save/fetch the transformed field name
var _LovFL = "field";
// FR: FoRm      - Key string used to save/fetch the form name
var _LovFR = "form";
// HR: HRef      - The URL of the main calling page (where the lovInupt is).
var _LovHR;
// IC: lastIeClick - used to keep the date of the last onClick event
//                   to determine if a second should be processed.
var _LovIC = 0;
// LD: LastDate - used to keep the date of the last onChange event to determine
// if the onClick event should be processed.
var _LovLD = 0;
// LI: LovId - The ID attribute passed in from the listOfValues element.
// var _LovLI;
// NM: NaMe - An object containing the name of the form, name of the
//            field and the transformed name of the field from/to which
//            the data goes. It also contains the partialTargets.
var _LovNM = null;
// PR: PartialRefresh - non-zero if we should use partial updates.
// PT: PartialTarget- The string used to send the partialTargets
//                    parameter on events
var _LovPT = "partialTargets";
// SC: stateCheck - The stateCheck callback passed in from the listOfValues
//                  element.
// var _LovSC;
// SF: Select function - This variable holds the user specified post select
//                       callback. If other than null, it will be called to
//                       transfer data back to the main page, otherwise, the
//                       default callback below will do what it can.
// The select function looks like:
//     function MyCallback(lovwin, formNm, event)
// where lovwin is the LOV window that is closing down
//
//       formNm is an object with the form name, field name, and
//              transformed field name.
// and
//       event  is the window closing event
//
var _LovSF = null;
// SL: SeLected - flag that tells whether a select has occurred.
// var _LovSL = false;
// SR: SouRce    - Key string used to save/fetch the source parameter
var _LovSR = "source";
// ST: SearchText- The string used to send the searchText parameter on events
var _LovST = "searchText";
// WN: WindowName- The name of the LOV modal window
var _LovWN = 'lovWindow';

////
//// Public Functions
////

/**
 * There are no public functions in this library
 */

////
//// Private Functions
////

///
/// Event and Window related functions
///

// VTF: Validate Text Field
//
// This function handles the onChange and onClick events for the lovInput field.
//
// When this is an onChange handler, we know that the user has made some sort
// of update. However, if the user has just cleared out the field, we don't
// want to fire validation as this may lead to a very expensive blind query
// So, we don't initiate validation on a blank field unless explicitly
// requested to do so.
//
// When this is an onClick handler, the user has explicitly requested a lookup.
// We can't just ignore that request, so if the field is blank, we'll validate
// it if the user requests, otherwise we'll directly open the LOV Window.
//
// Once we know we're going to validate, we just build up an URL pointing back
// to the current page, add a partialTarget id pointing to the lovInput text
// field, and fire an lovValidate event using firePartialChange. The event has
// a source parameter (the name of the lovInput element), and a searchText
// parameter (the contents of the field).
//
// In the handler for the lovValidate event, the client should try to find an
// exact match. If one is found, they will update the provider for the
// lovInput's text element and leave the lovInput's isValid attribute set to
// true. Otherwise, they should set up the data providers for the LOV table,
// and set the lovInput's isValid attribute to false.
//
// When the lovInput starts the partial refresh, it will check isValid. When
// everything is valid, it will just update the field with the new valid data.
// When invalid, a piece of javascript will be generated which makes a call to
// the _LovInputOLW function to open the LOV window.
//
//
// Inputs:
//    partialTargets: The set of partial targets to update.
//    validate: A callback in which the client can add/delete/update parameters
//              which will end up on the URL used to fire the partialChange.
//    input:    The lovInput text field DOM object.
//    validateBlanks: By default, we do not send a validation event if the text
//                    field is null or all whitespace. If the validateBlanks
//                    argument is true, we will validate anything.
//    paramObj: The object containing all the parameters needed to open the LOV
//              window. Since this parameter object will ONLY be sent down if
//              the window may be opened, and since the window can only be
//              directly opened in response to a click of the search icon, we
//              know that a valid parameter object means this is an onClick
//              handler.
//
// Returns: Almost always false (to keep from navigating to the top of the
//          page in response to the href="#"). We only return true in the case
//          of an onChange handler and an null field - otherwise we'd get stuck
//          in the field when running IE.
//
function _LovInputVTF(partialTargets,
                      validate,
                      input,
                      validateBlanks,
                      formSubmitted,
                      paramObj)
{
  var _click = (paramObj != (void 0));

  // OK, we've gotten here because one of the following things has happened:
  //   1.) user has changed the data and tabbed out
  //   2.) user has changed the data and clicked search
  //   3.) user has not changed the data but tabbed out
  //   4.) user has not changed the data but clicked search
  //
  // In all four of these cases, the data could be either:
  //   A.) null or all whitespace
  //   B.) some text
  //
  // Furthermore, this function is only called in the PPR environment (non-PPR
  // ignores the onChange and hard-wires onClick to OLW).
  //
  // In the above cases we get the following events:
  //    I.E.:  cases 1 & 2: onChange only; case 3: nothing;
  //           case 4: onClick only
  //    Mozilla:  case 1: onChange only; case 2: onChange followed by onClick;
  //              case 3: nothing; case 4: onClick only
  //
  // We want to validate EVERY onChange of type B, and onChange of typeA
  // only if validateBlanks is true. If we get an onClick right after an
  // onChange, then we're dealing with Mozilla case #2, and we can safely
  // ignore it (the onChange already fired the validation). Otherwise, we
  // have case #4. Here the user has clicked the icon even though there is an
  // existing value in the text field. They must REALLY want the LOV window.
  // In this case we bring up the LOV window without doing any validation.
  //
  // There is one known bug lurking in here. If the user changes the text to an
  // exact match and clicks the icon, we have case #2. We send the validate
  // event, the server recognizes that it's a match, sends back the same text,
  // and we redraw... it appears that *NOTHING* has happened in response to the
  // click. Sure, it's easy to say "just bring up the window in response to an
  // exact match," but we don't want the user to type an exact match, tab out
  // and have a window come up. We can't wait around for the click to come in
  // and process there, because in the case of a tab out, there won't be an
  // onClick coming. Hmmmph! We've addressed this by ALWAYS bringing up the
  // window. _LovInputOLW will detect an existing window, and ignore a second
  // request (the PPR request from the validation) to bring it up.

  // If this is onClick, check for a processed onClick or onChange a short
  // while ago. This filters out Mozilla case #2 and doubleClicks.
  var newDate = new Date();
  var form = null;

  if (_click)
  {
    if (_LovLD)
    {
      var diff = newDate - _LovLD;
      if ((diff >= 0) && (diff < 2000))
      {
        // This event is the second click of a double click which was processed
        // a short while ago. Everything is taken care of, we're done
        return false;
      }
    }
    // We're going to process this event, remember when it happened.
    _LovLD = newDate;
  }
  else if (_agent.isIE)
  {
    // I hate IE! When we set the timeout below in IE, this event gets
    // delivered twice. Skip the second one.
    if (_LovIC)
    {
      var diff = newDate - _LovIC;
      if ((diff >= 0) && (diff < 500))
      {
        return false;
      }
    }
    _LovIC = newDate;
  }

  // The default is to NOT send the validate event
  var _sendValEvent = false;

  // Assume the string is blank
  var strBlank = true;
  if (input)
  {
    // Check if the input string is all whitespace
    if (input.value)
    {
      // If there is any non-whitespace char, this search will
      // return a positive index.
      strBlank = (input.value.search(/\S/) < 0);
    }

    if (strBlank)
    {
      // OK, we did get a blank string. If the client wants to validate it,
      // go ahead.
      if ((validateBlanks != (void 0)) && validateBlanks)
      {
        // The only time we'll ever validate a blank string is if it's
        // changing to a blank string. If the string was blank before, and is
        // still blank, then we just bring up the LOV window.
        _sendValEvent = !_click;
      }
    }
    else
    {
      // The string is not blank. Here if the user has clicked the icon, we
      // always bring up the LOV, we don't do any validation. However, if the
      // user has changed the text, we DO want to validate it.
      _sendValEvent = !_click;
    }

    if (_sendValEvent)
    {
      // Only send the validate event if we've got some partial targets
      if (partialTargets)
      {
        var params = new Object();
        var URL = 0;
        var origParams = new Object();
        // If possible, get the destination from the form
        if (input.form != (void 0))
        {
          form = input.form;
          if (form.action != (void 0))
            URL = form.action;
        }
        URL = _LovInputDDP(URL, params);

        // clone the form action parameters
        for (var paramName in params)
          origParams[paramName] = params[paramName];

        if (input.id)
          params[_LovSR] = input.id;
        else
          params[_LovSR] = input.name;

        params[_LovST] = input.value;
        params[_LovPT] = partialTargets;

        // check for a validate callback, and let it update the params
        if (_LovInputMPC(validate, params, 'lovValidate', false))
        {
          var useSubmit = _LovInputUSF();

          for (paramName in params)
          {
            // Make sure we don't double encode any parameters that may have
            // already been encoded on the form action.
            if ((origParams[paramName] != null)
                && (origParams[paramName] == params[paramName]))
            {
              // This parameter is already on the action with it's current value
              delete params[paramName];
            }
            else
            {
              if (origParams[paramName] != null)
              {
                // This parameter has been updated by the callback
                delete origParams[paramName];
              }

              // The input field and/or any added parameter could have any
              // random character in it and does need to be encoded
              // (Bug #2720832).
              //
              // However, if we're going to submit the form, or if we'll end up
              // building a second form and submitting that, we should not
              // encode the input.
              if (!(useSubmit || formSubmitted))
                params[paramName] = _LovInputENC(params[paramName]);
            }
          }

          // To avoid Bug 2666324, we will not encode any of the original
          // parameters when adding the args to the URL.
          var encodeParams = false;

          if (useSubmit && !formSubmitted)
          {
            // Send down the original action with duplicates/updates removed
            var action = _LovInputUAA(URL, origParams, encodeParams);

            // we will be doing a partial update
            params[_getPartialParameter()] = true;

            // now build a form out of these parameters, and submit it.
            _LovInputSFS(params, form, action);
          }
          else
          {
            // The parameter objects do not overlap. Build one big
            // parameter list, and then build the URL from that.
            for (paramName in origParams)
              params[paramName] = origParams[paramName];

            // finally, fire the event
            var callStr = 'if (!_lovInputREI(new Date()))';
            if (formSubmitted)
            {
              var pStr = '{ ';
              var notFirst = false;
              for (paramName in params)
              {
                if (notFirst)
                  pStr += ", ";
                pStr += "'" + paramName + "' : '" + params[paramName] + "'";
                notFirst = true;
              }
              pStr += ' }';

              callStr += "_submitPartialChange('"
                         + form.name
                         + "', 0, "
                         + pStr
                         + ")";
              // _submitPartialChange(form, 0, params);
            }
            else
            {
              URL = _LovInputUAA(URL, params, encodeParams);
              callStr += "_firePartialChange('" + URL + "')";
              // _firePartialChange(URL);
            }
            // Bug 2807785
            // OK, this validate has happened because the user has entered
            // something into the textInput field, and then left the field. How
            // they left the field is very important. If they tabbed out, or
            // clicked on some whitespace, then we could just go ahead and send
            // the event. However if they clicked on a submit button, then
            // firing this event would disrupt that submit, so we wait a little
            // bit check to see if a submit has started, and if it hasn't,
            // go ahead and do the validate.
            var delay = 200;
            if (_agent.isIE)
              delay = 250;

            _setRequestedFocusNode(document, params[_LovSR], true);
            window.setTimeout(callStr, delay);
          }
        }
      }
    }
    // EVERY click opens the LOV window.
    if (_click)
      _LovInputOLW(partialTargets, paramObj, 1);
    else
      // every change has dirtied the form
      _navDirty = true;
  }
  return (strBlank && !_click);
}
// REI: Recent Event of Interest
//
// returns true if a reset or submit occurred recently
//
function _lovInputREI(nowDate)
{
  return (_recentReset(nowDate) || _recentSubmit(nowDate));
}
// OLW: Open LOV Window
//
// This function is called from the LovInputRenderer to
// open up the new LOV window. This is the main entry point,
// but it still isn't a public function. The interface is
// subject to change at any time.
//
// Inputs:
//   partialTargets: The partial targets to update
//   params: The parameter object containing all the useful fields.
//   direct: True if we got here by way of a direct click, false if through
//           a PPR script.
//
// Returns: nothing useful
//
function _LovInputOLW(partialTargets, params, direct)
{
  var dep = _getDependent(window, _LovWN);
  var paramObj = params;
  var isDirect = (direct == 1);

  if (paramObj == (void 0))
  {
    paramObj = new Object();
  }

  var isAction = _LovInputGPF(paramObj, 'A', false);
  var formName = _LovInputGPF(paramObj, 'F', 0);
  var fieldName = _LovInputGPF(paramObj, 'N', 0);

  // Field name is required when not an action, formName is always required
  if (!((isAction || fieldName) && formName))
    return;

  var fieldID = _LovInputGPF(paramObj, 'L', 0);
  var transFieldName = _LovInputGPF(paramObj, 'T', fieldName);

  var onInit = _LovInputGPF(paramObj, 'I', 0);
  var onSelect = _LovInputGPF(paramObj, 'S', 0);
  var evt = _LovEV;

  // The URL to open in the launched modal LOV window
  // If no destination specified, loop back to calling page.
  var URL = _LovInputGPF(paramObj, 'D', location.href);

  var sTxt;
  if (!isAction)
  {
    var fld = document[formName][transFieldName];
    if (!fld)
      return;

    // Get the search text out of the text input field
    sTxt = fld.value;
  }
  else
  {
    sTxt = _LovInputGPF(paramObj, 'R', '');
  }

  // Callback may or may not be null
  _LovSF = onSelect;
  _LovNM = new Object();
  // The main page has a function called _LovClose on it. This function
  // will end up submitting the lovClose event. Grab it now and store it away.
  _LovNM[_LovFR] = formName;
  _LovNM[_LovFL] = transFieldName;
  if (fieldID)
    _LovNM[_LovSR] = fieldID;
  else
    _LovNM[_LovSR] = fieldName;
  _LovNM[_LovPT] = partialTargets;

  // Check for an existing LOV window. If it already exists, then the user has
  // entered a value in the text field and then clicked the search icon. We
  // already opened a window in response to the click, this OMW call is a PPR
  // call in response to the validation, it can be ignored, the window is
  // already up.
  // We HAVE to make sure that the _LovNM object has been re-populated
  // BEFORE we exit or else no values will be returned.
  if (dep != (void 0))
  {
    return false;
  }

  var submitPrepare = _LovInputGPF(paramObj, 'P', false);

  if (submitPrepare && isDirect)
  {
    _submitPartialChange(formName, 0, {'event':'lovPrepare',
                                       'source':_LovNM[_LovSR],
                                       'searchText':sTxt,
                                       'partialTargets':partialTargets});
  }
  else
  {
    // build up an object which has all the parameters we'll eventually pass
    // we then pass this object off to the developer's init function for
    // any additions/changes.
    var prmObj = new Object();
    var filter = "lovFilter";
    URL = _LovInputDDP(URL, prmObj);

    prmObj[_LovSR] = _LovNM[_LovSR];
    prmObj[_LovST] = sTxt;

    // do the onInit callback
    if (!submitPrepare)
    {
      if (! _LovInputMPC(onInit, prmObj, filter, true))
        return false;
    }
    else
    {
      prmObj[_LovEV] = filter;
    }

    // build up the final URL
    URL = _LovInputUAA(URL, prmObj, true);

    // and save off the calling page URL
    _LovHR = location.href;

    // Finally, open the new window
    _LovInputOMW(URL, _LovInputWCB);
  }

  return false;
}
// GPF : Get Parameter Field
//
// Gets argument parameters from the parameter object given a particular key.
// The field key should be one of the single character argument abbreviations:
//
// A: isAction:       This openWindow call is from an LovOpenWindowAction,
//                    not an lovInput element.
//
// D: Dest:           The destination URL string.
//
// F: Formname:       The name of the form  which contains the initial
//                    searchText value (this is assumed to be at document level
//                    e.g. if the field is document.form1.txtval1.value,
//                    formName would be form1).
//
// I: onInit:         The user defined onLovInit function.
//
// L: fieLdid:        The ID of the field which contains the initial
//                    searchText value. This ID is used to set this field as
//                    a partial refresh target.
//
// N: fieldName:      The name of the field which contains the initial
//                    searchText value (this is assumed to be a child of the
//                    form given in formName. So, if the field is
//                    document.form1.txtval1.value, fieldName would be txtval1,
//                    and formName better be form1).
//
// R: seaRchtext:     The seeded searchText. Only used if isAction is true.
//
// S: onSelect:       The user defined onLovSelect function. See _LovSF above
//
// T: Transfieldname: The transformed version of fieldName. If this field is
//                    in a table, the name will have the table name, and row
//                    number added on. Accessing the field through the DOM
//                    will require this full transformed name. If the field
//                    name is not transformed (ie. transFieldName ==
//                    fieldName) this value will be null.
//
// Inputs:
// paramObj: The object in which we'll search for parameters.
//    field: A one-character field identifier (see above).
//     dflt: Value to return if no parameter is set.
//
// Returns:
//   The value from the lookup, or the given default if the parameter was not
//   set in the parameter object.
function _LovInputGPF(paramObj, field, dflt)
{
  var retVal;
  if (paramObj != (void 0))
  {
    retVal = paramObj[field];
  }

  if (retVal == (void 0))
    retVal = dflt;

  return retVal;
}
// MPC : Manage Parameter Callback
//
// This function executes one of the callback functions that allow the client
// to change the parameters being sent to an event. It checks for any changes
// to the event parameter, and if requested saves the original event name.
//
//
// Inputs:
//   callback:  The onLov{Validate,Init} callback. The signature is
//              callback(object params) where the params is just a
//              bunch of key/value pairs listing the parameters.
//   params:    The parameter object
//   eventName: The name of the event that will be raiesd with these params
//   keepEvent: If true, the client cannot override the event name.
//
// Returns: Whatever the client callback returned. If the client callback
//          returns TRUE, we'll post-process the parameter object. If it
//          returns false, we return immediately.
//
function _LovInputMPC(callback, params, eventName, keepEvent)
{
  var evt = _LovEV;

  // we don't want to send the event over to the callback, so save it locally
  var saveEvent = null;

  // only save an existing event name if requested
  if (keepEvent)
    saveEvent = params[evt];
  delete params[evt];

  if (callback)
  {
    // If the client callback returns false, no more processing, just return
    var rv = callback(params);
    if (!rv)
      return false;
  }

  // Now put the event parameter in. Precedence is:
  //    saved value (if keepEvent is true)
  //    default value
  if (!params[evt])
  {
    // If the callback didn't set any event name
    if (saveEvent)
    {
      //... and there was one on the URL, use it
      params[evt] = saveEvent;
    }
    else
    {
      // otherwise use the default
      params[evt] = eventName; // The default event name
    }
  }
  return true;
}

// OMW: Open Modal lov Window
//
// This function opens up a modal window displaying the destination page
// with the requested callback function attached.
// Because we want things to stay modal, we need to call through the
// FrameRedirect jsp.
//
// Inputs:
//   destination: URL of the page to display
//   callback: A standard window close callback (no third argument!)
//
// Returns: none you should depend on
//
function _LovInputOMW(destination, callback)
{
  // Only allow redirect to another page on this same machine
  var newURL = location.protocol + '//' + location.host;

  // add in the frame redirect JSP
  newURL += _jspDir + _getQuerySeparator(_jspDir) + '_t=';

  if (_LovInputUSF())
  {
    // If we've got a non-UTF encoding on the page,
    // call the recoding version of fred.
    newURL += 'fredRC&';

    if (_enc)
      newURL += "enc=" + _enc + "&";
  }
  else
    newURL += 'fred&';

  // Bug #2464627: Force the size to at least 750 x 550
  newURL +=  '_minWidth=750&_minHeight=550&';

  if (_configName)
    newURL += "configName=" + _configName + "&";

  if (_contextURI)
    newURL += "contextURI=" + _contextURI + "&";

  newURL += 'redirect=';

  if (destination.charAt(0) != '/')
  {
    // The destination is relative, we have to prepend the current directory
    // because we're about to move our base over to the JSP directory.
    var basePath=location.pathname;
    destination = (basePath.substr(0, basePath.lastIndexOf('/') + 1)
                   + destination);
  }

  // This is no longer the first argument, so gotta replace the '?'
  // FredJSP is pedantic enough to replace the delimiter after the
  // 'redirect=' with a '?' for us. If we didn't replace this ?, we'd end
  // up with two on the final URL which would cause much pain.
  newURL += destination.replace('?', '&');

  // OK, open the LOV window as a modal window
  lovw = openWindow(window, newURL, _LovWN, {width:750, height:550},
                    true, 'dialog', callback);

  // initialize these here because it has to be on the LOV window. At this
  // point, nothing has been selected, and the lovSelect event has not been
  // generated.
  lovw._LovSL = false;
  lovw._LovSF = _LovSF;
  lovw._LovNM = _LovNM;
}

//
// PWP: Pass Window Parameters
//
// This grabs parameters from the most likely places
// Inputs:
//   paramName: name of the parameter to get
//   winObj: The window from which to get the parameter
// Returns:
//   The value of the parameter gotten from the
//   windowObject, top, or windowObject's opener in that order.
//
function _LovInputPWP(paramName, winObj)
{
  var rv = window[paramName];

  if (rv  == (void 0))
  {
    if ((winObj != (void 0)) && winObj[paramName])
    {
      rv = winObj[paramName];
    }
    else if (top[paramName])
    {
      rv = top[paramName];
    }
    else if ((winObj.opener != (void 0)) && (winObj.opener[paramName]))
    {
      rv = winObj.opener[paramName];
    }
  }
  return rv;
}

//
// WCB: Windowclose CallBack
//
// This is the default callback for the closing of the LOV window
// Inputs:
//   lovwin: The window that closed calling this function
//   event: The close event (not used)
// Returns:
//   False in PPR to inhibit a redraw, true otherwise.
//
function _LovInputWCB(lovwin, event)
{
  var rv = true;
  // If there has been no selection, we don't send or submit anything.
  if (!lovwin._LovSL)
  {
    _LovInputSTC(lovwin.opener);
    return false;
  }

  // It's tempting to get rid of these PWP calls, and get the params from the
  // place where you KNOW they live, but they live in different places on
  // different browsers. Best to check all the likely places.
  _LovNM = _LovInputPWP('_LovNM', lovwin);

  // If the name object is null, then there has been a partial update of the
  // main page since we brought up the LOV window. This happens in the following
  // scenario: - The user enters enough text to match a single value
  //           - The user clicks the LOV icon
  //           - We send a validate event, and raise the window
  //           - The validate event completes, initiates a PPR redraw of the
  //             main page which hammers out JavaScript context.
  //           - The user selects the only selection available, and clicks the
  //             select button
  //           - We end up here, but there is no context.
  // This isn't really a problem since the PPR update has already updated the
  // text field with the only possible value. There is a bit of a problem here
  // in that if the user entered "San Fra", the LOV window came up with "San
  // Francisco" as the only selection, the user doesn't want that and presses
  // cancel. The user now expects "San Fra" to be in the text field, but it has
  // been updated with "San Francisco".

  if (_LovNM == null)
    return false;

  // If there's been a selection, assume we've dirtied the page
  lovwin.opener._navDirty = true;

  // if we're still in business...
  if (rv)
  {
    var params = new Object();
    params[_LovEV] = 'lovUpdate';
    params[_LovSR] = _LovNM[_LovSR];
    var partialTargets = _LovNM[_LovPT];

    // When done with an LOV window, the BLAF says we should focus on the
    // lovInput's textInput field.
    _setRequestedFocusNode(lovwin.opener.document, params[_LovSR], false);

    if (partialTargets)
    {
      params[_LovPT] = partialTargets;
    }

    // Look for the close function on the main page. The close function will
    // submit the given form with the given parameters.
    // We can't submit the form from here because submitForm expects the
    // document containing the form to be available as plain old 'document'.
    var cf = _LovInputPWP('_lovClose', lovwin);
    var frm = _LovNM[_LovFR];
    if ((cf != (void 0)) && (frm != (void 0)))
    {
      cf(frm, params, (partialTargets));
    }
    else
    {
      // If no close function found (this should never happen), then
      // fallback to a plain old event submission.
      var URL = _LovInputDDP(0, params);
      if (partialTargets)
      {
        URL = _LovInputUAA(URL, params, true);
        _firePartialChange(URL);
      }
      else
      {
        URL = _LovInputUAA(URL, params, true);
        location = URL;
      }
    }
  }
  return false;
}

///
///   URL manipulation functions
///

// DDP: Decompose Destination into Parts
// Inputs:
//   URL: The starting URL to break up
//   parts: An empty object which will be populated with the
//          parameters off the URL. The value will be indexed by the key.
//
// Returns:
//   The base URL (the original URL stripped of all args).
//
function _LovInputDDP(URL, parts)
{
  var base = URL;

  if (!URL)
  {
    if (_LovHR != (void 0))
    {
        base = _LovHR;
    }
    else if ((location != (void 0))
        && (location.href != (void 0)))
    {
        base = location.href;
    }
    else
    {
      // we have no possible recourse here.
      return "#";
    }
  }

  // We sometimes get a '#' added to the end of URLs, delete it here
  // (ONLY if it's the LAST character of the URL!)
  if (base.charAt(base.length - 1) == '#')
  {
    base = base.substr(0, base.length - 1);
  }

  var newURL = base;

  // Find the beginning of the argument section of the current URL
  var sepIndx = base.indexOf('?');

  // If no separator (and hence, no args) found, just keep the original URL
  if (sepIndx > 0)
  {
    // Otherwise, gotta split the args up
    newURL = base.substr(0, sepIndx);
    var query = base.substr(sepIndx + 1); // get just the args, drop the '?'
    var params = query.split('&');          // each arg separately

    for (var i = 0; i < params.length; i++)
    {
      var indx = params[i].indexOf('=');

      if (indx >= 0)
      {
        // If there's an '=', use everything before it as key,
        // and everything after as the value
        parts[params[i].substr(0, indx)] = params[i].substr(indx + 1);
      }
      else
      {
        // otherwise, use the whole param as a key to an empty string
        parts[params[i]] = "";
      }
    }
  }

  return newURL;
}
//
// UAA: URL Add Args
//
// Add all the args in the given object to the given URL
// Inputs:
//   URL: The base URL to which we are making changes.
//   paramObj: an object containing all the key value pairs.
//
// Returns: (string) the resulting new URL
//
function _LovInputUAA(URL, paramObj, encodeParams)
{
  // This URL has been sanitized, all the parameters are in the paramObj
  // Therefore, we know that the URL doesn't have any args,
  // so start with a '?'
  var sep = '?';
  var newURL = URL;

  if (newURL)
  {
    // go through the parameters adding them to the URL one at a time
    for (var key in paramObj)
    {
      var val = paramObj[key];
      newURL += (sep
                 + (encodeParams ? _LovInputENC(key) : key)
                 + '=');
      if (val)
        newURL += (encodeParams ? _LovInputENC(val) : val);
      // every arg after the first will be prefaced with an '&'
      sep = '&';
    }
  }
  return newURL;
}

// STC: Select Text after Cancel
//
// User is cancelling, select the text in the input field and set focus
//
// Inputs: win: The window containing the lovInput field.
//
// Returns: none you should depend on
//
function _LovInputSTC(win)
{
  // First, set the focus to the textInput and select the text.
  var nameObj = _LovInputPWP('_LovNM', win);

  var doc = null;
  var fld = null;
  if (nameObj && nameObj[_LovFR] && nameObj[_LovFL])
  {
    if ((win != null) && (win["document"]))
    {
      doc = win.document;
      fld = doc[nameObj[_LovFR]][nameObj[_LovFL]];
    }
  }

  if (fld)
  {
    // If fld is non null, we know that doc must be valid
    _pprFocus(fld, doc);
    // select all the text in the input field
    // so the user can blast or change it
    if (fld["select"] != null)
      fld.select();
  }
}

///
/// Button functions
///

// CBF: Cancel Button Function
//
// User is cancelling, close the LOV.
//
// Inputs: none
//
// Returns: none you should depend on
//
function _LovInputCBF()
{
  // We done. Go on home.
  top.close();
  return false;
}
// SBF: Select Button Function
//
// This function handles a click of the select button.
//
// Inputs: none
//
// Returns: none you should depend on
//
function _LovInputSBF(event)
{
  // OK, a select has happened, remember it.
  top._LovSL = true;

  // Another Navigator weirdness is that it won't allow alerts from a closing
  // window. We really have to call the onLovSelect callback here before the
  // window closes. This actually worked so well that we now extend do it the
  // same way on every browser.
  var nameObj = _LovInputPWP('_LovNM', top);
  var opn = top.opener;

  // If there's been a selection, assume we've dirtied the page
  opn._navDirty = true;

  var stateCheck = _LovInputPWP('_LovSC', window);
  var closeWin = true;
  if (stateCheck != (void 0) && stateCheck != '')
  {
    closeWin = eval(stateCheck + '(window, "_LOVResFrm")');
  }

  var selFunc = _LovInputPWP('_LovSF', top);

  // Now run the onLovSelect callback if there is one
  if (selFunc)
  {
    // If the client returns false from this callback, it should be treated as
    // if a select never happened.
    top._LovSL = selFunc(window,
                         opn.document[nameObj[_LovFR]][nameObj[_LovFL]],
                         event, opn);
  }

  // Netscape aborts the submit if the window goes away, so we can't just do a
  // submit followed by a top.close because the top.close happens locally, and
  // often completes before the submit. This causes the submit to be dropped.
  //
  // So, we defer the close of the LOV window until after the submit happens.
  // The submit will initiate a redraw, we catch it when the page unloads
  // and close the entire window then.
  //
  // added the top._unloadUIXDialog call to cover the Google pop-up
  // blocker bug where the body's onunload function does not get called
  // (bug 3184718).
  if (closeWin)
  {
    window.onunload = function(){top._unloadUIXDialog(window.event);
                                 top.close();};
  }

  // This *USUALLY* works... well, it doesn't work under Netscape on Solaris.
  // Netscape on Solaris doesn't redraw or deliver an onUnload event after the
  // submit!! Not only that, but calling the callback when the window is
  // closing leads to a Bus Error, so we call the WCB directly. So there!
  if (_agent.isNav && _agent.isSolaris)
  {
    window.onunload = function(){_LovInputWCB(top, event);};
  }
  submitForm(0, 0, {'event':'lovSelect', 'source':_LovLI});
}

// ENC: ENCode parameter text
//
// Encodes parameter text as well as possible on a given platform.
//
// Inputs: The string to encode
//
// Returns: The encoded string
//
function _LovInputENC(str)
{
  // Just Informational.
  // The following is the shift_jis encoding of "nippon" (means Japan).
  //   %93%FA%96%7B
  // And this is the utf8 one.
  //   %E6%97%A5%E6%9C%AC
  var encStr;
  var useEscape = _agent.isNav || _agent.isMac || _agent.atMost("ie", 5.49);

  if (!useEscape)
  {
    // JavaScript 1.5 added the handy encodeURIComponent which should work
    // with multibyte characters.
    encStr = encodeURIComponent(str);
  }
  else
  {
    // Earlier browsers need a little help.

    // There's really no useful function to encode multibyte strings, we do
    // what we can with what we have.

    // First just escape the string
    var escaped = escape(str);

    // Netscape's escape function doesn't encode the '+' char, and it would be
    // lost. Encode it here.
    var lastIdx = 0;
    var idx;
    encStr = "";

    while((idx = escaped.indexOf('+', lastIdx)) > 0 )
    {
      // explicitly encode each '+' char
      encStr += escaped.substring(lastIdx, idx) + '%2B';
      lastIdx = idx + 1;
    }
    // tack on anything that's left over
    encStr += escaped.substring(lastIdx);
  }
  return encStr;
}

//
// QSF: Quick Select Function
//
// This function gets called when the end-user clicks on the Lov Quick Select
// button in the ListOfValues table. It acts as if the user clicked on the
// single-selection radio button for the row, and then pressed the Select
// button. Therefore, this function programmatically selects the single
// selection's radio button for the table/row and then calls the _LovInputSBF
// function.
// Inputs:
//   tableName: The name of the table for which this button was selected.
//   row: zero-based index of the row.
//
// Returns: nothing
//
function _LovInputQSF(tableName, row)
{

  var proxy = new TableProxy(tableName);

  proxy.setSelectedRow(row);

  _LovInputSBF();
}

// USF: Use SubmitForm
//
// Returns true if the encoding, or some other factor, forces us to submit the
// validation event in a form rather than through a partialChange.
//
// Inputs: none
//
// Returns: true if a form should be used to send the event.
//
function _LovInputUSF()
{
  return _enc.toUpperCase() != "UTF-8";
}

// SFS: Second Form Submit
//
// This is a hack to make up for the fact that javascript can't encode to
// anything other than UTF-8. To support other character sets, we create a
// dummy form, move all the pertinent data over into it, and submit. On the
// server it looks the same, but on the client a new entry has been added to
// the history list.
//
// Inputs: params: all the parameters that would have gone on the URL
//         form: The form that the LOV currently lives in (we'll make a dummy
//               form with a name based on this form's name).
//
// Returns: none you should depend on
//
function _LovInputSFS(params, form, action)
{
  var doc = window.document;

  var formID = "_LovInput";
  if (form.id)
    formID += form.id;
  else if (form.name)
    formID += form.name;
  else
    formID += "DummyForm";

  var newForm = doc.createElement("form");
  newForm.id = formID;
  newForm.name = formID;
  newForm.target = form.target;
  newForm.method = form.method;
  if (action)
    newForm.action = action;
  else
    newForm.action = form.action;

  for (var paramName in params)
  {
    // build a hidden field to hold this parameter
    var hiddenField = doc.createElement("input");
    hiddenField.type = "hidden";
    hiddenField.name = paramName;
    hiddenField.value = params[paramName];
    newForm.appendChild(hiddenField);
  }

  doc.body.appendChild(newForm);

  // Bug #3058770
  // The fix for bug 2752257 relies on a form validater being present
  // It doesn't check for anything useful inside the validater, only that
  // there's something there. So, set up a dummy value.
  var val = "_"+ newForm.name + "Validater";
  var del = false;

  // If there's already something there, it'll serve our purpose.
  // Don't overwrite it.
  if (window[val] == (void 0))
  {
    del = true;
    window[val] = 1;
  }

  _submitPartialChange(newForm, 0, params);

  // Now remove the dummy validater
  // IE doesn't support deletion of window elements
  if (del && (!_agent.isIE))
    delete window[val];

  // and clean up the generated form
  doc.body.removeChild(newForm);
}

// SOE: Submit On Enter
//
// A copy of _submitOnEnter (built by the FormRenderer) which takes an event
// name, and source parameter. This function is used as an onKeyPress handler
// for the single searchText field in the basic search area (the advanced
// search mode would have multiple input fields). If the key pressed was the
// enter key, then the specified form will be submitted with the specified
// eventName and Source parameters, and FALSE will be returned (to halt
// further processing). For any other keypress TRUE will be returned.
//
// Inputs:  event: the user event that trigger the call to this function. This
//                 function is only used as a target of an onKeyPress handler,
//                 so this event will always be a keypress.
//           form: The form to submit - really the only form on the LOV page.
// generatedEvent: The event that will be generated when the form is submitted
//                 (currently this is hard-coded to 'lovFilter' in the
//                 listOfValues template.
//         source: The same source parameter that would go with any other
//                 lovFilter event (i.e. the ID of the lovInput element).
//
// Returns: FALSE, after a submit if the keyPress was a <cr>, TRUE otherwise.
//
function _LovInputSOE(event, form, generatedEvent, source)
{
  if (_getKC(event) == 13)
  {
    submitForm(form, 1, {'event': generatedEvent, 'source': source});
    return false;
  }
  return true;
}

// CoreFormat
//-----------------------------------------------------------------------------
/*
** Copyright (c) Oracle Corporation 2000-2002. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/

function _decimalFormat(
  number
  )
{
  return "" + number;
}


function _decimalParse(
  numberString
  )
{
  if (!numberString)
    return (void 0);

  // Get LocaleSymbols (from Locale.js)
  var symbols = getLocaleSymbols();
  if (symbols)
  {
    // We don't want leading or trailing grouping separators
    var grouping = symbols.getGroupingSeparator();
    if ((numberString.indexOf(grouping) == 0) ||
        (numberString.lastIndexOf(grouping) ==  (numberString.length - 1)))
      return (void 0);

    // Remove the thousands separator - which Javascript doesn't want to see
    var thousands = new RegExp("\\" + grouping, "g");
    numberString = numberString.replace(thousands, "");
    // Then change the decimal separator into a period, the only
    // decimal separator allowed by JS
    var decimal = new RegExp("\\" + symbols.getDecimalSeparator(),  "g");
    numberString = numberString.replace(decimal, ".");
  }

  // First, reject anything that's all spaces
  var i = numberString.length - 1;
  while (i >= 0)
  {
    if (numberString.charAt(i) != ' ')
      break;
    i--;
  }

  if (i >= 0)
  {
    // OK; it's non-empty.  Now, disallow exponential
    // notation, and then use some JS magic to exclude
    // non-numbers
    if ((numberString.indexOf('e') < 0) &&
        (numberString.indexOf('E') < 0) &&
        (((numberString * numberString) == 0) ||
         ((numberString / numberString) == 1)))
    {
      var result = parseFloat(numberString);
      if (!isNaN(result))
      {
        var integerDigits = numberString.length;
        var fractionDigits = 0;

        var sepIndex = numberString.lastIndexOf('.');
        if (sepIndex != -1)
        {
          integerDigits = sepIndex;
          fractionDigits = numberString.length - sepIndex -1;
        }
        
        var messageKey;
        
        if ((this._maxValue != (void 0)) &&
            (result  > this._maxValue))
        {
          messageKey = DecimalFormat.LV;
        }
        else if ((this._minValue != (void 0)) &&
                 (result  < this._minValue))
        {
          messageKey = DecimalFormat.MV;
        }
        else if ((this._maxPrecision != (void 0)) &&
                 (integerDigits  > this._maxPrecision))
        {
          messageKey = DecimalFormat.LID;
        }
        else if ((this._maxScale != (void 0)) &&
                 (fractionDigits  > this._maxScale))
        {
          messageKey = DecimalFormat.LFD;
        }

        if (messageKey != (void 0))
        {
          var messages = this._messages;
          
          if ((messages == (void 0)) ||
              (messages[messageKey] == (void 0)))
            return (void 0);  // default error format
          else
            return new ParseException(messages[messageKey]);
        }
        
        return result;
      }
    }
  }

  return (void 0);
}

function DecimalFormat(
  messages,
  maxPrecision,
  maxScale,
  maxValue,
  minValue)
{
  this._messages = messages;
  this._maxPrecision = maxPrecision;
  this._maxScale = maxScale;
  this._maxValue = maxValue;
  this._minValue = minValue;

  // for debugging
  this._class = "DecimalFormat";
}

DecimalFormat.prototype = new Format();
DecimalFormat.prototype.format = _decimalFormat;
DecimalFormat.prototype.parse  = _decimalParse;

// Less fraction digits
DecimalFormat.LFD = 'LFD';
// Less integer digits
DecimalFormat.LID   = 'LID';
// Less value
DecimalFormat.LV        = 'LV';
// More value
DecimalFormat.MV        = 'MV';


function _regExpFormat(
  value
  )
{
  return value;
}


function _regExpParse(
  parseString
  )
{
  var matchArr = parseString.match(this._pattern);
  
  if ((matchArr != (void 0)) && (matchArr[0] == parseString))
  {
    return parseString;
  }
  else
  {
    return (void 0);
  }
}


function RegExpFormat(
  pattern
  )
{
  this._class = "RegExpFormat";
  this._pattern = pattern;
}


RegExpFormat.prototype = new Format();
RegExpFormat.prototype.format = _regExpFormat;
RegExpFormat.prototype.parse  = _regExpParse;

// MarlinCore
//-----------------------------------------------------------------------------
/*
** Copyright (c) Oracle Corporation 2000-2003. All Rights Reserved.
**
** @version $Name:  $ ($Revision: 1.172 $) $Date: 2004/02/06 03:22:47 $
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/

/**
 * Note that the only function in this file that is supported is the openWindow
 * function.
 * All other functions may change without notice.
 */
// Object containing information about the user agent
var _agent = new Object();

// Object for the last time we submitted
var _lastDateSubmitted;

// Object for the last time we reset a form
var _lastDateReset = 0;

// Variables tracking the last time we validated a field, and the last time the
// validation actually failed.
var _lastDateValidated  = 0;
var _lastValidationFailure  = 0;

// Number of submits done on this page. This gives an accurate
// value of the depth of the history list (or distance to the page
// that preceeded this one). Well, it's acurate on Mozilla, it
// has little resemblance to reality on IE.
var _pprSubmitCount = 0;

// IE doesn't add the first partial refresh into the history list.
// Doesn't matter if it's a partialSubmit or a partialChange. However,
// subsequent submits are counted. This variable keeps track of whether we've
// done any action on this page.
var _pprSomeAction = false;

// Number of outstanding partial page request.  Partial
// page rendering keeps this ref count to distinguish
// between real partial page activity and back/refresh.
var _pprRequestCount = 0;

// Flag used by partial page rendering to mark whether the
// parent window has been unloaded.
var _pprUnloaded = false;

// Flag used by partial page rendering and the back issue
// to indicate whether or not we need to restore the saved inline scripts.
var _pprBackRestoreInlineScripts = false;

// We block input while a PPR request is in transit.
// _pprSavedCursor holds the cursor that the client had before the PPR event
var _pprSavedCursor = null;

// _pprSavedBlocking is true if we're blocked waiting on a PPR event
var _pprBlocking = false;

// We block using a special DIV element. This is it's name
var _pprdivElementName = '_pprBlockingDiv';

// A holder for the pending timeout (Gecko only).
var _pprBlockingTimeout = null;

// Keeps track of the last element to initiate a PPR request
var _pprEventElement = null;

// Keeps track of whether the user has actually made a choice from the popup
var _pprChoiceChanged = false;

// Object containing the initial state of a chosen form
var _initialFormState;
var _initialFormExclude = new Object();
var _initialFormStateName;
var _navDirty;

// this is the id of the component which gets the initial focus when the
// page loads.
var _initialFocusID = null;

// Certain UIX facilities can request that focus be set to a particular node,
// or the node AFTER a particular node following a PPR update. These three
// variables store the values needed to track down that node.
var _UixFocusRequestDoc = null;
var _UixFocusRequestID = null;
var _UixFocusRequestNext = false;

// _checkUnload is set on our body tag and is called when the window
// unloads. In our dialog windows, we call _checkUnload via an intermediary
// function _unloadUIXDialog to work around Google's pop-up blocker
// feature which blocks onUnload event handlers from window's opened
// with window.open.
// We don't want to call _checkUnload twice in a row,
// once from dialog code and once from body's onUnload event handler,
// so we block the second call in _unloadUIXDialog.
var _blockCheckUnloadFromDialog = false;


// variables needed if _submitForm was called before the form had
// completely rendered.
var _saveForm = null;
var _saveDoValidate = null;
var _saveParameters = null;
var _submitRejected = false;
var _inPartialSubmit = false;

/**
 * Return true if the agent is at least the specified agent type and version.
 */
function _atLeast(
  kind,
  version
  )
{
  return (!kind    || (kind    == _agent.kind))    &&
         (!version || (version <= _agent.version));
}


/**
 * Return true if the agent is at most the specified agent type and version.
 */
function _atMost(
  kind,
  version
  )
{
  return (kind == _agent.kind) && (version >= _agent.version);
}

function _supportsDOM()
{
  var retVal = false;

  if (_agent.isIE)
  {
    retVal = _agent.version >= 5.5;
  }
  else if (_agent.isNav)
  {
    retVal = false;
  }
  else if (_agent.isGecko)
  {
    retVal = true;
  }
  return retVal;
}

/**
 * initialize information about the agent
 */
function _agentInit()
{
  // convert all characters to lowercase to simplify testing
  var agentString = navigator.userAgent.toLowerCase();

  var version = parseFloat(navigator.appVersion);

  var isOpera   = false;
  var isIE      = false;
  var isNav     = false;
  var isGecko   = false;
  var isSafari = false;
  var kind      = "unknown";
  var isWindows = false;
  var isSolaris = false;
  var isMac     = false;

  if (agentString.indexOf("msie") != -1)
  {
    isIE = true;

    // extract ie's version from the ie string
    var matches = agentString.match(/msie (.*);/);
    version = parseFloat(matches[1]);
    kind = "ie";
  }
  else if (agentString.indexOf("opera") != -1)
  {
    isOpera = true
    kind = "opera";
  }
  else if ((agentString.indexOf("applewebkit") != -1) ||
           (agentString.indexOf("safari") != -1))
  {
    isSafari = true
    kind = "safari";
  }
  else if (agentString.indexOf("gecko/") != -1)
  {
    isGecko = true;
    kind = "gecko";
    version = 1.0;
  }
  else if ((agentString.indexOf('mozilla')    != -1) &&
           (agentString.indexOf('spoofer')    == -1) &&
           (agentString.indexOf('compatible') == -1))
  {
    if (version >= 5.0)
    {
      isGecko = true;
      kind = "gecko";
      version = 1.0;
    }
    else
    {
      isNav = true;
      kind = "nn";
    }
  }

  if (agentString.indexOf('win') != -1)
  {
    isWindows = true;
  }
  else if (agentString.indexOf('mac') != -1)
  {
    isMac = true;
  }
  else if (agentString.indexOf('sunos') != -1)
  {
    isSolaris = true;
  }

  _agent.isIE = isIE;
  _agent.isNav = isNav;
  _agent.isOpera = isOpera;
  _agent.isGecko = isGecko;
  _agent.isSafari = isSafari;
  _agent.version = version
  _agent.kind = kind;
  _agent.isWindows = isWindows;
  _agent.isSolaris = isSolaris;
  _agent.isMac = isMac;

  _agent.atLeast = _atLeast;
  _agent.atMost  = _atMost;
}


_agentInit();

// available features in ie
var _ieFeatures =
{
  channelmode:1, // ie 5.0
  copyhistory:1,
  directories:1,
  fullscreen:1,  // ie 5.0
  height:1,
  location:1,
  menubar:1,
  resizable:1,
  scrollbars:1,
  status:1,
  titlebar:1,    // ie 5.0 when trusted
  toolbar:1,
  width:1
};

// available features in Netscape
var _nnFeatures =
{
  alwayslowered:1,
  alwaysraised:1,
  copyhistory:1,
  dependent:1,
  directories:1,
  height:1,
  hotkeys:1,
  innerheight:1,
  innerwidth:1,
  location:1,
  menubar:1,
  outerwidth:1,
  outerheight:1,
  resizable:1,
  scrollbars:1,
  status:1,
  titlebar:1,
  toolbar:1,
  width:1,
  "z-lock":1
}

// override values for modeless windows. Values in this
// list can't be overridden by the caller for modeless windows
var _modelessFeatureOverrides =
{
};

// override values for modal windows. Values in this
// list can't be overridden by the caller for modal windows
var _modalFeatureOverrides =
{
};


var _featureDefaults =
{
  // default values for features of document windows
  document:
  {
    channelmode:false,
    copyhistory:true,
    dependent:false,
    directories:true,
    fullscreen:false,
    hotkeys:false,
    location:true,
    menubar:true,
    resizable:true,
    scrollbars:true,
    status:true,
    toolbar:true
  },
  // default values for features of dialog windows
  dialog:
  {
    channelmode:false,
    copyhistory:false,
    dependent:true,
    directories:false,
    fullscreen:false,
    hotkeys:true,
    location:false,
    menubar:false,
    resizable:true,
    scrollbars:true,
    status:true
  }
}


// featues that require signbing in order to be set
var _signedFeatures =
{
  alwayslowered:1,
  alwaysraised:1,
  titlebar:1,
  "z-lock":1
};

// features that are boolean values
var _booleanFeatures =
{
  alwayslowered:1,
  alwaysraised:1,
  channelmode:1,
  copyhistory:1,
  dependent:1,
  directories:1,
  fullscreen:1,
  hotkeys:1,
  location:1,
  menubar:1,
  resizable:1,
  scrollbars:1,
  status:1,
  titlebar:1,
  toolbar:1,
  "z-lock":1
};


/**
 * Gets the preferred width of the content
 */
function _getContentWidth(
  element,
  offsetWidth,
  offsetLeft
  )
{
  var children = element.childNodes;

  var isGecko = _agent.isGecko;

  var hasContentProp = (isGecko)
                         ? "tagName"
                         : "canHaveHTML"
  var maxWidth = 0;

  for (var i = 0; i < children.length; i++)
  {
    var currChild = children[i];

    if (currChild[hasContentProp] && (currChild.offsetWidth > 0))
    {
      var currWidth = 0;
      var currOffsetWidth = currChild["offsetWidth"];

      if (isGecko)
      {
        if (currOffsetWidth == offsetWidth)
        {
          currWidth = _getContentWidth(currChild,
                                       currOffsetWidth,
                                       currChild.offsetLeft);
        }
        else
        {
          currWidth = currOffsetWidth;
        }
      }
      else
      {
        currWidth = currChild["clientWidth"];

        if (currWidth == 0)
        {
          currWidth = _getContentWidth(currChild,
                                       currOffsetWidth,
                                       currChild.offsetLeft);
        }
      }

      if (currWidth > maxWidth)
      {
        maxWidth = currWidth;
      }
    }
  }

  // handle error cases
  if (maxWidth == 0)
    maxWidth = offsetWidth;

  // bogusly double the offset to guess the right margin...
  // However, in right to left languages, the left margin can get very large,
  // and cause the window to become huge (Bug 2846393). Limit it to an
  // empirically reasonable value.
  var marginWidth = 10;

  if (_isLTR() || (offsetLeft <= 5))
  {
      marginWidth = 2 * offsetLeft;
  }

  return maxWidth + marginWidth;
}

function _getTop(
  element
  )
{
  if (!_agent.isGecko)
  {
    return top;
  }
  else
  {
    var currWindow = (element)
                       ? element.window
                       : window;

    while (currWindow.parent && (currWindow.parent != currWindow))
    {
      currWindow = currWindow.parent;
    }

    return currWindow;
  }
}

/**
 * Sizes the window to its preferred size.
 */
function _sizeWin(
  element,
  extraWidth,
  extraHeight,
  params
  )
{
  var isGecko = _agent.isGecko;
  var isIE      = _agent.isIE;

  if (!(isGecko || (isIE && _agent.isWindows)))
    return;

  /*
  // =-= bts theoretically, this would be all we need to do for Mozilla,
  //         but the implementation appears to be sub-optimal for our case
  if (isGecko)
  {
    element.window.sizeToContent();
    return;
  }
  */

  var body =  element.window.document.body;

  if (body)
  {
    // width of the inside
    var newWidth = (!isIE && (body.scrollWidth > body.clientWidth))
                     ? body.scrollWidth
                     : _getContentWidth(body, body.offsetWidth, body.offsetLeft);
    var newHeight = 0;

    if (isGecko)
    {
      newHeight = body.offsetHeight + (window.outerHeight - window.innerHeight);

      // random extra space added in to make this work
      newHeight += 30;

      // add in the window frame plus padding
      newWidth  += (window.outerWidth - body.offsetWidth);
    }
    else
    {
      newHeight = body.scrollHeight + (body.offsetHeight - body.clientHeight);

      // if this method supported windows with toolbars, we would need to
      // add another 67 here, rather than the aribitrary 8 pixels
      newHeight += 21;

      // add in the padding plus bogus extra size
      newWidth += body.offsetWidth - body.clientWidth + 16;

      // add in the margins (MS bogusly uses Strings for these)
      newHeight += parseInt(body.topMargin) + parseInt(body.bottomMargin);
      newWidth  += parseInt(body.leftMargin) + parseInt(body.rightMargin);
    }

    //
    // allow the size to be a little bigger than currently necessary.
    // This is useful when we will be paging through multiple items and
    // later pages  might need a slightly larger window than the initial
    // page.
    if (extraWidth)
      newWidth += extraWidth;

    if (extraHeight)
      newHeight += extraHeight;

    // Make sure that width and height are at larger than min requested
    if (params != (void 0))
    {
      if (params['W'])
      {
        var minWidth = 0 + params['W'];
        if (newWidth < minWidth)
          newWidth = minWidth;
      }

      if (params['H'])
      {
        var minHeight = 0 + params['H'];
        if (newHeight < minHeight)
          newHeight = minHeight;
      }
    }

    // element.window.parent.resizeTo(newWidth, newHeight);
    _getTop(element).resizeTo(newWidth, newHeight);
  }
}


/**
 * onClick override for Netscape when modal windows are up.
 * Handles suppressing mouse clicks.
 */
function _onModalClickNN(
  event
  )
{
  if (_getValidModalDependent(self))
  {
    // eat the mouse click if we have a modal window
    return false;
  }
  else
  {
    // allow the mouse click to propagate down if we
    // don't have a modal window
    self.routeEvent(event);
    return true;
  }
}

//
// mozilla event handler for eating click events
var _mozClickEH = new Object();

function _onModalClickMoz(
  event
  )
{
  dump(event);
}

_mozClickEH["handleEvent"] = _onModalClickMoz;


/**
 * onFocus override when modal windows are up.
 * Handles moving the focus to the top and suppressing IE mouse clicks.
 */
function _onModalFocus()
{
  var theBody = self.document.body;

  var modalDependent = _getValidModalDependent(self);

  var hasCapture = _agent.atLeast("ie", 5) && _agent.isWindows;

  if (modalDependent)
  {
    modalDependent.focus();

    // reset the capture on IE since it is released whenever
    // the focus moves to another window
    if (hasCapture)
    {
      theBody.setCapture();
    }
  }
  else
  {
    if (hasCapture)
    {
      theBody.releaseCapture();
    }
  }
}

/**
 * renders transparent image for spacing
 */
function t(width,height)
{

  // if the transparent url is not null render img tag
  if ( _tURL != void 0 )
  {
    document.write('<img src="' + _tURL + '"');

    if (width!=void 0)
      document.write(' width="' + width + '"');
    if (height!=void 0)
      document.write(' height="' + height + '"');

    // if accessibility mode is not null, render alt attribute
    if (_axm != void 0)
      document.write(' alt=""');

    document.write('>');
  }
}

/**
 * Opens a new window, makes it active, and returns the Window
 * object.
 * <p>
 * @param parentWindow The window opening this window.  For modal
 *                     windows, this is the window that the
 *                     new window will be modal to. REQUIRED.
 * @param srcURL       URL for the contents of the new window.
 *                     REQUIRED.
 * @param windowName   Name that identifies the new window for
 *                     URLS. Required if a closeCallback is to be used.
 * @param features     Javascript object containing name/value pairs
 *                     of features to apply to the new window,
 * @param isModal      True if the window should be modal.
 * @param kind         Either 'document' or 'dialog'.  The default is
 *                     dependent on the value of isModal--'document'
 *                     for modeless, 'dialog' for modal.
 * @param closeCallback  Callback method called when the widnow is closed.
 *                      Note: Under certain adverse conditions (for example,
 *                      if the parentWindow navigates to a new page), it is
 *                      possible that the closeCallback may not be called.
 */
function openWindow(
  parentWindow,
  srcURL,
  windowName,
  features,
  isModal,
  kind,
  closeCallback
  )
{
  if (parentWindow)
  {
    // default modality if none specified
    if (isModal == (void 0))
      isModal = false;

    // if the kind isn't specified, default the kind off of the modality
    if (!kind)
    {
      kind = (isModal) ? "dialog" : "document";
    }

    // default the window name to "_blank" if no
    // name is specified, in order to guarantee that a new window is
    // opened
    if (!windowName)
      windowName = "_blank";

    //
    // pick the correct defaults and overrides
    //
    var defaults = _featureDefaults[kind];

    if (defaults == (void 0))
    {
      kind = "document";
      defaults = _featureDefaults[kind];
    }

    var overrides = (isModal)
                      ? _modalFeatureOverrides
                      : _modelessFeatureOverrides;

    // determine the avialable features
    var agentFeatures = (_agent.isIE)
                          ? _ieFeatures
                          : _nnFeatures;

    // we'll be hammering the features object, so make a copy
    var featuresCopy = null;
    if (features)
    {
      featuresCopy = new Object();
      for (var f in features)
      {
        featuresCopy[f] = features[f];
      }
    }

    //
    // build up the feature string
    //
    var featureString = "";

    // loop through the available features for this platform
    for (var featureName in agentFeatures)
    {
      // get the overridden value of the feature
      var featureValue = overrides[featureName];

      if (featureValue == (void 0))
      {
        // get the value of the feature if it isn't overridden
        if (featuresCopy)
        {
          featureValue = featuresCopy[featureName];
          delete featuresCopy[featureName];
        }

        // if no value, get the default value, if any
        if (featureValue == (void 0))
          featureValue = defaults[featureName];
      }

      if (featureValue != (void 0))
      {
        // check if this is a boolean value
        var isBoolean = _booleanFeatures[featureName] != (void 0);

        // output the value
        if (featureValue || !isBoolean)
        {
          // add the feature name
          featureString += featureName;

          // add the value for nonboolean features
          if (!isBoolean)
          {
            featureString += "=" + featureValue;
          }

          // add separator between this and the next
          featureString += ",";
        }
      }
    }

    // Now tack on all the extra features that the user has requested.
    // These may or may not have meaning to the browser's implementation of
    // window.open().
    for (var f in featuresCopy)
    {
      featureString += f;
      if (featuresCopy[f])
        featureString += "=" + featuresCopy[f];

      featureString += ",";
    }

    // trim off last separator
    if (featureString.length != 0)
    {
      featureString = featureString.substring(0, featureString.length - 1);
    }

    // register the closing callback
    if (closeCallback)
    {
      _setDependent(parentWindow, windowName, closeCallback);
    }

    // detect the bogus ie4 and turn off modal windows
    var atMostIE4 = _agent.atMost("ie", 4.99);
    var alphaFilter = false;

    // body of the aprent window
    var parentBody = parentWindow.document.body;

    if (isModal && !atMostIE4)
    {
      if (_agent.atLeast("ie", 4))
      {
        parentBody.style.filter = "alpha(opacity=50)";
        alphaFilter = true;
      }

      if ((_agent.atLeast("ie", 5) && _agent.isWindows))
      {
        parentBody.setCapture();
      }
      else if (_agent.isNav)
      {
        parentWindow.captureEvents(Event.CLICK);
        parentWindow.onclick = _onModalClickNN;
      }
      else if (_agent.isGecko)
      {
        // this should work, but doesn't appear to
        parentBody.addEventListener(Event.CLICK, _mozClickEH, true);
      }

      parentWindow.onfocus = _onModalFocus;
    }

    // open an empty window
    var newWindow = parentWindow.open(srcURL, windowName, featureString);

    /*
    // create the content for the window.  We use a frameset so that
    // the content can change without firing the window close event.
    var realSrcURL = "<html>";

    //realSrcURL += "<head><scr";
    //realSrcURL += 'ipt src="/images/jsLibs/Window.js"></head>';

    realSrcURL += '<frameset rows="100%,*" border="0" onunload="_checkUnload(event)"><frame src="' +
          srcURL +
          '"></frameset></html>';

    newWindow.document.write(realSrcURL);
    newWindow.document.close();
        */

    if (isModal && !atMostIE4)
    {
      _setDependent(parentWindow, "modalWindow", newWindow);
    }

    // make the active window
    newWindow.focus();


    // Set up a timer to make sure that we reset the alpha filter.
    if (alphaFilter)
    {
      parentWindow.setTimeout("_clearAlphaFilter()", 1000);
    }

    return newWindow;
  }
  else
  {
    return null;
  }
}


/**
 * Returns the object containing the dependent windows.
 */
function _getDependents(
  parentWindow,
  createIfNecessary
  )
{
  var depends;

  if (parentWindow)
  {
    depends = parentWindow["_dependents"];

    if (depends == (void 0))
    {
      if (createIfNecessary)
      {
        depends = new Object();
        parentWindow["_dependents"] = depends;
      }
    }
  }

  return depends;
}

/**
 * Get the named dependent
 */
function _getDependent(
  parentWindow,
  dependentName
  )
{
  var depends = _getDependents(parentWindow);
  var dependent;

  if (depends)
  {
    dependent = depends[dependentName];
  }

  return dependent;
}


/**
 * Sets the value of the named dependent
 */
function _setDependent(
  parentWindow,
  dependentName,
  dependentValue
  )
{
  var depends = _getDependents(parentWindow, true);

  if (depends)
  {
    depends[dependentName] = dependentValue;
  }
}


/**
 * Returns the dependent which is modal.
 */
function _getModalDependent(
  parentWindow
  )
{
  return _getDependent(parentWindow, "modalWindow");
}

/**
 * Returns the dependent which is modal and is still open.
 */
function _getValidModalDependent(
  parentWindow
  )
{
  var modalDependent = _getModalDependent(parentWindow);

  if (modalDependent)
  {
    if (modalDependent.closed)
    {
      _setDependent(parentWindow, "modalWindow", (void 0));
      modalDependent = (void 0);
    }
  }

  return modalDependent;
}


/**
 * Returns true if the passed in dependent is the modal dependent
 * of the parent window,
 */
function _isModalDependent(
  parentWindow,
  dependent
  )
{
  return (dependent == _getModalDependent(parentWindow));
}

// A self-rescheduling function which clears out alpha filtering when
// modal windows are closed.  The alpha filter is normally removed in
// the modal window's onunload() handler.  However, the onunload handler
// may not be called if the modal window is closed by the user before
// loading completes.  This function is used in conjunction with
// Window.setTimeout() to make sure that the alpha filter is cleared
// in this case.
function _clearAlphaFilter()
{
  if (_getValidModalDependent(self) != null)
  {
    // If the modal window is still shown, re-schedule ourselves
    self.setTimeout("_clearAlphaFilter()", 1000);
  }
  else
  {
    // No modal dependent - clear out the alpha filter and
    // don't bother rescheduling.
    self.document.body.style.filter = null;
  }
}


/**
 * Called by our modal windows when changes are applied and
 * the window is closed to make sure that the parent window
 * is updated appropriately. Due to bug 3184718 (Google
 * pop-up blocker does not call onUnload event handlers from
 * dialog windows) we cannot rely on _checkUnload being
 * called by the unload event. We call this function
 * instead which in turn calls _checkUnload.
 */
function _unloadUIXDialog(
  event
  )
{
  // _checkUnload is called from body's
  // unload event when
  // We use this flag to keep it from
  // running through _checkUnload function twice.
  // If Google's pop-up blocker is
  // not enabled, then _checkUnload is called from body's
  // unload event as well as from here.

  _blockCheckUnloadFromDialog = false;
  _checkUnload(event);
  _blockCheckUnloadFromDialog = true;
}

/**
 * Called by the unload handler of modal windows to call the event
 * handler and make sure that the parent window is updated appropriately
 * to show that no modal window is up anymore.
 */
function _checkUnload(
  event
  )
{
  // Make sure we don't run through this function twice
  // when we close a dialog. The
  // _unloadUIXDialog function blocks a second run
  // using the _blockCheckUnloadFromDialog flag.

  if (_blockCheckUnloadFromDialog)
  {
    _blockCheckUnloadFromDialog = false;
    return;
  }


  // Check to see if we are a modal window that has been
  // abandoned (who's parent has changed out from under us).
  // In this case, we skip the unload handler, since the
  // modal window no longer has permission to access its
  // parent, and JavaScript errors may occur
  if (_isModalAbandoned())
    return;

  // Check to see if we have an open modal child window
  var modalWindow = _getModalDependent(window);
  if (modalWindow != null)
  {
    // If we are being unloaded before our modal child has been
    // closed, that means that the user must have navigated
    // to a new page just before the modal window was displayed.
    // In this case, let's just close our modal child.  We need
    // to be extra careful to make sure that the modal child does
    // not try to access any properties on the parent window
    // when closing, because the modal child may no longer have
    // permission to access us at this point.  Set a property
    // on the modal window to let it know that it has been
    // abandoned.
    _setModalAbandoned(modalWindow);

    // Now we can safely close the modal window
    modalWindow.close();
  }

  _pprUnloaded = true;

  var topWindow = _getTop();

  if (!topWindow)
    return;

  var parentWindow = topWindow["opener"];

  if (!parentWindow)
    return;

  var unloader = _getDependent(parentWindow, self.name);

  if (_isModalDependent(parentWindow, self))
  {
    // remove the modal window
    _setDependent(parentWindow, "modalWindow", (void 0));

    parentWindow.onfocus = null;

    var parentBody = parentWindow.document.body;

    // release the ie mouse grab
    if (_agent.atLeast("ie", 4))
    {
      if (_agent.atLeast("ie", 5) && _agent.isWindows)
      {
        parentBody.releaseCapture();
      }
      parentBody.style.filter = null;
    }

    // release the netscape mouse grab
    if (_agent.isNav)
    {
      parentWindow.releaseEvents(Event.CLICK);

      // stop eating all of the mouse clicks
      parentWindow.onclick = null;
    }

    if (_agent.isGecko)
    {
      parentBody.removeEventListener(Event.CLICK,
                                     _mozClickEH,
                                     true);
    }
  }

  if (unloader != (void 0))
  {
    // remove our dependent info
    _setDependent(parentWindow, self.name, (void 0));

    // try the passed in event (netscape way first), then
    // try to get the event the IE way
    if (event == (void 0))
      event = self.event;

    // call the unloader with the unloading window and the event
    unloader(topWindow, event);
  }
}

// Tests whether the current window is an "abandoned"
// modal window.  This is a modal window who's parent
// window has navigated to a new page, in which case
// the modal window is out of context.
function _isModalAbandoned()
{
  // We look for the _abandoned property on the modal window.
  // Note that in the LOV case, we actually have two onunload
  // event handlers that need to be suppressed: The onunload
  // handler for the LOV window and the onunload handler for
  // the actual contents of the LOV window which are nested
  // within a frame.  So, we check for the _abandoned property
  // on the "top" window.
  var topWindow = _getTop();
  return topWindow._abandoned;
}

// Marks the specified modal window as abandoned
function _setModalAbandoned(
  modalWindow
  )
{
  // Just set the _abandoned property on the modal window.
  modalWindow._abandoned = true;
}


function _focusChanging()
{
  if (_agent.isIE)
  {
    return (window.event.srcElement != window.document.activeElement);
  }
  else
  {
    // Netscape gives us no good way of determining this
    return true;
  }
}


/**
 * Function that returns a single key/value pair String
 */
function _getKeyValueString(
  target,
  keyName,
  index
  )
{
  var value = target[keyName];

  if (typeof(value) == "function")
  {
    value = "[function]";
  }

  var separator = (_agent.isGecko)
                    ? ((index + 1) % 3 == 0)
                      ? '\n'
                      : '    '
                    : '\t';

  return keyName + ':' + value + separator;
}

function _dump(
  target
  )
{
  dump(target, {innerText:1, outerText:1, outerHTML:1, innerHTML:1});
}

/**
 * Utility for dumping the contents of a JavaScript object.
 */
function dump(
  target,
  suppressProps,
  name
  )
{
  var props = "";

  if (target)
  {
    // default the name if none provided
    if (!name)
    {
      name = target["name"];
    }

    //
    // Because we need to catch exceptions that IE throws if
    // for some object reads, we need to have separate ie and netscape
    // code for the exception catching.  Unfortunately, "try" and
    // catch are reserved words, so we have to dynamically create
    // our adding function so that Netscape doesn't choke when it
    // parses our file
    //
    // var adderContent = "return i + ':' + target[i] + '\\n';";
    var adderContent = "return _getKeyValueString(target, key, index);";

    // wrap adder content with a try and eat the bogus ie exception
    if (_agent.atLeast("ie", 5) || _agent.isGecko)
      adderContent = "try{" + adderContent + "}catch(e){return '';}";

    var adder = new Function("target", "key", "index", adderContent);
    var propCount = 0;
    var propArray = new Array();

    for (var key in target)
    {
      // don't add properties that should be suppressed
      if ((!suppressProps || !suppressProps[key]) && !key.match(/DOM/))
      {
        propArray[propCount] = key;
        propCount++;
      }
    }

    // sort the array so that we can find stuff
    propArray.sort();

    for (var i = 0; i < propArray.length; i++)
    {
      props += adder(target, propArray[i], i);
    }
  }
  else
  {
    // the object to dump was undefined
    name = "(Undefined)";
  }

  // tell the user that the object has no properties
  if (props == "")
  {
    props = "No properties";
  }

  alert(name + ":\n" + props);
}


/**
 * Calls the correct validations function for the form and returns true
 * if the validation succeeded.
 */
function _validateForm(
  form
  )
{
  var funcName = '_' + form.name + 'Validater';
  var formWind = window[funcName];
  if (formWind)
    return formWind(form);

  return false;
}


/**
 * Returns the next sibling that is not a comment
 */
function _getNextNonCommentSibling(
  parent,
  index
  )
{
  var children = parent.children;

  for (var i = index + 1; i < children.length; i++)
  {
    var child = children[i];

    if (child && (child.tagName != "!"))
    {
      return child;
    }
  }

  return null;
}


/**
 * Validate the specified field.
 */
function _valField(
  formName,
  nameInForm
  )
{
  if (nameInForm)
  {
    // get the target whose validation we want to run
    var target = document.forms[formName][nameInForm];

    // get its onblur function
    var blurFunc = target.onblur;

    if (blurFunc)
    {
      var valFunc = blurFunc.toString();

      // whack off the beginning and end of the function, leaving the content
      var valContents = valFunc.substring(valFunc.indexOf("{") + 1,
                                          valFunc.lastIndexOf("}"));

      var targetString = "document.forms['" +
                         formName +
                         "']['" +
                         nameInForm +
                         "']";

      // replace 'this' with the actual target
      valContents = valContents.replace(/this/, targetString);

      // trim off last argument
      var lastArg = valContents.lastIndexOf(",");

      valContents = valContents.substring(0, lastArg) + ")";

      // perform the validation
      eval(valContents);
    }
  }
}

function _validationAlert(errorString)
{
  // Show the error and note the time we finished this validation.
  // Record the validation both before and after the alert so that we
  // halt any validations caused by events triggered along with this
  // one, or by the closing of this alert.
  _recordValidation(true, 0);
  alert(errorString);
  _recordValidation(true, 0);
}

// Records the time of this validation event.
// If fail is set, this is a validation failure, that is noted also.
function _recordValidation(fail, now)
{
  if (!now)
    now = new Date();

  _lastDateValidated = now;
  if (fail)
    _lastValidationFailure = now;
}


// returns true if a validation has occurred "recently"
// failures: True means only report on recent failures, false means report on
//           any validation.
function _recentValidation(failures)
{
  var retVal = false;
  var timeWindow = 250;

  // Assuming that a reasonable user won't close the dialog and change the
  // text within a quarter of a second, we ignore any validations within
  // 250ms. of the last failed validation. The timings I've seen here range
  // in the 60 - 90 ms. range, but that is on fast development machines. We
  // could probably lower this to 150 if we're seeing dropped validations
  // for really fast, ambitious users.
  // With Macintosh IE, we manage to crash the browser!
  if (_agent.isMac)
  {
    // The iBook can have diffs of up to about 480 ms.
    // Call it 600 to be safe.
    timeWindow = 600;
  }

  var newDate = new Date();
  var diff;

  // If failures are requested, the caller is only interested in failures.
  // If simple validation requested, caller interested in any validation fail
  // or not.
  diff = newDate - _lastValidationFailure;
  if ((diff >= 0) && (diff < timeWindow))
  {
    retVal = true;
  }
  else if (!failures)
  {
    diff = newDate - _lastDateValidated;
    if ((diff >= 0) && (diff < timeWindow))
    {
      retVal = true;
    }
  }
  return retVal;
}

/**
 * Field validation function
 * @param err   The error string
 * @param input The field we are validating
 */
function _validateField(
  input,
  validationIndex,
  errorFormatIndex,
  emptyValidation,
  nextSibOK
  )
{
  var isNN = _agent.isNav;

  // don't validate under Netscape if tabbing to the next sibling is OK
  if (isNN && nextSibOK)
  {
    return;
  }

  // Bug #2205664 (also 2210697, and 2465351): Handling these validations is
  // fraught with peril on Netscape. The problem is that onBlur events stack
  // up. The user leaves the text field, an onBlur fires, we raise an alert
  // causing another onBlur to queue up, we finish handling the first,
  // return, and immediately get the second.
  // Bug 2465351 also mentions Mozilla, so it's been added for completeness.
  if (isNN || _agent.isMac || _agent.isGecko)
  {
    if (_recentValidation(false))
      return;
  }

  // determine whether we need to validate the field
  var doValidate = emptyValidation || (_getValue(input) != "");

  // We only validate if we aren't in the middle of validation in order
  // to avoid infinite lopps caused by the fact that the focus has already
  // moved to a new field when the validation fires, and if the validation
  // on the new field failed also, we would ping-pong between the fields
  // forever
  if (doValidate && !window._validating && _focusChanging())
  {
    if (nextSibOK)
    {
      var activeElement = window.document.activeElement;

      if (activeElement)
      {
        var parent = input.parentElement;

        if (parent == activeElement.parentElement)
        {
          var children = parent.children;

          for (var i = 0; i < children.length; i++)
          {
            if (input == children[i])
            {
              doValidate = (activeElement != _getNextNonCommentSibling(parent, i));
            }
          }
        }
      }
    }

    if (doValidate)
    {
      var validationError = _getValidationError(input, validationIndex);

      if (validationError)
      {
        // mark that we are in the middle of validation
        window._validating = input;

        input.select();

        // move the focus back to the failed field before showing the alert
        // to grab the user's attention about what failed.  We don't
        // do this for netscape because doing so in Netscape will cause
        // an infinite loop because Netscape appears to keep the onFocus
        // caused by moving the focus to the field and the onBlur
        // caused by the display of the alert queued up.  This hoses
        // us because it causes the window._validating flag to be cleared
        // and another round of validation to occur
        // ColorField has required validation on hidden field,
        // but cannot receive focus
        if (!isNN && (input.type != 'hidden'))
        {
          input.focus();

          // See if there's a specific position at which validation
          // failed;  if there is, we'll try to select everything after it.
          if (window["_failedPos"] != (void 0))
          {
            // IE style - createTextRange()
            if (input.createTextRange)
            {
              var rng = input.createTextRange();
              rng.moveStart("character", window["_failedPos"]);
              rng.select();
            }
            // Mozilla style.  Sadly, this won't work for TEXTAREA,
            // because of a Mozilla bug.
            else if (input.selectionStart != (void 0))
            {
              input.selectionStart = window["_failedPos"];
            }

            window["_failedPos"] = (void 0);
          }
        }

        // get the error String, if any
        var errorString = _getErrorString(input, errorFormatIndex,
                                          validationError);

        if (errorString)
        {
          // show the error and note the time we finished this validation.
          _validationAlert(errorString);
        }

        // move the focus back to the field after showing the alert for
        // Netscape
        // ColorField has required validation on hidden field,
        // but cannot receive focus.
        if (isNN && (input.type != 'hidden'))
        {
          input.focus();
        }
      }
    }
  }
}


/**
 * Field unvalidation function.
 *
 * If the input object is the currently validating object, reset the
 * validating object so that field validations can continue.  This method
 * is called from the onFocus handler of validating objects.
 */
function _unvalidateField(
  input
  )
{
  if (window._validating == input)
  {
    window._validating = void 0;
  }
}


/**
 * Attempts to submits the form, potentially firing validation and notifying
 * any Cabo onSubmit handlers registered on the form, returning
 * <code>true</code> if the submission actually occurred.
 * <p>
 * If the <code>doValidate</code> parameter is false, no validation will
 * be performed, and the form is guaranteed to be submitted.  Otherwise,
 * the form will be submitted if both the validation succeeds and any
 * registered Cabo onSubmit handlers do not return <code>false</code>.
 * <p>
 * @param form The form to submit.  This can either be the name of the form
 *             in the current <code>document</code>, the index of the form
 *             in the current <code>document</code> or the form itself.
 * @param doValidate boolean value specifying whether validation should
 *   occur before the form is submitted.  (As per a common Javascript
 *   idiom, it is acceptable to pass true/false as well as 0/1).  If
 *   this parameter is ommitted, it defaults to true.
 * @param parameters a single Javascript object that specifies
 *   all the additional key-value pairs to submit.  There must be
 *   pre-existing &lt;input type="hidden"&gt; elements as targets
 *   for each of these parameters.
 */
function submitForm(
  form,
  doValidate,
  parameters
  )
{
  // if the form was passed as a form name, get the form object
  if ((typeof form) == "string")
  {
    form = document[form];
  }
  // if the form was passed as a form index, get the form object
  else if ((typeof form) == "number")
  {
    form = document.forms[form];
  }

  // we had better have a form now
  if (!form)
    return false;

  // Check to see if submitForm is called before the form
  // has been rendered completely. If so, save the parameters
  // and return. At the end of the form, we always call _submitFormCheck
  // which will re-call submitForm if it had been rejected.
  // This is for bug 2752257.

  // Bug #3058770: In LovInput, we have to hack up a form for NLS. It's not
  // validated, so there is no real validater, we've just hacked one. The
  // submit always sets doValidate to false. Just make sure that you never use
  // this validater if doValidate is false (it might just be the value '1').
  var formComplete = window["_"+ form.name + "Validater"];

  if (formComplete == (void 0))
  {
    _saveForm = form;
    _saveDoValidate = doValidate;
    _saveParameters = parameters;
    if (form.target == "_pprIFrame")
    {
      _inPartialSubmit = true;
    }
    _submitRejected = true;
    // Do not submit the form,
    // since the form hasn't been rendered completely yet.
    return false;
  }
  _submitRejected = false; //just in case, clear it.
  _inPartialSubmit = false; //just in case, clear it.

  // Bug 1789483: ignore a second form submission that happens
  // less than 0.5 seconds after the first one
  var newDate = new Date();
  if (_recentSubmit(newDate))
    return;

  _lastDateSubmitted = newDate;

  //
  // default value for doValidate is true
  if (doValidate == (void 0))
    doValidate = true;

  // assume that we should submit the form
  var doSubmit = true;

  // validate the form if necessary, and don't submit the
  // form if validation fails
  if (doValidate && !_validateForm(form))
    doSubmit = false;

  //
  // If we have an onSubmit handler, call it
  //
  var onSubmit = window["_" + form.name + "_Submit"];

  if (onSubmit != (void 0))
  {
    // create function so that "return" is handled correctly,
    var func = new Function("doValidate", onSubmit);

    // install the function on the object so that "this" is
    // handled correctly
    form._tempFunc = func;

    // call the submit handler with the doValidate flag,
    var handlerResult = form._tempFunc(doValidate);

    // uninstall the temporary function
    form._tempFunc = (void 0);

    // if we're validating and the handler returns false,
    // don't submit the form
    if (doValidate && (handlerResult == false))
    {
      doSubmit = false;
    }
  }

  if (doSubmit)
  {
    // reset any hidden form values before submitting
    _resetHiddenValues(form);

    //
    // assign any dynamic values before submitting
    //
    var isDOM = _supportsDOM();
    var tempParams = new Object();

    if (parameters)
    {
      for (var paramName in parameters)
      {
        var paramValue = parameters[paramName];

        if (paramValue != (void 0))
        {
          var hiddenField = form[paramName];

          if (hiddenField)
          {
            hiddenField.value = paramValue;
          }
          else if (isDOM)
          {
            // as a convenience to the client, build a hidden field to hold
            // this parameter.
            var tmpField = document.createElement("input");
            tmpField.type = "hidden";
            tmpField.name = paramName;
            tmpField.value = parameters[paramName];
            form.appendChild(tmpField);
            tempParams[paramName] = tmpField;
          }

        }
      }
    }

    // finally submit the form
    form.submit();

    // And cleanup any temporary fields we provided for the client.
    // ...probably don't need to do this, but...
    for (var paramName in tempParams)
      form.removeChild(tempParams[paramName]);
  }

  return doSubmit;
}

/**
 * Checks if _submitForm had been called before the form had completely
 * rendered, and if so, recall it. This function is rendered at the end of the
 * form, so it is guaranteed that the form is complete when this is called.
 */
function _submitFormCheck()
{

  if (_submitRejected)
  {
    if (_inPartialSubmit)
    {
      _submitPartialChange(_saveForm, _saveDoValidate, _saveParameters);
      _inPartialSubmit = false;
    }
    else
    {
      submitForm(_saveForm, _saveDoValidate, _saveParameters);
    }
    _saveForm = null;
    _saveDoValidate = null;
    _saveParameters = null;
  }
}

/**
 * Attempts to reset the form, calling
 * any reset function calls registered on the form.
 * The form will be reloaded if any
 * reset function call returns <code>true</code>.
 * This function returns <code>true</code> if the page
 * had to be reloaded, and false otherwise.
 * <p>
 * @param form The form to submit.  This can either be the name of the form
 *             in the current <code>document</code>, the index of the form
 *             in the current <code>document</code> or the form itself.
 */
function resetForm(
  form
  )
{
  var doReload = false;

  // if the form was passed as a form name, get the form object
  if ((typeof form) == "string")
  {
    form = document[form];
  }
  // if the form was passed as a form index, get the form object
  else if ((typeof form) == "number")
  {
    form = document.forms[form];
  }

  // we had better have a form now
  if (!form)
    return false;

  var resetCallbacks= window[ "_" + form.name + "_Reset"];

  if (resetCallbacks && !doReload)
  {
    for (var i = 0; i < resetCallbacks.length; i++)
    {
      var trueResetCallback = unescape(resetCallbacks[i]);

      doReload = (eval(trueResetCallback));

    }
  }


  if ( doReload )
  {
    window.document.location.reload();
  }
  else
  {
    form.reset();
  }
  _lastDateReset = new Date();
  return doReload;
}



/**
 * Resets any Marlin-generated hidden values on the form passed in.
 * All hidden form values generated by Marlin on this form are set to
 * the empty string.
 */
function _resetHiddenValues(
  form
  )
{
  var resetFields = window["_reset" + form.name + "Names"];

  if (resetFields)
  {
    for (var i = 0; i < resetFields.length; i++)
    {
      var currField = form[resetFields[i]];

      if (currField)
      {
        currField.value = '';
      }
    }
  }
}


/**
 * Returns the value of a form element.
 */
function _getValue(formElement)
{
  var elementType = formElement.type

  if (!elementType && formElement.length)
  {
    elementType = formElement[0].type;
  }

  if (elementType == "checkbox")
  {
    return formElement.checked;
  }
  else if (elementType.substring(0,6) == "select")
  {
    var selectedIndex = formElement.selectedIndex;

    // selectedIndex exists and non-negative
    if (selectedIndex != (void 0) &&
        selectedIndex != null &&
        selectedIndex >= 0)
    {
      var opt = formElement.options[selectedIndex];
      var value = opt.value;
      if (!value)
      {
        // If there's no value, it could be for two reasons:
        //  (1) The user has only specified "text".
        //  (2) The user explicitly wanted "value" to be empty.
        // We can't really tell the difference between the two,
        // unless we assume that users will be consistent with
        // all options of a choice.  So, if _any_ option
        // has a value, assume this one was possibility (2)
        for (var i = 0; i < formElement.options.length; i++)
        {
          if (formElement.options[i].value)
            return value;
        }

        // OK, none had a value set - this is option (1) - default
        // the "value" to the "text"
        return opt.text;
      }

      return value;
    }

    // no selected value
    return "";
  }
  else if (elementType == "radio")
  {
    if (formElement.length)
    {
      for (var i = 0; i < formElement.length; i++)
      {
        if (formElement[i].checked)
        {
          return formElement[i].value;
        }
      }
    }
    else
    {
      if (formElement.checked)
      {
        return formElement.value;
      }
    }

    // no selected value
    return "";
  }
  else
  {
    return formElement.value;
  }
}


/**
 * Calls an array of validation functions and returns a single error
 * String.
 */
function _multiValidate(
  form,
  validators
  )
{
  var failures = "";

  // We check for any revent validation failures here, not just validations.
  // If a validation has been run on one field in the form (e.g. an onBlur), we
  // still need to run every other validation. However, if that one validation
  // failed, the user has seen one alert, don't bug them with a second til they
  // have fixed the first error.
  if (validators && !_recentValidation(true))
  {
    // get the list of different validations
    var validations = _getValidations(form);

    if (validations)
    {
      var firstFailure = true;

      // loop through the validations, building up the error string
      for (var i = 0; i < validators.length; i += 4)
      {
        // get the current form element to validate
        var currInput = form[validators[i + 1]];

        // if currInput is an array then multiple elements have the same name.
        // Only the first will be validated as subsequent values should be in sync
        var elementType = currInput.type;

        if (!elementType && currInput.length)
        {
          if (currInput[0].type != "radio")
          {
            currInput = currInput[0];
          }
        }

        // determine if we should skip validation if the input value is
        // empty
        var skipIfEmpty = validators[i + 3];

        var value = _getValue(currInput);

        // do the validation if either there isn't an additional
        // required check on this element, or this element has a value
        if (!(skipIfEmpty && (value == "")))
        {
          // evaluate the current submit validation
          var validationError = _getValidationError(currInput, validators[i],
                                                    validations);
          if (validationError)
          {
            // move the focus back to the first failed field
            if (firstFailure)
            {
              // ColorField has required validation on hidden field,
              // but cannot receive focus.
              if (currInput.focus && (currInput.type != 'hidden'))
                currInput.focus();

            if (currInput.type == "text")
                currInput.select();
              firstFailure = false;
            }

             // get the formatted error string for the current input and
             // formatIndex
            var errorString = _getErrorString(currInput,
                                              validators[i + 2],
                                              validationError);

            failures += '\n' + errorString;
          }
        }
      }
    }
    _recordValidation((failures.length > 0), 0);
  }

  return failures;
}


/**
 * Returns the id of an input element on either IE or Netscape, dealing
 * with the fact that Netscape doesn't support IDs locally.
 */
 function _getID(
   input
   )
 {
   if (!_agent.isNav)
   {
     // for non-Netscape return the ID directly
     var id = input.id;

     var inputType = input.type;

     if (!inputType && input.length)
       inputType = input[0].type;

     // for radio buttons, return ID of enclosing <span>
     if (inputType == "radio")
     {
       var inputParent;

       if (input.length)
       {
         inputParent = input[0].parentNode;
       }
       else
       {
         inputParent = input.parentNode;
       }

       id = inputParent.id;
     }

     return id;
   }
   else
   {
     var form = _getForm(input);
     // for loser Netscape, use table lookup
     var nameToID = window["_" + form.name + "_NameToID"];

     if (nameToID)
     {
       var name = _getName(input);
       return nameToID[name];
     }
   }
 }


/**
 * Returns the form of an input element on either IE or Netscape, dealing
 * with the fact that radio inputs do not directly support the form attribute.
 */
 function _getForm(
   input
   )
 {
   var form = input.form;

   if (form == (void 0))
   {
     var inputType = input.type;

     if (!inputType && input.length)
       inputType = input[0].type;

     // for radio buttons, return ID of enclosing <span>
     if (inputType == "radio" && input.length)
     {
       form = input[0].form;
     }
   }

   return form;
 }

/**
 * Returns the name of an input element on either IE or Netscape, dealing
 * with the fact that radio inputs do not directly support the name attribute.
 */
 function _getName(
   input
   )
 {
   var name = input.name;

   if (name == (void 0))
   {
     var inputType = input.type;

     if (!inputType && input.length)
       inputType = input[0].type;

     // for radio buttons, return ID of enclosing <span>
     if (inputType == "radio" && input.length)
     {
       name = input[0].name;
     }
   }

   return name;
 }

/**
 * Return true if the object or any of its prototypes'
 * are an instance of the specified object type.
 */
function _instanceof(
  obj,  // the object instance
  type  // the constructor function
)
{
  if (type == (void 0))
    return false;

  while (typeof(obj) == "object")
  {
    if (obj.constructor == type)
      return true;

    // walk up the prototype hierarchy
    obj = obj.prototype;
  }

  return false;
}

/**
 * Return the formatted error string for an input field
 * and an errorFormatIndex
 */
function _getErrorString(
  input,
  errorFormatIndex,
  validationError
  )
{
  var errorFormat;

  var form = _getForm(input);
  var value = _getValue(input);

  // use the message embedded in the validationError, if any
  if (_instanceof(validationError, window["ParseException"]))
  {
    errorFormat = validationError.parseString;
  }
  else
  {
    // get the list of different error formats
    var errorFormats = window["_" + form.name + "_Formats"];

    if (errorFormats)
    {
      // get the appropriate error format
      errorFormat = errorFormats[errorFormatIndex];
    }
  }

  if (errorFormat)
  {
    // get the mapping of id's to labels
    var labelMap = window["_" + form.name + "_Labels"];

    var label;

    // get the label for this input element, if one has been
    // associated using the ID of the input element
    if (labelMap)
    {
      label = labelMap[_getID(input)];
    }

    // get the mapping of id's to patterns
    var patternMap = window["_" + form.name + "_Patterns"];

    var pattern;

    // get the pattern for this input element, if one has been
    // associated using the ID of the input element
    if (patternMap)
    {
      pattern = patternMap[_getID(input)];
    }

    // format the error string, replacing the following tokens
    //   %value%    the value of the input element
    //   %label%    the value of the label
    //   %pattern%  the validation pattern
    var errorString = _formatErrorString(errorFormat,
                                         {
                                           "value":value,
                                           "label":label,
                                           "pattern":pattern
                                         });
    // return the error
    return errorString;
  }
}


/**
 * Returns the array of validation information used to validate the form at
 * submission time.
 */
function _getValidations(
  form
  )
{
  return window["_" + form.name + "_Validations"];
}


/**
 * Perform the error validation and return true if there is an error.
 */
function _getValidationError(
  input,           // form element to be validated
  validationIndex, // index of validation code
  validations      // the validations array, if available
  )
{
  if (!validations)
  {
    // get the list of different validations
    validations = _getValidations(input.form);
  }

  if (validations)
  {
    var validator = validations[validationIndex];

    if (validator)
    {
      // get the true validator by replacing any value token with
      // the value of the input
      var trueValidator = validator.replace(/%value%/g, "_getValue(input)");

      // return true/ParseException if a validation error has occurred
      return (eval(trueValidator));
    }
  }

  // no error
  return (void 0);
}


/**
 * Performs token replacement on the the error format, replacing each
 * token found in the token Object with the value for that token.
 */
function _formatErrorString(
  errorFormat, // error format string with embedded tokens to be replaced
  tokens       // tokens Object containin token names and values to be replaced
  )
{
  var currString = errorFormat;

  // loop through all of the tokens, replacing them if they are present
  for (var currToken in tokens)
  {
    var currValue = tokens[currToken];

    // if the token has no value, replace it with the empty string
    if (!currValue)
    {
      currValue = "";
    }

    // the tokens are delimited by '%' before and after the token
    var currRegExp = "%" + currToken + "%";

    // support tokens of the form {token} as well as %token%
    currString = currString.replace(new RegExp('{' + currToken + '}', 'g'),
                                    currRegExp);

    // Replace the token.  Don't use String.replace, as the value may
    // include dollar signs, which leads Netscape astray (bug 2242675)
    var indexOf = currString.indexOf(currRegExp);
    if (indexOf >= 0)
    {
      currString = currString.substring(0, indexOf) +
                   currValue +
                   currString.substring(indexOf + currRegExp.length);
    }
  }

  return currString;
}

/**
 * Chain two functions together returning whether the default
 * event handling should occur
 */
function _chain(
  evh1,        // event handler 1 string
  evh2,        // event handler 2 string
  target,      // target of event
  event,       // the fired event
  shortCircuit // shortcircuit if handler 1 false
  )
{
  var result1 = _callChained(evh1, target, event);

  if ( shortCircuit && (result1 == false))
    return false;

  var result2 = _callChained(evh2, target, event);

  // since undefined results should be evaluated as true,
  // return false only if either result1 or result2 return false
  return !((result1 == false) || (result2 == false));
}

function _callChained(
  handler,
  target,
  event
  )
{
  if (handler && (handler.length > 0))
  {
    // handle ie case, where we have no event parameter
    if (event == (void 0))
    {
      event = target.window.event;
    }

    // create function so that "return" is handled correctly,
    // use event parameter so that both ie and netscape
    // functions work
    var func = new Function("event", handler);

    // install the function on the object so that "this" is
    // handled correctly
    target._tempFunc = func;

    // evaluate the result
    var result = target._tempFunc(event);

    // clear the temporary function
    target._tempFunc = (void 0);

    // undefined results should be evaluated as true,
    return !(result == false);
  }
  else
  {
    return true;
  }
}

// Enforce the maximum length of a form element
function _checkLength(formElement, length, event)
{
  elementLength = formElement.value.length;
  if (elementLength > length)
  {
    formElement.value = formElement.value.substr(0, length);
    return false;
  }

  if (elementLength < length)
    return true;

  if ((elementLength == length) && (event.type == 'change'))
    return true;

  if (event && (event.which < 32))
     return true;

  return false;
}


/**
 * Cover for document.getElementById that works on IE 4.x
 */
function _getElementById(
  doc,
  id
  )
{
  //
  // If we arent' on Internet Explorers before 5,
  // use the DOM way of doing this
  //
  if ((_agent.kind != "ie") || (_agent.version >= 5))
  {
    var element = doc.getElementById(id);

    // IE's implementation of getElementById() is buggy.  If
    // the page contains an anchor which has the same name
    // as the requested id, IE will return the anchor, even
    // if the anchor's id attribute is not set.  So, make
    // sure that we actually get back an element with the
    // correct id.
    if ((element == null) || (element.id == id))
      return element;

    // If we get here, that means that IE has probably returned
    // an anchor instead of the desired element.  Let's scan
    // the entire DOM tree to find the element we want.
    return _findElementById(doc, id);
  }
  else
  {
    return doc.all[id];
  }
}

// A recursive method which searches the entire DOM tree
// to find the element with the specified ID
function _findElementById(
  node,
  id
  )
{
  // Check to see if the current node is the node
  // that we are looking for
  if (node.id == id)
    return node;

  // Check all children of the current node
  if (node.childNodes)
  {
    var childNodes = node.childNodes;
    for (var i = 0; i < childNodes.length; i++)
    {
      var foundNode = _findElementById(childNodes.item(i), id);
      if (foundNode != null)
        return foundNode;
    }
  }

  return null;
}

// Returns '?' or '&' depending on whether the
// baseURL already contains a query string
function _getQuerySeparator(baseURL)
{
  var lastChar = baseURL.charAt(baseURL.length - 1);
  if ((lastChar == '&') || (lastChar == '?'))
    return "";

  return (baseURL.indexOf('?') >= 0) ? '&' : '?';
}

/**
 * Adds a parameter to an existing URL, replacing the parameter if
 * it already exists
 */
function _addParameter(
  baseURL,
  paramName,
  paramValue
  )
{
  // check if we have parameters
  var queryIndex = baseURL.indexOf('?');

  if (queryIndex == -1)
  {
    // no parameters, so append to parameters
    return baseURL + '?' + paramName + '=' + paramValue;
  }
  else
  {
    // check if the parameter already exists
    var paramIndex = baseURL.indexOf(paramName + '=', queryIndex + 1);

    if (paramIndex == -1)
    {
      // parameter isn't in the URL
      return baseURL + '&' + paramName + '=' + paramValue;
    }
    else
    {
      //
      // replace the value of the parameter
      //
      var valueIndex = paramIndex + paramName.length + 1;

      // get the URL + the parameter
      var newString = baseURL.substring(0, valueIndex);

      // append the new value
      newString += paramValue;

      var lastIndex = baseURL.indexOf('&', valueIndex);

      if (lastIndex != -1)
      {
        // append the rest of the string after the replaced value
        newString += baseURL.substring(lastIndex, paramName.length);
      }

      return newString;
    }
  }
}

/**
 * Adds a parameter to the parameters object for form submission
 */
function _addFormParameter(
  parameters,
  paramName,
  paramValue
  )
{
  // Always create a new object, since we don't want to mess with
  // the caller's parameters
  var newParameters = new Object();

  // Copy over existing parameters
  if (parameters)
  {
    for (var name in parameters)
      newParameters[name] = parameters[name];
  }

  // Now set the new parameter value
  newParameters[paramName] = paramValue;

  return newParameters;
}

function _firePCUpdateMaster(
  firer,
  tableName,
  baseURL,
  targetID
  )
{
  var selectedVarName = tableName + '_dt';

  // get the name of the selected detail
  var selectedDetailName = window[selectedVarName];

  // we only need to change if these are different
  if (selectedDetailName != firer.id)
  {
    // save our id as the id of the selected detail
    window[selectedVarName] = firer.id;

    // if there is a selected detail, change its icon to unselected
    if (selectedDetailName)
    {
      // get the detail element that is selected
      var selectedDetail = _getElementById(document, selectedDetailName);

      if (selectedDetail)
      {
        // change our icon to selected
        // =-= bts temp hack of URL
        _updateDetailIcon(selectedDetail, '/marlin/cabo/images/cache/c-sdtl.gif');
      }
    }


    // change our icon to selected
    // =-= bts temp hack of URL
    _updateDetailIcon(firer, '/marlin/cabo/images/cache/c-dtl.gif');

    // actual fire the partial change
    _firePartialChange(baseURL, targetID);
  }
}

function _updateDetailIcon(
  sourceElement,
  iconURL
  )
{
  // do this the DOM way
  sourceElement.firstChild.src = iconURL;
}

// Fires a partial change event to the specified URL
function _firePartialChange(baseURL)
{
    var srcURL = _addParameter(baseURL,
                               _getPartialParameter(),
                               "true");

    // For now, we just use a single shared iframe "_pprIFrame".
    var iframe = _getElementById(document, '_pprIFrame');

    // Before we fire, update the request count
    _pprRequestCount++;

    // and block all mouse clicks until the update is done
    _pprStartBlocking(window);

    if (_agent.isIE)
    {
      // iframe.src = srcURL;
      //iframe.ownerDocument.location.replace(srcURL);
      //iframe.document.location.replace(srcURL);

      // ie 5.5 way of doing this
      iframe.contentWindow.location.replace(srcURL);
    }
    else
    {
      // do this a slightly more confusing way for Mozilla
      //dump(iframe.contentDocument.location);

      iframe.contentDocument.location.replace(srcURL);
    }
}

// Fires a partial page request via form submission.
// The args are the same as submitForm().  The
// partialTargets are passed in as parameters
function _submitPartialChange(
  form,
  doValidate,
  parameters
  )
{
  // Get the actual form object
  if ((typeof form) == "string")
    form = document[form];

  if (!form)
    return false;

  // Tack on the "partial" event parameter parameter
  parameters = _addFormParameter(parameters, _getPartialParameter(), "true");

  // We need to set the target of the form to be the PPR iframe.
  // Save the old target and set the new one.
  var oldTarget = form.target;
  form.target = "_pprIFrame";

  // Before we fire, update the request count
  _pprRequestCount++;

  var delta = 0;

  // IE only adds to the history list if something has already been done on
  // this page, so until there has been some action, we don't
  // increment/decrement at all.
  if ((!_agent.isIE) || parent._pprSomeAction)
  {
    delta = 1;
  }
  _pprSubmitCount += delta;

  parent._pprSomeAction = true;

  // block all mouse clicks until the submit is done
  _pprStartBlocking(window);

  // Submit the form
  var submitted = submitForm(form, doValidate, parameters);

  // If the form wasn't actually submitted, update the ref count
  if (!submitted)
  {
    _pprStopBlocking(window);
    _pprRequestCount--;
    _pprSubmitCount -= delta;
  }

  // Reset the form target
  form.target = oldTarget;
}

/**
 * Returns the encoded value of the "partial" event
 * parameter.
 */
function _getPartialParameter()
{
  // Check to see we've got a variable which defines the partial parameter
  if (window._pprPartialParam)
    return window._pprPartialParam;

  return "partial";
}

/**
 * Equivalent of targetItem.outerHTML = sourceItem.outerHTML for platforms
 * that don't support outerHTML, but do support innerHTML, or platforms that
 * don't suppoer outerHTML on all elements
 */
function _setOuterHTML(
  targetDocument,
  targetItem,
  sourceItem
  )
{
  var sourceTagName = sourceItem.tagName;

  if (_agent.isIE)
  {
    var useOuter = true;

    // is this a special IE leaf item that doesn't support outerHTML
    var isLeafItem = ((sourceTagName == "TD")      ||
                      (sourceTagName == "TH")      ||
                      (sourceTagName == "CAPTION"));

    // is this a special IE container item, that doesn't support
    // inner or outerHTML
    var isContainerItem = !isLeafItem &&
                          ((sourceTagName == "COL")      ||
                           (sourceTagName == "COLGROUP") ||
                           (sourceTagName == "TR")       ||
                           (sourceTagName == "TFOOT")    ||
                           (sourceTagName == "THEAD")    ||
                           (sourceTagName == "TBODY"));

    if (isLeafItem || isContainerItem)
    {
      // create an element with the correct attributes
      var newTargetItem = targetDocument.createElement(sourceTagName);
      // mergeAttributes, including the id
      newTargetItem.mergeAttributes(sourceItem, false);

      // copy over the content
      if (isLeafItem)
      {
        newTargetItem.innerHTML = sourceItem.innerHTML;
      }
      else
      {
        if (isContainerItem)
        {
          //
          // add all of the cloned child elements
          // firstChild gets the comments as elements. we will
          // need to filter those out.
          //
          var currCell = sourceItem.firstChild;

          while (currCell != null)
          {
            // add the cloned cell.
            // filter out the comment tag.
            while(currCell != null && currCell.tagName == "!")
            {
              currCell = currCell.nextSibling;
            }

            newTargetItem.appendChild(_setOuterHTML(targetDocument,
                                                    null,
                                                    currCell));


            currCell = currCell.nextSibling;

          }
        }
      }

      // replace the current child with the new child
      if (targetItem)
      {
        targetItem.parentNode.replaceChild(newTargetItem, targetItem);
      }
      else
      {
        targetItem = newTargetItem;
      }

      useOuter = false;
    }

    if (useOuter)
    {
      targetItem.outerHTML = sourceItem.outerHTML;
    }
  }
  else
  {
    // create the new Target item using a namespace to create this item
    // screws up Mozilla!
    var newTargetItem;

    // treat tr differently, since there is a Mozilla bug regarding innerHTML
    // and tr. It seems to strip out all the tds.
    if (sourceTagName != 'TR')
    {
      newTargetItem = targetDocument.createElement(sourceTagName);

      // Mozilla has a bug with copying innerHTML for the select element (#100175)
      // and it also has a problem when we use cloneNode (the scripts that
      // are in the iframe that get eval'd sometimes don't seem to get eval'd
      // correctly, and we get a missing function error). So, this is what
      // we came up with to work around that.
      if (sourceTagName == 'SELECT')
      {
        // the multiple attribute does not get copied over when we copy over
        // the source attributes, so do it here. This way when we set the
        // selected options below, multiple selections will be allowed.
        if (sourceItem.multiple)
        {
          newTargetItem.multiple = sourceItem.multiple;
        }
        for (var i = 0; i< sourceItem.options.length; i++)
        {
          var sourceOption = sourceItem.options[i];
          var newOption = new Option();

          newOption.value = sourceOption.value;
          newOption.text = sourceOption.text;
          newOption.selected = sourceOption.selected;


          newTargetItem.options[i] = newOption;
        }


      }
      else
      {
        // copy over the inner html
        var sourceInnerHTML = sourceItem.innerHTML;

        if ((sourceInnerHTML != null) && (sourceInnerHTML.length > 0))
        {

          newTargetItem.innerHTML = sourceItem.innerHTML;

        }

      }

      //
      // set the attributes of the source node
      //
      var sourceAttrs = sourceItem.attributes;

      for (var i = 0; i < sourceAttrs.length; i++)
      {
        newTargetItem.setAttribute(sourceAttrs[i].name, sourceAttrs[i].value);
      }


    }
    else
    {
      // for tr, use the importNode function. I'm limiting its
      // use to tr in case it is buggy, and to reduce the cases
      // where it is used. It might be a good thing to use in
      // the future. =-=jmw
      newTargetItem = targetDocument.importNode(sourceItem, true);

    }



    // Replace the current child with the new child.
    // We would like replaceChild(), but this causes problems on
    // Mozilla.  For example, in the PPR validation demo, if we use
    // replaceChild(), the validation demo contents disappear!  Using
    // insertBefore() followed by removeChild() seems to work just
    // fine though...
    // targetItem.parentNode.replaceChild(newTargetItem, targetItem);

    targetItem.parentNode.insertBefore(newTargetItem, targetItem);
    targetItem.parentNode.removeChild(targetItem);


  }

  return targetItem;
}

/**
 * Unload event handler for the partial iframe
 */
function _partialUnload()
{
  // We try to detect back events and jump back to the last
  // fully rendered page.  We consider an unload to be the
  // result of a back or refresh event if the _pprRequestCount
  // has not been incremented.  Note: If the parent window has
  // been unloaded (_pprUnloaded), this must be a refresh, so
  // do nothing.
  if ((parent._pprRequestCount <= 0) && !parent._pprUnloaded)
  {
    // This should be a no-op, but just in case...
    _pprStopBlocking(parent);

    if (!(_agent.isIE) && (parent.document.referrer != null))
    {
      // If we have a referrer, it's safer to jump to it than to try to
      // figure out how deep we are in the history list.
      // Although Microsoft claims that history.go takes either an integer or a
      // string, it appears that only the integer argument works.
      parent.history.go(parent.document.referrer);
    }
    else
    {
      var backOffset = -1;

      if (_agent.isIE)
      {
        // I'm sure this will come as a big surprise, but IE's history mechanism
        // is busted. It never seems to grab the first submit unless some other
        // action has taken place...
        if (parent._pprSomeAction)
        {
          backOffset = -(parent._pprSubmitCount);
        }
      }
      else if (parent._pprSubmitCount && (parent._pprSubmitCount > 0))
      {
        // On Mozilla, the submitCount will be accurate
        backOffset -= parent._pprSubmitCount;
      }

      // we're about to jump back, reset the submit (page) count.
      parent._pprSubmitCount = 0;
      parent._pprSomeAction = false;

      // Only jump back if the offset is actually negative
      if (backOffset < 0)
      {
        parent.history.go(backOffset);
      }
    }
  }
}

function _partialRedirect(URL)
{
  // ER #2603173
  // If an error occurred during a partial operation, the client
  // can call the method PartialPageUtils.forceRedirectURL(context, url)
  // with an URL to which we can redirect to show the error.
  if (URL && (parent._pprRequestCount > 0))
  {
    if (((typeof URL) == "string") && (URL.length > 0))
    {
      // We're going to an error page, make sure we clear all
      // tracking variables before the onUnload gets called.
      parent._pprRequestCount--;
      parent._pprSubmitCount = 0;
      parent._pprSomeAction = false;
      parent.location.href = URL;
      _pprStopBlocking(parent);
    }
  }
}


/**
 * function used to store the global variables
 * needed to load the libraries for IE
 */
function _pprLibraryStore(numberOfLibraries)
{
  // keep track of which libraries are loaded
  this.loadedStatus = new Array(numberOfLibraries);

  for (var i=0; i < numberOfLibraries; i++)
    this.loadedStatus[i] = false;

  // keep track of the total number of libraries to load
  this.total = numberOfLibraries;

  // an array to store each library script
  this.allLibraries = new Array(numberOfLibraries);
}
// stores the variables needed to load the libraries for IE
var _pprLibStore;

/**
 * This callback function gets called each time
 * a library is downloaded. This is for IE only.
 * index - an index which indicates which library is being loaded.
 *     s - the library contents.
 * Store the library scripts in an array. When they are all downloaded
 * execute the contents of each library.
 */
function _pprExecScript(index, s)
{
  if (_pprLibStore && _pprLibStore.allLibraries != (void 0))
  {
    _pprLibStore.allLibraries[index] = s;
    _pprLibStore.loadedStatus[index] = true;

    // Check to see if all the libraries are loaded. Start at the end
    // of the array, since those will most likely be the last loaded, so
    // we can return quicker.
    for (var index = _pprLibStore.total-1; index >= 0; index--)
    {
      // well, we see that there is a library not loaded, so return. When
      // they are all loaded, then we'll pass through and execute them.
      if (!_pprLibStore.loadedStatus[index])
        return;
    }

    // all the libraries are loaded, so loop through each and evaluate it.
    for (var i=0; i < _pprLibStore.total; i++)
    {
     parent.window.execScript(_pprLibStore.allLibraries[i]);
    }
    _pprLibStore = null;

  }
}

/**
 * returns an array with the libraries that need to be loaded due to the
 * PPR request and are not already cached.
 */
function _createToLoadArray()
{
  var toLoadArray = new Array();
  var toLoadIndex = 0;

  if (window["_pprLibraries"] != (void 0))
  {
    // loop through each library in _pprLibraries
    // if it is not in the cached libraries list, then
    // add it to an array which indicates that the library needs to be loaded.
    for (var i=0; i < _pprLibraries.length; i++)
    {

      if ((parent._cachedLibs == null)
          || (parent._cachedLibs.indexOf(_pprLibraries[i]) == -1))
      {
        toLoadArray[toLoadIndex++] = _pprLibraries[i];
      }
    }
  }

  return toLoadArray;
}

function _addLibraryToCache(libraryName)
{
  // always load ScriptEval, so don't put it in the _cachedLibs list.
  if ((libraryName.indexOf("ScriptEval")) == -1)
  {
    if (parent._cachedLibs == null)
      //=-=jmw does not work in some cases in IE for some unknown reason
      // parent._cachedLibs = new String(libraryName);
      parent._cachedLibs = "" + libraryName;
    else
      parent._cachedLibs += "," + libraryName;
  }
}

// Given the targetDocument and the array of libraries to load,
// download the libraries using IE's startDownload function.
function _loadScriptLibrariesIE(targetDocument, toLoadArray)
{
  if (toLoadArray == null) return;

  var downloadElement   = _getElementById(targetDocument, "_uixDownload");

  if (downloadElement == null) return;

  var numLibraries = toLoadArray.length;

  // initialize the global variables that are needed to load the libraries
  // in the callback.
  _pprLibStore = new _pprLibraryStore(numLibraries);

  // For each library, start the download and set up the callback
  // which will get called when the download is complete.
  for (var i = 0; i < numLibraries; i++)
  {
    var scriptStatement = "_pprExecScript(" + i + ", s);"
    // create a callback function that takes "s" as an argument
    // (this will be the script's content in string form) and
    // takes the scriptStatement as its body. This callback
    // function will then turn around and call _pprExecScript(i,s),
    // where i is the library index and s is the library string
    var downloadCallback = new Function("s",scriptStatement );
    downloadElement.startDownload(toLoadArray[i], downloadCallback);

    _addLibraryToCache(toLoadArray[i]);

  }
}

/**
 * Download the libraries that are defined in the iframe
 * into the targetDocument. The method for Gecko is
 * to create a script element in the targetDocument for
 * each library.
 */
function _loadScriptLibrariesGecko(targetDocument, toLoadArray)
{
  // we first walk through the list of library paths that are stored in
  // the _pprLibraries variable in the iframe.
  // As we get each path, we create a script element
  // with the library path as the src and add it to the target document.

  var iFrameElement   = _getElementById(targetDocument, "_pprIFrame");

  if (iFrameElement)
  {
    for (var i = 0; (i < toLoadArray.length); i++)
    {
      // first check to see if this library is already loaded.
      // if so, don't bother to load it again. However, always load
      // ScriptEval

       var newScriptElement= targetDocument.createElement("script");
       newScriptElement.setAttribute('src', toLoadArray[i]);

       iFrameElement.parentNode.insertBefore(newScriptElement, iFrameElement);

      _addLibraryToCache(toLoadArray[i]);
    }
  }
}


/**
 * Download the libraries that are defined in the iframe
 * into the targetDocument. The last library will
 * always be ScriptEval.js, which will evaluate the
 * scripts defined in the _pprScripts variable.
 */
function _loadScriptLibraries(targetDocument)
{
  if (window["_pprLibraries"] != (void 0))
  {
    var toLoadArray = _createToLoadArray();

    if (toLoadArray.length > 0)
    {
      if (_agent.isIE)
      {
        _loadScriptLibrariesIE(targetDocument, toLoadArray);
      }
      else
      {
        _loadScriptLibrariesGecko(targetDocument, toLoadArray);
      }
    }
  }
}


/**
 * Onload event handler for the partial iframe
 */
function _partialChange(navigationFormName)
{
  // If we don't have an outstanding request, don't do
  // anything.  Technically we shouldn't see this case,
  // but this might come up if our back button/history
  // handling code fails.
  if (parent._pprRequestCount <= 0)
    return;

  // Update the request count
  parent._pprRequestCount--;
  parent._pprSomeAction = true;

  // We need to do some pre-processing... Fix up links
  if (navigationFormName)
    _fixAllLinks(navigationFormName, parent);

  // use source document, since Mozilla claims that
  // the document object no longer exists
  // when we call write and close at the end
  // of this method
  var sourceDocument = document;
  var targetDocument = parent.document;

  var oldActiveElement = _getParentActiveElement();
  var newActiveElement = null;

  // We want to keep track of whether we are restoring the focus
  // to the actual component which should have the focus, or just
  // to the first focusable component that we can find.
  var isFirstFocusable = false;

  for (var i = 0; i < _pprTargets.length; i++)
  {

    var targetID = _pprTargets[i];

    //
    // copy over the content
    //
    var sourceItem = _getElementById(sourceDocument, targetID);
    var targetItem = _getElementById(targetDocument, targetID);

    if (sourceItem && targetItem)
    {
      // If the old activeElement is a descendent of the node that
      // is being replaced, we'll need to restore the focus after
      // we are done replacing content.
      var restoreFocus = _isDescendent(oldActiveElement, targetItem);

      _setOuterHTML(targetDocument, targetItem, sourceItem);

      // If that activeElement was in the subtree that was just
      // replaced, figure out what the new activeElement should be.
      if ((restoreFocus) && (newActiveElement == null))
      {
        // Make sure we have a current reference to the target item.
        // It seems that the call _setOuterHTML() can cause our
        // targetItem reference to become invalid.  The end result
        // is that we are not able to successfully traverse the
        // subtree under targetItem.  Getting the targetItem from
        // the document after _setOuterHTML() has been called fixes
        // this problem.
        targetItem = _getElementById(targetDocument, targetItem.id);

        // Get the new element to set the focus to
        newActiveElement = _getNewActiveElement(targetDocument,
                                                targetItem,
                                                oldActiveElement);

        if (newActiveElement == null)
        {
          newActiveElement = _getFirstFocusable(targetItem);
          if (newActiveElement != null)
            isFirstFocusable = true;
        }
        // if we're here, we'll take care of resetting the focus,
        // no need for the unblock to do it.
        parent._pprEventElement = null;
      }
    }
  }
  // Load the script libraries into target document
  _loadScriptLibraries(targetDocument);

  _saveScripts(targetDocument);

  // unblock
  _pprStopBlocking(parent);

  // Restore focus to new active element
  var req = _getRequestedFocusNode();
  if (req != null)
    newActiveElement = req;

  _restoreFocus(newActiveElement, isFirstFocusable, targetDocument);
  _setRequestedFocusNode(null, null, false);

  // Make sure form actions are up to date
  _updateFormActions(sourceDocument, targetDocument);

  // clear out the iframe content
// =-=ags For now, we avoid overwriting the iframe content, as the
//        additional write essentially breaks the history list on IE.
//        We should, however, investigate whether there is some way
//        to free up the memory used by the iframe without breaking
//        back button behavior
//  sourceDocument.write();
//  sourceDocument.close();
}

/* If the UIX facility needs to set focus to a particualr node after a PPR
 * update, calling this function saves off the data needed to find that node
 *
 * Args:
 *    doc : The document that the node lives in
 * nodeid : The id attached to the desired node
 *   next : If true, we'll try to focus on the node following the one above,
 *          otherwise, we'll try to focus on the requested node.
 */
function _setRequestedFocusNode(doc, nodeid, next)
{
  // we only allow one outstanding focus request
  top._UixFocusRequestDoc = doc;
  top._UixFocusRequestID = nodeid;
  top._UixFocusRequestNext = (next == true);
}


/* If a request was made to focus on a particular node, this function will
 * attempt to get that node.
 */
function _getRequestedFocusNode()
{
  if ((top._UixFocusRequestDoc != null)
      && (top._UixFocusRequestID != null))
  {
    var element = _getElementById(top._UixFocusRequestDoc,
                                  top._UixFocusRequestID);
    if (!element)
      return null;

    if (top._UixFocusRequestNext)
    {
      // If "next" was set, the caller doesn't want this node, but the next
      // one. Try to find something that'll accept focus.
      for (var next = element.nextSibling;
           next != null;
           next = next.nextSibling)
      {
        if (_isFocusable(next)
            // we actually DO want to "tab" to links
            || ((_agent.isIE) && (next.nodeName.toLowerCase() == 'a')))
        {
          element = next;
          break;
        }
      }
    }
    return element;
  }
  return null;
}

/**
 * Onload event handler for the partial iframe for full page response.
 * Takes two parameters: the onload and onunload handlers for the
 * full page.
 */
function _fullChange()
{
  // Update the request count
  if (parent._pprRequestCount > 0)
  {

    parent._pprRequestCount--;

    // Before we get the contents, insert a script which disables
    // document.write().  That way, when we write out the contents
    // to the parent window, we won't duplicate calls to document.write()
    // that have already been performed.
    var disableWriteItem = _getElementById(document, "_pprDisableWrite");
    disableWriteItem.text = "var _pprDocumentWrite = document.write;" +
                            "var _pprDocumentWriteln = document.writeln;" +
                            "document.write = new Function('return;');" +
                            "document.writeln = new Function('return;');";

    // Insert a script which re-enables document.write() after we've
    // finished writing out the contents to the parent window.
    var enableWriteItem = _getElementById(document, "_pprEnableWrite");
    enableWriteItem.text = "document.write = _pprDocumentWrite;" +
                           "document.writeln = _pprDocumentWriteln";

    // We also need to replace the onload/onunload event handlers
    // before we get the contents for the parent window.
    var bodyItem = document.body;

    // Save the old event handlers
    var iframeOnload = bodyItem.getAttribute("onload");
    var iframeOnunload = bodyItem.getAttribute("onunload");

    // Set up the new event handlers
    bodyItem.setAttribute("onload",
      _getCommentedScript(document, ("_pprFullOnload")));
    bodyItem.setAttribute("onunload",
      _getCommentedScript(document, ("_pprFullOnunload")));


    // Get the full content of the iframe document.  This should now have
    // the correct uncommented version of our partial page scripts, plus
    // scripts to disable and re-enable document.write().
    var content = _getDocumentContent();

    // We do not want the _fullChange script to be called again.
    // It will cause a javascript error if it is. Therefore,
    // we need to clear it out.
    var pattern =
      new RegExp("<script id=_pprFullChange>_fullChange\\(\\)</script>","i");
    content = content.replace(pattern, "");

    // Reset the event handlers
    bodyItem.setAttribute("onload", iframeOnload);
    bodyItem.setAttribute("onunload", iframeOnunload);

    // Write the full content to the parent window

    var targetDocument = parent.document;

    if (_agent.isIE)
    {
       // Explicitly set the charset on the target document
       // before writing contents - see bug 3296281
       var sourceCharset = document.charset;
       targetDocument.open();
       targetDocument.charset = sourceCharset;
    }

    targetDocument.write(content);
    targetDocument.close();

    // the scripts for the page will be loaded onLoad, see _checkLoad

  }
}

// Called by _partialChange() to ensure that the form action
// attributes are update to date.
function _updateFormActions(sourceDocument, targetDocument)
{
  var sourceForms = sourceDocument.forms;

  // Loop through all of the forms in the iframe, checking
  // the action attribute against the corresponding forms
  // in the parent window.  If any action attributes are
  // out of date, set the new action on the form in the
  // parent window.
  for (var i = 0; i < sourceForms.length; i++)
  {
    var sourceForm = sourceForms[i];

    // The partial page response may include forms which
    // have not had their contents updated.  We only care
    // about forms that contain partial targets...
    if (sourceForm.hasChildNodes())
    {
      // Get the source form's name and action
      var sourceName = sourceForm.name;
      var sourceAction = sourceForm.action;

      // Locate the target form
      var targetForm = targetDocument.forms[sourceName];

      if (targetForm)
      {
        var targetAction = targetForm.action;

        // If the target form's action is out of date,
        // set it to the new action.
        if (targetAction != sourceAction)
          targetForm.action = sourceAction;
      }
    }
  }
}

// Called by _partialChange to get the parent window's
// active element.
function _getParentActiveElement()
{
  // This is more complicated then it should be.  We should be
  // able to access the activeElement through parent.activeElement,
  // but when we do that IE gives us a non-readable node - and we
  // we can't get at any properties that we need, like id, parentNode,
  // etc...  This seems to be some sort of security restriction - it
  // is only a problem when we try to access the parent window's
  // active element from a child iframe.  This isn't a problem
  // if we access the local window's activeElement.
  //
  // As usual, we have a totally sick work around.  Since the parent
  // window can get at its own activeElement, we execute a script
  // in the parent window which saves away the active element in
  // a well known variable.  Then, we can access the activeElement
  // and all of its properties through this variable.
  //
  // Of course, none of this actually works on Mozilla, where
  // activeElement is not supported.

  if (parent.document.activeElement)
  {
    _eval(parent, "_saveActiveElement()");
    return parent._pprActiveElement;
  }

  return null;
}

// This function is executed by _getParentActiveElement() to
// save away the activeElement in a well known location where
// it can be accessed by the PPR iframe
function _saveActiveElement()
{
  if (window._pprEventElement)
    window._pprActiveElement = window._pprEventElement;
  else if (document.activeElement)
    window._pprActiveElement = document.activeElement;
  else
    window._pprActiveElement = null;
}


// Finds the node under the specified target item
// to which the focus should be restored.
// If the targetItem contains an element with the same
// id or name as the old activeElement, we set
// the focus to this item.  Otherwise we set the focus
// to the first focusable item.
function _getNewActiveElement(targetDocument, targetItem, oldActiveElement)
{
  // First check to see if we have a new element with the same
  // ID as the old active element.
  if (oldActiveElement.id)
  {
    var newActiveElement = _getElementById(targetDocument,
                                           oldActiveElement.id);
    if (_isFocusable(newActiveElement))
      return newActiveElement;
  }

  // =-=ags We should look for nodes with the same name

  return null;
}

// Returns the first focusable node under the specified node
function _getFirstFocusable(node)
{
  if ((node == null) || _isFocusable(node))
    return node;

  if (node.hasChildNodes)
  {
    var children = node.childNodes;
    for (var i = 0; i < children.length; i++)
    {
      var child = children[i];
      var firstFocusable = _getFirstFocusable(child);
      if (firstFocusable != null)
        return firstFocusable;
    }
  }

  return null;
}

// Restores the focus to the specified node
function _restoreFocus(node, isFirstFocusable, doc)
{
  if (node == null)
    return;

  // If we are in a scrolled DIV, restoring the focus to the
  // first focusable node may cause the DIV to scroll back to 0,0.
  // So, for now we just avoid restoring the focus in this situation.
  // In the future we should see less cases where scrolling occurs,
  // since we should do a better job locating the correct node to
  // receive the focus.
  var divNode = _getAncestorByName(node, "DIV");
  if (!divNode)
  {
    _pprFocus(node, doc);
  }
  else
  {
    var scrollTop = divNode.scrollTop;
    var scrollLeft = divNode.scrollLeft;

    // If we aren't scrolled at all, or if we are restoring the
    // focus to the correct focusable owner (and not just the
    // first focusable node), then restore the focus.  Otherwise,
    // we do nothing, in order to avoid unnecessary scrolling.
    if (((scrollTop == 0) && (scrollLeft == 0)) || !isFirstFocusable)
    {
      _pprFocus(node, doc);
    }
  }

  // Bug #2753958: IE doesn't seem to want to re-set the focus when we're
  // done with a PPR update if the input element happens to be enclosed
  // within a table. However, if we make a second request, the focus is set
  // correctly. This is pretty hacky, but it's limited to the one
  // interesting case. We can live with this.
  // =-= Check to see if this case is even needed anymore after
  //     the introduction of _pprFocus.
  if ((_agent.isIE)
      && (node.tagName == 'INPUT')
      && (_getAncestorByName(node, 'TABLE')))
  {
    _pprFocus(node, doc);
  }
}

// Returns an ancestor with the specified name
function _getAncestorByName(
  node,
  ancestorName
  )
{
  ancestorName = ancestorName.toUpperCase();

  while (node)
  {
    if (ancestorName == node.nodeName)
      return node;

    node = node.parentNode;
  }

  return null;
}

// Tests whether one node is a descendent of another
function _isDescendent(
  node,
  ancestorNode
  )
{
  if (node == null)
    return false;

  while (node.parentNode)
  {
    if (node == ancestorNode)
      return true;

    node = node.parentNode;
  }

  return false;
}

// Tests whether the specified node is focusable
function _isFocusable(node)
{
  if (node == null)
    return false;

  var name = node.nodeName.toLowerCase();

  // Links that have a destination are generally focusable
  if (('a' == name) && (node.href))
  {
    // We need to be careful on IE - it seems that
    // IE has problems setting the focus to links
    // which contain a single image.  We see this when
    // IE tries to set the focus to the link around the
    // previous icon in the table.  Actually, this does
    // not seem to be a problem if the link has its
    // id set, so we first check for that.

    // If we're not on IE, or if the link has an id,
    // the link should be focusable
    if (!_agent.isIE || (node.id))
      return true;

    // If we're on IE, we only consider the link to be
    // focusable if it has something other than a single
    // image for its contents.
    var children = node.childNodes;
    if ((children) && (children.length == 1))
    {
      var childName = children[0].nodeName;
      if ('img' == childName.toLowerCase())
        return false;
    }

    return true;
  }

  // Blow off any disabled elements
  if (node.disabled)
    return false;

  // Input elements are also usually focusable
  if ('input' == name)
  {
    // But don't set the focus to hidden fields
    return (node.type != 'hidden');
  }

  // Catch everything else here...
  return (('select' == name) ||
          ('button' == name) ||
          ('textarea' == name));
}

// The partial page response includes a single script element
// which contains all of the code that needs to be executed when
// the partial page changes are applied.  Oddly enough, the code
// is actually commented out to prevent execution within the
// iframe context.  We need to pull the code out from the comment
// braces (/* */) and execute the code in the parent window's
// context.  This function returns the uncommented script code
// that needs to be executed in the parent window.
function _getCommentedScript(doc, id)
{
  var scriptItem = _getElementById(doc, id);

  if (scriptItem != null)
  {
    var scriptContents = scriptItem.text;

    // Strip off the start/end comments
    var start = 0;
    var end = scriptContents.length - 1;

    while (start < end)
    {
      if (scriptContents.charAt(start) == '*')
        break;

      start++;
    }

    while (end > start)
    {
      if (scriptContents.charAt(end) == '*')
        break;

      end--;
    }

    // Rip out the actual script contents from within the comments
    return scriptContents.substring(start + 1, end);
  }

  return null;
}

// Evaluates the specified code in the target window
function _eval(targetWindow, code)
{
  if (code == null)
    return;

  // For IE, we use window.execScript().  For Mozilla, we use
  // window.eval().  It would be nice if we could use eval() on
  // IE too, but IE's implementation of eval() always executes
  // the script in the current context, even if some other
  // window is specified.
  if (_agent.isIE)
    targetWindow.execScript(code);
  else
    targetWindow.eval(code);
}

// Returns the full content (outer HTML) of the current document
function _getDocumentContent()
{
  // Would be nice if we could also include the doctype!

  if (_agent.isIE)
    return document.documentElement.outerHTML;

  // We use innerHTML to get the content on Mozilla, so we
  // need to build up the <html> element manually
  var content = "<html"

  // We need to explicitly add in any attrs
  var attrs = document.documentElement.attributes;
  for (var i = 0; i < attrs.length; i++)
  {
    content += " ";
    content += attrs[i].name;
    content += "=\""
    content += attrs[i].value;
    content += "\"";
  }

  content += ">";

  // Add in the inner content
  content += document.documentElement.innerHTML;

  // Close up the content
  content += "</html>";

  return content;
}

//
// Transform all links on a page to (conditionally)
// perform a form submit (using the given form).
// The targetWindow is the window in which the links
// will appear.  If this is called in response to a
// partial page update, the targetWindow is the
// parent window - and not the current (iframe's)
// window.
//
function _fixAllLinks(formName, targetWindow, exclude)
{
  // Save off the initial state of this form.
  _initialFormState = _getFormState(formName, exclude);
  _initialFormStateName = formName;
  if (exclude != (void 0))
    _initialFormExclude = exclude;

  // If we are collecting state for a different window,
  // copy the state over to the target window
  if (window != targetWindow)
  {
    if (targetWindow._initialFormState == null)
      targetWindow._initialFormState = new Object();

    var sourceWindowState = _initialFormState;
    var targetWindowState = targetWindow._initialFormState;

    for (var key in sourceWindowState)
      targetWindowState[key] = sourceWindowState[key];
  }

  var links = document.links;
  var currHrefWithAnchor = targetWindow.location.href + '#';
  var iframeWithAnchor = location.href + '#';

  // Look through all of the links
  for (var i = 0; i < links.length; i++)
  {
    var href = links[i].href;
    // If the link is actually pointing to anchor on this page, or
    // to javascript: or mailto:, don't mess with it!
    // Bug #3369822: A destination of '#' in the iFrame comes back as the full
    // submitted URL, so have to check against it too.
    if (!href
        || (href.substr(0, iframeWithAnchor.length) == iframeWithAnchor)
        || (href.substr(0, currHrefWithAnchor.length) == currHrefWithAnchor)
        || (href.substr(0, 11).toLowerCase() == "javascript:")
        || (href.substr(0, 7).toLowerCase() == "mailto:")
        || (href.indexOf("_noSv=M") >= 0))
    {
      continue;
    }

    // Similarly, any link with a target frame should be ignored
    if (links[i].target)
    {
      continue;
    }

    // Encode single-quotes correctly
    var hrefArr = href.split("'");
    href = hrefArr[0];
    for (var j = 1; j < hrefArr.length; j++)
      href = href + "\\'" + hrefArr[j];

    // Bug 2807885: bizarrely, setting "href" actually decodes
    // the href - at least it does on Mozilla and IE.  (It doesn't
    // on Netscape 4.)
    if (!_agent.isNav)
      href = escape(href);

    links[i].href = "javascript:_submitNav('" + formName + "','" + href + "')";
  }
}


//
// Store the current value of all elements in a form.
// Ignores all hidden fields.
//
function _getFormState(formName, exclude)
{
  var state = new Object();
  var frm = document[formName];

  for (var i = 0; i < frm.length; i++)
  {
    var name = frm.elements[i].name;
    if (name)
    {
      var element = frm[name];
      if (element)
      {
        if ((exclude != (void 0)) && (exclude[name] != (void 0)))
          continue;

        // Skip over hidden values
        if (!element.type || (element.type != 'hidden'))
          state[name] = _getValue(element);
      }
    }
  }

  return state;
}

/**
 * Determines if the "navigation form" is dirty, returning true if
 * the form is dirty.  This function can only be called if
 * the "navigationFormName" attribute was set on the UIX body component.
 *
 * @return true if the navigation form is dirty
 */
function isNavDirty()
{
  var dirty    = false;

  if (_navDirty)
    dirty = true;
  else
  {
    // Get the current state of the form
    var newState = _getFormState(_initialFormStateName, _initialFormExclude);
    // Compare it to the old state
    for (var key in newState)
    {
      if (newState[key] != _initialFormState[key])
      {
        dirty = true;
        break;
      }
    }
  }

  return dirty;
}


//
// Add a name to the list of excluded names
//
function _addNavExclude(exclude)
{
  if (_initialFormExclude != (void 0))
    _initialFormExclude[exclude] = 1;
  else
  {
    _initialFormExclude = new Object();
    _initialFormExclude[exclude] = 1;
  }
}

//
// Submit a form navigation.  Note that we only bother submitting if
// the form is "dirty"; this relies on fixAllLinks() having been called
// to store the state of that form.
//
//
function _submitNav(formName, uri)
{
  // The form is "dirty": send a "navigate" event, and store
  // off the URL that we would have navigated to
  if (isNavDirty())
  {
    var onNav = window["_onNavigate"];
    if ((onNav == (void 0)) || !(onNav(formName, uri) == false))
      submitForm(formName, 0, {'event':'navigate','uri':uri});
  }
  // Nope, not dirty: just navigate to the page
  else
    document.location.href = uri;
}


/**
 * Called to identify the input field from an event
 * This is called not only below, but also from LovInput.js.
 */
function _getInputField(event)
{
  var input = (void 0);
  var src = (void 0);

  if (window.event)
  {
    kc = window.event.keyCode;
    src = window.event.srcElement;
  }
  else if (event)
  {
    kc = event.which;
    src = event.target;
  }

  if (src != (void 0)
      && (src.tagName == "INPUT" ||
          src.tagName == "TEXTAREA" ))
    input = src;

  return input;
}

/**
 * Called when a field receives focus.
 * Prepares for a later reset of this field by saving its current value.
 */
function _enterField(
  event
  )
{
  var input;
  var src;
  var retv = true;

  var input = _getInputField(event);

  if (input != (void 0))
  {
    input.form._mayResetByInput = false;

    if (input != window._validating)
    {
      // save the last valid value for later restoration
      input._validValue = input.value;
    }
    retv = false;
  }

  return retv;
}

/**
 * Resets the form input to its last valid value.
 * This function is called from the onKeyDown for a form input.
 * If called twice in succession for the same form, with the
 * escape keycode both times, this function will reset the form.
 */
function _resetOnEscape(event)
{
  var kc;
  var input = _getInputField(event);

  if (input != (void 0))
  {
    var form = input.form;

    if (kc == 27)  // escape keycode
    {
      // reset the form input to its last valid value
      // providing there is no selection (consistent with IE)

      var hasSelection = false;

      if ((input.selectionStart != (void 0)) &&
          (input.selectionEnd   != (void 0)))
      {
        hasSelection = (input.selectionStart != input.selectionEnd);
      }
      else if (document.selection)
      {
        hasSelection = (document.selection.createRange().text.length != 0);
      }

      if (!hasSelection)
      {
        // always reset the field
        input.value = input._validValue;

        // determine if a full form reset is required
        if (form._mayResetByInput == true)
        {
          // reset the form
          // unset the flag for form reset
          form.reset();
          form._mayResetByInput = false;
        }
        else
        {
          // set the flag for form reset
          form._mayResetByInput = true;
        }
      }

      // consume this event to prevent any browser behavior from kicking in
      return false;
    }
    else // any keycode other than escape
    {
      // unset the flag for form reset
      // since some other key was pressed
      form._mayResetByInput = false;
    }
  }
  return true;
}


/**
 * Called by the load handler of each document body to prepare event handlers
 * for forms and fix links to submit the navigation form.
 */
function _checkLoad(
  event,
  navFormName,
  excludeList
  )
{
  // first restore page state
  restorePartialPageState();

  for (var i = 0; i < document.forms.length; i++)
  {
    var form = document.forms[i];

    // Note: event listener functions must already be defined above
    //       no forward references
    if (form.addEventListener) // DOM events
    {
      form.addEventListener('focus', _enterField, true);
      form.addEventListener('keydown', _resetOnEscape, true);
    }
    else if (form.attachEvent) // IE5 events
    {
      form.attachEvent('onfocusin', _enterField);
      form.attachEvent('onkeydown', _resetOnEscape);
    }
  }

  if (navFormName != (void 0))
  {
    var exclude = new Object();

    if (excludeList != (void 0))
    {
      for (var j = 0; j < excludeList.length; j++)
        exclude[excludeList[j]] = 1;
    }

    _fixAllLinks(navFormName, window, exclude);
  }

  // If we're inside a frameset, and the top frame wants
  // reloads blocked, install a _noReload handler.
  // chain _monitor also if that is already set.
  if ((self != top) && top["_blockReload"])
  {
    // If _monitor is already set on document.onkeydown, then
    // chain _noReload and _monitor by calling another function which
    // calls both: _monitorNoReload.
    // _monitor is set on IE only.
    if ((_agent.isIE)
        && (document.onkeydown != null)
        && (((document.onkeydown).toString().indexOf('_monitor')) > 0))
    {
      document.onkeydown = _monitorNoReload;
    }
    else
    {
      document.onkeydown = _noReload;
    }
  }

  // Set initialFocus if necessary
  if ((!_agent.isNav) && (_initialFocusID != null))
  {
    var myElement = _getElementById(document, _initialFocusID);
    if (myElement && myElement.focus)
    {
      myElement.focus();
      if (myElement.type == 'text')
        myElement.select();
    }
  }

  if (!_agent.isNav)
    _loadScriptLibraries(document);

}


//
// Event handle that blocks keys that lead to a page reloading.
//
function _noReload(e)
{
  if (!e) e=window.event;
  var kc=e.keyCode;
  // F5 and Ctrl-R
  if ((kc==116)||(kc==82 && e.ctrlKey))
  {
    if (e.preventDefault) e.preventDefault();
    e.keyCode=0;
    return false;
  }
}

//
// Event handle that monitors access keys and
// blocks keys that lead to a page reloading.
// used for our dialogs.
//
function _monitorNoReload(e)
{
  if (_agent.isIE)
    _monitor(e);
  return _noReload(e);
}


//
// Deliver a client event with the specified type, source and parameters
// to the handler body.
//
function _handleClientEvent(type, source, params, handlerBody)
{
  var event = new Object();
  event.type = type;
  event.source = source;
  event.params = params;
  var func = new Function("event", handlerBody);
  return func(event);
}


//
// APIs dealing with cookies.  We currently have no supported
// public functions.  _getCookie() and _setCookie() are good candidates.
//

function _getCookie(name)
{
  var dc = document.cookie;

  var value = "";
  var prefix = name + "=";
  if (dc)
  {
    // Look for the cookie name in in the middle.
    var begin = dc.indexOf("; " + prefix);

    if (begin < 0)
    {
      // Not there: look for it at the beginning
      begin = dc.indexOf(prefix);
      if (begin > 0)
        begin = -1;
    }
    else
      // Found it - now skip over the colon and space
      begin += 2;

    if (begin >= 0)
    {
      var end = dc.indexOf(";", begin);
      if (end < 0)
        end = dc.length;

      value = unescape(dc.substring(begin + name.length + 1, end));
    }
  }

  return value;
}

//
// Sets a cookie value.
// This function isn't especially general (yet) as it doesn't
// allow overriding the domain, path, or expiry.
//
function _setCookie(name, value)
{
  // Compute the domain to scope as widely as is legit
  // by scoping off.
  // =-=AEW "localhost" just doesn't work.  I don't know how to
  // deal with this...  It'll confuse developers, even though
  // it will never have any real impact.
  var domain = window.location.host;

  /* =-=AEW The current Oracle Cookie design guidelines require
     cookies to _not_ be scoped widely.  So don't!
  var periodIndex = domain.indexOf(".");
  if ((periodIndex >= 0) &&
      (domain.lastIndexOf(".") != periodIndex))
  {
    // But don't scope off anything that's entirely a number,
    // since then we're probably dealing with an IP address
    var startOfDomain = domain.substr(0, periodIndex);
    if (!(((startOfDomain * startOfDomain) == 0) ||
          ((startOfDomain / startOfDomain) == 1)))
      domain = domain.substr(periodIndex);
  }
  */

  var colonIndex = domain.indexOf(":");
  if (colonIndex >= 0)
    domain = domain.substr(0, colonIndex);

  // Expire 10 years after today
  var expires = new Date();
  expires.setFullYear(expires.getFullYear() + 10);

  // And here's the cookie:
  // (Reordering the parameters seemed to break some browsers)
  var curCookie = name + "=" + value +
      "; path=/;domain=" + domain + "; expires=" + expires.toGMTString();

  document.cookie = curCookie;
}


//
// Set a single value in the UIX cookie
//
function _setUIXCookie(index, value)
{
  var arry = _getUIXCookie();
  arry[index] = value;

  // Rebuild the encoded value
  var encodedValue = arry[0];
  for (var i = 1; i < arry.length; i++)
  {
    encodedValue = encodedValue + "^" + arry[i];
  }

  _setCookie("oracle.uix", encodedValue);
}


//
// Extract the decoded form of the UIX cookie
//
function _getUIXCookie()
{
  var encodedValue = _getCookie("oracle.uix");
  var arry;
  if (encodedValue)
    arry = encodedValue.split("^");
  else
    arry = new Array("0", "", "");

  return arry;
}


//
// Defaults the time zone
//
function _defaultTZ()
{
  var tz = _getUIXCookie()[2];

  // If the time zone has already been set, then bail:  however,
  // time zones that start with "GMT" are inherently unreliable,
  // because they won't track daylight savings.  For such time zones,
  // always replace the value.
  if (tz && (tz.indexOf("GMT") != 0))
  {
    return;
  }

  // Set the correct time zone.
  _setUIXCookie(2, _getTimeZoneID());
}


//
// Compute the time zone ID, of form GMT+-XX:YY
//
function _getTimeZoneID()
{
  // Get the time zone offset, then flip the sign, as this
  // is opposite in meaning to the time zone ID
  var tzOffset = -(new Date()).getTimezoneOffset();
  var newTZ;

  // Build up the name of the time zone
  if (tzOffset > 0)
    newTZ = "GMT+";
  else
  {
    newTZ = "GMT-";
    tzOffset = -tzOffset;
  }

  var minutes = "" + tzOffset % 60;
  if (minutes.length == 1)
    minutes = "0" + minutes;
  return (newTZ + (Math.floor(tzOffset / 60)) + ":" + minutes);
}


/**
 * _monitor: monitor the document's onkeydown presses.
 * Activate the uix button that has the access key that matches the
 * key that is pressed along with the ALT key (ALT+accessKey).
 * This is called for IE, not on Mac, only.
 *
 * Background: we want the accesskey to work on our UIX
 * buttons (which are links wrapped around an image)
 * like it does for a <button> html element, and that is
 * it activates the button rather than sets focus like it does for a link.
 * (This already works as expected in Mozilla)
 * This javascript code must know that the element is our uix button.
 * In ButtonRenderer, we render a bogus attribute for this purpose,
 * called "uixbtn".
 * We also register document.onkeydown=_monitor on the page.
 */
function _monitor(e)
{

  // when the user holds down only the accessKey, this triggers a mouse
  // down event over and over with a keyCode of 18. So...
  // we make sure not to call the _findAccessKey function if
  // the user is holding down just the access key.

  var windowEvent = window.event;

  if ((windowEvent.altKey == true) && (windowEvent.ctrlKey == false) &&
      (windowEvent.keyCode != null) && (windowEvent.keyCode != 18 )
      && (!windowEvent.repeat) )
  {
    var keyPressed = String.fromCharCode(window.event.keyCode);

    var accessKeyNode = _getNodeWithAccessKey(document, keyPressed);

    // if the element is a UIX Button, make it act like an accelerator

    if (accessKeyNode != null && (accessKeyNode.getAttribute("uixbtn") != null))
    {

      // now I need to find the node to click...

      // if label, get the "for" attribute value.
      // That is the id of the element to click
      if (accessKeyNode.htmlFor)
      {
          var id=accessKeyNode.htmlFor;
          accessKeyNode = (id != null)
                          ? window.document.getElementById(id)
                          : null;
      }

      if (accessKeyNode != null)
      {
        // IE acts like it sets the focus before it triggers the access key,
        // so we will do the same.

        accessKeyNode.focus();
        accessKeyNode.click();
      }

    } // end if (accessKeyNode != null)
  }
  return true;
}

/**
 * This function is passed a DOM Node object and a keyboard key
 * that was pressed at the same time as the ALT key.
 * return the node with the access key, otherwise null.
 */
function _getNodeWithAccessKey(node, keyPressed)
{
  var keyPressedUpper = keyPressed.toUpperCase();
  var keyPressedLower = keyPressed.toLowerCase();

  var accessKeyObject =
  {
    activeFound:false,
    firstAccessKeyNode:null,
    accessKeyNode:null
  }

  // start looking through the DOM Node tree from the beginning
  // for the access key the user pressed.
  // The node returned will be the first one
  // found starting from the current active element, otherwise we will use the
  // first one found in the DOM tree regardless of the current active element.

  accessKeyObject = _findAccessKey(document,
                                   accessKeyObject,
                                   keyPressedUpper,
                                   keyPressedLower);

  var accessKeyNode = accessKeyObject.accessKeyNode;
  var firstAccessKeyNode = accessKeyObject.firstAccessKeyNode;

  // if we didn't find accessKeyNode in the tree from the active node
  // to the end of the tree,
  // just use the first element we found with the correct access key.

  if ( (accessKeyNode == null) && (firstAccessKeyNode != null) )
  {
    accessKeyNode = firstAccessKeyNode;
  }
  return accessKeyNode;
}

/**
 * This function is passed a DOM Node object, a local accessKeyObject,
 * and the keyPressed in upper and lower cases.
 * This function checks to see if the node
 * represents an HTML tag; i.e., if the node is an Element object. It
 * recursively calls itself on each of the children of the node, testing
 * them in the same way. If you invoke this function by passing it the
 * Document object, it traverses the entire DOM tree.

 * Find the element with the access key.
 * first, store the first element with the access key encountered in
 * the accessKeyObject.firstAccessKeyNode.
 * Then, find the first element with the access key
 * that is found after the focus element,
 * since this is how it works on IE for <button></button>
 * return accessKeyObject if the access key node found, otherwise return null.
 */
function _findAccessKey(node, accessKeyObject, keyPressedUpper, keyPressedLower)
{
  // check all nodes of type ELEMENT_NODE
  if (node.nodeType == 1 /*Node.ELEMENT_NODE*/)
  {
    // check if the _keyPressed  is the access key
    if ( (node.accessKey == keyPressedUpper) ||
          (node.accessKey == keyPressedLower))
    {
      if (accessKeyObject.activeFound == true)
      {
        // Great! Found accessKey on an element after the focus, so return
        // the node. The recursion will unwind and pass out this node to
        // the original call.
        accessKeyObject.accessKeyNode = node;
        return accessKeyObject;
      }
      else if (accessKeyObject.firstAccessKeyNode == null)
      {
        // the focus hasn't been found yet, but store away this node
        // in the global variable, since it is the first one found
        accessKeyObject.firstAccessKeyNode = node;
      }
    }
    // If the focus is on an element with the access key, IE activates the NEXT
    // element, which is why I set the _activeFound flag after checking the
    // access key.
    if (node == document.activeElement)
    {
      accessKeyObject.activeFound = true;
    }
  } // end check if it is an element node

  // Now get all children of n
  // Loop through the children
  // Recurse on each one

  var children = node.childNodes;
  for(var i=0; i < children.length; i++)
  {
    var accessKeyObject =
    _findAccessKey( children[i],
                    accessKeyObject,
                    keyPressedUpper,
                    keyPressedLower);

     if (accessKeyObject.accessKeyNode != null)
     {
      return accessKeyObject; // stop looping.
     }
  }
  return accessKeyObject;
}


//
// Returns true if the value contains nothing but strings
//
function _isEmpty(value)
{
  var s = "" + value;
  var i = 0;
  while (i < s.length)
  {
    if (s.charAt(i) != ' ')
      return false;
    i++;
  }

  return true;
}

//
// Returns true if the current document reads left to right.
//
function _isLTR()
{
  return document.documentElement["dir"].toUpperCase() == "LTR";
}

//
// _pprConsumeClick: Helps implement blocking. This function just consumes
//                   every click that falls within the body.
//
function _pprConsumeClick(event)
{
  if (_agent.isIE)
  {
    var body = document.body;
    if ((event.x < body.offsetLeft) || (event.y < body.offsetTop)
        || (event.x > body.offsetWidth) || (event.y > body.offsetHeight))
    {

      // OK, we've caught an event outside the body of the document. Assume
      // that the user is clicking somewhere on the menu bar, or anther window.
      // At this point, we release the mouse and continue (that's better than
      // keeping the user in limbo).

      _pprStopBlocking(window);
    }
  }
  // Mozilla is much better about this, we only catch the clicks within
  // the div element expanded to the full size of the body, so every click
  // gets blocked.
  return false;
}

//
// _pprStartBlocking: Starts consuming every click (to implement blocking)
//
function _pprStartBlocking(win)
{
  // In order to force the nice impatient user to allow a PPR update to
  // complete, we block all mouse clicks between the start of a PPR update, and
  // the end. We do this by building a dummy DIV element and having it grab all
  // clicks. On Mozilla, we just expand it to cover the entire body as a glass
  // frame. On IE, we leave the DIV at zero size, but route every click to it.
  if (!win._pprBlocking)
  {
    var body = win.document.body;
    var divElement = _getElementById(win.document, _pprdivElementName);

    if (divElement)
    {
      if (_agent.isGecko)
      {
        var w = body.offsetWidth + 16;
        var h = body.offsetHeight;
        // expand the div element to cover the entire page.
        divElement.style.height = h;

        // add in 16 pixels to cover the scrollbar area
        divElement.style.width = w;

        // If the user clicks the stop button, then we'll be stuck blocking.
        // So we don't hang, this timeout will clear the block in eight
        // seconds whether we've finished or not, but first we clear any
        // previously existing timeout.
        if (top._pprBlockingTimeout != null)
        {
          top.clearTimeout(top._pprBlockingTimeout);
        }
        top._pprBlockingTimeout = top.setTimeout("_pprStopBlocking(window);",
                                                 8000);
      }
      else if (_agent.isIE)
      {
        // DIV captures every event
        divElement.setCapture();

        // save off the element we'll return focus to
        _pprEventElement = window.document.activeElement;

        // catch keyPress events too...
        if (win._pprEventElement)
          divElement.focus();

        // save current cursor and display a wait cursor
        top._pprSavedCursor = body.style.cursor;
        body.style.cursor = "wait";
      }
      win._pprBlocking = true;
    }
  }
}

//
// _pprStopBlocking: Finishes up the blocking, releases the page back
//                   to normal processing
//
function _pprStopBlocking(win)
{
  if (win._pprBlocking)
  {
    var divElement = _getElementById(win.document, _pprdivElementName);
    if (_agent.isGecko)
    {
      // just resize to zero
      divElement.style.width = 0;
      divElement.style.height = 0;

      // If we've set a timeout, clear it now.
      if (top._pprBlockingTimeout != null)
      {
        top.clearTimeout(top._pprBlockingTimeout);
        top._pprBlockingTimeout == null;
      }
      // Mozilla doesn't support setting focus to a DIV. Have to find a way to
      // eat the keyPress events.
    }
    else if (_agent.isIE)
    {
      // Stop capturing mouse and keyPress events, and reset the cursor to
      // whatever it was before.
      divElement.releaseCapture();
      if (win._pprEventElement)
        win._pprEventElement.focus();
      win.document.body.style.cursor = top._pprSavedCursor;
    }
    win._pprEventElement = null;
  }
  win._pprBlocking = false;
}

// handle the onClick or onBlur for an IE SELECT element
// Returns true if the user has finally made a selection, and is ready to go.
function _pprChoiceAction()
{
  // this function is only needed to handle IE's weird select element
  if (!_agent.isIE)
    return true;

  var rv = false;

  // This gets called as both onClick and onBlur, but both really only want
  // to submit the event if a change has been made.
  if ((!top._pprBlocking) && (_pprChoiceChanged))
  {
    // clear the choice tracker
    _pprChoiceChanged = false;
    rv = true;
  }
  return rv;
}

// handle the onChange for an IE SELECT element
function _pprChoiceChangeEvent(event)
{
  if (!_agent.isIE)
    return true;

  // Just remember the fact that a change has occurred.
  if (!top._pprBlocking)
    _pprChoiceChanged = true;

  return true;
}

// Get the keycode from an event
function _getKC(event)
{
  if (window.event)
    return window.event.keyCode;
  else if (event)
    return event.which;
  return -1;
}


// Returns true if a form has been submitted a "short" time before newDate
function _recentSubmit(newDate)
{
  if (_lastDateSubmitted)
  {
    var diff = newDate - _lastDateSubmitted;
    if ((diff >= 0) && (diff < 200))
      return true;
  }
  return false;
}

// Returns true if a form has been reset a "short" time before newDate
function _recentReset(newDate)
{
  if (_lastDateReset)
  {
    var diff = newDate - _lastDateReset;
    if ((diff >= 0) && (diff < 200))
      return true;
  }
  return false;
}

/*
 * After updates we often can't just set focus to a node, it has to be prepared
 * in a browser specific way (different idosyncracies cause poor focus
 * behavior).
 */
function _pprFocus(node, doc)
{
  if (_agent.isIE)
  {
    // On IE, if a node has focus and we update it, then setting focus to it
    // seems to have no effect. Setting the focus to another node, then back to
    // the target seems to work correctly. Here we set the focus to a hidden
    // field.
    var divnode = _getElementById(doc, _pprdivElementName);
    if ((divnode) && (divnode["focus"]))
      divnode.focus();

    // IE often needs a bit more coaxing to really set the focus
    node.setActive();
  }
  node.focus();
}

// For PPR Back issue:
// save the page's contents into a hidden, disabled field.
// This function gets called when the beforeunload event fires, which is
// an IE-only event.
function _savePageStateIE()
{
  // the PPR Back Button support only works for IE.
  if (!_agent.isIE)
    return;

  var theSpan = _getElementById(document, "_pprPageContent");
  if (theSpan == null)
    return;

  var savePageField = _getElementById(document, "_pprSavePage");
  if (savePageField == null)
    return;

  savePageField.value = theSpan.outerHTML;

}

// For PPR Back issue:
// save the scripts and libraries with every partial page response
// so that the scripts and libraries can be re-loaded when the user
// comes back to the page after navigating off of it. (this is called
// from _partialChange())
function _saveScripts(targetDocument)
{
  // the PPR Back Button support only works for IE.
  if (!_agent.isIE)
    return;

  // save inline scripts by appending them to the previous saved scripts.
  var saveScriptElement= _getElementById(targetDocument, "_pprSaveScript");

  if (saveScriptElement != null)
  {
    var _pprScripts = _getCommentedScript(document, "_pprScripts");
    saveScriptElement.value =
      saveScriptElement.value + " " + _pprScripts;
  }

  // save the library by appending to the already saved libraries
  // if it isn't already there.
  var saveLibrariesElement = _getElementById(targetDocument, "_pprSaveLib");

  if (saveLibrariesElement != null && (window["_pprLibraries"] != (void 0)))
  {

    for (var i = 0; (i < _pprLibraries.length); i++)
    {

      if (saveLibrariesElement.value.indexOf(_pprLibraries[i]) == -1)
      {
        if (saveLibrariesElement.value != "")
         saveLibrariesElement.value += "," + _pprLibraries[i];
        else
          saveLibrariesElement.value += _pprLibraries[i];
      }
    }
  }
}


// For PPR Back issue:
// restore the page state when the user comes back to the page
// after navigating off of it.
// This includes copying the saved html, loading the saved libraries,
// and eval'ing the inline scripts (done from ScriptEval.js).
// This function is called from _checkLoad
function restorePartialPageState()
{
  // the PPR Back Button support only works for IE.
  if (!_agent.isIE)
    return;

  // get the saved page state from the hidden, disabled field
  // and write it to the page
  var savePageField   = _getElementById(document, "_pprSavePage");

  // if the field doesn't exist or if there was nothing saved in it, return.
  if (savePageField == null || savePageField.value == "")
    return;

  var theSpan = _getElementById(document, "_pprPageContent");

  if (theSpan == null)
    return;

  theSpan.outerHTML = savePageField.value;

  // next, get the libraries out of the saved field, and parse each one,
  // and reload.
  var saveLibrariesElement = _getElementById(document, "_pprSaveLib");
  if (saveLibrariesElement != null && saveLibrariesElement.value != "")
  {
    var libraries = saveLibrariesElement.value.split(",");
    _loadScriptLibrariesIE(document, libraries);
  }

  // Note: The inline javascript is eval'd in ScriptEval.js, so that it
  // is guaranteed that the libraries are all loaded first.
  // set this variable so that the saved script will be loaded, as opposed
  // to the script that is in _pprScripts, which gets loaded with every
  // ppr response.
  _pprBackRestoreInlineScripts = true;
}



// DateFormat
//-----------------------------------------------------------------------------
/*
** Copyright (c) Oracle Corporation 2000. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/


// External variables used:
//  _dfLenient:  if set at all, parsing will be lenient.  If not
//    set, parsing will be strict.
//  _df2DYS: Sets the two-digit year start.

var _AD_ERA = void 0;
var _dfLenient;

function _getADEra()
{
  if (_AD_ERA == (void 0))
  {
    _AD_ERA = new Date(0);
    _AD_ERA.setFullYear(1);
  }
  
  return _AD_ERA;
}

function _simpleDateFormat(
  formatTime
  )
{
  var stringHolder = new Object();
  stringHolder.value ="";
  
  var pattern = this._pattern;
  if (typeof pattern != "string")
    pattern = pattern[0];
    
  _doClumping(pattern,
              this._localeSymbols,
              _subformat,
              formatTime,
              stringHolder);
  
  return stringHolder.value;
}


/**
 * Parses a String into a Date using the current object's pattern.  If the
 * parsing fails, undefined will be returned.
 */
function _simpleDateParse(
  parseString
  )
{
  var pattern = this._pattern;
  if (typeof pattern == "string")
  {
    return _simpleDateParseImpl(parseString,
                                pattern,
                                this._localeSymbols);
  }
  else
  { 
    var i;
    for (i = 0; i < pattern.length; i++)
    {
      var date = _simpleDateParseImpl(parseString,
                                      pattern[i],
                                      this._localeSymbols);
      if (date != (void 0))
        return date;
    }
  }
}

function _simpleDateParseImpl(
  parseString,
  parsePattern,
  localeSymbols)
{
  var parseContext = new Object();
  parseContext.currIndex = 0;
  parseContext.parseString = parseString;
  parseContext.parsedHour = (void 0);
  parseContext.parsedMinutes = (void 0);
  parseContext.parsedSeconds = (void 0);
  parseContext.parsedMilliseconds = (void 0);
  parseContext.isPM = false;
  parseContext.parsedBC = false;
  parseContext.parsedFullYear = (void 0);
  parseContext.parsedMonth = (void 0);
  parseContext.parsedDate = (void 0);
  parseContext.parseException = new ParseException();

  var parsedTime = new Date(0);
  parsedTime.setDate(1);

  // parse the time
  if (_doClumping(parsePattern,
                  localeSymbols,
                  _subparse,
                  parseContext,
                  parsedTime))
  {
    if (parseString.length != parseContext.currIndex)
    {
      return (void 0);
    }

    // Set the parsed year, if any;  adjust for AD vs. BC
    var year = parseContext.parsedFullYear;
    if (year != (void 0))
    {
      // convert year to BC
      if (parseContext.parsedBC)
      {
        year = _getADEra().getFullYear() - year;
      }

      parsedTime.setFullYear(year);
      parseContext.parsedFullYear = year;
    }

    // Set the parsed month, if any
    var month = parseContext.parsedMonth;
    if (month != (void 0))
      parsedTime.setMonth(month);

    // Set the parsed day-of-month, if any
    var date = parseContext.parsedDate;
    if (date != (void 0))
      parsedTime.setDate(date);

    // Set the parsed hour, if any.  Adjust for AM vs. PM
    var hour = parseContext.parsedHour;
    if (hour != (void 0))
    {
      if (parseContext.isPM && (hour < 12))
      {
        hour += 12;
      }

      parsedTime.setHours(hour);
      parseContext.parsedHour = hour;
    }

    // Set the parsed minutes, if any
    var minutes = parseContext.parsedMinutes;
    if (minutes != (void 0))
      parsedTime.setMinutes(minutes);

    // Set the parsed seconds, if any
    var seconds = parseContext.parsedSeconds;
    if (seconds != (void 0))
      parsedTime.setSeconds(seconds);

    // Set the parsed milliseconds, if any
    var milliseconds = parseContext.parsedMilliseconds;
    if (milliseconds != (void 0))
      parsedTime.setMilliseconds(milliseconds);

    // so far we have done a lenient parse
    // now we check for strictness
    if (!_isStrict(parseContext, parsedTime))
    {
      return (void 0);
    }

    return parsedTime;
  }
  else
  {
    // failure
    return (void 0);
  }
}

/**
 * Determine whether the parsed time is a strictly parsed value.
 */
function _isStrict(
  parseContext,
  parsedTime)
{
  var checks = ["FullYear", "Month", "Date", "Hours", "Minutes",
                "Seconds", "Milliseconds"];

  for (var i=0; i < checks.length; i++)
  {
    var parsed = "parsed" + checks[i];

    if (parseContext[parsed] != (void 0) &&
        parseContext[parsed] != parsedTime["get" + checks[i]]())
    {
      // failure for strict parsing
      return false;
    }
  }

  return true;
}


/**
 * Clump up similar runs of pattern characters from the format patter and
 * call the subfunction for each result.  Return whether the clumping
 * succeeded.
 */
function _doClumping(
  formatPattern,
  localeSymbols,
  subFunction,
  param,
  outValue
  )
{  
  var formatLength = formatPattern.length;
  var inQuote      = false;
  var kindCount    = 0;
  var lastChar     = void 0;
  var startIndex   = 0;
  
  for (var i = 0; i < formatLength; i++)
  {
    var currChar = formatPattern.charAt(i);
    
    if (inQuote)
    {
      if (currChar == "\'")
      {
        inQuote = false;
        
        // handle to single quotes in a row as escaping the quote
        // by not skipping it when outputting
        if (kindCount != 1)
        {
          startIndex++;
          kindCount--;
        }

        // output the quoted text
        if (!subFunction(formatPattern,
                         localeSymbols,
                         "\'",
                         startIndex,
                         kindCount,
                         param,
                         outValue))
        {
          // failure
          // alert("failure at " + startIndex + " with " + lastChar);
          return false;
        }
        
        kindCount = 0;
        lastChar  = void 0;
      }
      else
      {
        // keep adding characters to the escaped String
        kindCount++;
      }
    }
    else
    {
      // the characters that we are collecting have changed
      if (currChar != lastChar)
      {
        if (kindCount != 0)
        {       
          // output the previously collected string
          if (!subFunction(formatPattern,
                           localeSymbols,
                           lastChar,
                           startIndex,
                           kindCount,
                           param,
                           outValue))
          {
            // failure
            //alert("failure at " + startIndex + " with " + lastChar);
            return false;
          }
          
          kindCount = 0;
          lastChar  = void 0;
        }
        
        if (currChar == '\'')
        {
          inQuote = true;
        }
  
        startIndex = i;      
        lastChar = currChar;
      }

      // keep collecting this kind of character together
      kindCount++;
    }   
  }
  
  // output any left over substring being collected
  if (kindCount != 0)
  {
    if (!subFunction(formatPattern,
                     localeSymbols,
                     lastChar,
                     startIndex,
                     kindCount,
                     param,
                     outValue))
    {
      // failure
      //alert("failure at " + startIndex + " with " + lastChar);
      return false;
    }
  }
  
  // success
  return true;
}


/**
 * Format a clump of pattern elements using the specified time.
 */
function _subformat(
  inString,
  localeSymbols,
  formatType,
  startIndex,
  charCount,
  time,
  stringHolder
  )
{    
  // string to append to the toString
  var appendString = null;
  
  var amPMAdjust = false;
    
  if ((formatType >= 'A') && (formatType <= 'Z') ||
      (formatType >= 'a') && (formatType <= 'z'))
  {
    switch (formatType)
    {
      case 'D': // day in year
        appendString = "(Day in Year)";
        break;
      
      case 'E': // day in week
      {
        var dayOfWeek = time.getDay();
        
        appendString = (charCount <= 3)
                         ? localeSymbols.getShortWeekdays()[dayOfWeek]
                         : localeSymbols.getWeekdays()[dayOfWeek];
      }
      break;
      
      case 'F': // day of week in month
        appendString = "(Day of week in month)";
        break;
      
      case 'G': // era designator
      {
        var eras = localeSymbols.getEras();
        
        appendString = (time.getTime() < _getADEra().getTime())
                         ? eras[0]
                         : eras[1];
 
      }
      break;
                  
      case 'M': // month in year
      {
        var monthIndex = time.getMonth();
        
        if (charCount <= 2)
        {
          // use the month number
          appendString = _getPaddedNumber(monthIndex + 1, charCount);
        }
        else if (charCount == 3)
        {
          // use the month abbreviation
          appendString = localeSymbols.getShortMonths()[monthIndex];
        }
        else
        {
          // use the full month name
          appendString = localeSymbols.getMonths()[monthIndex];
        }
      }
      break;
      
      case 'S': // millisecond (0 - 999)
        appendString = _getPaddedNumber(time.getMilliseconds(), charCount);
        break;
      
      case 'W': // week in month
        appendString = "(Week in Month)";
        break;
      
      case 'a': // am/pm marker
      {
        var amPMs = localeSymbols.getAmPmStrings();
        
        appendString =  (_isPM(time.getHours()))
                          ? amPMs[1]
                          : amPMs[0];
      }
      break;
      
      case 'd': // day in month
        appendString = _getPaddedNumber(time.getDate(), charCount);
        break;
      
      case 'h': // hour in am/pm (1-12)
        hours = time.getHours();
        
        if (_isPM(hours))
          hours -= 12;
        
        if (hours == 0)
          hours = 12;

        appendString = _getPaddedNumber(hours, charCount);
        break;
      
      case 'K': // hour in am/pm (0-11)
         hours = time.getHours();
        
        if (_isPM(hours))
           hours -= 12;
         
        appendString = _getPaddedNumber(hours, charCount);
        break;

      case 'k': // hour in day (1-24)
        hours = time.getHours();
        if (hours == 0)
          hours = 24;
        appendString = _getPaddedNumber(hours, charCount);
        break;

      case 'H': // hour in day (0-23)
        appendString = _getPaddedNumber(time.getHours(), charCount);
        break;


      case 'm': // minute in hour 0 - 59)
        appendString = _getPaddedNumber(time.getMinutes(), charCount);
        break;
 
      case 's': // seconds in minute 0 - 59)
        appendString = _getPaddedNumber(time.getSeconds(), charCount);
        break;
   
      case 'w': // week in year
        appendString = "(Week in year)";
        break;
      
      case 'y': // year
      {
        var year = time.getFullYear();
        
        // truncate 2 and 1 digit years to that number of digits
        var maxDigits = (charCount <= 2)
                          ? charCount
                          : (void 0);
                          
                          
        appendString = _getPaddedNumber(year, charCount, maxDigits);
      }
      break;
        
        
      case 'z': // timezone
      {
        var gmtHourOffset = time.getTimezoneOffset() / 60;
        
        appendString  = "GMT";
        
        if (gmtHourOffset > 0) appendString += "+";
        
        appendString += _getPaddedNumber(gmtHourOffset, 2);
        
      }
      break;
    
      default:
        // do nothing rather than throw an exception
        appendString = "";
    }
  }
  else
  {
    // all other results are literal
    appendString = inString.substring(startIndex, startIndex + charCount);
  }
  
  stringHolder.value += appendString;
  
  // formatting should never fail
  return true;
}


/**
 * Parse a substring using a clump of format elements.
 */
function _subparse(
  inString,
  localeSymbols,
  formatType,
  startIndex,
  charCount,
  parseContext,
  parsedTime
  )
{
  // Start index of the string being parsed (as opposed
  // to startIndex, which is the index on the format mask)
  var inStartIndex = parseContext.currIndex;


  if ((formatType >= 'A') && (formatType <= 'Z') ||
      (formatType >= 'a') && (formatType <= 'z'))
  {
    switch (formatType)
    {
      case 'D': // day in year
        // skip this number
        if (_accumulateNumber(parseContext, 3) == (void 0))
        {
          return false;
        }
        break;
      
      case 'E': // day in week
      {
        // extract the day but do nothing with it, as there is not setDay()
        // on Date
        var dayIndex = _matchArray(parseContext,
                                   (charCount <= 3)
                                     ? localeSymbols.getShortWeekdays()
                                     : localeSymbols.getWeekdays());
                                     
        if (dayIndex == (void 0))
        {
          return false;
        }
      }
      break;
      
      case 'F': // day of week in month
        // skip this number
        if (_accumulateNumber(parseContext, 2) == (void 0))
        {
          return false;
        }
        break;
      
      case 'G': // era designator
      {
        var eraIndex = _matchArray(parseContext, localeSymbols.getEras());
        
        if (eraIndex != (void 0))
        {
          if (eraIndex == 0)
          {
            parseContext.isBC = true;
          }
        }
        else
        {
          return false;
        }
      }
      break;
                  
      case 'M': // month in year
      {
        var monthIndex;
        var monthOffset = 0;

        if (charCount <= 2)
        {
          // match month number
          monthIndex = _accumulateNumber(parseContext, 2);
          
          // subtract 1 from the monthIndex to make it 0-based
          monthOffset = -1;
        }
        else
        {
          var nameArray = (charCount == 3)
                            ? localeSymbols.getShortMonths()
                            : localeSymbols.getMonths();

          monthIndex = _matchArray(parseContext, nameArray);
        }

        if (monthIndex != (void 0))
        {
          parseContext.parsedMonth = (monthIndex + monthOffset);
        }
        else
        {
          return false;
        }
      }
      break;

      case 'S': // millisecond (0 - 999)
      {
        var milliseconds = _accumulateNumber(parseContext, 3);

        if (milliseconds != (void 0))
        {
          parseContext.parsedMilliseconds = milliseconds;
        }
        else
        {
          return false;
        }
      }
      break;
      
      case 'W': // week in month
        // skip this number
        if (_accumulateNumber(parseContext, 2) == (void 0))
        {
          return false;
        }
        break;
      
      case 'a': // am/pm marker
      {
        var amPMIndex = _matchArray(parseContext,
                                    localeSymbols.getAmPmStrings());
        
        if (amPMIndex == (void 0))
        {
          return false;
        }
        else
        {
          if (amPMIndex == 1)
          {
            parseContext.isPM = true;
          }
        }
      }
      break;
      
      case 'd': // day in month
      {
        var dayOfMonth = _accumulateNumber(parseContext, 2);
                
        if (dayOfMonth != (void 0))
        {
          parseContext.parsedDate = dayOfMonth;
        }
        else
        {
          return false;
        }
      }
      break;
        
      case 'h': // hour in am/pm (1-12)
      case 'k': // hour in day (1-24)
      case 'H': // hour in day (0-23)
      case 'K': // hour in am/pm (0-11)
      {
        var hour = _accumulateNumber(parseContext, 2);
        
        if (hour != (void 0))
        {
          if ((formatType == 'h') && (hour == 12))
            hour = 0;
          if ((formatType == 'k') && (hour == 24))
            hour = 0;

          parseContext.parsedHour = hour;
        }
        else
        {
          return false;
        }
      }
      break;


      case 'm': // minute in hour 0 - 59)
      {
        var minutes = _accumulateNumber(parseContext, 2);
        
        if (minutes != (void 0))
        {
          parseContext.parsedMinutes = minutes;
        }
        else
        {
          return false;
        }
      }
      break;
      
      case 's': // seconds in minute 0 - 59)
      {
        var seconds = _accumulateNumber(parseContext, 2);

        if (seconds != (void 0))
        {
          parseContext.parsedSeconds = seconds;
        }
        else
        {
          return false;
        }
      }
      break;

      case 'w': // week in year
        // skip this number
        if (_accumulateNumber(parseContext, 2) == (void 0))
        {
          return false;
        }
        break;

      case 'y': // year
      {
        var year = _accumulateNumber(parseContext, 4);
        var enteredChars = parseContext.currIndex - inStartIndex;
        // if we have a 2-digit year, add in the default year
        if (year != (void 0))
        {
          if ((enteredChars > 2) &&
              (charCount <= 2) &&
              (year <= 999))
          {
            // Block bonus characters;  if they've specified
            // a two-year mask, and there's more than two characters,
            // there might be a problem.  But allow four digits.
            return false;
          }
          else if ((charCount <= 2) && (year >= 0) && (year <= 100))
          {
            year = _fix2DYear(year);
          }
          else if (charCount == 4)
          {
            // Bug 2169562: For four-digit year formats, reject
            // three-year entries.  Fair enough!
            if (enteredChars == 3)
              return false;    
            if (enteredChars <= 2)
              year = _fix2DYear(year);
          }

          // There is no year "0"
          if (year == 0)
            return false;

          parseContext.parsedFullYear = year;
        }
        else
        {
          return false;
        }
      }
      break;
        
        
      case 'z': // timezone
      {
        // consume the GMT portion
        if (!_matchText(parseContext, "GMT"))
        {
          return false;
        }
        
        // consume the plus or minus
        if (_matchArray(parseContext, ["-", "+"]) == (void 0))
        {
          return false;
        }
        
        // accumulate the offset number
        if (_accumulateNumber(parseContext, 2) == (void 0))
        {
          return false;
        }
      }
      break;
    
      default:
    }
  }
  else
  {
    // consume constants
    return _matchText(parseContext,
                      inString.substring(startIndex, startIndex + charCount));
  }
  
  // match succeeded
  return true;
}


/**
 * Fix two-digit years.
 */
function _fix2DYear(year)
{
  var defaultCentury;

  if (_df2DYS != (void 0))
  {
    // year               51    01
    // offsetYear       1950  1950
    // defaultCentury   1900  1900
    // year             1951  1901
    // year             1951  2001
    var offsetYear = _df2DYS;
    defaultCentury = offsetYear - (offsetYear % 100);

    year += defaultCentury;
    if (year < offsetYear)
      year += 100;
  }
  else
  {
    var currentYear = new Date().getFullYear();
    defaultCentury = currentYear - (currentYear % 100) - 100;

    year += defaultCentury;
 
    // if the new year is now more than 80 years in the past,
    // then it is actually a date in the future, so add the 100 years
    // back in.  The 80 years rule, matches Java's spec
    if (year + 80 < currentYear)
    {
      year += 100;
    }
  }

  return year;
}


/**
 * Match the current text against an array of possibilities, returning
 * the index of the succesful match, or undefined if no match succeeded.
 */
function _matchArray(
  parseContext,
  matchArray
  )
{
  for (var i = 0; i < matchArray.length; i++)
  {
    if (_matchText(parseContext, matchArray[i]))
    {
      return i;
    }
  }
  
  // no match
  return (void 0);
}


/**
 * Match the specified text in a case insensitive manner,
 * returning true and updating the
 * <code>parseContext</code> if the match succeeded.
 */
function _matchText(
  parseContext,
  text
  )
{
  // if no text to match then match will fail
  if (!text)
    return false;

  // get the length of the text to match
  var textLength  = text.length;

  var currIndex   = parseContext.currIndex;
  var parseString = parseContext.parseString;
  
  // determine whether we have enough of the parseString left to match
  if (textLength > parseString.length - currIndex)
  {
    return false;
  }

  //
  // Convert to lowercase for case insensitive match
  //
  // =-= bts Maybe toLocaleLowerCase would be better, but that would cause
  //         problems if the browser locale were different from the application
  //         locale.
  //  
  var parseText  = parseString.substring(currIndex, currIndex + textLength);
  var parseMatch = parseText.toLowerCase();
  var textMatch  = text.toLowerCase();
  
  if (parseMatch != textMatch)
    return false;
    
  // update the current parseContext
  parseContext.currIndex += textLength;
  
  return true;
}
 

/**
 * Accumlates and returns a number at this location or undefined, if
 * there is no number.
 */
function _accumulateNumber(
  parseContext,
  maxLength
  )
{
  var startIndex  = parseContext.currIndex;
  var currIndex   = startIndex;
  var parseString = parseContext.parseString;
  var parseLength = parseString.length;
  if (parseLength > currIndex + maxLength)
    parseLength = currIndex + maxLength;

  var currValue = 0;

  // gather up all of the digits
  while (currIndex < parseLength)
  {
    var currDigit = parseDigit(parseString.charAt(currIndex));

    if (!isNaN(currDigit))
    {
      // add on the digit and shift over the results
      currValue *= 10;
      currValue += currDigit;

      currIndex++;
    }
    else
    {
      break;
    }
  }

  if (startIndex != currIndex)
  {
    // update the current parseContext
    parseContext.currIndex = currIndex;

    // return the numeric version
    return currValue;
  }
  else
  {
    // no number at this location
    return (void 0);
  }
}


/**
 * Returns true if the hour index is considered PM.
 */
function _isPM(
  hours
  )
{
  return (hours >= 12);
}


/**
 * Pad out a number with leading 0's to meet the minDigits digits or
 * truncate to meet the minDigits.
 */
function _getPaddedNumber(
  number,
  minDigits,
  maxDigits
  )
{  
  var stringNumber = number.toString();
  
  //
  // pad out any number strings that are too short
  //
  if (minDigits != (void 0))
  {    
    var addedDigits = minDigits - stringNumber.length;
  
    while (addedDigits > 0)
    {
      stringNumber = "0" + stringNumber;
      addedDigits--;
    }
  }
  
  //
  // truncate any number strings that are too long
  //
  if (maxDigits != (void 0))
  {
    var extraDigits = stringNumber.length - maxDigits;
    
    if (extraDigits > 0)
    {
      stringNumber = stringNumber.substring(extraDigits,
                                            extraDigits + maxDigits);
    }
  }
  
  return stringNumber;
}


/**
 * Construct a SimpleDateFormat with the specifed date pattern for
 * the specified locale.
 */
function SimpleDateFormat(
  pattern,
  locale
  )
{
  // for debugging
  this._class = "SimpleDateFormat";
  
  // save the Locale elements for the specified locale, or client locale
  // if no locale is specified
  this._localeSymbols = getLocaleSymbols(locale);

  // =-= bts need to change default pattern to match JDK
  if (pattern == (void 0))
    pattern = this._localeSymbols.getShortDatePatternString();

  // Bug 2002065: 
  // Be forgiving of users who prefer a different separator
  // and alternative month styles.
  if (_dfLenient)
  {
    if (typeof(pattern) == "string" ) 
      pattern = [pattern];

    // now we have guaranteed an array, we can extend it to include
    // more forgiving patterns

    // first add in replacements for month parsing
    var baseCount = pattern.length;
    for (var i=0; i < baseCount; i++)
    {
      if (pattern[i].indexOf('MMM') != -1)
      {
        pattern[pattern.length] = pattern[i].replace(/MMM/g, 'MM');
        pattern[pattern.length] = pattern[i].replace(/MMM/g, 'M');
      }
    }

    // now add support for all of the above with any of
    // the separators below
    var baseCount = pattern.length;
    for (var i=0; i < baseCount; i++)
    {
      if (pattern[i].indexOf('/') != -1)
      {
        pattern[pattern.length] = pattern[i].replace(/\//g, '-');
        pattern[pattern.length] = pattern[i].replace(/\//g, '.');
      }
      if (pattern[i].indexOf('-') != -1)
      {
        pattern[pattern.length] = pattern[i].replace(/-/g, '/');
        pattern[pattern.length] = pattern[i].replace(/-/g, '.');
      }
      if (pattern[i].indexOf('.') != -1)
      {
        pattern[pattern.length] = pattern[i].replace(/\./g, '-');
        pattern[pattern.length] = pattern[i].replace(/\./g, '/');
      }
    }
  }

  this._pattern = pattern;
}

SimpleDateFormat.prototype = new Format();
SimpleDateFormat.prototype.format = _simpleDateFormat;
SimpleDateFormat.prototype.parse  = _simpleDateParse;

// TableProxy
//-----------------------------------------------------------------------------
/*
** Copyright (c) Oracle Corporation 2000. All Rights Reserved.
**
**34567890123456789012345678901234567890123456789012345678901234567890123456789
*/

//
// Global functions for retrieving information about a form element
// rendered in a table. Clients can use these methods to retrieve the
// name of the table, the row of the element, or the name element
// before its name was transformed.
//

// Get the table name from one of the table's form elements. Clients
// can then use this name to construct a TableProxy object.
function getTableName(
  formElement
  )
{
  var elementName = formElement.name;
  if (elementName == (void 0))
    return;

  var colonIndex = elementName.indexOf(":");
  if (colonIndex >= 0)
    return elementName.substring(0, colonIndex);
}

// Get the table row index from one of the table's form elements. The
// index is zero-based.
function getTableRow(
  formElement
  )
{
  var elementName = formElement.name;
  if (elementName == (void 0))
    return;

  var colonIndex = elementName.lastIndexOf(":");
  if (colonIndex >= 0)
  {
    var rowString = elementName.substring(colonIndex + 1);
    return Number(rowString);
  }
}

// Get the element name from one of a table's form elements. This
// is the name of the element before it is rendered into a table.
function getTableElementName(
  formElement
  )
{
  var elementName = formElement.name;
  if (elementName == (void 0))
    return;

  var nameIndex = elementName.indexOf(":");
  if (nameIndex >= 0)
  {
    var rowIndex = elementName.lastIndexOf(":");
    if (rowIndex >= 0)
    {
      return elementName.substring((nameIndex + 1), rowIndex);
    }
  }
}

// Select all the rows of the named multi-selection table.
function tableSelectAll(
  tableName
  )
{
  new TableProxy(tableName).selectAll();
}

// Deselect all the rows of the named multi-selection table.
function tableSelectNone(
  tableName
  )
{
  var proxy = new TableProxy(tableName);
  proxy.selectNone();
}

//
// TableProxy Object
//

// TableProxy instances can be used to determine information about
// a TableBean at runtime and gain access to form data it contains.
// A TableProxy can be constructed from just the name of the table,
// or it can additionally be supplied with the name of the form
// in which the table resides and the window DOM object in which
// the target table resides.
//
// If the form name is not supplied, it will be determined at runtime
// by checking all forms for the existence of the table.
//
// If the window name is not supplied the document of the caller
// will be used.

// Constructor
function TableProxy(
  tableName,
  formName,
  window
  )
{
  this.tableName = tableName;

  // get the length field
  var tableLengthName = tableName + ":length";

  // use the specified window to find the table,
  // if one is provided
  var targetDocument = document;

  if (window != (void 0))
  {
    targetDocument = window.document;
  }

  // get the form name
  if (formName != (void 0))
  {
    this.formName = formName;
  }
  else
  {
    // find the first form with this table
    var formsLength = targetDocument.forms.length;

    for (var i = 0; i < formsLength; i++)
    {
      if (targetDocument.forms[i][tableLengthName] != (void 0))
      {
        this.formName = targetDocument.forms[i].name;
        break;
      }
    }
  }

  // point to the form itself
  this._form = targetDocument.forms[this.formName];

  // store the table length
  this.length = this._form[tableLengthName].value;
}

//
// Public prototype method hookups
//

//
// Calling getValue() on a TableProxy will return the base value
// of the rows displayed on screen for that table. Thus, the
// absoloute row number of a given selected row, would be
// getValue + selectedRow.
//
TableProxy.prototype.getValue = _getTableValue;

//
// Calling getLength() on a TableProxy will return the number
// of rows displayed on screen for that table.
//
TableProxy.prototype.getLength = _getTableLength;

//
// Calling getSelectedRow() on a TableProxy will return the
// zero-based index of the currently selected row in a
// TableBean with a SingleSelectionBean. Calling this method
// when no rows are selected should return -1. Calling it
// on a TableBean with no SingleSelectionBean, or a
// MultipleSelectionBean, is undefined.
//
TableProxy.prototype.getSelectedRow = _getSelectedRow;

//
// Calling getSelectedRows() on a TableProxy will return an
// Array containing the indices of the selected rows. The
// indices will be zero-based. Calling this method
// on a TableBean with no MultipleSelectionBean, or a
// SingleSelectionBean, is undefined.
//
TableProxy.prototype.getSelectedRows = _getSelectedRows;

//
// Calling setSelectedRows(row) on a TableProxy will select
// the row in a single or multiple selection table.  The
// row number is zero-based. Calling this method
// on a TableBean with no MultipleSelectionBean or a
// SingleSelectionBean, does nothing.
//
TableProxy.prototype.setSelectedRow = _setSelectedRow;

//
// Calling getFormElement(elementName, row) on a TableProxy
// will return the element with the specified name for the
// given table row. Clients can the read or write information
// to that form element using standard JavaScript calls.
//
TableProxy.prototype.getFormElement = _getFormElement;

//
// Calling isMultiSelect on a TableProxy will return true if the proxied
// table is a multiple selection table.
//
TableProxy.prototype.isMultiSelect = _isMulti;

//
// Calling selectAll() on a TableProxy will select all
// rows in a multiple selection table and mark the "selectMode"
// field of the table to the value "all". It will take no action
// on a table without multiple selection.
//
TableProxy.prototype.selectAll = _selectAll;

//
// Calling getSelectAll() on a TableProxy will return true if
// the table is a multi-select table AND select all has been set.
//
TableProxy.prototype.getSelectAll = _getSelectAll;

//
// Calling selectNone() on a TableProxy will deselect all
// rows in a multiple selection table and mark the "selectMode"
// field of the table to the value "none". It will take no action
// on a table without multiple selection.
//
TableProxy.prototype.selectNone = _selectNone;

//
// Calling getSelectNone() on a TableProxy will return true if
// the table is a multi-select table AND select none has been set.
//
TableProxy.prototype.getSelectNone = _getSelectNone;

//
// private prototypes
//

TableProxy.prototype._multiSelect = _multiSelect;

//
// Method implementations
//

// has selection
function _hasSelection()
{
  var form = this._form;
  // test for single selection
  var testSelection = this.tableName + ":selected";
  if (form[testSelection] != (void 0))
    return true;

  // test for multiple selection: the text key should be the same as
  // UIConstants.SELECT_MODE_KEY
  testSelection = this.tableName + ":selectMode";
  return (form[testSelection] != (void 0));
}

// get the value (the base row number)
function _getTableValue()
{
  var valueVarName = "_" + this.tableName + "_value";
  if (window[valueVarName] != (void 0))
    return window[valueVarName] - 1;
  // If the variable hasn't been written, assume we're at the start
  return 0;
}

// get the selected row
function _getSelectedRow()
{
  // if no selection, return -1
  if (!(this._hasSelection()))
    return -1;

  var currElement = this._form[this.tableName + ":selected"];
  if (currElement == (void 0))
    return -1;
  else if (currElement.length != (void 0))
  {
    var radioButton;

    for (var i = 0; i < currElement.length; i++)
    {
      radioButton = currElement[i];
      if ((radioButton != (void 0)) && radioButton.checked)
        return i;
    }
  }
  else if (currElement.checked)
  {
    return currElement.value;
  }

  return -1;
}

// get the selected row
function _getSelectedRows()
{
  // if no selection, return -1
  if (!(this._hasSelection()))
    return -1;

  var currElement;
  var result = new Array();
  var selectedCount = 0;

  for (var i = 0; i < this.length; i++)
  {
    currElement = this.tableName + ":selected:" + i;
    currElement = this._form[currElement];
    if ((currElement != (void 0)) && currElement.checked)
    {
      result[selectedCount] = i;
      selectedCount++;
    }
  }

  return result;
}

// get the total rows displayed
function _getTableLength()
{
  return this.length;
}

// get the form element for a table, column, row
function _getFormElement(
  elementName,
  row
  )
{
  var name = this.tableName + ":" + elementName + ":" + row;

  return this._form[name];
}
// is this a multiple selection table?
function _isMulti()
{
 var zeroElement = this.tableName + ":selected:0";
 zeroElement = this._form[zeroElement];
 return (zeroElement != (void 0));
}

// select all rows in a multiple select table
function _selectAll()
{
  this._multiSelect(true);
}

// True if selectAll has been clicked
function _getSelectAll()
{
  if (!(this._hasSelection()))
    return false;

  var selectModeName = this.tableName + ":selectMode";
  var selectModeField = this._form[selectModeName];
  if (selectModeField != (void 0))
  {
    return selectModeField.value == "all";
  }
  return false;
}

// deselect all rows in a multiple select table
function _selectNone()
{
  this._multiSelect(false);
}

// True if selectNone has been clicked
function _getSelectNone()
{
  if (!(this._hasSelection()))
    return false;

  var selectModeName = this.tableName + ":selectMode";
  var selectModeField = this._form[selectModeName];
  if (selectModeField != (void 0))
  {
    return selectModeField.value == "none";
  }
  return false;
}

function _multiSelect(
  selectAll
  )
{
  // if no selection, return
  if (!(this._hasSelection()))
    return;

  var currElement;
  var selectedCount = 0;

  for (var i = 0; i < this.length; i++)
  {
    currElement = this.tableName + ":selected:" + i;
    currElement = this._form[currElement];
    if ((currElement != (void 0)) &&
        (!currElement.disabled))
    {
      currElement.checked = selectAll;
    }
  }

  // set the select mode field. the text key should be the same as
  // UIConstants.SELECT_MODE_KEY
  var selectModeName = this.tableName + ":selectMode";
  var selectModeField = this._form[selectModeName];
  if (selectModeField != (void 0))
  {
    if (selectAll)
      selectModeField.value = "all";
    else
      selectModeField.value = "none";
  }
}

function _tableProxyToString()
{
  var value = "TableProxy, tableName=" + this.tableName + "\n";
  value += "form=" + this.formName + "\n";
  value += "_hasSelection=" + this._hasSelection() + "\n";
  value += "selectedRow=" + this.getSelectedRow() + "\n";
  value += "selectedRows=" + this.getSelectedRows() + "\n";
  value += "getLength=" + this.getLength() + "\n";
  value += "selectMode=" ;

  var selectModeName = this.tableName + ":selectMode";
  var selectModeField = this._form[selectModeName];
  if (selectModeField != null)
    value += selectModeField.value + "\n";
  else
    value += "NULL\n";

  return value;
}

// set the selected row
function _setSelectedRow(row)
{
  if (row == (void 0))
    return;

  // get the single selection element
  var currElement = this._form[this.tableName + ":selected"];

  if (currElement == (void 0))
  {
    // if no single selection element, try for the multi-selection element.
    currElement = this._form[this.tableName + ":selected:"+row];
  }

  if (currElement == (void 0))
  {
    // if no elements have that name, return
    return;
  }
  else if (currElement.length != (void 0))
  {

    // loop through each element and find the one with the value
    // of row. Then set the checked attribute to true.

    for (var i = 0; i < currElement.length; i++)
    {

      if ((currElement[i] != (void 0)) && (currElement[i].value==row))
        currElement[i].checked = true;
    }
  }
  else if (currElement.value == row)
  {
    currElement.checked = true;
  }

  return;
}


//
// "private" prototype method hookups
//
TableProxy.prototype._hasSelection = _hasSelection;
TableProxy.prototype.toString = _tableProxyToString;
